<?php
/**
 * Thrive Themes - https://thrivethemes.com
 *
 * @package thrive-apprentice
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Silence is golden
}

/**
 * Class TVA_Course
 */
class TVA_Course extends TVA_Term {
	/**
	 * @var int
	 */
	protected $term_id;

	/**
	 * TVA_Course constructor.
	 *
	 * @param $term_id
	 * @param array $filters
	 */
	public function __construct( $term_id, $filters = array() ) {

		$this->filters = $filters;
		$this->term_id = (int) $term_id;

		$args = $this->get_args();

		if ( class_exists( 'WP_Term_Query' ) ) {
			$term_query = new WP_Term_Query( $args );
		}

		foreach ( $term_query->terms as $term ) {
			if ( $term->term_id === $this->term_id ) {
				$this->term = $term;
			}
		}

		if ( empty( $this->term ) ) {
			return false;
		}

		$this->get_settings();
		$this->term = $this->get_term();
	}

	/**
	 * @return array
	 */
	protected function get_args() {
		$args = array(
			'taxonomy'   => TVA_Const::POST_TAXONOMY,
			'hide_empty' => false,
			'meta_query' => array(
				'relation' => 'AND',
			),
		);

		if ( isset( $this->filters['published'] ) ) {
			$args['meta_query'][] = array(
				'key'     => 'tva_status',
				'value'   => 'publish',
				'compare' => 'IN',
			);
		} elseif ( isset( $this->filters['private'] ) ) {
			$args['meta_query'][] = array(
				'key'     => 'tva_status',
				'value'   => 'private',
				'compare' => 'IN',
			);
		} else {
			$args['meta_query'][] = array(
				'key'     => 'tva_status',
				'value'   => array( 'publish', 'draft' ),
				'compare' => 'IN',
			);
		}

		$args['meta_key'] = 'tva_order';
		$args['orderby']  = 'meta_value_num';
		$args['order']    = 'ASC';

		return $args;
	}

	/**
	 * This will return an un-grouped array of elements which are children of an element
	 *
	 * @param $id
	 *
	 * @return array
	 */
	public function get_element_children( $id ) {
		$elements = $this->search_in_modules( $id, $this->term->modules );

		if ( empty( $elements ) ) {
			$elements = $this->search_in_chapters( $id, $this->term->chapters );
		}

		return $elements;
	}

	/**
	 * @param $id
	 * @param $modules
	 *
	 * @return array
	 */
	public function search_in_modules( $id, $modules ) {
		$elements = array();

		if ( ! empty( $modules ) ) {
			foreach ( $modules as $module ) {

				// we've found the element let's return it's children
				if ( $module->ID == $id ) {
					if ( ! empty( $module->lessons ) ) {
						return $module->lessons;
					}

					if ( ! empty( $module->chapters ) ) {
						foreach ( $module->chapters as $chapter ) {
							$elements[] = $chapter;
							$elements   = array_merge( $elements, $chapter->lessons );
						}

						return $elements;
					}
				} else {
					$elements = $this->search_in_chapters( $id, $module->chapters );

					if ( ! empty( $elements ) ) {
						return $elements;
					}
				}
			}
		}

		return $elements;
	}

	/**
	 *
	 * @param $id
	 * @param $chapters
	 *
	 * @return array
	 */
	public function search_in_chapters( $id, $chapters ) {
		$elements = array();

		foreach ( $chapters as $chapter ) {
			if ( $chapter->ID == $id ) {
				return $chapter->lessons;
			}

		}

		return $elements;
	}


}