<?php
/**
 * Created by PhpStorm.
 * User: User
 * Date: 3/23/2018
 * Time: 17:46
 */

class TVA_Chapters_Controller extends TVA_REST_Controller {
	/**
	 * @var string
	 */
	public $base = 'chapters';

	/**
	 * @var
	 */
	public $post_id;
	/**
	 * @var
	 */
	public $course_id;

	/**
	 * @var WP_REST_Request $request
	 */
	public $request = array();

	/**
	 * @var array
	 */
	public $settings = array();

	/**
	 * The course Object
	 *
	 * @var TVA_Course array
	 */
	public $course;

	/**
	 * Published items
	 *
	 * @var array
	 */
	public $published = array();

	/**
	 * Register Routes
	 */
	public function register_routes() {

		register_rest_route( self::$namespace . self::$version, '/' . $this->base, array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'new_chapter' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/(?P<ID>[\d]+)', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_chapter' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'edit_chapter' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_order/', array(
			array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => array( $this, 'update_chapter_order' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
		) );
		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/move_chapters/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'move_chapters' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
		) );
		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/group_as_chapter/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'group_as_chapter' ),
				'permission_callback' => array( $this, 'chapters_permissions_check' ),
				'args'                => array(),
			),
		) );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function new_chapter( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$args = array(
			'post_title'     => $request->get_param( 'post_title' ),
			'post_type'      => TVA_Const::CHAPTER_POST_TYPE,
			'post_parent'    => $request->get_param( 'post_parent' ),
			'post_status'    => $request->get_param( 'post_status' ),
			'comment_status' => $request->get_param( 'comment_status' ),
		);

		$result = wp_insert_post( $args );

		if ( ! is_wp_error( $result ) ) {
			$term_id = $request->get_param( 'course_id' );

			update_post_meta( $result, 'tva_chapter_order', $request->get_param( 'order' ) );
			wp_set_object_terms( $result, (int) $term_id, TVA_Const::POST_TAXONOMY );

			$lessons         = $request->get_param( 'lessons' );
			$post            = get_post( $result );
			$post->course_id = $term_id;
			$post->order     = (int) $request->get_param( 'order' );

			if ( ! empty( $lessons ) ) {
				$post->lessons = array();

				foreach ( $lessons as $lesson ) {
					wp_update_post( array( 'ID' => $lesson['ID'], 'post_parent' => $result ) );

					$lesson['post_parent'] = $result;
					$post->lessons[]       = $lesson;
				}
			}

			return new WP_REST_Response( $post, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function edit_chapter( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$chapter_id = $request->get_param( 'ID' );
		$post_title = $request->get_param( 'post_title' );

		$args = array(
			'ID'          => $chapter_id,
			'post_title'  => $post_title,
			'post_status' => $request->get_param( 'post_status' ),
		);

		$update = wp_update_post( $args );
		$post   = get_post( $chapter_id );

		if ( ! is_wp_error( $update ) ) {
			/**
			 * We don't need those data and we unset them because they trigger unnecessary 'change' events in js
			 */
			unset( $post->post_modified );
			unset( $post->post_modified_gmt );

			$this->course_id = $request->get_param( 'course_id' );

			$this->handle_membership_plugin_setup();

			return new WP_REST_Response( $post, 200 );
		}

		return new WP_Error( 'no-results', __( $update->get_error_message(), TVA_Const::T ) );
	}

	/**
	 * Handle the membership plugin setup, like adding things needed to the DB
	 */
	public function handle_membership_plugin_setup() {
		global $tva_db;

		$course         = tva_get_course_by_id( $this->course_id );
		$user_settings  = TVA_Settings::instance();
		$this->settings = $user_settings->get_settings();

		foreach ( $this->settings['membership_plugin'] as $membership ) {
			$tva_db->tva_protection_manager( $course, $membership['tag'] );
		}
	}

	/**
	 * Update the order to show the chapters to
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function update_chapter_order( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		/**
		 * @var WP_REST_Request $request
		 */
		$order   = $request->get_param( 'order' );
		$post_id = $request->get_param( 'ID' );

		update_post_meta( $post_id, 'tva_chapter_order', $order );

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * Move chapters around
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function move_chapters( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$ids    = $request->get_param( 'ids' );
		$result = wp_set_object_terms( $ids, (int) $request->get_param( 'course_id' ), TVA_Const::POST_TAXONOMY );

		if ( empty( $result ) || is_wp_error( $result ) ) {
			return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
		}

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function delete_chapter( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$chapter_id = $request->get_param( 'ID' );

		$terms = wp_get_post_terms( $chapter_id, TVA_Const::POST_TAXONOMY );

		if ( ! empty( $terms ) ) {
			$this->course_id = $terms[0]->term_id;

			$this->course = new TVA_Course( $this->course_id );
			$children     = $this->course->get_element_children( $chapter_id );

			foreach ( $children as $child ) {
				wp_delete_post( $child->ID );
			}
		}
		$post   = get_post( $chapter_id );
		$delete = wp_delete_post( $chapter_id );

		if ( $post->post_parent > 0 ) {

			$this->unpublish_parent( $post->post_parent );

		}

		if ( ! is_wp_error( $delete ) ) {
			return new WP_REST_Response( $this->published, 200 );
		}

		return new WP_Error( 'delete_failed', __( 'Failed to delete Chapter. Please try again later!', TVA_Const::T ) );
	}

	/**
	 * Unpublish the module if needed
	 *
	 * @param $items
	 *
	 * @return WP_REST_Response
	 */
	/**
	 * Recursively unpublish parents
	 *
	 * @param $parent_id
	 */
	private function unpublish_parent( $parent_id ) {
		$this->course = new TVA_Course( $this->course_id );
		$children     = $this->course->get_element_children( $parent_id );
		$published    = false;

		foreach ( $children as $child ) {
			if ( $child->post_status == 'publish' ) {
				$published = true;
			}
		}

		if ( ! $published ) {
			$parent = get_post( $parent_id );
			wp_update_post( array( 'ID' => $parent->ID, 'post_status' => 'draft' ) );
			$this->published[] = (int) $parent_id;
		}
	}

	/**
	 * Check if user is logged in and is an administrator
	 *
	 * @return bool
	 */
	public function chapters_permissions_check() {
		return current_user_can( 'manage_options' );
	}

	/**
	 * Group items as chapters
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function group_as_chapter( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$items = $request->get_param( 'items' );

		foreach ( $items as $item ) {
			$children        = $item['item_ids'];
			$this->course_id = $item['course_id'];

			$route   = '/' . self::$namespace . self::$version . '/' . $this->base;
			$chapter = new WP_REST_Request( 'POST', $route );
			$chapter->set_query_params( $item );
			$chapter->set_header( 'X-TVA-Request-ID', get_option( 'tva_request_id' ) );


			$response = rest_do_request( $chapter );
			$server   = rest_get_server();
			$data     = $server->response_to_data( $response, false );

			if ( $data ) {
				$course          = new TVA_Courses_Controller();
				$course->term_id = $this->course_id;

				foreach ( $children as $child ) {
					$post            = get_post( $child );
					$old_post_parent = $post->post_parent;

					wp_update_post( array( 'ID' => $child, 'post_parent' => $data->ID ) );


					if ( $old_post_parent > 0 ) {
						$old_post = get_post( $post->post_parent );
						if ( $old_post->post_type === TVA_Const::CHAPTER_POST_TYPE ) {
							$course->unpublish_parent( $old_post_parent );
						}
					}
				}
			}
		}

		$course = new TVA_Course( $this->course_id );

		return new WP_REST_Response( $course->get_term(), 200 );
	}
}