<?php

/**
 * Class TVA_Courses_Controller
 */
class TVA_Courses_Controller extends TVA_REST_Controller {
	/**
	 * @var string
	 */
	public $base = 'courses';

	/**
	 * @var
	 */
	public $term_id;

	/**
	 * @var WP_REST_Request $request
	 */
	public $request;

	/**
	 * @var array
	 */
	public $settings = array();

	/**
	 * The course Object
	 *
	 * @var TVA_Course array
	 */
	public $course;

	/**
	 * Published items
	 *
	 * @var array
	 */
	public $published = array();

	/**
	 * Register Routes
	 */
	public function register_routes() {

		register_rest_route( self::$namespace . self::$version, '/' . $this->base, array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'new_course' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_courses' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/(?P<ID>[\d]+)', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_course' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'edit_course' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_order/', array(
			array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => array( $this, 'update_courses_order' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/search_users', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'search_users' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/bulk_action', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'handle_bulk_action' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_posts_order', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'update_posts_order' ),
				'permission_callback' => array( $this, 'courses_permissions_check' ),
				'args'                => array(),
			),
		) );
	}

	/**
	 * Return all existing courses
	 *
	 * @return array
	 */
	public function get_courses() {
		return tva_get_courses();
	}

	/**
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function new_course( $request ) {
		/**
		 * We should add the new course
		 *
		 * @var WP_REST_Request $request
		 */
		$this->request = $request;
		$name          = $this->request->get_param( 'name' );
		$result        = '';

		$args = array(
			'description' => $request->get_param( 'description' ),
			'slug'        => $request->get_param( 'slug' ),
		);

		$course = get_term_by( 'name', $name, TVA_Const::POST_TAXONOMY );

		if ( $course ) {
			$this->term_id = $course->term_id;
			$return        = array(
				'term_id'          => $course->term_id,
				'term_taxonomy_id' => $course->term_id,
			);
		} else {
			$result = wp_insert_term( $name, TVA_Const::POST_TAXONOMY, $args );

			if ( is_wp_error( $result ) ) {
				// If the category already exists we will import the lessons in the same category
				if ( isset( $result->error_data['term_exists'] ) ) {
					$this->term_id              = $result->get_error_data();
					$return['term_id']          = $this->term_id;
					$return['term_taxonomy_id'] = $this->term_id;
				} else {
					return new WP_Error( 'no-results', __( $result->get_error_message(), TVA_Const::T ) );
				}
			} else {
				$this->term_id = $result['term_id'];
				$return        = $result;
			}
		}

		$return['url'] = get_term_link( $this->term_id, TVA_Const::POST_TAXONOMY );
		$this->update_term_meta();

		/**
		 * When importing the courses, instead of endlessly  doing api
		 * calls for each lesson of the course, we're adding the lessons here
		 * (!!! a newly created course only has lessons if it's imported !!!)
		 */
		$lessons = $request->get_param( 'lessons' );
		if ( ! empty( $lessons ) ) {
			$inserted_lessons = array();
			$has_comments     = false;
			$order            = 0;
			foreach ( $lessons as $lesson ) {
				if ( $lesson['checked'] == true ) {
					$args = array(
						'post_title'    => $lesson['post_title'],
						'post_name'     => $lesson['post_title'],
						'post_type'     => TVA_Const::POST_TYPE,
						'post_excerpt'  => $lesson['post_excerpt'],
						'post_category' => array( $this->term_id ),
						'post_status'   => 'draft',
					);

					$post_id = wp_insert_post( $args );

					if ( ! is_wp_error( $post_id ) ) {
						$inserted_lessons[] = $post_id;
						$comments           = get_comments( array( 'post_id' => $lesson['old_id'] ) );

						if ( ! empty( $comments ) ) {
							if ( $has_comments === false ) {
								$has_comments             = true;
								$return['comment_status'] = 'open';
								update_term_meta( $this->term_id, 'tva_comment_status', 'open' );
							}

							foreach ( $comments as $comment ) {
								$comment->comment_post_ID = $post_id;
								wp_update_comment( $comment->to_array() );
							}
						}

						/**
						 * Import the Featured Image
						 */
						$featured_image = has_post_thumbnail( $lesson['old_id'] ) ? get_the_post_thumbnail_url( $lesson['old_id'], 'full' ) : '';

						if ( ! empty( $featured_image ) ) {
							update_post_meta( $post_id, 'tva_cover_image', $featured_image );
						}

						/**
						 * Import the lessond type, and if it's a video import the urls and other data
						 */
						$lesson_type = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_lesson_type', true );
						$video_type  = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_type', true );
						$audio_type  = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_audio_type', true );
						$video_extra = array();
						$url         = '';

						if ( $lesson_type == 'video' ) {
							if ( $video_type == 'youtube' ) {

								$url             = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_url', true );
								$hide_related    = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_hide_related', true );
								$hide_logo       = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_hide_logo', true );
								$hide_controls   = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_hide_controls', true );
								$hide_title      = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_hide_title', true );
								$autoplay        = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_autoplay', true );
								$hide_fullscreen = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_youtube_hide_fullscreen', true );
								if ( $hide_related ) {
									$video_extra['hide-related'] = 1;
								}
								if ( $hide_logo ) {
									$video_extra['hide-logo'] = 1;
								}
								if ( $hide_controls ) {
									$video_extra['hide-controls'] = 1;
								}
								if ( $hide_title ) {
									$video_extra['hide-title'] = 1;
								}
								if ( $autoplay ) {
									$video_extra['autoplay'] = 1;
								}
								if ( $hide_fullscreen ) {
									$video_extra['hide-full-screen'] = 1;
								}
							} elseif ( $video_type == 'vimeo' ) {
								$url = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_vimeo_url', true );
							} elseif ( $video_type == 'custom' ) {
								$url = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_custom_url', true );
							} else {
								$video_type = 'custom';
								$url        = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_video_custom_embed', true );
							}

						} elseif ( $lesson_type == 'audio' ) {
							// we only import soundcloud url
							if ( $audio_type == 'soundcloud' ) {
								$video_type = '';
								$url        = get_post_meta( $lesson['old_id'], '_thrive_meta_appr_audio_soundcloud_url', true );
							}
						} else {
							$video_type = '';
						}

						/**
						 * we should update video options
						 */
						$media_type = '';

						if ( ( $lesson_type == 'audio' ) && ( $audio_type == 'soundcloud' ) ) {
							$media_type = 'soundcloud';
						} elseif ( $lesson_type == 'video' ) {
							$media_type = $video_type;
						}

						if ( ! empty( $media_type ) ) {
							$media_args = array(
								'media_type'          => $media_type,
								'media_url'           => $url,
								'media_extra_options' => $video_extra,
							);

							update_post_meta( $post_id, 'tva_post_media', $media_args );
						}

						update_post_meta( $post_id, 'tva_lesson_type', $lesson_type );
						wp_set_object_terms( $post_id, $this->term_id, TVA_Const::POST_TAXONOMY );

						/**
						 * Get the TCB content
						 */

						$content_before_more = get_post_meta( $lesson['old_id'], 'tve_content_before_more', true );
						$content_more_found  = get_post_meta( $lesson['old_id'], 'tve_content_more_found', true );
						$save_post           = get_post_meta( $lesson['old_id'], 'tve_save_post', true );
						$custom_css          = get_post_meta( $lesson['old_id'], 'tve_custom_css', true );
						$user_custom_css     = get_post_meta( $lesson['old_id'], 'tve_user_custom_css', true );
						$page_events         = get_post_meta( $lesson['old_id'], 'tve_page_events', true );
						$updated_post        = get_post_meta( $lesson['old_id'], 'tve_updated_post', true );
						$search_content      = get_post_meta( $lesson['old_id'], 'tve_search_content', true );
						$globals             = get_post_meta( $lesson['old_id'], 'tve_globals', true );
						$icon_pack           = get_post_meta( $lesson['old_id'], 'thrive_icon_pack', true );
						$masonry             = get_post_meta( $lesson['old_id'], 'tve_has_masonry', true );
						$typefocus           = get_post_meta( $lesson['old_id'], 'tve_has_typefocus', true );
						$wistia_popover      = get_post_meta( $lesson['old_id'], 'tve_has_wistia_popover', true );

						if ( ! empty( $lesson['post_content'] ) ) {
							$non_tcb_content = '<div class="tve_wp_shortcode thrv_wrapper"><div class="tve_shortcode_raw" style="display: none">___TVE_SHORTCODE_RAW__' . $lesson['post_content'] . '__TVE_SHORTCODE_RAW___</div></div>';
							$save_post       .= $non_tcb_content;
							$updated_post    .= $non_tcb_content;
						}

						/**
						 * put the TCB content in
						 */
						update_post_meta( $post_id, 'tve_content_before_more', $content_before_more );
						update_post_meta( $post_id, 'tve_content_more_found', $content_more_found );
						update_post_meta( $post_id, 'tve_save_post', $save_post );
						update_post_meta( $post_id, 'tve_custom_css', $custom_css );
						update_post_meta( $post_id, 'tve_user_custom_css', $user_custom_css );
						update_post_meta( $post_id, 'tve_page_events', $page_events );
						update_post_meta( $post_id, 'tve_updated_post', $updated_post );
						update_post_meta( $post_id, 'tve_search_content', $search_content );
						update_post_meta( $post_id, 'tve_globals', $globals );
						update_post_meta( $post_id, 'thrive_icon_pack', $icon_pack );
						update_post_meta( $post_id, 'tve_has_masonry', $masonry );
						update_post_meta( $post_id, 'tve_has_typefocus', $typefocus );
						update_post_meta( $post_id, 'tve_has_wistia_popover', $wistia_popover );

						/** @var WP_Post $post */
						$post               = get_post( $post_id );
						$post->course_id    = $this->term_id;
						$post->tcb_edit_url = tva_get_editor_url( $post_id );
						$post->cover_image  = $featured_image;
						$post->lesson_type  = $lesson_type;
						$post->post_media   = get_post_meta( $post_id, 'tva_post_media', $args );

						$args = array(
							'posts_per_page' => - 1,
							'post_type'      => TVA_Const::POST_TYPE,
							'post_status'    => array( 'publish', 'draft' ),
							'tax_query'      => array(
								array(
									'taxonomy' => TVA_Const::POST_TAXONOMY,
									'field'    => 'term_id',
									'terms'    => array( $this->term_id ),
									'operator' => 'IN',
								),
							),
						);

						$posts = get_posts( $args );

						foreach ( $posts as $other_post ) {
							$other_post = tva_get_post_data( $other_post );
							if ( $order < $other_post->order ) {
								$order = $other_post->order;
							}
						}

						$post->order = ++ $order;
						update_post_meta( $post->ID, 'tva_lesson_order', $post->order );

						$return['lessons'][] = $post;
						update_post_meta( $lesson['old_id'], 'tva_imported', true );
					};
				}
			}

			if ( $has_comments === true ) {
				foreach ( $inserted_lessons as $lesson_id ) {
					wp_update_post(
						array( 'ID' => $lesson_id, 'comment_status' => 'open' )
					);
				}

				foreach ( $return['lessons'] as $lesson ) {
					$lesson->comment_status = 'open';
				}
			}
		}

		do_action( 'tva_after_save_course', $return, $request );

		if ( $return ) {
			return new WP_REST_Response( $return, 200 );
		}

		return new WP_Error( 'no-results', __( $result->get_error_message(), TVA_Const::T ) );
	}

	/**
	 * Update term meta
	 */
	public function update_term_meta() {
		$term_id        = $this->term_id;
		$user_settings  = TVA_Settings::instance();
		$this->settings = $user_settings->get_settings();

		update_term_meta( $term_id, 'tva_cover_image', $this->request->get_param( 'cover_image' ) );
		update_term_meta( $term_id, 'tva_order', $this->request->get_param( 'order' ) );
		update_term_meta( $term_id, 'tva_level', $this->request->get_param( 'level' ) );
		update_term_meta( $term_id, 'tva_logged_in', $this->request->get_param( 'logged_in' ) );
		update_term_meta( $term_id, 'tva_message', $this->request->get_param( 'message' ) );
		update_term_meta( $term_id, 'tva_roles', $this->request->get_param( 'roles' ) );
		update_term_meta( $term_id, 'tva_topic', $this->request->get_param( 'topic' ) );
		update_term_meta( $term_id, 'tva_author', $this->request->get_param( 'author' ) );
		update_term_meta( $term_id, 'tva_status', $this->request->get_param( 'status' ) );
		update_term_meta( $term_id, 'tva_description', $this->request->get_param( 'description' ) );
		update_term_meta( $term_id, 'tva_label', $this->request->get_param( 'label' ) );
		update_term_meta( $term_id, 'tva_label_name', $this->request->get_param( 'label_name' ) );
		update_term_meta( $term_id, 'tva_label_color', $this->request->get_param( 'label_color' ) );
		update_term_meta( $term_id, 'tva_excluded', $this->request->get_param( 'excluded' ) );
		update_term_meta( $term_id, 'tva_membership_ids', $this->request->get_param( 'membership_ids' ) );
		update_term_meta( $term_id, 'tva_bundle_ids', $this->request->get_param( 'bundle_ids' ) );
		update_term_meta( $term_id, 'tva_video_status', $this->request->get_param( 'video_status' ) );
		update_term_meta( $term_id, 'tva_term_media', $this->request->get_param( 'term_media' ) );
		update_term_meta( $term_id, 'tva_comment_status', $this->request->get_param( 'comment_status' ) );

		/**
		 * Check if course comment status has benn changed, if so update comment status for all lessons within that course
		 */
		if ( TVA_Const::TVA_IS_COURSE_COMMENT_STATUS_CHANGED !== $this->request->get_param( 'comment_status_changed' ) ) {
			$ids = $this->request->get_param( 'post_ids' );

			if ( isset( $ids ) && is_array( $ids ) ) {
				TVA_Lessons_Controller::update_lesons_comment_status( $ids, $this->request->get_param( 'comment_status' ) );
			}
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}
	}

	/**
	 * Handle the membership plugin setup, like adding things needed to the DB
	 */
	public function handle_membership_plugin_setup() {
		/**
		 * we have the course in the request but it's an array, and for consistency
		 * with other actions done on the $tva_db class we get the course object rather
		 * than keeping the array and doing a lot of other checks afterwords
		 */
		$course = tva_get_course_by_id( $this->term_id );

		global $tva_db;
		$settings = $this->settings;

		foreach ( $settings['membership_plugin'] as $membership ) {
			$tva_db->tva_protection_manager( $course, $membership['tag'] );
		}
	}

	/**
	 * Check if the user has permissions to do API calls
	 *
	 * @return bool
	 */
	public function courses_permissions_check() {
		return current_user_can( 'manage_options' );
	}

	/**
	 * Delete the course
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function delete_course( $request ) {
		/**
		 * @var WP_REST_Request $request
		 */
		$term_id = $request->get_param( 'ID' );

		/**
		 * Also delete the course lessons
		 */
		$args = array(
			'posts_per_page' => - 1,
			'post_type'      => array( TVA_Const::POST_TYPE, TVA_Const::CHAPTER_POST_TYPE, TVA_Const::MODULE_POST_TYPE ),
			'post_status'    => array( 'publish', 'draft' ),
			'tax_query'      => array(
				array(
					'taxonomy' => TVA_Const::POST_TAXONOMY,
					'field'    => 'term_id',
					'terms'    => array( $term_id ),
					'operator' => 'IN',
				),
			),
		);

		$posts = get_posts( $args );

		$this->delete_course_comments( $term_id );
		$result = wp_delete_term( $term_id, TVA_Const::POST_TAXONOMY );

		if ( ! empty( $posts ) ) {
			foreach ( $posts as $post ) {

				if ( empty( $this->settings ) ) {
					$user_settings  = TVA_Settings::instance();
					$this->settings = $user_settings->get_settings();
				}

				if ( $this->settings['membership_plugin'] ) {
					global $tva_db;
					foreach ( $this->settings['membership_plugin'] as $membership ) {
						$fn = 'delete_' . $membership['tag'] . '_restrictions';
						$tva_db->$fn( $post );
					}
				}

				wp_delete_post( $post->ID, true );
			}
		}

		if ( $result ) {
			return new WP_REST_Response( $result, 200 );
		}

		return new WP_Error( 'no-results', __( 'No course was deleted!', TVA_Const::T ) );
	}

	/**
	 * Delete course comments
	 *
	 * @param $course_id
	 */
	public function delete_course_comments( $course_id ) {
		$hidden_post_id = get_option( 'tva_course_hidden_post_id' );
		$args           = array(
			'post_id'    => $hidden_post_id,
			'meta_key'   => 'tva_course_comment_term_id',
			'meta_value' => $course_id,
		);

		$comments = get_comments( $args );

		if ( ! empty( $comments ) ) {
			foreach ( $comments as $comment ) {
				wp_delete_comment( $comment->comment_ID, true );
				delete_comment_meta( $comment->comment_ID, 'tva_course_comment_term_id', $course_id );
			}
		}
	}

	/**
	 * Edit course
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function edit_course( $request ) {
		/**
		 * We should add the new course
		 */
		$this->request = $request;
		$this->term_id = $this->request->get_param( 'ID' );

		$args = array(
			'name'        => $this->request->get_param( 'name' ),
			'description' => '',
			'slug'        => $this->request->get_param( 'slug' ),
		);

		$result = wp_update_term( $this->term_id, TVA_Const::POST_TAXONOMY, $args );

		$result['url'] = get_term_link( $result['term_id'], TVA_Const::POST_TAXONOMY );

		$this->update_term_meta();

		do_action( 'tva_after_save_course', $result, $request );

		if ( $result ) {
			return new WP_REST_Response( $result, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * @param $status
	 * Update comment status for all courses and lessons
	 */
	public static function tva_update_courses_comment_status( $status ) {
		$terms = get_terms( array( 'taxonomy' => TVA_Const::POST_TAXONOMY ) );

		foreach ( $terms as $term ) {
			update_term_meta( $term->term_id, 'tva_comment_status', $status );
		}

		$args = array(
			'post_type'      => array( TVA_Const::POST_TYPE, TVA_Const::MODULE_POST_TYPE ),
			'posts_per_page' => - 1,
			'post_status'    => array( 'publish', 'draft' ),
		);

		$posts = get_posts( $args );

		foreach ( $posts as $post ) {
			wp_update_post( array( 'ID' => $post->ID, 'comment_status' => $status ) );
		}
	}

	/**
	 * Update courses order
	 *
	 * @param $request
	 *
	 * @return WP_REST_Response
	 */
	public function update_courses_order( $request ) {
		/**
		 * @var WP_REST_Request $request
		 */
		$new_order = $request->get_param( 'new_order' );
		foreach ( $new_order as $term_id => $order ) {
			update_term_meta( $term_id, 'tva_order', $order );
		}

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * @param $request
	 *
	 * @return array
	 */
	public function search_users( $request ) {

		/**
		 * @var WP_REST_Request $request
		 */
		$term   = $request->get_param( 'term' );
		$result = array();

		$args  = array(
			'orderby'     => 'login',
			'order'       => 'ASC',
			'search'      => '*' . $term . '*',
			'count_total' => false,
			'fields'      => 'all',
		);
		$users = get_users( $args );

		foreach ( $users as $user ) {
			$result[] = array(
				'id'    => $user->data->ID,
				'type'  => $user->data->user_email,
				'label' => $user->data->display_name,
				'url'   => get_avatar_url( $user->data->ID ),
			);
		}

		return $result;
	}

	/**
	 * @param  WP_REST_Request $request
	 *
	 * @return mixed
	 */
	public function handle_bulk_action( $request ) {
		if ( ! self::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$user_settings  = TVA_Settings::instance();
		$this->settings = $user_settings->get_settings();
		$this->term_id  = $request->get_param( 'course_id' );
		$data           = $request->get_param( 'items' );
		$action         = $request->get_param( 'action' );

		$fn = $action . '_bulk_items';

		if ( $action === 'move' ) {
			$high_level = $request->get_param( 'high_level' );
			$result     = $this->$fn( $data, $high_level );
		} else {
			$result = $this->$fn( $data );
		}


		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}

		return $result;
	}

	/**
	 * Bulk delete items
	 *
	 * @param $items
	 *
	 * @return string
	 */
	public function delete_bulk_items( $items ) {
		$posts = array();
		foreach ( $items as $item ) {
			$posts[] = get_post( $item );

			wp_delete_post( $item );
		}

		foreach ( $posts as $post ) {
			if ( ( $post instanceof WP_Post ) && $post->post_parent > 0 ) {

				$this->unpublish_parent( $post->post_parent );
			}
		}

		return tva_get_course_by_id( $this->term_id );
	}

	/**
	 * Bulk publish items
	 *
	 * @param $items
	 *
	 * @return WP_REST_Response
	 */
	public function publish_bulk_items( $items ) {
		$this->course = new TVA_Course( $this->term_id );

		foreach ( $items as $item ) {
			wp_update_post( array( 'ID' => $item, 'post_status' => 'publish' ) );
			$this->published[] = (int) $item;

			$post = get_post( $item );
			if ( $post->post_parent > 0 ) {
				$this->publish_parent( $post->post_parent );

			}
		}

		return new WP_REST_Response( $this->published, 200 );
	}

	/**
	 * Bulk unpublish items
	 *
	 * @param $items
	 *
	 * @return WP_REST_Response
	 */
	public function unpublish_bulk_items( $items ) {
		foreach ( $items as $item ) {
			wp_update_post( array( 'ID' => $item, 'post_status' => 'draft' ) );
			$this->published[] = (int) $item;
			$post              = get_post( $item );

			if ( $post->post_parent > 0 ) {
				$this->unpublish_parent( $post->post_parent );

			}
		}

		return new WP_REST_Response( $this->published, 200 );
	}

	/**
	 * Recursively unpublish parents
	 *
	 * @param $parent_id
	 */
	public function unpublish_parent( $parent_id ) {
		$this->course = new TVA_Course( $this->term_id );
		$children     = $this->course->get_element_children( $parent_id );
		$published    = false;

		foreach ( $children as $child ) {
			if ( $child->post_status == 'publish' ) {
				$published = true;
			}
		}

		if ( ! $published ) {
			$parent = get_post( $parent_id );
			if ( $parent instanceof WP_Post ) {

				wp_update_post( array( 'ID' => $parent->ID, 'post_status' => 'draft' ) );
				$this->published[] = (int) $parent_id;

				if ( $parent->post_parent > 0 ) {
					$this->unpublish_parent( $parent->post_parent );
				}
			}
		}
	}

	/**
	 * Recursively publish parents
	 *
	 * @param $parent_id
	 */
	private function publish_parent( $parent_id ) {
		foreach ( $this->course->posts as $post ) {

			if ( $post->ID === (int) $parent_id ) {
				wp_update_post( array( 'ID' => $post->ID, 'post_status' => 'publish' ) );
				$this->published[] = (int) $parent_id;
				if ( $post->post_parent > 0 ) {
					$this->publish_parent( $post->post_parent );
				}
			}
		}
	}

	/**
	 * Bulk move items
	 *
	 * @param $items
	 *
	 * @return WP_REST_Response
	 */
	public function move_bulk_items( $items, $high_level ) {
		$new_course_id = '';
		$response      = array();

		$order        = 0;
		$this->course = new TVA_Course( $items[0]['course_id'] );
		$term         = $this->course->get_term();

		foreach ( $items as $item ) {
			$new_course_id = (int) $item['course_id'];
			wp_set_object_terms( $item['ID'], (int) $item['course_id'], TVA_Const::POST_TAXONOMY );

			if ( $item['post_type'] === 'tva_' . $high_level ) {
				wp_update_post( array( 'ID' => $item['ID'], 'post_parent' => $item['post_parent'] ) );
				$old_parents[] = get_post( $item['old_post_parent'] );

				$elements = $this->course->get_element_children( $item['post_parent'] );
				$plural   = $high_level . 's';

				if ( $term->content_type === $plural ) {
					update_post_meta( $item['ID'], $item['post_type'] . '_order', count( $term->$plural ) );
				} else {

					if ( $elements && 0 === $order ) {
						foreach ( $elements as $element ) {
							if ( $element->order > $order ) {
								$order = $element->order;
							}
						}
					}

					if ( ! empty( $term->lessons ) ) {
						foreach ( $term->lessons as $element ) {
							if ( $element->order > $order ) {
								$order = $element->order;
							}
						}
					}

					++ $order;
					update_post_meta( $item['ID'], $item['post_type'] . '_order', $order );
				}

				if ( 'publish' === $item['post_status'] ) {
					$this->publish_parent( $item['post_parent'] );
				}
			}
		}

		$old_parents = array_column( $items, 'old_post_parent' );
		foreach ( $old_parents as $old_parent ) {
			$this->unpublish_parent( $old_parent );
		}


		$response[] = tva_get_course_by_id( $this->term_id );

		if ( $new_course_id != $this->term_id ) {
			if ( $this->settings['membership_plugin'] ) {
				/**
				 * Update membership options for the course in which we moved items
				 */
				$this->term_id = $new_course_id;
				$this->handle_membership_plugin_setup();
			}

			$response[] = tva_get_course_by_id( $new_course_id );
		}

		return new WP_REST_Response( $response, 200 );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function update_posts_order( $request ) {
		$data          = $request->get_param( 'items' );
		$this->term_id = $request->get_param( 'course_id' );
		$old_posts     = array();

		foreach ( (array) $data as $value ) {
			$old_posts[] = get_post( $value['ID'] );

			wp_update_post( array( 'ID' => $value['ID'], 'post_parent' => $value['post_parent'] ) );
			update_post_meta( (int) $value['ID'], $value['post_type'] . '_order', $value['order'] );
		}

		/**
		 * Update the status for the post/s which receive the item
		 */
		foreach ( $data as $value ) {
			if ( $value['post_parent'] > 0 ) {
				$post_parent = get_post( $value['post_parent'] );

				$this->change_post_status( $post_parent );

				if ( $post_parent->post_parent > 0 ) {
					$module = get_post( $post_parent->post_parent );
					$this->change_post_status( $module );
				}
			}
		}

		/**
		 * Update the status for the post/s from which the item has been added
		 */
		foreach ( $old_posts as $post ) {
			if ( $post->post_parent > 0 ) {
				$parent = get_post( $post->post_parent );
				$this->change_post_status( $parent );

				if ( $parent->post_parent > 0 ) {
					$module = get_post( $parent->post_parent );
					$this->change_post_status( $module );
				}
			}
		}

		$user_settings  = TVA_Settings::instance();
		$this->settings = $user_settings->get_settings();
		$this->term_id  = $request->get_param( 'course_id' );

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}

		$term = new TVA_Course( $this->term_id );

		return new WP_REST_Response( $term->get_term(), 200 );
	}

	/**
	 * @param $post
	 */
	public function change_post_status( $post ) {
		$publish      = 'draft';
		$this->course = new TVA_Course( $this->term_id );
		$pb_children  = wp_list_filter( $this->course->posts, array( 'post_parent' => $post->ID ) );

		foreach ( $pb_children as $child ) {
			if ( $child->post_status === 'publish' ) {
				$publish = 'publish';
			}
		}

		wp_update_post( array( 'ID' => $post->ID, 'post_status' => $publish ) );
	}

	/**
	 * @todo remove this
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return bool
	 */
	public static function is_valid_request( $request ) {
		return true;
	}
}
