<?php

class TVA_Lessons_Controller extends TVA_REST_Controller {

	/**
	 * @var string
	 */
	public $base = 'lessons';

	/**
	 * @var
	 */
	public $post_id;
	/**
	 * @var
	 */
	public $course_id;

	/**
	 * @var WP_REST_Request $request
	 */
	public $request = array();

	/**
	 * @var array
	 */
	public $settings = array();

	/**
	 * Register Routes
	 */
	public function register_routes() {

		register_rest_route( self::$namespace . self::$version, '/' . $this->base, array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'new_lesson' ),
				'permission_callback' => array( $this, 'lessons_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/(?P<ID>[\d]+)', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_lesson' ),
				'permission_callback' => array( $this, 'lessons_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'edit_lesson' ),
				'permission_callback' => array( $this, 'lessons_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_order/', array(
			array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => array( $this, 'update_lessons_order' ),
				'permission_callback' => array( $this, 'lessons_permissions_check' ),
				'args'                => array(),
			),
		) );
		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/move_lessons/', array(
			array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => array( $this, 'move_lessons' ),
				'permission_callback' => array( $this, 'lessons_permissions_check' ),
				'args'                => array(),
			),
		) );
	}

	/**
	 * Add a new lesson to the db
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function new_lesson( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		/**
		 * We should add the new course
		 */
		$this->request = $request;

		$args = array(
			'post_title'     => $this->request->get_param( 'post_title' ),
			'post_type'      => TVA_Const::POST_TYPE,
			'post_excerpt'   => $this->request->get_param( 'post_excerpt' ),
			'comment_status' => $this->request->get_param( 'comment_status' ),
			'post_category'  => array( $this->request->get_param( 'course_id' ) ),
			'post_status'    => 'draft',
			'post_parent'    => $this->request->get_param( 'post_parent' ),
		);

		$result = wp_insert_post( $args );

		if ( ! is_wp_error( $result ) ) {
			$this->post_id = $result;

			$this->update_post_meta();

			$post            = get_post( $result );
			$post->course_id = (int) $this->request->get_param( 'course_id' );

			$post = tva_get_post_data( $post );

			$template = array(
				'lesson_type'    => $this->request->get_param( 'lesson_type' ),
				'post_media'     => $this->request->get_param( 'post_media' ),
				'comment_status' => $this->request->get_param( 'comment_status' ),
			);

			update_term_meta( $post->course_id, 'tva_term_lesson_template', $template );

			return new WP_REST_Response( $post, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * Add/Update the lesson meta
	 */
	public function update_post_meta() {
		$post_id     = $this->post_id;
		$cover_image = $this->request->get_param( 'cover_image' );
		$order       = $this->request->get_param( 'order' );

		update_post_meta( $post_id, 'tva_cover_image', $cover_image );
		update_post_meta( $post_id, 'tva_post_media', $this->request->get_param( 'post_media' ) );
		update_post_meta( $post_id, 'tva_lesson_type', $this->request->get_param( 'lesson_type' ) );

		if ( isset( $order ) ) {
			update_post_meta( $post_id, 'tva_lesson_order', $order );
		}

		$this->course_id = (int) $this->request->get_param( 'course_id' );

		wp_set_object_terms( $post_id, $this->course_id, TVA_Const::POST_TAXONOMY );

		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}

	}

	/**
	 * Handle the membership plugin setup, like adding things needed to the DB
	 */
	public function handle_membership_plugin_setup() {
		global $tva_db;

		$course   = tva_get_course_by_id( $this->course_id );
		$settings = $this->settings;

		foreach ( $settings['membership_plugin'] as $membership ) {
			$tva_db->tva_protection_manager( $course, $membership['tag'] );
		}


	}

	/**
	 * Delete the lesson
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function delete_lesson( $request ) {
		/**
		 * @var WP_REST_Request $request
		 */
		$post_id = $request->get_param( 'ID' );

		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			global $tva_db;
			$lesson = get_post( $post_id );

			foreach ( $this->settings['membership_plugin'] as $setting ) {
				$fn = 'delete_' . $setting['tag'] . '_restrictions';
				$tva_db->$fn( $lesson );
			}

		}

		$result = wp_delete_post( $post_id, true );

		if ( ! is_wp_error( $result ) ) {
			return new WP_REST_Response( true, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * Edit the lesson
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function edit_lesson( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$this->request = $request;

		$args = array(
			'post_title'     => $this->request->get_param( 'post_title' ),
			'post_type'      => TVA_Const::POST_TYPE,
			'post_excerpt'   => $this->request->get_param( 'post_excerpt' ),
			'post_status'    => $this->request->get_param( 'post_status' ),
			'ID'             => $this->request->get_param( 'ID' ),
			'comment_status' => $this->request->get_param( 'comment_status' ),
			'post_parent'    => $this->request->get_param( 'post_parent' ),
		);

		$result = wp_update_post( $args );

		if ( ! is_wp_error( $result ) ) {
			$this->post_id = $result;

			$this->update_post_meta();

			return new WP_REST_Response( $result, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * @param $request
	 *
	 * Update the order to show the lessons to
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function update_lessons_order( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		/**
		 * @var WP_REST_Request $request
		 */
		$new_order = $request->get_param( 'new_order' );

		foreach ( $new_order as $post_id => $data ) {
			if ( ! $this->course_id ) {
				$post            = get_post( (int) $post_id );
				$terms           = wp_get_post_terms( $post->ID, TVA_Const::POST_TAXONOMY );
				$this->course_id = $terms[0]->term_id;
			}

			wp_update_post(
				array(
					'ID'          => $post_id,
					'post_parent' => $data['parent'],
				)
			);
			update_post_meta( $post_id, 'tva_lesson_order', $data['order'] );
		}

		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * Move lessons form one course to another
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function move_lessons( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		/**
		 * @var WP_REST_Request $request
		 */
		$this->request = $request;
		$ids           = $request->get_param( 'ids' );

		foreach ( $ids as $post_id ) {
			$this->post_id = $post_id;
			$result        = wp_set_object_terms( $post_id, (int) $this->request->get_param( 'course_id' ), TVA_Const::POST_TAXONOMY );

			if ( empty( $result ) || is_wp_error( $result ) ) {
				return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
			}
		}

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * Check if user is logged in and is an administrator
	 *
	 * @return bool
	 */
	public function lessons_permissions_check() {
		return current_user_can( 'manage_options' );
	}

	/**
	 * Update comment status for all lessons which belongs to a given course
	 *
	 * @param array $ids
	 * @param       $comment_status
	 */
	public static function update_lesons_comment_status( $ids, $comment_status ) {

		if ( is_array( $ids ) ) {
			foreach ( $ids as $id ) {
				wp_update_post(
					array(
						'ID'             => $id,
						'comment_status' => $comment_status,
					) );
			}
		}
	}
}
