<?php
/**
 * Created by PhpStorm.
 * User: User
 * Date: 3/24/2018
 * Time: 16:38
 */

class TVA_Modules_Controller extends TVA_REST_Controller {
	/**
	 * @var string
	 */
	public $base = 'modules';

	/**
	 * @var
	 */
	public $post_id;
	/**
	 * @var
	 */
	public $course_id;

	/**
	 * @var WP_REST_Request $request
	 */
	public $request = array();

	/**
	 * @var array
	 */
	public $settings = array();

	/**
	 * Register Routes
	 */
	public function register_routes() {

		register_rest_route( self::$namespace . self::$version, '/' . $this->base, array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'new_module' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/(?P<ID>[\d]+)', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_module' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'edit_module' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_order/', array(
			array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => array( $this, 'update_module_order' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );
		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/move_modules/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'move_modules' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );
		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/update_post_status/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'update_post_status' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/group_as_module/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'group_as_module' ),
				'permission_callback' => array( $this, 'modules_permissions_check' ),
				'args'                => array(),
			),
		) );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function new_module( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$args        = array(
			'post_title'     => $request->get_param( 'post_title' ),
			'post_excerpt'   => $request->get_param( 'post_excerpt' ),
			'post_status'    => $request->get_param( 'post_status' ),
			'post_type'      => TVA_Const::MODULE_POST_TYPE,
			'comment_status' => $request->get_param( 'comment_status' ),
		);
		$cover_image = $request->get_param( 'cover_image' );
		$result      = wp_insert_post( $args );
		$chapters    = $request->get_param( 'chapters' );
		$lessons     = $request->get_param( 'lessons' );

		if ( ! is_wp_error( $result ) ) {
			update_post_meta( $result, 'tva_cover_image', $cover_image );
			update_post_meta( $result, 'tva_module_order', $request->get_param( 'order' ) );

			$term_id = $request->get_param( 'course_id' );
			wp_set_object_terms( $result, (int) $term_id, TVA_Const::POST_TAXONOMY );
			$post = get_post( $result );

			$post->order        = (int) $request->get_param( 'order' );
			$post->course_id    = $term_id;
			$post->cover_image  = $cover_image;
			$post->post_excerpt = $request->get_param( 'post_excerpt' );
			$post->chapters     = array();
			$post->lessons      = array();

			$childs = ! empty( $chapters ) ? $chapters : $lessons;

			if ( is_array( $childs ) ) {
				foreach ( $childs as $child ) {
					wp_update_post(
						array(
							'ID'             => $child['ID'],
							'post_parent'    => $result,
							'comment_status' => $request->get_param( 'comment_status' ),
						)
					);

					$child['post_parent'] = $result;

					if ( ! empty( $chapters ) ) {
						if ( is_array( $child['lessons'] ) ) {
							foreach ( $child['lessons'] as $key => $lesson ) {
								wp_update_post(
									array(
										'ID'             => $lesson['ID'],
										'comment_status' => $request->get_param( 'comment_status' ),
									)
								);

								$lesson['comment_status'] = $request->get_param( 'comment_status' );
								$child['lessons'][ $key ] = $lesson;
							}
						}

						$post->chapters[] = $child;
					} elseif ( ! empty( $lessons ) ) {
						$post->lessons[] = $child;
					}
				}
			}

			return new WP_REST_Response( $post, 200 );
		}

		return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function edit_module( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$module_id             = $request->get_param( 'ID' );
		$post_title            = $request->get_param( 'post_title' );
		$module_description    = $request->get_param( 'post_excerpt' );
		$cover_image           = $request->get_param( 'cover_image' );
		$chapters              = $request->get_param( 'chapters' );
		$lessons               = $request->get_param( 'lessons' );
		$comment_status_changed = $request->get_param( 'comment_status_changed' );

		$args = array(
			'ID'             => $module_id,
			'post_title'     => $post_title,
			'post_excerpt'   => $module_description,
			'post_status'    => $request->get_param( 'post_status' ),
			'comment_status' => $request->get_param( 'comment_status' ),
		);

		$update = wp_update_post( $args );
		$post   = get_post( $module_id );
		update_post_meta( $module_id, 'tva_cover_image', $cover_image );

		if ( $comment_status_changed ) {
			foreach ( (array) $chapters as $chapter ) {
				wp_update_post(
					array(
						'ID'             => $chapter['ID'],
						'comment_status' => $request->get_param( 'comment_status' ),
					)
				);

				foreach ( (array) $chapter['lessons'] as $lesson ) {
					wp_update_post(
						array(
							'ID'             => $lesson['ID'],
							'comment_status' => $request->get_param( 'comment_status' ),
						)
					);
				}
			}

			foreach ( (array) $lessons as $lesson ) {
				wp_update_post(
					array(
						'ID'             => $lesson['ID'],
						'comment_status' => $request->get_param( 'comment_status' ),
					)
				);
			}
		}

		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->course_id = $request->get_param( 'course_id' );
			$this->handle_membership_plugin_setup();
		}

		if ( ! is_wp_error( $update ) ) {
			unset( $post->post_modified );
			unset( $post->post_modified_gmt );

			return new WP_REST_Response( $post, 200 );
		}

		return new WP_Error( 'no-results', __( $update->get_error_message(), TVA_Const::T ) );
	}

	/**
	 * Handle the membership plugin setup, like adding things needed to the DB
	 */
	public function handle_membership_plugin_setup() {
		global $tva_db;

		$course   = tva_get_course_by_id( $this->course_id );
		$settings = $this->settings;

		foreach ( $settings['membership_plugin'] as $membership ) {
			$tva_db->tva_protection_manager( $course, $membership['tag'] );
		}
	}

	/**
	 * Update the order to show the modules to
	 *
	 * @param $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function update_module_order( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		/**
		 * @var WP_REST_Request $request
		 */
		$order   = $request->get_param( 'order' );
		$post_id = $request->get_param( 'ID' );

		update_post_meta( $post_id, 'tva_module_order', $order );

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function delete_module( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}
		$module_id = $request->get_param( 'ID' );
		$terms     = wp_get_post_terms( $module_id, TVA_Const::POST_TAXONOMY );

		if ( ! empty( $terms ) ) {
			$this->course_id = $terms[0]->term_id;
			$course          = new TVA_Course( $this->course_id );
			$children        = $course->get_element_children( $module_id );

			foreach ( $children as $child ) {
				wp_delete_post( $child->ID );
			}
		}

		$delete = wp_delete_post( $module_id );

		$course = new TVA_Course( $this->course_id );
		$term   = $course->get_term();
		$course->ensure_correct_order( $term->modules );


		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}


		if ( ! is_wp_error( $delete ) ) {
			return new WP_REST_Response( array(), 200 );
		}

		return new WP_Error( 'delete_failed', __( 'Failed to delete module. Please try again later!', TVA_Const::T ) );
	}

	/**
	 * Move modules from one course to another
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function move_modules( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$id     = $request->get_param( 'ids' );
		$result = wp_set_object_terms( $id, (int) $request->get_param( 'course_id' ), TVA_Const::POST_TAXONOMY );

		if ( empty( $result ) || is_wp_error( $result ) ) {
			return new WP_Error( 'no-results', __( $result, TVA_Const::T ) );
		}

		return new WP_REST_Response( true, 200 );
	}

	/**
	 * @param WP_REST_Request $request
	 *
	 * @return int|WP_Error
	 */
	public function update_post_status( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$status  = $request->get_param( 'post_status' );
		$post_id = $request->get_param( 'post_id' );

		return wp_update_post( array( 'ID' => $post_id, 'post_status' => $status ) );
	}

	/**
	 * Check if user is logged in and is an administrator
	 *
	 * @return bool
	 */
	public function modules_permissions_check() {
		return current_user_can( 'manage_options' );
	}

	/**
	 * Group items as modules
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function group_as_module( $request ) {
		if ( ! TVA_Courses_Controller::is_valid_request( $request ) ) {
			return new WP_Error( 'invalid-request', __( 'Invalid Request. Please reload the page', TVA_Const::T ) );
		}

		$items = $request->get_param( 'items' );

		foreach ( $items as $item ) {
			$children        = $item['item_ids'];
			$this->course_id = $item['course_id'];

			$route  = '/' . self::$namespace . self::$version . '/' . $this->base;
			$module = new WP_REST_Request( 'POST', $route );
			$module->set_query_params( $item );
			$module->set_header( 'X-TVA-Request-ID', get_option( 'tva_request_id' ) );

			$response = rest_do_request( $module );
			$server   = rest_get_server();
			$data     = $server->response_to_data( $response, false );

			if ( $data ) {
				$course          = new TVA_Courses_Controller();
				$course->term_id = $this->course_id;

				foreach ( $children as $child ) {
					$post            = get_post( $child );
					$old_post_parent = $post->post_parent;

					wp_update_post(
						array(
							'ID'             => $child,
							'post_parent'    => $data->ID,
							'comment_status' => $item['comment_status'],
						)
					);

					if ( $old_post_parent > 0 ) {
						$course->unpublish_parent( $old_post_parent );
					}
				}
			}
		}

		if ( empty( $this->settings ) ) {
			$user_settings  = TVA_Settings::instance();
			$this->settings = $user_settings->get_settings();
		}

		if ( $this->settings['membership_plugin'] ) {
			$this->handle_membership_plugin_setup();
		}

		$course = new TVA_Course( $this->course_id );

		return new WP_REST_Response( $course->get_term(), 200 );
	}
}