<?php

class TVA_Topics_Controller extends TVA_REST_Controller {

	public $base = 'topics';

	/**
	 * Register Routes
	 */
	public function register_routes() {

		register_rest_route( self::$namespace . self::$version, '/' . $this->base, array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'save_topics' ),
				'permission_callback' => array( $this, 'topics_permissions_check' ),
				'args'                => array(),
			),
		) );

		register_rest_route( self::$namespace . self::$version, '/' . $this->base . '/(?P<ID>[\d]+)', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_topics' ),
				'permission_callback' => array( $this, 'topics_permissions_check' ),
				'args'                => array(),
			),
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'edit_topics' ),
				'permission_callback' => array( $this, 'topics_permissions_check' ),
				'args'                => array(),
			),
		) );
	}

	public function save_topics( $request ) {
		/**
		 * We should add the new topic
		 */
		$topics = get_option( 'tva_filter_topics', array() );
		$id     = 0;
		if ( ! empty( $topics ) ) {
			/**
			 * Get the biggest ID so we can create our new one
			 */
			foreach ( $topics as $topic ) {
				if ( $topic['ID'] > $id ) {
					$id = $topic['ID'];
				}
			}
		}

		$id ++;
		$icon_param = $request->get_param( 'icon' );
		$icon       = preg_replace( '#^https?://#', '//', $icon_param );
		$model      = array(
			'ID'    => $id,
			'color' => $request->get_param( 'color' ),
			'icon'  => $icon,
			'title' => $request->get_param( 'title' ),
		);

		$topics[] = $model;
		$result   = update_option( 'tva_filter_topics', $topics );

		if ( $result ) {
			return new WP_REST_Response( $model, 200 );
		}

		return new WP_Error( 'no-results', __( 'No topic was updated!', TVA_Const::T ) );
	}

	public function topics_permissions_check( $request ) {
		return current_user_can( 'manage_options' );
	}

	public function delete_topics( $request ) {
		$response = array();
		$id       = $request->get_param( 'ID' );

		$topics = get_option( 'tva_filter_topics', array() );
		foreach ( $topics as $key => $topic ) {
			if ( $topic['ID'] == $id ) {
				unset( $topics[ $key ] );
			}
		}
		$topics = array_values( $topics );

		$courses = tva_get_courses();
		foreach ( $courses as $course_term ) {
			if ( $course_term->topic == $id ) {
				$response['edited'][] = $course_term->term_id;
				update_term_meta( $course_term->term_id, 'tva_topic', 0 );
			}
		}
		$result = update_option( 'tva_filter_topics', $topics );

		if ( $result ) {
			return new WP_REST_Response( $response, 200 );
		}

		return new WP_Error( 'no-results', __( 'No topic was deleted!', TVA_Const::T ) );
	}

	public function edit_topics( $request ) {
		$id         = $request->get_param( 'ID' );
		$icon_param = $request->get_param( 'icon' );
		$icon       = preg_replace( '#^https?://#', '//', $icon_param );
		$model      = array(
			'ID'    => $id,
			'color' => $request->get_param( 'color' ),
			'icon'  => $icon,
			'title' => $request->get_param( 'title' ),
		);

		$topics = get_option( 'tva_filter_topics', array() );

		foreach ( $topics as $key => $topic ) {
			if ( $topic['ID'] == $id ) {

				$topics[ $key ] = $model;
			}
		}

		$result = update_option( 'tva_filter_topics', $topics );

		if ( $result ) {
			return new WP_REST_Response( $model, 200 );
		}

		return new WP_Error( 'no-results', __( 'No topic was updated!', TVA_Const::T ) );
	}
}
