/**
 * Created by Pop Aurelian on 21-Feb-17.
 */

var ThriveAppFront = ThriveAppFront || {};

( function ( $ ) {

	//  add class to menu box

	if ( $( '.tva-header .tva-inner-header' ).children().length > 1 ) {
		$( '.tva-header .tva-inner-header div' ).last().addClass( 'tva-dropdownmenu' );
	}

	//	detect edge of menu so that the dropdown items dont flow outside the window

	$( '.tva-header > div > ul li' ).on( 'mouseenter', function ( e ) {
		if ( $( 'ul', this ).length ) {
			var elm = $( 'ul:first', this ),
				off = elm.offset(),
				l = off.left,
				w = elm.width(),
				docW = $( '.tva-header' ).width();

			var isEntirelyVisible = ( l + w <= docW );

			if ( ! isEntirelyVisible ) {
				$( this ).parent().addClass( 'edge' );
				$( this ).find( "ul" ).addClass( 'edge' );
			}
		}
	} );

	$( '.tva-header > div' ).click( function () {
		$( this ).toggleClass( "visible" );
	} );

	var markLesson = function ( completed ) {

		if ( ThriveAppFront.lesson.post_type !== 'tva_lesson' ) {
			return;
		}

		var data = {
			lesson_id: ThriveAppFront.lesson_page,
			completed: false
		};

		if ( completed ) {
			data.completed = true;
		}


		$.ajax( {
			headers: {
				'X-WP-Nonce': ThriveAppFront.nonce
			},
			type: 'POST',
			data: data,
			url: ThriveAppFront.routes.frontend + '/mark_lesson/'
		} ).done( function ( response, status, options ) {
		} );
	};

	if ( ! ThriveAppFront.is_inner_frame ) {

		var tva_search_key;

		if ( ThriveAppFront.course_page && ThriveAppFront.allowed && ! ThriveAppFront.is_admin ) {
			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveAppFront.nonce
				},
				type: 'POST',
				data: {
					course_id: ThriveAppFront.course_page
				},
				url: ThriveAppFront.routes.frontend + '/process_conversion/'
			} ).done( function ( response, status, options ) {
				//todo: should we do anything with the response ?
			} );
		}

		if ( ThriveAppFront.lesson_page && ThriveAppFront.allowed && ! ThriveAppFront.is_admin ) {
			markLesson();
		}

		$( '.tva-finish-lesson' ).on( 'click', function ( e ) {
			markLesson( true );
		} );

		$( '.tva-filter-checkbox-container' ).on( 'click', function ( e ) {
			e.preventDefault();

			var clear = $( this ).find( '.tva-filter-course-clear-filters' ),
				filters_container = $( '.tva-filters-wrapper' ),
				search_container = $( '.tva-search-wrapper' ),
				checked = $( this ).find( '.tva-filter' ).is( ':checked' );

			$( this ).find( '.tva-filter' ).prop( 'checked', ! checked );

			if ( clear.length > 0 ) {
				filters_container.find( '.tva-filter-checkbox-container' ).each( function () {
					$( this ).children( "div:first" ).removeClass( 'tva-filter-checkbox-selected' );
				} );
				filters_container.find( '.tva-filter' ).each( function () {
					$( this ).prop( "checked", false );
				} );
				search_container.find( '.tva-search-input' ).val( '' );
			}

			setFilterCheckbox( this, redirectAfterFiltering );

			if ( clear.length > 0 ) {
				// handle the plain permalink case
				if ( tvaUrlParamExists( 'page_id', window.location.href ) ) {
					var page_id = tvaGetParameterByName( 'page_id', window.location.href );
					var alteredURL = "?page_id=" + page_id;
				} else {
					var alteredURL = tvaRemoveParam( "filters", window.location.href );
				}
				window.history.pushState( {}, document.title, alteredURL );
			}

		} );

		/**
		 * Whenever the user changes any character on the search field we should filter everything
		 */
		var tvaTextInput = document.getElementById( 'tva_front_search' );
		var tvaTimeout = null;

		/**
		 * Add loader with some delay on course search
		 * @param e
		 */
		if ( tvaTextInput ) {

			tvaTextInput.onkeyup = function () {

				var self = this;

				if ( tvaTimeout ) {
					clearTimeout( tvaTimeout );
				}

				tvaTimeout = setTimeout( function () {
					if ( tva_search_key !== self.value ) {
						redirectAfterFiltering();
					}
				}, 300 );
			};

			$( '.tva-search-submit' ).live( 'click', function () {
				redirectAfterFiltering();
			} );
		}

		function setFilterCheckbox( elem, callback ) {

			var checkbox = $( elem ).find( '.tva-filter' );

			if ( checkbox.is( ":checked" ) ) {
				$( elem ).children( "div:first" ).addClass( 'tva-filter-checkbox-selected' );
			} else {
				$( elem ).children( "div:first" ).removeClass( 'tva-filter-checkbox-selected' );
			}

			if ( typeof callback === 'function' ) {
				callback();
			}
		}

		$( 'a.tva-page-numbers' ).live( 'click', function () {

			var page = $( this ).attr( 'data-tva-page-nr' );

			redirectAfterFiltering( page );
		} );

		function tvaRemoveParam( key, sourceURL ) {

			var rtn = sourceURL.split( "?" )[ 0 ];

			// handle the plain permalink case
			if ( tvaUrlParamExists( 'page_id', window.location.href ) ) {

				var page_id = tvaGetParameterByName( 'page_id', window.location.href );
				rtn = rtn + "?page_id=" + page_id;

				return rtn;
			}

			return rtn;
		}

		function tvaUrlParamExists( param, url ) {

			if ( url.indexOf( '?' + param + '=' ) != - 1 ) {
				return true;
			} else if ( url.indexOf( '&' + param + '=' ) != - 1 ) {
				return true;
			}

			return false
		}

		function tvaGetParameterByName( name, url ) {

			if ( ! url ) {
				url = window.location.href;
			}
			name = name.replace( /[\[\]]/g, "\\$&" );
			var regex = new RegExp( "[?&]" + name + "(=([^&#]*)|&|#|$)" ),
				results = regex.exec( url );
			if ( ! results ) {
				return null;
			}
			if ( ! results[ 2 ] ) {
				return '';
			}
			return decodeURIComponent( results[ 2 ].replace( /\+/g, " " ) );
		}

		function redirectAfterFiltering( page ) {

			var loading_css_class = 'tva-loading',
				course_container = $( '.tva-courses-container' );

			if ( true === course_container.hasClass( loading_css_class ) ) {
				return;
			}

			var loader = '<div class="tva-front-loader tva-spinner-css tva-spinner-scope"><div class="tva-spinner-rolling"><div></div></div></div>',
				filter_terms = $( '.tva-filter-checkbox-container input:checkbox:checked' ).map( function () {
					return $( this ).val();
				} ).get();

			tva_search_key = $( '.tva-search-input' ).val();

			course_container.addClass( loading_css_class );
			course_container.append( loader );
			course_container.find( ".tva-course-card" ).remove();

			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveAppFront.nonce
				},
				type: "POST",
				url: ThriveAppFront.routes.frontend + '/filters/',
				data: {
					s: tva_search_key,
					terms: filter_terms,
					page: page
				}
			} ).done( function ( response ) {

				course_container.removeClass( loading_css_class );
				course_container.empty().append( response );

				if ( ThriveAppFront.index_page ) {
					course_container.find( '.tva-course-description' ).addClass( 'scrollbar-macosx' ).scrollbar();
				}

				$( 'html, body' ).animate( {
					scrollTop: 0
				}, 400 );

				$( '.ttw-uni-page-numbers' ).on( 'click', function () {
					var page = $( this ).attr( 'data-tva-page-nr' );
					redirectAfterFiltering( page );
				} );

				showClearFiltersButton();
			} );

		}

		showClearFiltersButton();

		function showClearFiltersButton() {
			var activeFilters = $( '.tva-filters-container' ).find( '.tva-filter' ).is( ':checked' );
			var display = '';

			/**
			 * By default tva-clear-filters had inline block display so we keep it that way
			 */
			activeFilters === true ? display = 'inline-block' : 'none';
			$( '.tva-clear-filters' ).css( 'display', display );
		}
	}

	/**
	 * validate apprentice register page
	 */
	if ( ThriveAppFront.tva_register_page == true ) {

		function validateEmail( email ) {
			var regex = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
			return regex.test( email );
		}

		var password, confirm_password;

		jQuery( 'input[type="password"]' ).on( 'change', function () {
			password = jQuery( "input[name='user_pass']" ).val();
			confirm_password = jQuery( "input[name='confirm_pass']" ).val();
		} );


		jQuery( '#tva-register-button' ).on( 'click', function ( e ) {
			if ( ( validateEmail( jQuery( "input[name=user_email]" ).val() ) != true ) ) {
				e.preventDefault();
				jQuery( ".tva-password-mismatch" ).fadeIn( "slow" );
				jQuery( '.tva-invalid-email' ).fadeIn( "slow" );
			}

			jQuery( 'input[name="user_pass"], input[name="user_login"]' ).each( function () {
				if ( jQuery( this ).val() == '' ) {
					e.preventDefault();
					setTimeout( function () {
						jQuery( ".tva-required-filed" ).fadeOut( "slow" );
						jQuery( ".tva-invalid-email" ).fadeOut( "slow" );
					}, 5000 );

				}

				if ( ( password != confirm_password ) ) {
					jQuery( ".tva-invalid-password" ).fadeIn( "slow" );
					e.preventDefault();
				}
			} );

			if ( password == undefined ) {
				$( 'input[name="user_pass"]' ).parent().next( 'p' ).fadeIn( 'slow' );
			}

			if ( confirm_password == undefined ) {
				$( 'input[name="confirm_pass"]' ).parent().next( 'p' ).fadeIn( 'slow' );
				$( '#tva-register-button' ).addClass( 'tva-btn-margin-top' );
			}


		} );

		jQuery( 'input[name="user_pass"], input[name="confirm_pass"]' ).on( 'focusin', function () {
			jQuery( ".tva-invalid-password" ).fadeOut( "slow" );
		} );

		jQuery( '#ta-registration-form input[type=email], input[name="user_pass"], input[name="confirm_pass' ).on( 'focusin', function () {
			jQuery( ".tva-required-filed" ).fadeOut( "slow" );
			jQuery( ".tva-invalid-email" ).fadeOut( "slow" );
		} );

		jQuery( '#ta-registration-form input[type=email]' ).on( 'focusout', function () {
			var self = jQuery( this ),
				data = {
					email: self.val()
				};
			jQuery.ajax( {
				type: "POST",
				url: ThriveAppFront.routes.frontend + '/check_email/',
				data: data
			} ).done( function ( response ) {
				if ( response == true ) {
					jQuery( '#tva-email-error-wrapper' ).fadeIn( 'slow' );
					setTimeout( function () {
						jQuery( '#tva-email-error-wrapper' ).fadeOut( 'slow' )
					}, 5000 );
				}
				jQuery( '#tva-register-button' ).prop( "disabled", response );
			} );
		} );
	}

	jQuery( '.tva-register-url' ).on( 'click', function () {
		document.cookie = 'tva_lesson_to_redirect=' + window.location.href + '; path=/';
	} );

	/**
	 * Handle comments form
	 */
	if ( ( ThriveAppFront.course.comment_status == 'open' || ThriveAppFront.lesson.comment_status == 'open' ) && ThriveAppFront.has_comment_plugin == false ) {

		$( document ).ready( function () {
			comments.tvaGetGuestData( $( '.tva-comments-form' ) );
			var post_id = Object.keys( ThriveAppFront.lesson ).length > 0 ? ThriveAppFront.lesson.ID : ThriveAppFront.course_ref_post_id;

			/**
			 * Get available comments
			 */
			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveAppFront.nonce
				},
				type: "POST",
				url: ThriveAppFront.routes.frontend + '/comments/',
				data: {
					comment_data: {
						comment_post_ID: post_id,
						comment_term_ID: post_id == ThriveAppFront.course_ref_post_id ? ThriveAppFront.course.ID : ''
					}
				}
			} ).done( function ( response ) {
				comments.comment_count = response.comment_count;
				comments.all_comments = response.comments;
				comments.user_can_manage_comments = response.approve;
				comments.renderCommentsList();
			} );

		} );

		$( '#tva-comments-list' ).on( 'click', '.tva-reply-button', function ( e ) {

			/**
			 * Close any reply form that are already open
			 */
			$( '.tva-comment-form' ).each( function ( index, item ) {
				if ( $( item ).parent().attr( 'data-comment-id' ) !== $( e.currentTarget ).parent().attr( 'data-comment-id' ) ) {
					$( item ).slideUp( 'slow' );
				}
			} );

			$( this ).parent( '.tva-single-comment' ).find( '.tva-comment-form' ).slideToggle();

			if ( ThriveAppFront.is_user_logged_in == false ) {
				comments.tvaGetGuestData( $( this ) );
			}
		} );

		$( '.tva-frontend-template' ).on( 'click', '#tva-submit-comment', function () {
			var self = $( this ),
				comment = comments.tvaValidateComment( self );

			if ( false === comment ) {
				return;
			}

			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveAppFront.nonce
				},
				type: "POST",
				url: ThriveAppFront.routes.frontend + '/create_comment/',
				data: {
					comment_data: comment
				}
			} ).done( function ( response ) {
				$( '#tva-comments-list' ).html( '' );
				$( self ).parent( '.tva-comments-form' ).find( '.tva-comment-content' ).val( '' );
				comments.childs = '';
				comments.html = '';
				comments.comment_count = response.comment_count;

				comments.renderCommentListHeader();

				_.each( response.comments, function ( comment ) {
					comments.html += '<div class="tva-comment-holder">';
					comments.renderComment( comment );
					comments.html += '</div>';
				} );

				$( '#tva-comments-list' ).html( comments.html );

			} ).fail( function ( error ) {
				$( self ).parent().find( '.tva-error-message' ).remove();
				comments.tvaShowError( error.status, self )
			} );

		} );

		/**
		 * Comments object
		 */
		var comments = {
			comment_count: 0,
			all_comments: [],
			childs: '',
			html: '',
			user_can_manage_comments: false,

			/**
			 * Render a parent comment ant it's childs
			 * @param item
			 */
			renderComment: function ( item ) {
				this.html += '<div class="tva-single-comment" data-comment-id="' + item.comment_ID + '">';
				this.html += '<div class="tva-comment-author">' + item.comment_author + '</div>';

				if ( item.comment_approved != true ) {
					var state = this.user_can_manage_comments === true ? ThriveAppFront.t.This : ThriveAppFront.t.Your;
					this.html += '<div class="tva-comment-approval">';
					this.html += state + " " + ThriveAppFront.t.Moderation;
					this.html += '</div>';
				}

				this.html += '<p class="tva-comment">' + item.comment_content + '</p>';
				this.tvaCommentReplyForm();
				this.html += '</div>';

				if ( ( item.children !== null ) && ( item.children.length > 0 ) ) {
					this.childs = item.children;
					this.html += '<div class="tva-children-list">';
					var self = this;

					_.each( this.childs, function ( child ) {
						self.renderComment( child );
					} );

					this.html += '</div>';
				}
			},

			/**
			 * Validate comment before submit
			 * @param context
			 */
			tvaValidateComment: function ( context ) {
				context.parent().find( '.tva-error-message' ).remove();

				var comment_data = this.tvaPrepareCommentData( context );

				if ( comment_data.comment_content == '' ) {
					return this.tvaShowError( 'invalid_comment_content', context );
				}

				if ( comment_data.comment_author == '' ) {
					return this.tvaShowError( 'invalid_comment_author', context );
				}

				if ( comment_data.comment_author_email == '' ) {
					return this.tvaShowError( 'missing_author_email', context );
				}

				var email_regex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
					url_regex = /(http(s)?:\/\/.)(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/g;

				if ( email_regex.test( comment_data.comment_author_email ) === false ) {
					return this.tvaShowError( 'invalid_author_email', context );
				}

				if ( ( comment_data.comment_author_url != '' ) && ( url_regex.test( comment_data.comment_author_url ) === false ) ) {
					return this.tvaShowError( 'invalid_url', context )
				}

				return comment_data;
			},

			renderCommentListHeader: function () {
				this.html += '<div class="tva-comments-number">';
				if ( comments.comment_count === 1 ) {
					this.html += '<p>' + ThriveAppFront.t.SingleComment + '</p>';
				} else if ( comments.comment_count > 1 ) {
					this.html += '<p>' + this.comment_count + ' ' + ThriveAppFront.t.MultipleComments + '</p>';
				}
				this.html += '</div>';
			},

			/**
			 * Render comments list
			 */
			renderCommentsList: function () {
				var self = this;
				this.renderCommentListHeader();
				this.html += '</div>';
				_.each( this.all_comments, function ( item ) {
					self.html += '<div class="tva-comment-holder">';
					self.renderComment( item );
					self.html += '</div>';
				} );

				$( '#tva-comments-list' ).append( this.html );
			},

			/**
			 * Render a child comment
			 * @returns {string}
			 */
			tvaCommentReplyForm: function () {
				this.html += '<button class="tva-reply-button">' + ThriveAppFront.t.Reply + '</button>';
				this.html += '<div class="tva-comment-form">';
				this.html += '<textarea class="tva-comment-content"></textarea>';

				if ( ! ThriveAppFront.current_user.ID ) {
					this.html += '<div class="tva-guest-fileds">';
					this.html += '<p class="tva-guest">' + ThriveAppFront.t.Guest + '</p>';
					this.html += '<span>' + ThriveAppFront.t.GuestName + '</span>';
					this.html += '<input type="text" class="tva-comment-input" name="name">';
					this.html += '<span>' + ThriveAppFront.t.GuestEmail + '</span>';
					this.html += '<input type="text" class="tva-comment-input" name="email">';
					this.html += '<span>' + ThriveAppFront.t.GuestUrl + '</span>';
					this.html += '<input type="text" class="tva-comment-input" name="url">';
					this.html += '</div>';
				}

				this.html += '<button id="tva-submit-comment" >' + ThriveAppFront.t.SubmitComment + '</button>';
				this.html += '</div>';
			},

			/**
			 * Prepare comment data
			 * @param context
			 * @returns data
			 */
			tvaPrepareCommentData: function ( context ) {
				var parent_id = context.parent().parent().attr( 'data-comment-id' );
				var post_id = Object.keys( ThriveAppFront.lesson ).length > 0 ?
					ThriveAppFront.lesson.ID :
					ThriveAppFront.course_ref_post_id,

					comment_author = ThriveAppFront.is_user_logged_in != false ?
						ThriveAppFront.current_user.display_name :
						context.parent().find( 'input[name="name"]' ).val(),

					comment_author_email = ThriveAppFront.is_user_logged_in != false ?
						ThriveAppFront.current_user.user_email :
						context.parent().find( 'input[name="email"]' ).val(),

					comment_author_url = ThriveAppFront.is_user_logged_in != false ?
						ThriveAppFront.current_user.user_url :
						context.parent().find( 'input[name="url"]' ).val(),

					data = {
						comment_post_ID: post_id,
						comment_content: context.parent().find( '.tva-comment-content' ).val(),
						comment_parent: parent_id !== undefined ? parent_id : 0,
						comment_author: comment_author,
						comment_author_email: comment_author_email,
						comment_author_url: comment_author_url,
						user_id: ThriveAppFront.current_user.ID ? ThriveAppFront.current_user.ID : '',
						comment_type: ''
					};

				data.comment_term_ID = Object.keys( ThriveAppFront.lesson ).length === 0 ? ThriveAppFront.course.ID : '';

				return data;
			},

			/**
			 * Handle comment errors
			 * @param key
			 * @param context
			 * @returns {boolean}
			 */
			tvaShowError: function ( key, context ) {
				var html = '';

				switch ( key ) {

					/**
					 * 409 and 429 are the codes for wp errors
					 */
					case 409:
						html = '<div class="tva-error-message">' + ThriveAppFront.t.DuplicateComment + '</div>';
						context.parent().append( html );
						return false;
						break;

					case 429:
						html = '<div class="tva-error-message">' + ThriveAppFront.t.CommentToFast + '</div>';
						context.parent().append( html );
						return false;
						break;

					case 'invalid_comment_content':
						html = '<div class="tva-error-message">' + ThriveAppFront.t.EmptyComment + '</div>';
						context.parent().find( '.tva-comment-content' ).after( html );
						return false;
						break;

					case 'invalid_comment_author':
						html = '<div class="tva-error-message">' + ThriveAppFront.t.EmptyName + '</div>';
						context.parent().find( 'input[name="name"]' ).after( html );
						return false;
						break;

					case 'missing_author_email':
						html = '<div class="tva-error-message">' + ThriveAppFront.t.EmptyEmail + '</div>';
						context.parent().find( 'input[name="email"]' ).after( html );
						return false;
						break;

					case 'invalid_author_email':
						html = '<div class="tva-error-message">' + ThriveAppFront.t.InvalidEmail + '</div>';
						context.parent().find( 'input[name="email"]' ).after( html );
						return false;
						break;

					case 'invalid_url':
						html = '<div class="tva-error-message">' + ThriveAppFront.t.InvalidUrl + '</div>';
						context.parent().find( 'input[name="url"]' ).after( html );
						return false;
						break;

					default:
						return true;
						break;

				}
			},

			/**
			 * Get user data from cookies
			 * @param context
			 */
			tvaGetGuestData: function ( context ) {
				if ( ThriveAppFront.is_user_logged_in != true ) {
					var guest_name = decodeURIComponent( this.tvaGetCookieData( 'tva_cookie_user_name' ) ).slice( 1, - 1 ),
						guest_email = decodeURIComponent( this.tvaGetCookieData( 'tva_cookie_user_email' ) ).slice( 1, - 1 ),
						guest_url = decodeURIComponent( this.tvaGetCookieData( 'tva_cookie_user_url' ) ).slice( 1, - 1 );

					context.parent().find( 'input[name="name"]' ).val( guest_name );
					context.parent().find( 'input[name="email"]' ).val( guest_email );
					context.parent().find( 'input[name="url"]' ).val( guest_url.replace( /\\\//g, "/" ) );
				}
			},

			/**
			 * Get data from a specific cookie
			 * @param cname
			 * @returns {*}
			 */
			tvaGetCookieData: function ( cname ) {
				var name = cname + "=";
				var ca = document.cookie.split( ';' );
				for ( var i = 0; i < ca.length; i ++ ) {
					var c = ca[ i ];
					while ( c.charAt( 0 ) == ' ' ) {
						c = c.substring( 1 );
					}
					if ( c.indexOf( name ) == 0 ) {
						return c.substring( name.length, c.length );
					}
				}
				return "";
			}
		};
	}

	$( '.tva-cm-lesson' ).on( 'mousedown', function () {
		$( this ).addClass( 'tva-lesson-pressed' );
	} );

	$( '.tva-cm-lesson' ).on( 'mouseup', function () {
		$( this ).removeClass( 'tva-lesson-pressed' );
	} );

	$( document ).ready( function () {

		var $course_desc = $( '.tva-course-description' );

		if ( ThriveAppFront.index_page ) {
			$course_desc.addClass( 'scrollbar-macosx' ).scrollbar();
		}

		if ( typeof tve_frontend_options !== 'undefined' && ! tve_frontend_options.is_editor_page ) {
			//This code must be executed only in frontend

			var transport = tva_get_url_param( 'pp' );
			if ( parseInt( ThriveAppFront.is_checkout ) && parseInt( ThriveAppFront.is_user_logged_in ) && transport && transport.toLowerCase() === 'sendowl' && ( tva_get_url_param( 'pid' ) || tva_get_url_param( 'bid' ) ) ) {

				var data = [];
				data.push( {
					name: 'payment_processor',
					value: tva_get_url_param( 'pp' )
				} );

				data.push( {
					name: 'bid',
					value: tva_get_url_param( 'bid' )
				} );

				data.push( {
					name: 'pid',
					value: tva_get_url_param( 'pid' )
				} );

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveAppFront.nonce
					},
					type: 'POST',
					data: data,
					url: ThriveAppFront.routes.user + '/process_redirect/'
				} ).done( function ( response ) {
					if ( response.message ) {
						TCB_Front.toast( response.message, false );
					}

					if ( response.redirect ) {
						document.location.href = response.redirect;
					}
				} ).fail( function ( response ) {
					TCB_Front.toast( response.responseJSON.message, true );

				} );
			}


			/**
			 * Gets an url param value from param name
			 *
			 * @param paramName
			 *
			 * @returns {*|number}
			 */
			function tva_get_url_param( paramName ) {
				var results = new RegExp( '[\?&]' + paramName + '=([^&#]*)' ).exec( window.location.search );

				return ( results !== null ) ? results[ 1 ] || 0 : false;
			}

			/**
			 * Process the checkout elements
			 */
			ThriveGlobal.$j( '.thrv-checkout' ).each( function () {

				$( this ).find( '[data-type="password"] input' ).attr( 'type', 'password' );

				var _states_with_no_state_bar = [ 'forgot_password', 'reset_confirmation' ],
					error_class = 'tcb-form-error';

				/**
				 * Toggle checkout states
				 *
				 * @param event
				 */
				function show_state( event ) {
					event.stopPropagation();
					event.preventDefault();

					var $target = ThriveGlobal.$j( this ),
						$checkout = $target.closest( '.thrv-checkout' );

					$checkout.find( '.tcb-active-state' ).removeClass( 'tcb-active-state' );
					$checkout.find( '.tcb-tva-checkout-form-wrapper' ).addClass( 'tcb-permanently-hidden' );

					$target.addClass( 'tcb-active-state' );
					$checkout.find( '.tcb-tva-checkout-form-wrapper[data-instance="' + $target.attr( 'data-instance' ) + '"]' ).removeClass( 'tcb-permanently-hidden' );
				}

				function switch_state_lnk( event ) {
					event.stopPropagation();
					event.preventDefault();

					var $target = ThriveGlobal.$j( this ),
						$checkout = $target.closest( '.thrv-checkout' ),
						_instance = $target.attr( 'data-go_to_instance' );

					$checkout.find( '.tcb-tva-checkout-form-wrapper' ).addClass( 'tcb-permanently-hidden' );
					$checkout.find( '.tcb-tva-checkout-form-wrapper[data-instance="' + _instance + '"]' ).removeClass( 'tcb-permanently-hidden' );

					if ( _instance === 'login' ) {
						$checkout.find( '.tcb-active-state' ).removeClass( 'tcb-active-state' );
						$checkout.find( '.thrv-button-group-item[data-instance="' + _instance + '"]' ).addClass( 'tcb-active-state' );
					}

					toggle_instance_bar( $checkout, _instance );
				}

				function toggle_instance_bar( $checkout, current_state ) {
					$checkout.find( '.thrv-button-group' ).toggleClass( 'tcb-permanently-hidden', _.contains( _states_with_no_state_bar, current_state ) );
				}

				function on_input( event ) {
					var input = event.currentTarget;

					if ( input.classList.contains( error_class ) ) {
						input.nextSibling.remove();
						input.classList.remove( error_class );
					}
				}

				function on_focus() {
					this.placeholder = '';
				}

				function on_blur() {
					this.placeholder = this.getAttribute( 'data-placeholder' );
				}

				function on_submit( event ) {
					event.stopPropagation();
					event.preventDefault();

					var $target = ThriveGlobal.$j( this ),
						$checkout = $target.closest( '.thrv-checkout' ),
						$current_from = $target.closest( 'form' ),
						data = $target.serializeArray(),
						hash = $this.find( 'input[name="config"]' ).val(),
						indexed_array = {},
						errors_to_show = '',
						$submit_button = $target.find( 'button' );

					if ( ! hash ) {
						return false;
					}

					$.map( data, function ( n, i ) {
						indexed_array[ n[ 'name' ] ] = n[ 'value' ];
					} );

					_.some( indexed_array, function ( value, index ) {
						if ( value.length === 0 ) {
							errors_to_show = 'empty_fields';
							return true;
						}

						if ( index === 'email' && ! tve_is_email( value ) ) {
							errors_to_show = 'email';
							return true;
						}

						if ( index === 'confirm_password' && value !== indexed_array[ 'password' ] ) {
							errors_to_show = 'passwords_not_match';
							return true;
						}
					} );

					if ( errors_to_show.length > 0 ) {
						var form_config = tve_unserialize( TCB_Front.Base64.decode( hash ) ),
							error = document.createElement( 'span' );
						error.classList.add( error_class );
						error.innerText = form_config.error_messages[ errors_to_show ];

						if ( errors_to_show === 'empty_fields' ) {
							$current_from.find( 'input[type="text"], input[type="email"], input[type="password"]' ).not( '.' + error_class ).filter( function () {
								return ! this.value;
							} ).addClass( error_class ).after( error );
						} else if ( errors_to_show === 'email' ) {
							$current_from.find( 'input[type="email"]' ).not( '.' + error_class ).addClass( error_class ).after( error );
						} else if ( errors_to_show === 'passwords_not_match' ) {
							$current_from.find( 'input[type="password"]' ).not( '.' + error_class ).addClass( error_class ).after( error );
						}

						return false;
					}

					$submit_button.attr( 'disabled', 'disabled' );

					data.push( {
						name: 'payment_processor',
						value: tva_get_url_param( 'pp' )
					} );

					data.push( {
						name: 'bid',
						value: tva_get_url_param( 'bid' )
					} );

					data.push( {
						name: 'pid',
						value: tva_get_url_param( 'pid' )
					} );

					var route = get_route_name( $target.attr( 'data-route' ) );

					$.ajax( {
						headers: {
							'X-WP-Nonce': ThriveAppFront.nonce
						},
						type: 'POST',
						data: data,
						url: ThriveAppFront.routes.user + '/' + route + '/'
					} ).fail( function ( response ) {

						if ( _.contains( [
							'existing_user_email',
							'existing_user_login'
						], response.responseJSON.code ) ) {
							var form_config = tve_unserialize( TCB_Front.Base64.decode( hash ) ),
								error = document.createElement( 'span' );

							error.classList.add( error_class );
							var html = form_config.error_messages[ 'existing_user_email' ];
							html = html.replace( /\[a\]/g, '<a href="javascript:void(0)" class="tva-switch-instance-link" data-go_to_instance="login">' );
							html = html.replace( /\[\/a\]/g, '</a>' );

							jQuery( error ).html( html );

							$current_from.find( 'input[type="email"]' ).not( '.' + error_class ).addClass( error_class ).after( error );
						} else {
							TCB_Front.toast( response.responseJSON.message, true );
						}


					} ).done( function ( response ) {
						if ( response.message ) {
							TCB_Front.toast( response.message, false );
						}

						if ( response.state ) {
							$checkout.find( '.tcb-tva-checkout-form-wrapper' ).addClass( 'tcb-permanently-hidden' );
							$checkout.find( '.tcb-tva-checkout-form-wrapper[data-instance="' + response.state + '"]' ).removeClass( 'tcb-permanently-hidden' );

							toggle_instance_bar( $checkout, response.state )
						}

						if ( response.redirect ) {
							document.location.href = response.redirect;
						}
					} ).always( function () {

						$submit_button.removeAttr( 'disabled' );
					} );
				}

				/**
				 * Get route name
				 *
				 * @param route
				 * @returns {string|*}
				 */
				function get_route_name( route ) {
					var name = '';

					switch ( route ) {
						case 'register':
							name = 'tva_register';
							break;

						default:
							name = route;
							break;
					}

					return name;
				}

				/**
				 * Bind Events for checkout elements
				 *
				 * @param $btn_group
				 */
				function bind_events( $btn_group ) {
					$btn_group.off( 'click.thrv-button-group-item' ).on( 'click.thrv-button-group-item', '.thrv-button-group-item', show_state );
				}


				var $this = $( this ),
					$form = $this.find( 'form' );

				bind_events( $this.find( '.thrv-button-group' ) );

				/**
				 * Capture Submit Events
				 */
				$form.off( 'submit' ).on( 'submit', on_submit );

				var fields = [
					'input[type="text"]',
					'input[type="email"]',
					'input[type="password"]'
				];

				$form.find( fields.join( ',' ) ).on( 'input', on_input ).on( 'focus', on_focus ).on( 'blur', on_blur );

				/**
				 * Switch instance link
				 */
				$this.off( 'click.tva-switch-instance-link' ).on( 'click.tva-switch-instance-link', '.tva-switch-instance-link', switch_state_lnk );

				$this.find( '.thrv-button-group-item[data-default="true"]' ).click();

			} );
		}
	} );

	if ( ThriveAppFront.is_editor_page ) {
		TVE.add_filter( 'tcb_cloud_template_download', change_cloud_action );
	}

	function change_cloud_action( data ) {
		if ( ThriveAppFront.is_checkout || ThriveAppFront.is_thankyou ) {
			data.action = 'tva_cloud_templates'
		}

		return data;
	}

} )( jQuery );
