<?php
/**
 * Thrive Themes - https://thrivethemes.com
 *
 * @package thrive-apprentice
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Silence is golden
}

/**
 * modify the localization parameters for the javascript on the editor page (in editing mode)
 */
add_filter( 'tcb_editor_javascript_params', 'tva_editor_javascript_params', 10, 3 );

/**
 * Check the license
 */
add_filter( 'tcb_user_can_edit', 'tva_editor_check_license' );

/**
 * Custom Layouts
 */
add_filter( 'tcb_custom_post_layouts', 'tva_editor_layout', 10, 3 );

/**
 * Disable Revision Manager For Thrive Apprentice Pages
 */
add_filter( 'tcb_has_revision_manager', 'tva_disable_revision_manager', 10, 1 );

/**
 * Add some Apprentice post types to Architect Post Grid Element Banned Types
 */
add_filter( 'tcb_post_grid_banned_types', 'tva_add_post_grid_banned_types', 10, 1 );

add_filter( 'architect.branding', 'tva_architect_branding' );

/**
 * Adds TA product to TVA
 */
add_filter( 'tcb_element_instances', 'tva_add_product_to_tcb', 10, 1 );

/**
 * Include TVA Checkout Component Menu
 */
add_filter( 'tcb_menu_path_checkout', 'tva_include_checkout_menu', 10, 1 );
add_filter( 'tcb_menu_path_checkout_form', 'tva_include_checkout_form_menu', 10, 1 );

/**
 * Adds extra script(s) to the main frame
 */
add_action( 'tcb_main_frame_enqueue', 'tva_add_script_to_main_frame', 10, 0 );

/**
 * Adds extra SVG icons to editor page
 */
add_action( 'tcb_editor_iframe_after', 'tva_output_extra_control_panel_svg', 10, 0 );

/**
 * Loading templates of checkout element
 */
add_filter( 'tcb_backbone_templates', 'tve_include_backbone_template', 10, 1 );
add_filter( 'tcb_post_editable', 'tve_allow_architect_editing', 10, 3 );

/**
 * Allow Apprentice Pages to be edit with TAr on LpBuild
 */
add_filter( 'tve_post_type_can_use_landing_page', 'tva_post_type_can_use_landing_page', 10, 1 );

add_filter( 'tcb_can_use_landing_pages', 'tva_can_use_landing_page_templates', 10, 1 );
add_filter( 'tcb_has_templates_tab', 'tva_can_use_landing_page_templates', 10, 1 );

/**
 * Include Thrive Apprentice texts to TAR Editor
 */
add_filter( 'tcb_js_translate', 'tva_include_items_to_js_translate', 10, 1 );
add_filter( 'tcb_inline_shortcodes', 'tva_add_sendowl_product_shortcode', 10, 1 );

///////////////////////////////////////////////////////////////
/// 			    CLOUD TEMPLATES API                     ///
/// ///////////////////////////////////////////////////////////
add_filter( 'tcb_main_frame_localize', 'tva_checkout_page_templates', 10, 1 );
add_action( 'wp_ajax_tva_cloud_templates', 'tva_ajax_landing_page_cloud' );

/**
 * Include Modal Template for checkout
 */
add_filter( 'tcb_modal_templates', 'tva_include_tcb_modals' );

/**
 * Include Backbone templates
 *
 * @param $files
 *
 * @return array
 */
function tva_include_tcb_modals( $files ) {

	if ( apply_filters( 'tva_include_tcb_modals', TVA_Const::tva_is_checkout_page() ) ) {
		$files[] = TVA_Const::plugin_path( 'tcb-bridge/views/modals/checkout.php' );
	}

	return $files;
}

/**
 * Include backbone templates for TCB
 */
add_filter( 'tcb_backbone_templates', 'include_tcb_templates' );

/**
 * Include templates for checkout
 *
 * @param $templates
 *
 * @return array
 */
function include_tcb_templates( $templates ) {
	global $post;

	if ( apply_filters( 'include_tcb_templates', TVA_Const::tva_is_checkout_page( $post->ID ) ) ) {
		$templates['checkout-item'] = TVA_Const::plugin_path( 'tcb-bridge/views/cloud-templates/checkout-item.phtml' );
	}

	return $templates;
}

/**
 * Get A template
 */
function tva_ajax_landing_page_cloud() {

	if ( empty( $_POST['task'] ) ) {
		$error = __( 'Invalid request', 'thrive-cb' );
	}

	if ( ! isset( $error ) ) {

		try {
			switch ( $_POST['task'] ) {
				case 'download':
					$template = isset( $_POST['template'] ) ? $_POST['template'] : '';
					$post_id  = isset( $_POST['post_id'] ) ? $_POST['post_id'] : '';
					if ( empty( $template ) ) {
						throw new Exception( __( 'Invalid template', 'thrive-cb' ) );
					}
					$force_download = defined( 'TCB_CLOUD_DEBUG' ) && TCB_CLOUD_DEBUG;
					if ( ! $force_download ) {
						$transient_name = 'tva_template_download_' . $template;
						if ( get_transient( $transient_name ) === false ) {
							$force_download = true;
							set_transient( $transient_name, 1, DAY_IN_SECONDS );
						}
					}
					$downloaded = tve_get_downloaded_templates();

					if ( $force_download || ! array_key_exists( $template, $downloaded ) || tve_get_landing_page_config( $template ) === false ) {

						/**
						 * this will throw Exception if anything goes wrong
						 */
						add_filter( 'tcb_cloud_request_params', 'tva_filter_cloud_request_params' );

						TCB_Landing_Page_Cloud_Templates_Api::getInstance()->download( $template, 2 );

						remove_filter( 'tcb_cloud_request_params', 'tva_filter_cloud_request_params' );
					}

					tcb_landing_page( $post_id )->set_cloud_template( $template );

					wp_send_json(
						array(
							'success' => true,
						)
					);
			}
		} catch ( Exception $e ) {
			wp_send_json(
				array(
					'success' => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}

	wp_die();
}

/**
 * Rewrite landing page templates localization for checkout and thankyou page templates
 *
 * @param $data
 *
 * @return mixed
 */
function tva_checkout_page_templates( $data ) {

	$type = TVA_Const::get_page_type();

	if ( $type ) {
		$data['cloud_lp_templates'] = tva_get_cloud_templates( $type );
	}

	return $data;
}

/**
 * Get Landing Page Templates
 *
 * @param $type string checkout|thankyou
 *
 * @return array|mixed
 */
function tva_get_cloud_templates( $type ) {

	$transient_name = 'tva_cloud_cache_' . $type;

	unset( $GLOBALS['tcb_lp_cloud_error'] );

	if ( defined( 'TCB_CLOUD_DEBUG' ) && TCB_CLOUD_DEBUG ) {
		delete_transient( $transient_name );
	}

	$cache_for = apply_filters( 'tcb_apprentice_cloud_cache', 3600 * 8 );
	$templates = get_transient( $transient_name );

	if ( false === $templates ) {
		/**
		 * Make sure the correct TA template sets are returned.
		 */
		add_filter( 'tcb_cloud_request_params', 'tva_filter_cloud_request_params' );
		try {
			$templates = TCB_Landing_Page_Cloud_Templates_Api::getInstance()->getTemplateList();
			set_transient( $transient_name, $templates, $cache_for );

		} catch ( Exception $e ) {

			/* save the error message to display it in the LP modal */
			$GLOBALS['tcb_lp_cloud_error'] = $e->getMessage();
			$templates                     = array();
		}
		/**
		 * Immediately remove this filter so that further calls from TAr do not return TA template sets
		 */
		remove_filter( 'tcb_cloud_request_params', 'tva_filter_cloud_request_params' );
	}

	return $templates;
}


/**
 * Add Sendowl Shortcode
 *
 * @param $shortcodes
 *
 * @return array
 */
function tva_add_sendowl_product_shortcode( $shortcodes ) {

	if ( ! empty( $_REQUEST[ TVE_FRAME_FLAG ] ) && ( TVA_Const::tva_is_checkout_page() || TVA_Const::tva_is_thankyou_page() ) ) {
		$shortcode = array(
			__( 'Thrive Product', 'thrive-apprentice' ) => array(
				array(
					'name'   => __( 'Product Name', 'thrive-apprentice' ),
					'option' => __( 'Product Name', 'thrive-apprentice' ),
					'value'  => 'tva_sendowl_product',
				),
			),
		);

		$shortcodes = array_merge( $shortcodes, $shortcode );
	}

	return $shortcodes;
}

/**
 * Disable landing page functionality when TCB is not active
 *
 * @param $value
 *
 * @return bool
 */
function tva_can_use_landing_page_templates( $value ) {

	if ( TVA_Const::tva_is_checkout_page() || TVA_Const::tva_is_thankyou_page() ) {

		return defined( 'EXTERNAL_TCB' ) && EXTERNAL_TCB === 1;
	}

	return $value;
}

/**
 * Inject apprentice_pages post type in list of post types which can be landing pages
 *
 * @param $post_types array
 *
 * @return array $post_types
 */
function tva_post_type_can_use_landing_page( $post_types ) {

	$post_types[] = 'apprentice_pages';

	return $post_types;
}

/**
 * Allow Apprentice lessons modules and chapters and the checkout page to be editable with TCB
 *
 * @param $return
 * @param $post_type
 * @param $post_id
 *
 * @return bool
 */
function tve_allow_architect_editing( $return, $post_type, $post_id ) {

	if ( ! EXTERNAL_TCB ) {
		$user_settings = TVA_Settings::instance();
		$settings      = $user_settings->get_settings();
		if ( TVA_Const::POST_TYPE === $post_type || $post_id === (int) $settings['checkout_page']['ID'] ) {
			return $return;
		}

		return false;
	}

	return $return;
}

/**
 * Is Editable with TCB
 *
 * @param int|string $post_or_type
 *
 * @return bool
 */
function tva_is_editable( $post_or_type ) {

	$post_or_type = is_numeric( $post_or_type ) ? get_post_type( $post_or_type ) : $post_or_type;

	global $post;

	$return = in_array( $post_or_type, array( TVA_Const::POST_TYPE ) );
	$return = $return || ( $post instanceof WP_Post && TVA_Const::tva_is_checkout_page() );
	$return = $return || ( defined( 'DOING_AJAX' ) && DOING_AJAX );

	return apply_filters( 'tva_is_editable', $return, $post_or_type );
}

function tva_editor_check_license( $valid ) {

	if ( ! tva_is_apprentice() ) {
		return $valid;
	}

	if ( ! tva_license_activated() ) {
		add_action( 'wp_print_footer_scripts', 'tva_license_warning' );

		return false;
	}

	return true;
}

/**
 * Disable Revision Manager For Quiz Builder Pages
 *
 * @param bool $status
 *
 * @return bool
 */
function tva_disable_revision_manager( $status = true ) {
	$post_type = get_post_type();

	if ( tva_is_editable( $post_type ) ) {
		return ! $status;
	}

	return $status;
}

/**
 * Add some Apprentice post types to Architect Post Grid Element Banned Types
 *
 * @param array $banned_types
 *
 * @return array
 */
function tva_add_post_grid_banned_types( $banned_types = array() ) {
	$banned_types[] = TVA_Const::COURSE_POST_TYPE;

	return $banned_types;
}

/**
 * Set JS params
 *
 * @param $js_params
 * @param $post_id
 *
 * @return mixed
 */
function tva_editor_javascript_params( $js_params, $post_id ) {

	$user_settings = TVA_Settings::instance();
	$settings      = $user_settings->get_settings();

	if ( ( ! tva_is_apprentice() && ! ( $post_id === (int) $settings['checkout_page']['ID'] ) ) || EXTERNAL_TCB ) {
		return $js_params;
	}

	/** clear out any data that's not necessary on the editor  */
	$js_params['landing_page']          = '';
	$js_params['landing_page_config']   = array();
	$js_params['landing_pages']         = array();
	$js_params['page_events']           = array();
	$js_params['landing_page_lightbox'] = array();
	$js_params['style_families']        = tve_get_style_families();
	$js_params['style_classes']         = array(
		'Flat'    => 'tve_flt',
		'Classy'  => 'tve_clsy',
		'Minimal' => 'tve_min',
	);
	$js_params['read_more_tag']         = false;

	return $js_params;
}

function tva_editor_layout( $current_templates, $post_id, $post_type ) {
	if ( ! tva_is_editable( $post_type ) ) {
		return $current_templates;
	}

	/* flat is the default style for Thrive Quiz Builder designs */
	global $tve_style_family_classes;
	$tve_style_families = tve_get_style_families();
	$style_family       = 'Flat';
	$style_key          = 'tve_style_family_' . strtolower( $tve_style_family_classes[ $style_family ] );
	/* Style family */
	wp_style_is( $style_key ) || tve_enqueue_style( $style_key, $tve_style_families[ $style_family ] );

	return $current_templates;
}

/**
 * show a box with a warning message and a link to take the user to the license activation page
 * this will be called only when no valid / activated license has been found
 *
 * @return mixed
 */
function tva_license_warning() {

	return include TVA_Const::plugin_path( 'admin/views/license_inactive.php' );
}

/**
 * @param string $logo_url
 *
 * @return string
 */
function tva_architect_branding( $logo_url ) {
	if ( tva_is_editable( get_post_type() ) ) {
		$logo_url = TVA_Const::plugin_url( 'admin/img/ta-logo-dashboard.png' );
	}

	return $logo_url;
}

/**
 * Adds checkout element to TAR Editor
 *
 * @param array $elements
 *
 * @return array
 */
function tva_add_product_to_tcb( $elements = array() ) {

	if ( ( tva_is_editable( get_post_type() ) && TVA_Const::tva_is_checkout_page() ) || ( isset( $_REQUEST['type'] ) && 'checkout' === $_REQUEST['type'] ) ) {

		require_once dirname( __FILE__ ) . '/editor-elements/class-tcb-checkout-element.php';
		require_once dirname( __FILE__ ) . '/editor-elements/class-tcb-checkout-form-element.php';

		$elements['checkout']      = new TCB_Checkout_Element( 'checkout' );
		$elements['checkout_form'] = new TCB_Checkout_Form_Element( 'checkout_form' );
	}

	return $elements;
}

/**
 * Gets Checkout Element Menu Component file path
 *
 * @param $component_file_path string
 *
 * @return string
 */
function tva_include_checkout_menu( $component_file_path ) {

	if ( tva_is_editable( get_post_type() ) ) {

		return TVA_Const::plugin_path( 'tcb-bridge/editor-layouts/menus/checkout.php' );
	}

	return $component_file_path;
}

/**
 * Gets CheckoutForm Element Menu Component file path
 *
 * @param $component_file_path string
 *
 * @return string
 */
function tva_include_checkout_form_menu( $component_file_path ) {

	if ( tva_is_editable( get_post_type() ) ) {

		return TVA_Const::plugin_path( 'tcb-bridge/editor-layouts/menus/checkout_form.php' );
	}

	return $component_file_path;
}

function tva_add_script_to_main_frame() {
	if ( tva_is_editable( get_post_type() ) ) {
		tva_enqueue_script( 'tva-internal-editor', TVA_Const::plugin_url( 'tcb-bridge/assets/js/tva-tcb-internal.js' ), array( 'tve-main' ) );
		tva_enqueue_style( 'tva-ce-preview-list', TVA_Const::plugin_url( 'tcb-bridge/assets/css/main_frame.css' ) );
	}
}

/**
 * Loading checkout element's templates only in editor
 *
 * @param array $templates
 *
 * @return array
 */
function tve_include_backbone_template( $templates = array() ) {
	if ( tva_is_editable( get_post_type() ) ) {
		$templates = array_merge( $templates, tve_dash_get_backbone_templates( plugin_dir_path( dirname( __FILE__ ) ) . 'tcb-bridge/editor-layouts/backbone', 'backbone' ) );
	}

	return $templates;
}

/**
 * Outputs Extra SVG Icons to editor page (Control Panel)
 */
function tva_output_extra_control_panel_svg() {

	if ( tva_is_editable( get_post_type() ) ) {
		include dirname( __FILE__ ) . '/assets/fonts/tva-main.svg';
	}
}

/**
 * Include Thrive Apprentice Text to TAR Editor
 *
 * @param array $translate_arr
 *
 * @return array
 */
function tva_include_items_to_js_translate( $translate_arr = array() ) {

	if ( tva_is_editable( get_post_type() ) ) {
		if ( ! empty( $translate_arr['cf_errors'] ) && is_array( $translate_arr['cf_errors'] ) ) {
			$translate_arr['cf_errors']['existing_user_email'] = array(
				'label' => __( 'Email Is Already Used', 'thrive-apprentice' ),
			);
		}
	}

	return $translate_arr;
}

function tva_filter_cloud_request_params( $params ) {

	global $post;

	$post_id = null;
	$type    = null;

	if ( $post instanceof WP_Post ) {
		$post_id = $post->ID;
	} else {
		/**
		 * $page_id is sent over $_REQUEST when zip template is imported from page settings
		 */
		$post_id = ! empty( $_REQUEST['page_id'] ) ? $_REQUEST['page_id'] : null;
	}

	/**
	 * loading a template through ajax $post_id param is used
	 */

	if ( ! isset( $post_id ) ) {
		$post_id = ! empty( $_REQUEST['post_id'] ) ? $_REQUEST['post_id'] : null;
	}

	if ( isset( $post_id ) ) {
		$type = TVA_Const::get_page_type( $post_id );
	}

	if ( isset( $type ) ) {
		$params['appr'] = '1';
		$params['type'] = $type;
	}

	return $params;
}
