<?php
/**
 * Thrive Themes - https://thrivethemes.com
 *
 * @package thrive-apprentice
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Silence is golden
}

tva_overwrite_post();
?>

<?php include_once( TVA_Const::plugin_path( 'templates/header.php' ) ); ?>

<?php
$course = tva_get_course_by_slug(
	$term,
	array(
		'published'  => true,
		'protection' => true,
	)
);

$user_settings = TVA_Settings::instance();
$settings      = $user_settings->get_settings();
?>

<?php $topic = tva_get_topic_by_id( $course->topic ); ?>
<div class="tva-cm-redesigned-breadcrumbs">
	<?php tva_custom_breadcrumbs(); ?>
</div>
<div class="tva-frontend-template" id="tva-course-overview">
	<div class="tva-container">
		<div class="tva-course-head tva-course-head-<?php echo $topic['ID']; ?> tva-course-type-<?php echo $course->course_type_class; ?>">
			<div class="tva-course-icon">
				<img class="tva-course-icon" src="<?php echo ! empty( $topic['icon'] ) ? $topic['icon'] : TVA_Const::get_default_course_icon_url(); ?>"/>
				<span class="tva-lesson-name"><?php echo $topic['title']; ?></span>
			</div>
			<div class="tva-uni-course-type">
				<i></i>
				<span><?php echo tva_is_course_guide( $course ) ? __( 'Guide', 'thrive-apprentice' ) : $course->course_type; ?></span>
			</div>
		</div>
		<div class="tva-featured-image-container-single" <?php echo ! $course->cover_image ? 'style="background-color: ' . $topic['color'] . '; opacity: 0.7"' : ''; ?>>
			<div class="tva-image-overlay"></div>
			<?php if ( ! empty( $course->cover_image ) ) : ?>
				<div style="background-image:url('<?php echo $course->cover_image; ?>')" class="tva-image-as-bg"></div>
			<?php endif; ?>
		</div>

		<section
				class="tva-course-section <?php echo ( ( tva_is_course_guide( $course ) ) ) && ! is_active_sidebar( 'tva-sidebar' ) ? 'tva-course-guide-section' : ''; ?>">

			<div class="tva-archive-content">
				<?php $sendowl_referer = isset( $_SERVER['HTTP_REFERER'] ) && ! empty( $_SERVER['HTTP_REFERER'] ) && strpos( $_SERVER['HTTP_REFERER'], 'transactions.sendowl' ) !== false; ?>
				<?php if ( $course->logged_in && 'redirect' === $settings['thankyou_page']['type'] && $sendowl_referer ) : ?>
					<?php $message = str_replace( '[course_name]', $course->name, $settings['thankyou_page_message'] ); ?>
					<?php echo $message; ?>
				<?php endif; ?>
				<h1 class="tva_course_title"><?php echo $course->name; ?></h1>
				<div class="tva_page_headline_wrapper tva-course-counters">
					<div class="tva-course-numbers">
						<?php if ( $course->modules_count > 0 ) : ?>
							<span class="item-value"><?php echo $course->modules_count; ?> </span>
							<span class="item-name <?php echo 1 === $course->modules_count ? 'tva_course_module' : 'tva_course_modules'; ?>">
									<?php echo 1 === $course->modules_count ? $settings['template']['course_module'] : $settings['template']['course_modules']; ?>
								</span>
						<?php endif; ?>
						<?php if ( $course->chapters_count > 0 ) : ?>
							<span class="item-value">
									<?php echo $course->chapters_count; ?>
								</span>
							<span class="item-name <?php echo 1 === $course->chapters_count ? 'tva_course_chapter' : 'tva_course_chapters'; ?> ">
								<?php echo 1 === $course->chapters_count ? $settings['template']['course_chapter'] : $settings['template']['course_chapters']; ?>
							</span>
						<?php endif; ?>
						<?php if ( $course->published_lessons_count > 0 ) : ?>
							<span class="item-value">
								<?php echo $course->published_lessons_count; ?>
							</span>
							<span class="item-name <?php echo 1 === $course->published_lessons_count ? 'tva_course_lesson' : 'tva_course_lessons'; ?>">
								<?php echo 1 === $course->published_lessons_count ? $settings['template']['course_lesson'] : $settings['template']['course_lessons']; ?>
							</span>
						<?php endif; ?>

						<span class="tva-course-dificulty">
								<?php if ( 0 != $course->level ) : ?>
									<?php foreach ( $levels as $level ) : ?>
										<?php if ( $level['ID'] == $course->level ) : ?>
											<?php echo $level['name']; ?>
										<?php endif; ?>
									<?php endforeach; ?>
								<?php endif; ?>
							</span>
					</div>

					<p class="tva_page_headline tva_page_headline_text">
						<?php echo isset( $settings['template']['page_headline_text'] ) ? $settings['template']['page_headline_text'] : TVA_Const::TVA_ABOUT; ?>
					</p>
					<?php if ( true == $course->video_status ) : ?>
						<div class="tva-featured-video-container-single">
							<?php echo $course->term_video_embed; ?>
						</div>
					<?php endif ?>
				</div>

				<div class="tva-course-description">
						<span class="tva_paragraph">
							<?php echo $course->description; ?>
						</span>
				</div>

				<?php if ( $course->allowed ) : ?>
					<a class="tva_start_course tva_main_color_bg" href="<?php echo tva_get_start_course_url( $course ); ?>">
						<?php echo isset( $settings['template']['start_course'] ) ? $settings['template']['start_course'] : TVA_Const::TVA_START; ?>
					</a>
				<?php endif; ?>
			</div>
			<div class="tva-cm-container">
				<h1 class="tva_course_structure"><?php echo $settings['template']['course_structure']; ?></h1>

				<?php if ( ! empty( $course->modules ) ) : ?>
					<?php foreach ( $course->modules as $module ) : ?>
						<?php echo tva_generate_module_html( $module ); ?>
					<?php endforeach; ?>
				<?php elseif ( ! empty( $course->chapters ) ) : ?>
					<?php foreach ( $course->chapters as $chapter ) : ?>
						<?php echo tva_generate_chapter_html( $chapter ); ?>
					<?php endforeach; ?>
				<?php else : ?>
					<?php if ( ! empty( $course->lessons ) ) : ?>
						<?php foreach ( $course->lessons as $lesson ) : ?>
							<?php echo tva_generate_lesson_html( $lesson, true ); ?>
						<?php endforeach; ?>
					<?php endif; ?>
				<?php endif ?>
			</div>

			<?php
			if ( ( 'open' === $course->comment_status ) && ( true === $course->can_comment ) ) {
				TVA_Db::setCommentsStatus();
				comments_template( '', true );
				echo apply_filters( 'comment_form_submit_field', '' );
				tva_add_tcm_triggers();
			}
			?>
		</section>
		<?php if ( is_active_sidebar( 'tva-sidebar' ) ) : ?>
			<aside class="tva-sidebar-container">
				<div class="tva-sidebar-wrapper">
					<?php dynamic_sidebar( 'tva-sidebar' ); ?>
				</div>
			</aside>
		<?php endif; ?>
	</div>
</div>

<?php echo tva_add_apprentice_label(); ?>

<?php include_once( TVA_Const::plugin_path( 'templates/footer.php' ) ); ?>
