/**
 * Created by Pop Aurelian on 21-Feb-17.
 */

var ThriveApp = ThriveApp || {};
ThriveApp.util = ThriveApp.util || {};

/**
 * Settings for the underscore templates
 * Enables <##> tags instead of <%%>
 *
 * @type {{evaluate: RegExp, interpolate: RegExp, escape: RegExp}}
 */
_.templateSettings = {
	evaluate: /<#([\s\S]+?)#>/g,
	interpolate: /<#=([\s\S]+?)#>/g,
	escape: /<#-([\s\S]+?)#>/g
};

/**
 * Uppercase the 1st letter in string
 *
 * @param str
 * @returns string
 */
ThriveApp.util.upperFirst = function ( str ) {
	if ( ! str ) {
		return '';
	}

	return str.toLowerCase().charAt( 0 ).toUpperCase() + str.slice( 1 );
};

ThriveApp.util.hierarchy = {
	0: 'lesson',
	1: 'chapter',
	2: 'module'
};

ThriveApp.util.levels = {
	lesson: 0,
	chapter: 1,
	module: 2
};

/**
 * Decisions are based on hierarchy
 * 0 - no decission has to be made
 * 1 - lower level items need to be included within this level
 * 2 - this item needs to be included into a higher level and the user has to decide where
 *
 * @param item_level
 * @param model
 * @returns {number}
 */
ThriveApp.util.userHasToDecide = function ( item_level, model ) {

	var decision = 0,
		lowest = item_level - 2,
		low = item_level - 1,
		high = ++ item_level,
		highest = ++ item_level,
		d_model = ! model ? ThriveApp.globals.active_course : model;

	if ( ( ThriveApp.util.hierarchy[ lowest ] && d_model.get( ThriveApp.util.hierarchy[ lowest ] + 's' ).length > 0 ) || ( ThriveApp.util.hierarchy[ low ] && d_model.get( ThriveApp.util.hierarchy[ low ] + 's' ).length > 0 ) ) {
		decision = 1;
	} else if ( ( ThriveApp.util.hierarchy[ high ] && d_model.get( ThriveApp.util.hierarchy[ high ] + 's' ) && d_model.get( ThriveApp.util.hierarchy[ high ] + 's' ).length > 0 ) || ( ThriveApp.util.hierarchy[ highest ] && d_model.get( ThriveApp.util.hierarchy[ highest ] + 's' ) && d_model.get( ThriveApp.util.hierarchy[ highest ] + 's' ).length > 0 ) ) {
		decision = 2;
	}

	return decision;
};

/**
 * Un-map All the couse elements so we have a place where to search data when needed without any hierarchy
 * We keep the elements as they are so we have references and won't need to update them every time a change is made
 * @param course
 * @returns {*}
 */
ThriveApp.util.getCourseElements = function ( course ) {
	var elements = new ThriveApp.models.CourseElements( {} );

	elements.get( 'all' ).reset();
	elements.get( 'modules' ).reset();
	elements.get( 'chapters' ).reset();
	elements.get( 'lessons' ).reset();

	if ( course.get( 'modules' ).length > 0 ) {
		course.get( 'modules' ).each( function ( module ) {
			if ( module.get( 'chapters' ) ) {
				module.get( 'chapters' ).each( function ( chapter ) {
					if ( chapter.get( 'lessons' ) ) {
						chapter.get( 'lessons' ).each( function ( lesson ) {
							elements.get( 'lessons' ).add( lesson );
							elements.get( 'all' ).add( lesson );
						} );
					}
					elements.get( 'chapters' ).add( chapter );
					elements.get( 'all' ).add( chapter );
				} );
			}

			if ( module.get( 'lessons' ) ) {
				module.get( 'lessons' ).each( function ( lesson ) {
					elements.get( 'lessons' ).add( lesson );
					elements.get( 'all' ).add( lesson );
				} );
			}

			elements.get( 'modules' ).add( module );
			elements.get( 'all' ).add( module );
		} );

		return elements;
	}

	if ( course.get( 'chapters' ).length > 0 ) {
		course.get( 'chapters' ).each( function ( chapter ) {
			if ( chapter.get( 'lessons' ) ) {
				chapter.get( 'lessons' ).each( function ( lesson ) {
					elements.get( 'lessons' ).add( lesson );
					elements.get( 'all' ).add( lesson );
				} );
			}

			elements.get( 'chapters' ).add( chapter );
			elements.get( 'all' ).add( chapter );
		} );

		return elements;
	}

	if ( course.get( 'lessons' ).length > 0 ) {
		course.get( 'lessons' ).each( function ( lesson ) {
			elements.get( 'lessons' ).add( lesson );
			elements.get( 'all' ).add( lesson );
		} );
	}

	return elements;
};


/**
 * Slugify string
 *
 * @param word
 */
ThriveApp.util.slugify = function ( word ) {
	return ThriveApp.util.split( word, "-" ).toLowerCase();
};

/**
 * Split words
 *
 * @param word
 * @param delim
 */
ThriveApp.util.split = function ( word, delim ) {
	delim = delim || " ";
	var replacement = "$1" + delim + "$2";
	word = ThriveApp.util.to_title_case( word ).replace( / /g, '' );
	return word.replace( /([A-Z]+)([A-Z][a-z])/g, replacement ).replace( /([a-z\d])([A-Z])/g, replacement );
};

/**
 * To Title Case
 *
 * @param str
 */
ThriveApp.util.to_title_case = function ( str ) {
	return str.replace( /\w\S*/g, function ( txt ) {
		return txt.charAt( 0 ).toUpperCase() + txt.substr( 1 ).toLowerCase();
	} );
};

/**
 * Clear remove a specific editor or an array of editors
 *
 * @param editor
 * @param ignore
 */
ThriveApp.util.clearMCEEditor = function ( editor, ignore ) {
	if ( typeof tinymce === 'undefined' || ! tinymce ) {
		return;
	}
	if ( typeof editor === 'string' ) {
		if ( ignore != editor ) {
			var _current = tinymce.get( editor );
			if ( _current ) {
				_current.destroy();
				try {
					tinymce.execCommand( 'mceRemoveControl', true, element )
				} catch ( e ) {}
			}
		}
	} else {
		editor.forEach( function ( element ) {
			if ( ignore != element ) {
				var _current = tinymce.get( element );
				if ( _current ) {
					_current.destroy();
					try {
						tinymce.execCommand( 'mceRemoveControl', true, element )
					} catch ( e ) {}

				}
			}
		} );
	}
};

ThriveApp.util.addCoverImage = function ( model ) {
	var frame;

	// If the media frame already exists, reopen it.
	if ( frame ) {
		frame.open();
		return;
	}

	// Create a new media frame
	frame = wp.media( {
		title: 'Select or Upload a Cover Image',
		button: {
			text: 'Use this Image'
		},
		library: {type: 'image'},
		multiple: false  // Set to true to allow multiple files to be selected
	} );

	// When an image is selected in the media frame...
	frame.on( 'select', function () {
		var attachment = frame.state().get( 'selection' ).first().toJSON();
		model.set( {cover_image: attachment.url} );
	} );

	// Finally, open the modal on click
	frame.open();
};

ThriveApp.util.renderMCE = function ( id, model, prop ) {
	setTimeout( function () {
		ThriveApp.util.clearMCEEditor( id );
		ThriveApp.util.editorInit( id, model, prop );
	}, 0 );
};

ThriveApp.util.editorInit = function ( id, model, prop ) {
	if ( typeof tinymce === 'undefined' || ! tinymce ) {
		return;
	}
	mce_reinit = ThriveApp.util.build_mce_init( {
		mce: window.tinyMCEPreInit.mceInit[ 'tve_tinymce_tpl' ],
		qt: window.tinyMCEPreInit.qtInit[ 'tve_tinymce_tpl' ]
	}, id );

	tinyMCEPreInit.mceInit = $.extend( tinyMCEPreInit.mceInit, mce_reinit.mce_init );
	tinyMCEPreInit.mceInit[ id ].setup = function ( editor ) {
		editor.on( 'init', function () {
			editor.setContent( model.get( prop ) );
		} );
		editor.on( 'change', function ( e ) {
			var value = tinymce.get( id );

			model.set( prop, value.getContent() );

		} );
	};
	tinymce.init( tinyMCEPreInit.mceInit[ id ] );

};

/**
 * Build the editor
 *
 * @param defaults
 * @param _id
 * @returns {{mce_init: {}, qt_init: {}}}
 */
ThriveApp.util.build_mce_init = function ( defaults, _id ) {
	var mce = {}, qt = {};
	mce[ _id ] = jQuery.extend( true, {}, defaults.mce );
	qt[ _id ] = jQuery.extend( true, {}, defaults.qt );

	qt[ _id ].id = _id;

	mce[ _id ].selector = '#' + _id;
	mce[ _id ].body_class = mce[ _id ].body_class.replace( 'tve_tinymce_tpl', _id );

	return {
		'mce_init': mce,
		'qt_init': qt
	};
};

/**
 * ajax-suggest post search for an input
 * if fetch_single is passed and it's not empty, it will also fetch the selected post from the server and populate the input with it
 *
 * @param {object} $input jquery wrapper over the autocomplete input
 * @param {object} options map of autocomplete options to control jquery ui autocomplete
 * @constructor
 */
ThriveApp.PostSearch = function ( $input, options ) {
	options = options || {};
	options.no_value_callback = options.no_value_callback || jQuery.noop;
	options.change_callback = options.change_callback || jQuery.noop;

	function matches() {
		var regex;

		if ( ! ( regex = $input.data( 'allow-regex' ) ) ) {
			return false;
		}

		return $input.val().match( new RegExp( regex ) );
	}

	var defaults = {
		appendTo: $input.parent(),
		minLength: 2,
		delay: 200,
		change: function ( event, ui ) {
			if ( ! ui.item && ! $input.data( 'value-filled' ) && ! matches() ) {
				$input.val( '' );
				options.no_value_callback.apply( $input, arguments );
			}
			$input.data( 'value-filled', null );

			if ( matches() ) {
				options.change_callback.apply( $input, arguments );
			}
		}
	};

	options = jQuery.extend( true, defaults, options );

	if ( ! options.source ) {
		options.source = options.url;
	}

	jQuery.ajaxSetup( {
		headers: {'X-WP-Nonce': ThriveApp.nonce}
	} );

	$input.autocomplete( options ).data( "ui-autocomplete" )._renderItem = function ( ul, item ) {
		var _class = '';
		if ( options.collection && options.collection.length ) {
			var model = options.collection.findWhere( {id: item.id} );
			if ( model ) {
				_class = 'tva-selected-post';
			}
		}

		return jQuery( "<li class='" + _class + "'>" ).append( "<span class='tva-ps-result-title'>" + item.label + "</span><span class='tva-ps-result-email'>" + "(" + item.type + ")" + "</span>" ).appendTo( ul );
	};

	$input.on( 'blur', function () {
		if ( ! jQuery.trim( this.value ).length ) {
			options.no_value_callback.apply( $input, arguments );
		}
	} );

	if ( options.fetch_single && typeof options.fetch_single === 'number' ) {
		$input.addClass( 'ui-autocomplete-loading' );
		jQuery.ajax( {
			url: ThriveApp.router.courses,
			data: {
				id: options.fetch_single
			},
			success: function ( result ) {
				$input.data( 'value-filled', 1 ).val( result.title ).removeClass( 'ui-autocomplete-loading' ).next( 'label' ).addClass( 'tvd-active' );
			}
		} );
	}

	if ( options.fetch_single && typeof options.fetch_single === 'string' ) {
		$input.data( 'value-filled', 1 ).val( options.fetch_single ).next( 'label' ).addClass( 'tvd-active' );
	}
};

ThriveApp.util.Editor = {
	iframe: '',
	apply: function ( selector, prop, value, extra, hover_out ) {
		if ( ! extra ) {
			extra = '';
		}

		var $elements = this.iframe.find( selector );

		_.each( $elements, function ( element ) {
			if ( ! hover_out ) {
				jQuery( element ).css( prop, value + extra );
			} else {

				if ( extra === true ) {
					extra = '';
				}

				jQuery( element ).hover( function () {
					var sub_elem = prop === 'fill' ? $( this ).find( 'svg' ) : $( this );
					sub_elem.css( prop, value + extra );
				}, function () {
					var sub_elem = prop === 'fill' ? $( this ).find( 'svg' ) : $( this );
					sub_elem.css( prop, hover_out + extra );
				} );
			}
		} )
	},
	font_to_head: function ( url ) {
		var element = this.iframe.find( '#tva_google_font' );

		if ( element ) {
			element.remove();
		}

		var link = '<link rel="stylesheet" id="tva_google_font" href="' + url + '" type="text/css" media="all">';
		jQuery( this.iframe.find( 'head' ) ).append( link );
	},
	text: function ( selector, value ) {

		var $elements = this.iframe.find( selector );

		_.each( $elements, function ( element ) {
			jQuery( element ).text( value );
		} )
	},
	attrChange: function ( selector, value, attr ) {

		var $elements = this.iframe.find( selector );

		_.each( $elements, function ( element ) {
			jQuery( element ).attr( attr, value );
		} )
	},
	image: function ( selector, value, extra ) {
		var $elements = this.iframe.find( selector );

		_.each( $elements, function ( element ) {
			jQuery( element ).empty().append( '<img src="' + value + '"/>' );
		} )
	},
	emptyElement: function ( element ) {

		if ( element ) {
			this.iframe.find( element ).empty();
		}
	},
	init: function () {
		this.iframe = jQuery( 'iframe#tva_frame' ).contents();

		this.iframe.find( 'body' ).css( 'overflow', 'initial' );
		this.iframe.find( 'a' ).click( function ( event ) {
			event.preventDefault();
		} );
	}
};

ThriveApp.util.makeBulkActionsStiky = function () {
	var position = $( '.tva-course-container' )[ 0 ].getBoundingClientRect(),
		$container = $( '.tva-bulk-actions-container' );

	position.top < 0 ? $container.addClass( 'tva-fixed-bulk-actions' ) : $container.removeClass( 'tva-fixed-bulk-actions' );
};

ThriveApp.util.isCollection = function ( collection ) {
	return ( typeof collection === 'object' ) && ( collection instanceof Backbone.Collection );
};

ThriveApp.util.getPlural = function ( str ) {
	return str + 's';
};

ThriveApp.util.rebindWistiaFancyBoxes = function () {
	/**
	 * rebind the wistia listeners
	 */
	if ( window.rebindWistiaFancyBoxes ) {
		window.rebindWistiaFancyBoxes();
	}
};

ThriveApp.util.getKeyByValue = function ( array, value ) {
	for ( var key in array ) {
		if ( array[ key ] === parseInt( value ) ) {
			return key;
		}
	}
};

/**
 * Check if email is valid
 *
 * @param email
 * @returns {boolean}
 */
ThriveApp.util.isEmail = function ( email ) {
	var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

	return re.test( String( email ).toLowerCase() );
};

/**
 * Remove empty lines from a string
 *
 * @param data
 * @returns {*}
 */
ThriveApp.util.removeEmptyLine = function ( data ) {

	if ( typeof data !== 'string' ) {
		return false;
	}

	data = data.trim();

	if ( data.length === 0 ) {
		return false;
	}

	var chunks = data.split( '\n' ),
		parts = [];

	$.each( chunks, function ( key, value ) {
		if ( value.trim().length ) {
			parts.push( value );
		}
	} );

	return parts.join( '\n' );
};

/**
 * Check for a valid url
 *
 * @param url
 * @returns {boolean}
 */
ThriveApp.util.isUrl = function ( url ) {
	var pattern = new RegExp( '^(https?:\\/\\/)?' + // protocol
	                          '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.?)+[a-z]{2,}|' + // domain name
	                          '((\\d{1,3}\\.){3}\\d{1,3}))' + // ip (v4) address
	                          '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + //port
	                          '(\\?[;&amp;a-z\\d%_.~+=-]*)?' + // query string
	                          '(\\#[-a-z\\d_]*)?$', 'i' );

	return pattern.test( url );
};
;var ThriveApp = ThriveApp || {};
ThriveApp.util = ThriveApp.util || {};

/**
 * Storage handler when localStorage is not available
 */
var FallbackStorage = {
	setItem: function ( key, value ) {
	},
	getItem: function ( key ) {
	},
	removeItem: function ( key ) {
	}
};

/**
 * Main object constructor
 *
 * @constructor
 */
function ThriveAppStorage() {
	this.api = window.localStorage || FallbackStorage;
	this.listening = false;
	this.listeners = {};
}

/**
 * Persist a value in a key
 *
 * @param {String} key
 * @param {*} value
 * @returns {ThriveAppStorage}
 */
ThriveAppStorage.prototype.set = function ( key, value ) {
	if ( ! key || typeof value === 'undefined' || value === null ) {
		return this;
	}

	if ( typeof value === 'object' ) {
		value = JSON.stringify( value );
	}
	try {
		this.api.setItem( key, value );
	} catch ( e ) {
	}

	return this;
};

/**
 *
 * @param {String} key
 * @returns {*|undefined}
 */
ThriveAppStorage.prototype.get = function ( key ) {
	if ( ! key ) {
		return undefined;
	}

	var value = this.api.getItem( key );
	if ( ! value ) {
		return value;
	}
	try {
		return JSON.parse( value );
	} catch ( e ) {
		return value;
	}
};

/**
 * Removes an item from storage
 *
 * @param {String} key
 *
 * @return {ThriveAppStorage} allow chained calls
 */
ThriveAppStorage.prototype.unset = function ( key ) {
	if ( ! key ) {
		return this;
	}
	try {
		this.api.removeItem( key );
	} catch ( e ) {

	}

	return this;
};

ThriveAppStorage.prototype._listen = function () {

	var self = this,
		change = function ( e ) {
			if ( ! e ) {
				e = window.event;
			}

			var all = self.listeners[e.key];

			if ( all && all.length ) {
				all.forEach( function ( listener ) {
					listener( JSON.parse( e.newValue ), JSON.parse( e.oldValue ) );
				} );
			}
		};

	if ( window.addEventListener ) {
		window.addEventListener( 'storage', change, false );
	} else if ( window.attachEvent ) {
		window.attachEvent( 'onstorage', change );
	} else {
		window.onstorage = change;
	}
};


ThriveAppStorage.prototype.on = function ( key, fn ) {
	if ( this.listeners[key] ) {
		this.listeners[key].push( fn );
	} else {
		this.listeners[key] = [fn];
	}

	if ( this.listening === false ) {
		this._listen();
		this.listening = true;
	}
};

ThriveAppStorage.prototype.off = function ( key, fn ) {
	var listener = listeners[key];
	if ( listener.length > 1 ) {
		listener.splice( listener.indexOf( fn ), 1 );
	} else {
		listeners[key] = [];
	}
};

/**
 *
 * @param {String} component
 * @param {String} field
 * @param {Boolean|Number|Object|String} value
 */
ThriveAppStorage.prototype.setComponentMeta = function ( component, field, value ) {

	var meta = this.get( 'tcb-components-display' ) || {};

	meta[component] = meta[component] || {};
	meta[component][field] = value;

	this.set( 'tcb-components-display', meta );
};

/**
 *
 * @returns {ThriveAppStorage} a storage instance
 */
ThriveAppStorage.instance = function () {
	if ( ! this._instance ) {
		this._instance = new ThriveAppStorage();
	}

	return this._instance;
};

ThriveApp.util.localStorage = ThriveAppStorage.instance();;var ThriveApp = ThriveApp || {};
ThriveApp.util = ThriveApp.util || {};

ThriveApp.util.FontManager = {

	/**
	 * Google API url
	 */
	_api: 'https://www.googleapis.com/webfonts/v1/webfonts',

	/**
	 * Google Fonts API key
	 */
	_key: 'AIzaSyDJhU1bXm2YTz_c4VpWZrAyspOS37Nn-kI',

	/**
	 * Add import rule for font in head css
	 * @param {object} font
	 * @return {int} rule index
	 */
	import_font: function ( font ) {
		var removeIndex = 0,
			imports = TVE.CSS_Rule_Cache.getImports();

		/* check the rest of the imported fonts to see if we imported this font again */
		_.each( imports, function ( f, i ) {
			var parsed = this.parse_link( f );

			if ( parsed.family === font.family ) {
				/* if we find the same font imported before, we get the variants and we remove it */
				font.variants = font.variants.concat( parsed.variants );

				TVE.CSS_Rule_Cache.removeRule( i - (removeIndex ++) );
			}
		}, this );

		/* store unique variants only */
		font.variants = font.variants.filter( function ( value, index, self ) {
			return self.indexOf( value ) === index
		} );

		return TVE.CSS_Rule_Cache.insertImport( this.generate_link( font ) );
	},

	/**
	 * Get all imported fonts in the head css
	 * @returns {{}}
	 */
	get_imported_fonts: function () {
		var imports = TVE.CSS_Rule_Cache.getImports(),
			fonts = {}, self = this;

		imports.forEach( function ( font ) {
			var f = self.parse_link( font );
			fonts[f.family] = f;
		} );

		return fonts
	},
	/**
	 * Generate api url from font object
	 * @param font Object with family/variants/subsets
	 *
	 * @returns {string}
	 */
	generate_link: function ( font ) {
		var apiUrl = [];

		apiUrl.push( '//fonts.googleapis.com/css?family=' );
		apiUrl.push( font.family.replace( / /g, '+' ) );
		apiUrl.push( ':' );
		apiUrl.push( font.variants.join( ',' ) );
		apiUrl.push( '&subset=' );
		apiUrl.push( font.subset);

		return apiUrl.join( '' );
	},
	parse_link: function ( url ) {
		var match = /family=([a-zA-Z\+\d]*):([A-Za-z0-9,]*)&subset=(\w*)/g.exec( url ),
			font = {};

		if ( match.length > 3 ) {
			font.family = match[1].replace( /\+/g, ' ' );
			font.variants = match[2].split( ',' );
			font.subsets = match[3];
		}

		return font;
	},
	/**
	 * Get font name from import url
	 * @param link
	 * @returns string
	 */
	read_font_name: function ( link ) {
		var match = link.match( /family=((\w|\+)*)/ ),
			font = link;

		if ( match.length > 1 ) {
			font = match[1].replace( /\+/g, ' ' );
		}

		return font;
	},
	/**
	 * Check if a font is used in the head css or not.
	 * @param font {string}
	 * @returns {boolean}
	 */
	is_font_used: function ( font ) {
		if ( ! font ) {
			return false;
		}

		/* if the font is sent as link, we parse it first */
		if ( font.indexOf( 'fonts.googleapis.com' ) !== - 1 ) {
			font = this.read_font_name( font );
		}

		var re = new RegExp( 'font-family: [\"]?' + font, 'g' ),
			css = TVE.CSS_Rule_Cache.toText( CSSRule.MEDIA_RULE ),
			fontFound = re.test( css );

		/**
		 * If no block elements match the font ( added through head_css )
		 * also look for inline styles ( e.g. for text elements )
		 */
		if ( ! fontFound ) {
			TVE.Editor_Page.editor.find( TVE.TEXT_ALL ).find( 'span[style]' ).each( function () {
				if ( this.style.fontFamily && this.style.fontFamily.replace( /"/g, '' ) === font ) {
					fontFound = true;

					return false;
				}
			} );
		}

		return fontFound;
	},
	/**
	 * Get Google Fonts
	 * @returns {Array}
	 */
	get_google_fonts: function () {
		/* try and get google fonts from local storage */
		var fonts = ThriveApp.util.localStorage.get( 'ta_google_fonts' );

		if ( ! fonts || ! fonts.length ) {
			/* if we don't have the fonts in the local storage we get them from the API */
			jQuery.ajax( {
				method: 'GET',
				url: this._api,
				dataType: 'json',
				async: false,
				cache: true,
				data: {key: this._key}
			} ).done( function ( response ) {
				fonts = response.items || [];
			} ).fail( function () {
				fonts = [];
			} );
			/* and save them afterwards  */
			ThriveApp.util.localStorage.set( 'ta_google_fonts', fonts );
		}

		return fonts;
	}
};;/**
 * Created by Pop Aurelian on 21-Feb-17.
 */

/**
 * Fuck off IE TASK: TA-901
 * @type {*|(function(*=): boolean)}
 */
Number.isInteger = Number.isInteger || function ( value ) {
	return typeof value === 'number' && isFinite( value ) && Math.floor( value ) === value;
};

var ThriveApp = ThriveApp || {};
ThriveApp.models = ThriveApp.models || {};
ThriveApp.collections = ThriveApp.collections || {};

( function ( $ ) {
	/**
	 * Base Model
	 */
	ThriveApp.models.Base = Backbone.Model.extend( {
		idAttribute: 'ID',
		/**
		 * deep-json implementation for backbone models - flattens any abject, collection etc from the model
		 *
		 * @returns {Object}
		 */
		toDeepJSON: function () {
			var obj = $.extend( true, {}, this.attributes );
			_.each( _.keys( obj ), function ( key ) {
				if ( ! _.isUndefined( obj[ key ] ) && ! _.isNull( obj[ key ] ) && _.isFunction( obj[ key ].toJSON ) ) {
					obj[ key ] = obj[ key ].toJSON();
				}
			} );
			return obj;
		},
		/**
		 * deep clone a backbone model
		 * this will duplicate all included collections, models etc located in the attributes field
		 *
		 * @returns {ThriveApp.models.Base}
		 */
		deepClone: function () {
			return new this.constructor( this.toDeepJSON() );
		},
		/**
		 * ensures the same instance of a collection is used in a Backbone model
		 *
		 * @param {object} data
		 * @param {object} collection_map map with object keys and collection constructors
		 */
		ensureCollectionData: function ( data, collection_map ) {
			_.each( collection_map, _.bind( function ( constructor, key ) {
				if ( ! data[ key ] ) {
					return true;
				}
				var instanceOf = this.get( key ) instanceof constructor;
				if ( ! instanceOf ) {
					data[ key ] = new constructor( data[ key ] );
					return true;
				}
				this.get( key ).reset( data[ key ] );
				data[ key ] = this.get( key );
			}, this ) );
		},
		validation_error: function ( field, message ) {
			return {
				field: field,
				message: message
			};
		},
		/**
		 * Set nonce header before every Backbone sync.
		 *
		 * @param {string} method.
		 * @param {Backbone.Model} model.
		 * @param {{beforeSend}, *} options.
		 * @returns {*}.
		 */
		sync: function ( method, model, options ) {
			var beforeSend;

			options = options || {};

			options.cache = false;

			if ( ! _.isUndefined( ThriveApp.nonce ) && ! _.isNull( ThriveApp.nonce ) ) {
				beforeSend = options.beforeSend;

				options.beforeSend = function ( xhr ) {
					xhr.setRequestHeader( 'X-WP-Nonce', ThriveApp.nonce );
					xhr.setRequestHeader( 'X-TVA-Request-ID', ThriveApp.request_id );

					if ( beforeSend ) {
						return beforeSend.apply( this, arguments );
					}
				};
			}

			return Backbone.sync( method, model, options );
		},
		isValidProtocol: function ( url ) {
			if ( url ) {
				return ( url.indexOf( document.location.protocol ) === - 1 ) ? false : true;
			}
			return false;
		}
	} );

	/**
	 * Base Collection
	 */
	ThriveApp.collections.Base = Backbone.Collection.extend( {
		/**
		 * helper function to get the last item of a collection
		 *
		 * @return Backbone.Model
		 */
		last: function () {
			return this.at( this.size() - 1 );
		},
		sync: function ( method, model, options ) {
			var beforeSend;

			options = options || {};

			options.cache = false;

			if ( ! _.isUndefined( ThriveApp.nonce ) && ! _.isNull( ThriveApp.nonce ) ) {
				beforeSend = options.beforeSend;

				options.beforeSend = function ( xhr ) {
					xhr.setRequestHeader( 'X-WP-Nonce', ThriveApp.nonce );

					if ( beforeSend ) {
						return beforeSend.apply( this, arguments );
					}
				};
			}

			return Backbone.sync( method, model, options );
		}
	} );

	/**
	 * Breadcrumb Link
	 */
	ThriveApp.models.BreadcrumbLink = ThriveApp.models.Base.extend( {
		defaults: {
			ID: '',
			hash: '',
			label: '',
			full_link: false
		},
		/**
		 * we pass only hash and label, and build the ID based on the label
		 *
		 * @param {object} att
		 */
		initialize: function ( att ) {
			if ( ! this.get( 'ID' ) ) {
				if ( att.label ) {
					this.set( 'ID', att.label.split( ' ' ).join( '' ).toLowerCase() );
				}
			}
			this.set( 'full_link', att.hash.match( /^http/ ) );
		},
		/**
		 *
		 * @returns {String}
		 */
		get_url: function () {
			return this.get( 'full_link' ) ? this.get( 'hash' ) : ( '#' + this.get( 'hash' ) );
		}
	} );

	/**
	 * Breadcrumbs Collection
	 */
	ThriveApp.collections.Breadcrumbs = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Base.extend( {
			defaults: {
				hash: '',
				label: ''
			}
		} ),
		/**
		 * helper function allows adding items to the collection easier
		 *
		 * @param {string} route
		 * @param {string} label
		 */
		add_page: function ( route, label ) {
			var _model = new ThriveApp.models.BreadcrumbLink( {
				hash: route,
				label: label
			} );
			return this.add( _model );
		}
	} );

	/**
	 * Topic Model
	 */
	ThriveApp.models.Topic = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: {
			title: 'New Topic',
			icon: '',
			color: '#00c1ef',
			checked: 1,
			default_icon: ThriveApp.data.defaults.icon
		},
		url: function () {
			var url = ThriveApp.routes.topics;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		}
	} );

	/**
	 * Label Model
	 */
	ThriveApp.models.Label = ThriveApp.models.Topic.extend( {
		defaults: {
			title: 'New Label',
			color: '#58a545',
			label_color: '#58a545',
			checked: 1,
		},
		url: function () {
			var url = ThriveApp.routes.labels;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		}
	} );

	/**
	 * Collection of topics
	 */
	ThriveApp.collections.TopicsCollection = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Topic,
		comparator: function ( topicA, topicB ) {
			if ( topicA.get( 'ID' ) > topicB.get( 'ID' ) ) {
				return - 1;
			} // before
			if ( topicB.get( 'ID' ) > topicA.get( 'ID' ) ) {
				return 1;
			} // after
			return 0; // equal
		}
	} );

	/**
	 * Collection of labels
	 */
	ThriveApp.collections.LabelsCollection = ThriveApp.collections.TopicsCollection.extend( {
		model: ThriveApp.models.Label,
		comparator: function ( labelA, labelB ) {
			if ( labelA.get( 'ID' ) > labelB.get( 'ID' ) ) {
				return - 1;
			} // before
			if ( labelB.get( 'ID' ) > labelA.get( 'ID' ) ) {
				return 1;
			} // after
			return 0; // equal
		}
	} );

	ThriveApp.models.MediaSelect = ThriveApp.models.Base.extend( {
		defaults: function () {
			return {
				media_options: {
					youtube: {
						name: 'Youtube',
						extra: {},
						validate_domain: true
					},
					vimeo: {
						name: 'Vimeo',
						validate_domain: true
					},
					wistia: {
						name: 'Wistia',
						validate_domain: true
					},
					custom: {
						name: 'Custom',
						validate_domain: false
					}
				},
				media_type: 'youtube',
				media_url: '',
				media_extra_options: {}
			}
		},
		initialize: function () {
			if ( this.get( 'media_type' ) === 'soundcloud' ) {
				this.set( {
					media_options: {
						soundcloud: {
							name: 'SoundCloud',
							validate_domain: true
						},
						custom: {
							name: 'Custom',
							validate_domain: false
						}
					}
				} )
			}
		},

		/**
		 * Check if youtube url is valid
		 * @param url
		 * @param errors array of error messages
		 * @returns {boolean}
		 */
		is_youtube_url: function ( url, errors ) {

			if ( typeof url === 'undefined' ) {
				url = '';
			}

			if ( ( url.indexOf( 'youtube' ) === - 1 ) && ( url.indexOf( 'youtu' ) === - 1 ) ) {
				errors.push( this.validation_error( 'media_url', ThriveApp.t.invalidDomainUrl ) );

				return false;
			}

			return true;
		},

		/**
		 * Check if url is Video one
		 * @param url
		 * @param errors array of error messages
		 * @returns {boolean}
		 */
		is_vimeo_url: function ( url, errors ) {

			if ( typeof url === 'undefined' ) {
				url = '';
			}

			if ( url.indexOf( 'vimeo' ) === - 1 ) {
				errors.push( this.validation_error( 'media_url', ThriveApp.t.invalidDomainUrl ) );

				return false;
			}

			return true;
		},

		/**
		 * Check if wistia url is valid
		 * @param url
		 * @param errors array of error messages
		 * @returns {boolean}
		 */
		is_wistia_url: function ( url, errors ) {

			if ( typeof url === 'undefined' ) {
				url = '';
			}

			if ( ! this.isValidProtocol( url ) ) {
				var msg = ThriveApp.t.InvalidProtocol + document.location.protocol;

				errors.push( this.validation_error( 'media_url', msg ) );
				return false;
			}

			if ( url.indexOf( 'wistia' ) === - 1 ) {
				errors.push( this.validation_error( 'media_url', ThriveApp.t.invalidDomainUrl ) );

				return false;
			}

			return true;
		},

		/**
		 * Check if soundcloud url is valid
		 *
		 * @param url
		 * @param errors array of error messages
		 * @returns {boolean}
		 */
		is_soundcloud_url: function ( url, errors ) {
			if ( typeof url === 'undefined' ) {
				url = '';
			}

			if ( url.indexOf( 'soundcloud' ) === - 1 ) {
				errors.push( this.validation_error( 'media_url', ThriveApp.t.invalidDomainUrl ) );

				return false;
			}

			return true;
		},

		validate: function ( attrs ) {

			var errors = [];

			if ( attrs.media_url === '' ) {
				var msg = attrs.media_type === 'custom' ? ThriveApp.t.NoEmbedCode : ThriveApp.t.NoURL;

				return this.validation_error( 'media_url', msg );
			}

			var callback = 'is_' + attrs.media_type + '_url';

			if ( typeof this[ callback ] === 'function' && this[ callback ]( attrs.media_url, errors ) === false ) {
				return errors;
			}
		}
	} );

	ThriveApp.models.MediaSelect.prototype.isValidModel = function ( attrs ) {
		var errors = [];

		if ( attrs.media_url == '' ) {
			errors.push( this.validation_error( 'media_url', ThriveApp.t.NoURL ) );
		}

		var regexp = /(http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/,
			domainRegex = new RegExp( attrs.media_type, 'g' );

		if ( ! regexp.test( attrs.media_url ) ) {
			errors.push( this.validation_error( 'media_url', ThriveApp.t.URLNotValid ) );
		}

		if ( errors.length ) {
			return errors;
		}
	};

	/**
	 * Course Model
	 */
	ThriveApp.models.Course = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: function () {
			return {
				name: '',
				slug: '',
				description: '',
				topic: 0,
				label: 0,
				display: 1,
				level: 0,
				roles: {},
				cover_image: '',
				conversions: 0,
				author: ThriveApp.data.defaults.author,
				logged_in: 0,
				url: '',
				status: 'draft',
				state: ThriveApp.t.normal_state,
				message: ThriveApp.t.NotLoggedIn,
				excluded: 0,
				video_status: false,
				lesson_type: '',
				comment_status: ThriveApp.data.settings.comment_status,
				comment_status_changed: false,
				select_all: false,
				lessons_count: 0,
				taxonomy: 'tva_courses',
				membership_ids: {
					membermouse: [],
					memberpress: [],
					wishlist: [],
					sendowl: []
				},
				bundle_ids: {
					membermouse: [],
					memberpress: [],
					sendowl: []
				},
				lesson_template: {
					lesson_type: 'text',
					post_media: {
						media_extra_options: [],
						media_type: '',
						media_url: ''
					}

				}
			}
		},
		initialize: function () {
			this.set_data();
		},
		set_data: function () {
			this.set( {level_name: this.getLevelName()} );
			this.set( {topic_name: this.getTopicName()} );
			this.set( {topic_color: this.getTopicColor()} );
			this.set( {label_name: this.getLabelName()} );
			this.set( {label_color: this.getLabelColor()} );
			this.set( 'lessons', new ThriveApp.collections.Lessons( this.get( 'lessons' ) ) );
			this.set( 'chapters', new ThriveApp.collections.Chapters( this.get( 'chapters' ) ) );
			this.set( 'modules', new ThriveApp.collections.Modules( this.get( 'modules' ) ) );
			this.get( 'lesson_template' ).comment_status = this.get( 'comment_status' );
			this.checkData();
			this.checkStatus();
		},
		url: function () {
			var url = ThriveApp.routes.courses;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		getLabelName: function () {
			var label_id = this.get( 'label' ) ? this.get( 'label' ) : 0;
			var label = ThriveApp.globals.labels.findWhere( {ID: parseInt( label_id )} );

			return label ? label.get( 'title' ) : '';
		},
		getLabelColor: function () {
			var label_id = this.get( 'label' ) ? this.get( 'label' ) : 0;
			var label = ThriveApp.globals.labels.findWhere( {ID: parseInt( label_id )} );

			return label ? label.get( 'color' ) : '';
		},
		getTopicName: function () {
			// use parseint just to be sure the ID will be an intiger and not a string
			var topic = ThriveApp.globals.topics.findWhere( {ID: parseInt( this.get( 'topic' ) )} );

			return topic ? topic.get( 'title' ) : '';
		},
		getTopicColor: function () {
			// use parseint just to be sure the ID will be an intiger and not a string
			var topic = ThriveApp.globals.topics.findWhere( {ID: parseInt( this.get( 'topic' ) )} );

			return topic ? topic.get( 'color' ) : '';
		},
		getLevelName: function () {
			// use parseint just to be sure the ID will be an intiger and not a string
			var level = ThriveApp.globals.levels.findWhere( {ID: parseInt( this.get( 'level' ) )} );

			return level && level.get( 'name' ) ? level.get( 'name' ) : '';
		},

		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [],
				selected_levels = false;

			if ( ! attrs.name ) {
				errors.push( this.validation_error( 'course_name', ThriveApp.t.InvalidName ) );
			}

			if ( ! attrs.slug ) {
				errors.push( this.validation_error( 'course_slug', ThriveApp.t.InvalidSlug ) );
			}

			if ( ! attrs.author.user_login ) {
				errors.push( this.validation_error( 'author', ThriveApp.t.NoAuthor ) );
			}

			if ( attrs.excluded < 0 || ! Number.isInteger( parseFloat( attrs.excluded ) ) ) {
				errors.push( this.validation_error( 'course_excluded', ThriveApp.t.IntegerNumber ) );
			}

			if ( ! this.isValidProtocol( attrs.video_url ) && attrs.video_type == 'wistia' ) {
				errors.push( this.validation_error( 'video_url', ThriveApp.t.InvalidProtocol + document.location.protocol ) );
			}

			if ( errors.length ) {
				return errors;
			}
		},
		validateMemberships: function () {
			if ( this.get( 'logged_in' ) && ThriveApp.globals.settings.get( 'membership_plugin' ).length ) {
				var valid = false;
				ThriveApp.globals.settings.get( 'membership_plugin' ).each( function ( plugin ) {
					var tag = plugin.get( 'tag' );

					if ( Object.keys( this.get( 'membership_ids' )[ tag ] ).length > 0 || ( this.get( 'bundle_ids' )[ tag ] && Object.keys( this.get( 'bundle_ids' )[ tag ] ).length > 0 ) ) {
						valid = true;
					}
				}, this );

				return valid;
			}

			return true;
		},
		hasChildren: function () {
			if ( this.get( 'lessons' ).length > 0 ) {
				return 'lessons';
			}

			if ( this.get( 'chapters' ).length > 0 ) {
				return 'chapters';
			}

			if ( this.get( 'modules' ).length > 0 ) {
				return 'modules';
			}

			return false;
		},
		checkData: function () {
			if ( this.get( 'modules' ).length > 0 ) {
				if ( this.get( 'chapters' ).length > 0 ) {
					this.get( 'chapters' ).reset();
				}

				if ( this.get( 'lessons' ).length > 0 ) {
					this.get( 'lessons' ).reset();
				}
			}

			if ( ( this.get( 'chapters' ).length > 0 ) && ( this.get( 'lessons' ).length > 0 ) ) {
				this.get( 'lessons' ).reset();
			}
		},
		checkStatus: function () {
			var status = 'draft';

			if ( this.get( 'status' ) === 'publish' ) {
				var children = this.hasChildren();

				if ( children ) {
					status = this.get( children ).findWhere( {post_status: 'publish'} ) ? 'publish' : 'draft';
				}
			}

			if ( status !== this.get( 'status' ) ) {
				this.set( {status: status} );
				this.save();
			}
		}
	} );

	ThriveApp.collections.SelectedItems = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Base,
		/**
		 * deep-json implementation for backbone collection - flattens a collection of models
		 *
		 * @returns {Array|Object}
		 */
		toDeepJson: function () {

			/**
			 * flattens all collections within a model..
			 *
			 * @todo not sure if this should be here or we should implement it on model base
			 * it may look like a "struto-camila" in the end!!!!
			 */
			if ( this instanceof Backbone.Model ) {
				return ThriveApp.collections.SelectedItems.prototype.parseModel.call( this )
			}

			var models = $.extend( true, [], this.models ),
				self = this,
				items = [];

			_.each( models, function ( model, index ) {
				model = self.parseModel( model );

				items[ index ] = model;
			} );

			return items;
		},
		checkItems: function ( model, checked ) {
			var modules = this.where( {type: 'module'} );

			/**
			 * When we select chapters and lessons we need to also check if they're part of the same module,
			 * otherwise we need to reset the selected items collection
			 */
			if ( checked && this.length > 0 && ! this.findWhere( {type: 'module'} ) && ThriveApp.globals.course_elements.get( 'modules' ).length > 0 && model.get( 'type' ) !== 'module' ) {
				var module_id,
					reset = false;
				this.each( function ( item ) {
					if ( item.get( 'post_parent' ) > 0 && ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: item.get( 'post_parent' )} ) ) {
						module_id = item.get( 'post_parent' );
					} else {
						var parent = ThriveApp.globals.course_elements.get( 'chapters' ).findWhere( {ID: item.get( 'post_parent' )} );
						module_id = parent.get( 'post_parent' );
					}
				} );

				var post_parent = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: model.get( 'post_parent' )} );

				if ( post_parent.get( 'type' ) === 'module' && post_parent.get( 'ID' ) !== module_id ) {
					reset = true;
				}

				if ( post_parent.get( 'type' ) !== 'module' && post_parent.get( 'post_parent' ) !== module_id ) {
					reset = true;
				}
			}


			if ( ( this.length > 0 && model.get( 'type' ) === 'module' && modules.length === 0 ) || ( model.get( 'post_parent' ) > 0 && modules.length > 0 && ! this.findWhere( {ID: model.get( 'post_parent' )} ) ) || reset ) {
				this.updateValues( {checked: false}, true );
				this.reset();
			}

			model.set( {checked: checked} );
			checked ? this.add( model ) : this.remove( model, {silent: true} );

			this.setChildrenChecked( model, checked );

			if ( ! checked && model.get( 'post_parent' ) > 0 ) {
				this.setParentsUnchecked( model );
			}
		},
		setChildrenChecked: function ( model, checked ) {
			var children = model.hasChildren();

			if ( children ) {
				model.get( children ).each( function ( item ) {
					item.set( {checked: checked} );
					item.trigger( 'item_checked' );

					checked ? this.add( item ) : this.remove( item, {silent: true} );
					var item_children = item.hasChildren();
					if ( item_children ) {
						item.get( item_children ).each( function ( lesson ) {
							lesson.set( {checked: checked} );
							checked ? this.add( lesson ) : this.remove( lesson, {silent: true} );
						}, this );
					}
				}, this );
			}
		},
		setParentsUnchecked: function ( model ) {
			var parent = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: parseInt( model.get( 'post_parent' ) )} ),
				remove_other = false;

			if ( parent ) {
				parent.set( {checked: false} );
				this.remove( parent, {silent: true} );

				if ( parent.get( 'type' ) === 'module' ) {
					remove_other = true;
				}

				if ( parent.get( 'post_parent' ) > 0 ) {
					var module = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: parseInt( parent.get( 'post_parent' ) )} );
					if ( module ) {
						remove_other = true;
						module.set( {checked: false} );
						this.remove( module, {silent: true} );
					}
				}
			}

			if ( remove_other ) {
				this.removeOtherModules();
			}

		},
		removeOtherModules: function () {
			var other_modules = this.where( {type: 'module'} );
			if ( other_modules.length > 0 ) {
				_.each( other_modules, function ( other_module ) {
					this.remove( other_module, {silent: true} );
					other_module.set( {checked: false} );
					this.setChildrenChecked( other_module, false );
				}, this );

			}
		},
		getHighestSelectedLevel: function () {
			if ( this.length === 0 ) {
				return false;
			}
			var level = 0;

			this.each( function ( item ) {
				var item_level = ThriveApp.util.levels[ item.get( 'type' ) ];
				if ( level < item_level ) {
					level = item_level;
				}
			} );

			return level;
		},
		parseCollection: function ( collection ) {
			var models = $.extend( true, [], collection.models ),
				self = this,
				items = [];

			_.each( models, function ( item, index ) {
				items[ index ] = ThriveApp.collections.SelectedItems.prototype._call( 'parseModel', item );
			} );

			return items;
		},
		parseModel: function ( model ) {
			model = this instanceof Backbone.Model ? this : model;

			var obj = $.extend( true, {}, model.attributes ),
				self = this,
				json_model = model.toJSON();

			_.each( _.keys( obj ), function ( key ) {
				if ( ! _.isUndefined( obj[ key ] ) && ! _.isNull( obj[ key ] ) && ( obj[ key ] instanceof Backbone.Collection ) ) {
					json_model[ key ] = ThriveApp.collections.SelectedItems.prototype._call( 'parseCollection', obj[ key ] );
				}
			} );

			return json_model;
		},
		_call: function ( callback, args ) {
			return this instanceof Backbone.Model ?
				ThriveApp.collections.SelectedItems.prototype[ callback ].call( this ) :
				this[ callback ]( args )
		},
		_pluck: function ( keys ) {
			if ( typeof keys !== 'object' ) {
				return;
			}

			function parseModels( collection ) {
				var result = [],
					models = $.extend( true, [], collection.models );

				_.each( models, function ( model, index ) {
					var data = {};

					_.each( keys, function ( key ) {
						if ( ( typeof key === 'string' ) && ! _.isUndefined( model.get( key ) ) ) {
							data[ key ] = ThriveApp.util.isCollection( model.get( key ) ) ?
								parseModels( model.get( key ) ) : //ensure recursive calls for nested levels of collections
								model.get( key );
						}

						if ( key === 'order' ) {
							data[ key ] = index;
						}
					} );
					result.push( data );
				} );

				return result;
			}

			return parseModels( this );
		},
		pluckValues: function ( values ) {
			return this._pluck( values );
		},
		listValue: function ( value ) {
			if ( typeof value !== 'string' ) {
				return;
			}

			function parseModels( collection ) {
				var result = collection.pluck( value ),
					models = $.extend( true, [], collection.models );

				_.each( models, function ( model ) {
					var childs = model.hasChildren();

					if ( ThriveApp.util.isCollection( model.get( childs ) ) ) {
						var concat = parseModels( model.get( childs ) );
						result = result.concat( concat );
					}
				} );

				return result;
			}

			return parseModels( this );
		},
		updateValues: function ( keys, with_childrens ) {
			if ( typeof keys !== 'object' ) {
				return;
			}

			/**
			 * TODO: Move this outside or in _utils.js
			 * @param collection
			 *
			 * @param with_childrens
			 */
			function parseModels( collection, with_childrens ) {
				var models = $.extend( true, [], collection.models );
				_.each( models, function ( model, index ) {
					Object.keys( keys ).forEach( function ( key ) {
						if ( typeof key === 'string' ) {
							var obj = {};

							// allways ensure the corect order for a model
							if ( key === 'order' ) {
								obj[ key ] = index;
							} else {
								obj[ key ] = keys[ key ];
							}

							model.set( obj );
						}
					} );

					var childs = model.hasChildren();
					if ( ( with_childrens === true ) && ThriveApp.util.isCollection( model.get( childs ) ) ) {
						parseModels( model.get( childs ), with_childrens );
					}
				} );
			}

			parseModels( this, with_childrens );
		}
	} );

	/**
	 * Collection of courses which includes the collection of lessons
	 */
	ThriveApp.collections.Courses = ThriveApp.collections.SelectedItems.extend( {
		model: ThriveApp.models.Course,
		/**
		 * @param a
		 * @param b
		 * @returns {number}
		 */
		comparator: function ( a, b ) {
			return parseInt( b.get( 'order' ) ) - parseInt( a.get( 'order' ) );
		}
	} );

	ThriveApp.collections.ImportCourses = ThriveApp.collections.Courses.extend( {
		url: function () {
			return ThriveApp.routes.settings + '/get_old_courses_lessons/';
		}
	} );

	/**
	 * Level Model
	 */
	ThriveApp.models.Level = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: {
			name: ''
		},
		url: function () {
			var url = ThriveApp.routes.levels;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [];
			if ( ! attrs.name ) {
				errors.push( this.validation_error( 'add_level', ThriveApp.t.NoLevel ) );
			}

			if ( errors.length ) {
				return errors;
			}
		}
	} );
	/**
	 * Collection of Levels
	 */
	ThriveApp.collections.Levels = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Level
	} );

	/**
	 * Level Model
	 */
	ThriveApp.models.Role = ThriveApp.models.Base.extend( {} );
	/**
	 * Collection of Levels
	 */
	ThriveApp.collections.Roles = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Role
	} );

	/**
	 * Level Model
	 */
	ThriveApp.models.Lesson = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: function () {
			return {
				post_title: '',
				post_excerpt: '',
				post_parent: 0,
				lesson_type: 'text',
				post_media: {
					media_type: '',
					media_url: '',
					media_extra_options: {}
				},
				cover_image: '',
				state: ThriveApp.t.normal_state,
				post_status: 'draft',
				course_id: '',
				types: {
					text: {
						label: 'Text Lesson',
						type: 'text'
					},
					audio: {
						label: 'Audio Lesson',
						type: 'audio'
					},
					video: {
						label: 'Video Lesson',
						type: 'video'
					}
				},
				type: 'lesson',
				item_type: 'Lesson',
				checked: false,
				comment_status: 'closed'
			}
		},
		url: function () {
			var url = ThriveApp.routes.lessons;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [];

			if ( ! attrs.post_title ) {
				errors.push( this.validation_error( 'item_name', ThriveApp.t.InvalidName ) );
			}

			if ( attrs.lesson_type !== 'text' ) {
				var media_select = new ThriveApp.models.MediaSelect( attrs.post_media );
				if ( ! media_select.isValid() ) {
					_.each( media_select.validationError, function ( error ) {
						errors.push( error );
					} );

				}

			}

			if ( errors.length ) {
				return errors;
			}
		},
		changeParentsStatus: function () {
			var collection = this.collection,
				changed_parents = [],
				current = this.get( 'post_status' ),
				post_status = current === 'draft' ? 'publish' : 'draft',
				same_status = collection.where( {post_status: post_status} );

			if ( ( ( post_status === 'publish' && same_status.length === 0 ) || ( post_status === 'draft' && same_status.length === ( this.collection.length - 1 ) ) ) && this.get( 'post_parent' ) > 0 ) {
				var parent_model = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: this.get( 'post_parent' )} );
				if ( parent_model ) {
					if ( parent_model instanceof ThriveApp.models.Chapter && parent_model.get( 'post_parent' ) > 0 ) {
						var module_model = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: parent_model.get( 'post_parent' )} );
						if ( module_model ) {
							var chapters_same_status = module_model.get( 'chapters' ).where( {post_status: post_status} );

							if ( ( post_status === 'publish' && chapters_same_status.length === 0 ) || ( post_status === 'draft' && chapters_same_status.length === ( module_model.get( 'chapters' ).length - 1 ) ) ) {
								module_model.set( {post_status: post_status} );
								changed_parents.push( module_model )
							}

						}
					}
					parent_model.set( {post_status: post_status} );
					changed_parents.push( parent_model )
				}
			}

			return changed_parents;
		},
		hasChildren: function () {
			return false;
		},
		checkData: function () {
			return true;
		},
		set_data: function () {
			return true;
		}
	} );
	/**
	 * Collection of Levels
	 */
	ThriveApp.collections.Lessons = ThriveApp.collections.SelectedItems.extend( {
		model: ThriveApp.models.Lesson,
		/**
		 * Used to sort the collection
		 *
		 * @param model
		 * @returns {*}
		 */
		comparator: function ( model ) {
			return parseInt( model.get( 'order' ) );
		}
	} );

	/**
	 * Settings model
	 */
	ThriveApp.models.Settings = ThriveApp.models.Base.extend( {
		defaults: {
			per_page: '',
			preview_option: true,
			apprentice_label: true,
			index_page: {
				ID: '',
				name: ''
			},
			template: {
				font_family: '',
				font_source: 'safe',
				main_color: '',
				course_title: '',
				course_title_color: '',
				page_headline: '',
				page_headline_color: '',
				lesson_headline: '',
				chapter_headline: '',
				module_headline: '',
				paragraph: '',
				paragraph_color: '',
				view_lesson: '',
				logo_type: false
			}
		},
		initialize: function () {
			this.set( {sendowl_products: new ThriveApp.collections.Base( this.get( 'sendowl_products' ) )} );
			this.set( {sendowl_bundles: new ThriveApp.collections.Base( this.get( 'sendowl_bundles' ) )} );
		},
		url: function () {
			var url = ThriveApp.routes.settings + '/save_settings/';

			return url;
		},
		validate: function ( attrs, options ) {
			var errors = [];

			if ( ! attrs.per_page ) {
				errors.push( this.validation_error( 'per_page', ThriveApp.t.InvalidPerPage ) );
			}

			if ( ! attrs.index_page.ID ) {
				errors.push( this.validation_error( 'index_page', ThriveApp.t.InvalidIndexPage ) );
			}

			if ( ( attrs.loginform === true ) && ( attrs.index_page.ID === attrs.register_page.ID ) ) {
				errors.push( this.validation_error( 'index_page', ThriveApp.t.InvalidPagesSetup ) );
				errors.push( this.validation_error( 'register_page', ThriveApp.t.InvalidPagesSetup ) );
			}

			if ( errors.length ) {
				return errors;
			}

		},
		changeTemplate: function ( template ) {
			TVE_Dash.showLoader();

			var self = this;
			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveApp.nonce
				},
				type: 'POST',
				url: ThriveApp.routes.settings + '/get_user_settings/',
				data: {
					template: template.toJSON()
				}
			} ).done( function ( response, status, options ) {
				if ( response ) {
					self.set( {old_template: response.template} );
				}
				self.set( {template: template.toJSON()} );
			} ).always( function () {
				TVE_Dash.hideLoader();
			} );
		},
		importLessons: function () {
			TVE_Dash.showLoader();

			var self = this;
			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveApp.nonce
				},
				type: 'POST',
				url: ThriveApp.routes.settings + '/import_lessons/'
			} ).done( function ( response, status, options ) {
				if ( response ) {
					if ( response.length > 0 ) {
						ThriveApp.globals.courses.add( response );
						TVE_Dash.success( ThriveApp.t.SuccessImport )
					} else {
						TVE_Dash.success( ThriveApp.t.NoNewCoursesImported )
					}

				}
			} ).always( function () {
				TVE_Dash.hideLoader();
			} );
		},

		/**
		 * Generates the checkout page url for a SendOwl product: simple or bundle
		 *
		 * @param product Backbone.Model
		 * @param collection string default is 'products'
		 * @returns string
		 */
		get_checkout_page_url: function ( product, collection ) {

			var id,
				url = this.get( 'checkout_endpoint' ) || this.get( 'checkout_page_url' );

			if ( typeof collection === 'undefined' ) {
				collection = 'products';
			}

			if ( false === ( product instanceof Backbone.Model ) ) {
				product = this.get( 'sendowl_' + collection ).first();
			}

			if ( product instanceof Backbone.Model ) {
				id = product.get( 'id' );
			}

			if ( url.length && id ) {

				var glue = url.indexOf( '?' ) !== - 1 ? '&' : '?';

				url += glue + 'pp=Sendowl&' + ( collection === 'bundles' ? 'bid' : 'pid' ) + '=' + id;
			}

			return url;
		}
	} );

	ThriveApp.models.AvailableSettings = ThriveApp.models.Base.extend( {
		defaults: {
			'tooltip': 'data-position="top" data-tooltip="' + ThriveApp.t.OptionUnavailable + '"'
		},
		url: function () {
			return ThriveApp.routes.settings + '/get_available_settings/';
		}
	} );

	/**
	 * Model for one template
	 */
	ThriveApp.models.Template = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: {
			name: ''
		}
	} );

	/**
	 * The collection of templates
	 */
	ThriveApp.collections.Templates = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Template
	} );

	/**
	 * Web safe Fonts model
	 */
	ThriveApp.models.Font = ThriveApp.models.Base.extend( {} );

	/**
	 * A collection of our web safe fonts
	 */
	ThriveApp.collections.Fonts = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Font
	} );
	/**
	 * Font model for Google
	 */
	ThriveApp.models.GoogleFont = ThriveApp.models.Base.extend( {} );

	/**
	 * Fonts collection for Google
	 */
	ThriveApp.collections.GoogleFonts = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.GoogleFont
	} );
	/**
	 * Collection for all the membership plugins
	 */
	ThriveApp.collections.MembershipsCollection = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Base
	} );

	/**
	 * Membermouse membership and bundle model
	 */
	ThriveApp.models.MembermouseModel = ThriveApp.models.Base.extend( {
		defaults: {
			id: '',
			name: 'None'
		}
	} );

	/**
	 * Membermouse memberships and bundles collection
	 */
	ThriveApp.collections.MembermouseCollection = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.MembermouseModel

	} );

	/**
	 * Sendowl membership and bundle model
	 */
	ThriveApp.models.SendowlModel = ThriveApp.models.Base.extend( {
		defaults: {
			id: '',
			name: 'None'
		}
	} );

	/**
	 * Sendowl memberships and bundles collection
	 */
	ThriveApp.collections.SendowlCollection = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.SendowlModel

	} );

	ThriveApp.models.WishlistModel = ThriveApp.models.Base.extend( {
		defaults: {
			id: '',
			name: 'None'
		}
	} );

	ThriveApp.collections.WishlistCollection = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.WishlistModel
	} );

	ThriveApp.models.Chapter = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: {
			post_title: '',
			post_excerpt: '',
			data_field: 'chapter_name',
			state: ThriveApp.t.normal_state,
			edit_text: ThriveApp.t.edit_chapter,
			post_status: 'draft',
			course_id: '',
			post_parent: 0,
			type: 'chapter',
			tooltip_text: '',
			item_type: 'Chapter',
			expanded: false,
			checked: false

		},
		initialize: function () {
			this.set_data();
		},
		set_data: function () {
			this.set( 'lessons', new ThriveApp.collections.Lessons( this.get( 'lessons' ) ) );
		},
		checkData: function () {
			return true;
		},
		url: function () {
			var url = ThriveApp.routes.chapters;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [];

			if ( ! attrs.post_title ) {
				errors.push( this.validation_error( 'item_name', ThriveApp.t.InvalidName ) );
			}

			if ( errors.length ) {
				return errors;
			}
		},
		hasChildren: function () {
			if ( this.get( 'lessons' ).length > 0 ) {
				return 'lessons';
			}

			return false;
		}
	} );

	/**
	 * Collection of chapters which includes the collection of lessons
	 */
	ThriveApp.collections.Chapters = ThriveApp.collections.SelectedItems.extend( {
		model: ThriveApp.models.Chapter,
		/**
		 * Used to sort the collection
		 *
		 * @param model
		 * @returns {*}
		 */
		comparator: function ( model ) {
			return parseInt( model.get( 'order' ) );
		}
	} );

	ThriveApp.models.Module = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: {
			post_title: '',
			post_excerpt: '',
			data_field: 'module_name',
			state: ThriveApp.t.normal_state,
			edit_text: ThriveApp.t.edit_module,
			post_status: 'draft',
			course_id: '',
			type: 'module',
			cover_image: '',
			tooltip_text: '',
			item_type: 'Module',
			post_parent: 0,
			expanded: false,
			checked: false
		},
		initialize: function () {
			this.set_data();
		},
		set_data: function () {
			this.set( 'chapters', new ThriveApp.collections.Chapters( this.get( 'chapters' ) ) );
			this.set( 'lessons', new ThriveApp.collections.Lessons( this.get( 'lessons' ) ) );
			this.checkData();
		},
		url: function () {
			var url = ThriveApp.routes.modules;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [];

			if ( ! attrs.post_title ) {
				errors.push( this.validation_error( 'item_name', ThriveApp.t.InvalidName ) );
			}

			if ( errors.length ) {
				return errors;
			}
		},
		toDeepJson: function () {
			return ThriveApp.collections.SelectedItems.prototype.toDeepJson.call( this );
		},
		hasChildren: function () {
			if ( this.get( 'lessons' ).length > 0 ) {
				return 'lessons';
			}

			if ( this.get( 'chapters' ).length > 0 ) {
				return 'chapters';
			}

			return false;
		},
		checkData: function () {
			if ( ( this.get( 'chapters' ).length > 0 ) && ( this.get( 'lessons' ).length > 0 ) ) {
				this.get( 'lessons' ).reset();
			}
		}
	} );

	ThriveApp.models.ModuleParent = ThriveApp.models.Base.extend( {
		defaults: {
			name: 'Select Module'
		}
	} );

	ThriveApp.models.ChapterParent = ThriveApp.models.Base.extend( {
		defaults: {
			name: 'Select Chapter'
		}
	} );


	/**
	 * Collection of modules which includes the collection of lessons and chapters
	 */
	ThriveApp.collections.Modules = ThriveApp.collections.SelectedItems.extend( {
		model: ThriveApp.models.Module,
		/**
		 * Used to sort the collection
		 *
		 * @param model
		 * @returns {*}
		 */
		comparator: function ( model ) {
			return parseInt( model.get( 'order' ) );
		}
	} );

	ThriveApp.models.CourseElements = ThriveApp.models.Base.extend( {
		defaults: {
			modules: {},
			chapters: {},
			lessons: {},
			all: {}
		},
		initialize: function () {
			this.set_data()
		},
		set_data: function () {
			this.set( {modules: new ThriveApp.collections.Base( this.get( 'modules' ) )} );
			this.set( {chapters: new ThriveApp.collections.Base( this.get( 'chapters' ) )} );
			this.set( {lessons: new ThriveApp.collections.Base( this.get( 'lessons' ) )} );
			this.set( {all: new ThriveApp.collections.Base( this.get( 'all' ) )} );
		}
	} );

	ThriveApp.models.BulkActions = ThriveApp.models.Base.extend( {
		defaults: {
			module_group: {},
			chapter_group: {},
			items_move: {},
			publish: {}
		}
	} );

	ThriveApp.models.Log = ThriveApp.models.Base.extend( {
		idAttribute: 'id',
		defaults: {
			type: '',
			identifier: '',
			product: 'None',
			data: {},
			date: ''
		}
	} );

	ThriveApp.collections.Logs = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Log
	} );

	ThriveApp.models.Stack = ThriveApp.models.Base.extend( {
		idAttribute: 'id',
		defaults: {
			type: '',
			product: 'None',
			data: {},
			date: ''
		},
		initialize: function () {
			this.set( {data: new ThriveApp.collections.Logs( this.get( 'data' ) )} );
		}
	} );

	ThriveApp.collections.Stacks = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Stack
	} );

	ThriveApp.models.Type = ThriveApp.models.Base.extend( {
		defaults: {
			type: '',
			checked: true
		}
	} );

	ThriveApp.collections.Types = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.Type
	} );

	ThriveApp.models.LogsSettings = ThriveApp.models.Base.extend( {
		defaults: {
			limit: 20,
			types: [],
			s: ''
		}
	} );

	ThriveApp.models.SendowlCustomer = ThriveApp.models.Base.extend( {
		idAttribute: 'ID',
		defaults: function () {
			return {
				display_name: '',
				user_email: '',
				user_login: '',
				edit_url: '',
				membership_ids: [],
				bundle_ids: []
			}
		},
		initialize: function () {

			if ( this.get( 'ID' ) ) {
				this.set( {ID: parseInt( this.get( 'ID' ) )} );
			}

			if ( this.get( 'buyer_name' ) ) {
				this.set( {
					display_name: this.get( 'buyer_name' ),
					user_login: this.get( 'buyer_email' )
				} );
			}

			if ( this.get( 'buyer_email' ) ) {
				this.set( {user_email: this.get( 'buyer_email' )} );
			}

			if ( parseInt( this.get( 'is_valid' ) !== 1 ) ) {
				this.set( {is_valid: 0} );
			}
		},
		url: function () {
			var url = ThriveApp.routes.users;

			if ( this.get( 'ID' ) || this.get( 'ID' ) === 0 ) {
				url += '/' + this.get( 'ID' );
			}

			return url;
		},
		/**
		 * Overwrite Backbone validation
		 * Return something to invalidate the model
		 *
		 * @param {Object} attrs
		 * @param {Object} options
		 */
		validate: function ( attrs, options ) {
			var errors = [];

			if ( ! attrs.user_login ) {
				errors.push( this.validation_error( 'user_login', ThriveApp.t.MissingUserLogin ) );
			}

			if ( ! attrs.user_email ) {
				errors.push( this.validation_error( 'user_email', ThriveApp.t.MissingEmail ) );
			}

			if ( errors.length ) {
				return errors;
			}

			if ( ! ThriveApp.util.isEmail( attrs.user_email ) ) {
				return this.validation_error( 'user_email', ThriveApp.t.InvalidCustomerEmail );
			}
		}
	} );

	ThriveApp.collections.SendowlCustomers = ThriveApp.collections.SelectedItems.extend( {
		model: ThriveApp.models.SendowlCustomer
	} );

	ThriveApp.models.OrderItem = ThriveApp.models.Base.extend( {
		initialize: function () {
			this.set( {
				product_id: parseInt( this.get( 'product_id' ) )
			} )
		}
	} );

	ThriveApp.collections.OrderItems = ThriveApp.collections.Base.extend( {
		model: ThriveApp.models.OrderItem
	} );

} )( jQuery );;/**
 * Created by Pop Aurelian on 21-Feb-17.
 */
var ThriveApp = ThriveApp || {};
ThriveApp.views = ThriveApp.views || {};

( function ( $ ) {

	$( function () {

		/**
		 * Modal Steps View
		 * If a wizard is needed implement or extend this view
		 */
		ThriveApp.views.ModalSteps = TVE_Dash.views.Modal.extend( {
			stepClass: '.tva-modal-step',
			currentStep: 0,
			$step: null,
			events: {
				'click .tva-modal-next-step': "next",
				'click .tva-modal-prev-step': "prev"
			},
			afterRender: function () {
				this.steps = this.$el.find( this.stepClass ).hide();
				this.gotoStep( 0 );
				return this;
			},
			gotoStep: function ( index ) {
				var step = this.steps.hide().eq( index ).show(),
					self = this;
				this.$step = step;
				setTimeout( function () {
					self.input_focus( step );
				}, 50 );

				this.currentStep = index;

				return this;
			},
			next: function () {
				this.beforeNext();
				this.gotoStep( this.currentStep + 1 );
				this.afterNext();
			},
			prev: function () {
				this.beforePrev();
				this.gotoStep( this.currentStep - 1 );
				this.afterPrev();
			},
			beforeNext: function () {
				return this;
			},
			afterNext: function () {
				return this;
			},
			beforePrev: function () {
				return this;
			},
			afterPrev: function () {
				return this;
			},
			bind_zclip: function () {
				TVE_Dash.bindZClip( this.$el.find( 'a.tvd-copy-to-clipboard' ) );
			}
		} );

		ThriveApp.views.ModalGreetings = ThriveApp.views.ModalSteps.extend( {
			template: TVE_Dash.tpl( 'modals/greetings' ),
			empty_check: 0,
			events: {
				'click .tva-modal-next-step': "next",
				'click .tva-modal-prev-step': "prev",
				'click .tva-go-to-templates': 'nextStep',
				'click .tva-go-to-wizard': 'goToWizard',
				'click .tva-go-to-dashboard': 'goToDashBoard',
				'input #tva-apprentice-page': function ( e ) {
					this.empty_check = 1;
					var current_val = $( e.currentTarget ).val();
					if ( ! current_val ) {
						this.empty_check = 0;
						this.$( '#tva-new-apprentice-page' ).removeAttr( 'disabled' );
					}
				},
				'input #tva-new-apprentice-page': function ( e ) {
					this.model.get( 'index_page' ).ID = '';
					this.model.get( 'index_page' ).name = '';
					if ( $( e.currentTarget ).val() === '' ) {
						this.$( '#tva-apprentice-page' ).removeAttr( 'disabled' );
						this.empty_check = 0;
					} else {
						this.empty_check = 1;
						this.$( '#tva-apprentice-page' ).attr( 'disabled', 'disabled' );
					}
				}
			},
			afterRender: function () {
				this.steps = this.$el.find( this.stepClass ).hide();
				this.gotoStep( 0 );

				var $post_search = this.$( '#tva-apprentice-page' ),
					model = this.model,
					self = this;

				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',
					select: function ( event, ui ) {
						model.get( 'index_page' ).ID = parseInt( ui.item.id );
						model.get( 'index_page' ).name = ui.item.label;
						model.set( 'apprentice_url', ui.item.url );
					},
					search: function () {
						model.set( {index_page: {}} );
						self.$( '#tva-new-apprentice-page' ).removeAttr( 'disabled' );
					},
					open: function () {
						model.set( {index_page: {}} );
						self.$( '#tva-new-apprentice-page' ).attr( 'disabled', 'disabled' );
					},
					close: function ( event, ui ) {
						if ( ! model.get( 'index_page' ).ID ) {
							self.$( '#tva-new-apprentice-page' ).removeAttr( 'disabled' );
						}
					},
					fetch_single: model.get( 'index_page' )
				} );
			},
			nextStep: function () {
				this.tvd_clear_errors();
				var self = this;

				if ( this.empty_check == 0 ) {
					TVE_Dash.err( ThriveApp.t.NotEmpty );
					return;
				}

				TVE_Dash.showLoader();
				this.model.set( {first_time: true, preview_notification: true} );
				if ( ! this.model.get( 'index_page' ).ID ) {
					var value = this.$( '#tva-new-apprentice-page' ).val();

					if ( value ) {
						$.ajax( {
							headers: {
								'X-WP-Nonce': ThriveApp.nonce
							},
							type: 'POST',
							url: ThriveApp.routes.settings + '/create_new_page/',
							data: {
								page_name: value
							}
						} ).done( function ( response, status, options ) {
							self.model.get( 'index_page' ).ID = response.ID;
							self.model.get( 'index_page' ).name = value;
							self.model.set( 'apprentice_url', response.apprentice_url );
							self.model.set( 'preview_url', response.preview_url );

							if ( ! self.model.isValid() ) {
								return self.tvd_show_errors( self.model );
							}

							self.next();
						} ).always( function () {
							TVE_Dash.hideLoader();
						} );

					} else {
						TVE_Dash.err( ThriveApp.t.NoIndexInvalidIndex );
						TVE_Dash.hideLoader();
					}
				} else {
					if ( ! this.model.isValid() ) {
						return this.tvd_show_errors( this.model );
					}

					$.ajax( {
						headers: {
							'X-WP-Nonce': ThriveApp.nonce
						},
						type: 'POST',
						url: ThriveApp.routes.settings + '/update_provisional_index_page/',
						data: {
							index_page: self.model.get( 'index_page' )
						}
					} ).done( function ( response, status, options ) {
						self.model.set( {apprentice_url: response.apprentice_url} );
						self.model.set( {preview_url: response.preview_url} );
						self.next();
					} ).always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			},
			goToWizard: function () {
				this.close();
				ThriveApp.router.navigate( "#template_settings", {trigger: true} );
			},
			goToDashBoard: function () {
				TVE_Dash.showLoader();

				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response ) {
							self.close();
							ThriveApp.router.navigate( "#dashboard", {trigger: true} );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();

					} );
				}

			}
		} );

		/**
		 * General settings modal
		 */
		ThriveApp.views.ModalSettings = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/settings' ),
			events: {
				'click .tvd-modal-submit': 'save',
				'input #tva-per-page': 'setCourses',
				'click #tva-load-scripts': 'setLoadScripts',
				'click #tva-auto-login': 'setAutoLogin',
				'click #tva-show-loginform': 'setLoginForm',
				'click #tva-show-apprentice-label': 'setApprenticeLabel',
				'click #tva-comements-status': 'setCommentsStatus'
			},
			register_view: null,
			initial_comment_status: null,
			afterRender: function () {
				var $post_search = this.$( '#tva-apprentice-page' ),
					$login_page_search = this.$( '#tva-apprentice-login-page' ),
					model = this.model;
				this.initial_comment_status = model.get( 'comment_status' );
				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',
					select: function ( event, ui ) {
						model.get( 'index_page' ).ID = parseInt( ui.item.id );
						model.get( 'index_page' ).name = ui.item.label;
					},
					search: function () {
						model.set( {index_page: {}} );
					},
					open: function () {
						model.set( {index_page: {}} );
					},
					fetch_single: model.get( 'index_page' )
				} );

				this.register_view = new ThriveApp.views.RegisterPage( {
					el: this.$( '#tva-register-container' ),
					model: ThriveApp.globals.settings
				} );

				if ( model.get( 'loginform' ) == true ) {
					this.register_view.render();
				}
			},
			isNormalInteger: function ( str ) {
				var n = Math.floor( Number( str ) );
				return String( n ) === str && n > 0;
			},
			setCourses: function ( e ) {
				if ( ! this.isNormalInteger( $( e.currentTarget ).val() ) ) {
					this.$( '.tvd-modal-submit' ).attr( 'disabled', 'disabled' );
					return TVE_Dash.err( ThriveApp.t.IntegerFieldValidation );
				}

				this.$( '.tvd-modal-submit' ).removeAttr( 'disabled' );
				this.model.set( {per_page: $( e.currentTarget ).val()} )
			},
			setLoadScripts: function ( e ) {
				var value = $( e.currentTarget ).is( ':checked' );

				this.model.set( {load_scripts: value} );
			},
			setAutoLogin: function ( e ) {
				var value = $( e.currentTarget ).is( ':checked' );

				this.model.set( {auto_login: value} );
			},
			setLoginForm: function ( e ) {
				var value = $( e.currentTarget ).is( ':checked' );
				this.model.set( {loginform: value} );

				value === true ?
					this.register_view.render() :
					this.register_view.$el.empty()
			},
			setApprenticeLabel: function ( e ) {
				var value = $( e.currentTarget ).is( ':checked' );

				this.model.set( {apprentice_label: value} );
			},
			setCommentsStatus: function ( e ) {
				var value = $( e.currentTarget ).is( ':checked' );

				value === true ? this.model.set( {comment_status: 'open'} ) : this.model.set( {comment_status: 'closed'} );
			},
			save: function () {
				this.tvd_clear_errors();
				var changed = this.initial_comment_status != this.model.get( 'comment_status' ) ? true : false;
				this.model.set( {is_comment_status_changed: changed} );

				if ( ! this.model.isValid() ) {
					return this.tvd_show_errors( this.model );
				}

				TVE_Dash.showLoader();
				var self = this,
					xhr = this.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						// should we do something with the response ?
						if ( response ) {
							self.model.set( {
								apprentice_url: response.apprentice_url,
								preview_url: response.preview_url
							} );
							self.model.trigger( 'tva_render_header' );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
						self.close();
					} );
				}

			}
		} );

		ThriveApp.views.ModalImportLessons = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/import' ),
			events: {
				'click .tva-no': 'setImport',
				'click .tva-yes': 'setImport'
			},
			setImport: function ( e ) {
				TVE_Dash.showLoader();
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/set_import/'
				} ).done( function ( response, status, options ) {
					self.model.set( {import: response} );
					self.close();
					if ( $( e.currentTarget ).hasClass( 'tva-yes' ) ) {
						ThriveApp.router.navigate( "#import", {trigger: true} );
					}
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		/**
		 * View to render delete modal
		 */
		ThriveApp.views.ConfirmBulkDelete = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/bulkdelete' ),
			events: {
				'click .tvd-modal-submit': 'deleteBulkItems',
				'click .tvd-modal-close': 'closeModal'
			},
			afterInitialize: function ( args ) {
				_.bindAll( this, 'keyAction' );
				$( document ).bind( 'keydown', this.keyAction );
			},
			keyAction: function ( e ) {
				var code = e.which;
				if ( code == 13 ) {
					this.deleteBulkItems();
				}

				/**
				 * Unbind when esc is pressed
				 */
				if ( code === 27 ) {
					$( document ).unbind( 'keydown', this.keyAction );
				}
			},
			deleteBulkItems: function () {
				TVE_Dash.showLoader();
				var self = this;

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce,
						'X-TVA-Request-ID': ThriveApp.request_id
					},
					type: 'POST',
					url: ThriveApp.routes.courses + '/bulk_action/',
					data: {
						action: 'delete',
						items: ThriveApp.globals.selected_items.listValue( 'ID' ),
						course_id: ThriveApp.globals.active_course.get( 'ID' )
					}
				} ).done( function ( response, status, options ) {
					if ( response ) {
						TVE_Dash.success( ThriveApp.t.ItemDeleted );
						ThriveApp.globals.active_course.trigger( 'tva_after_action', {
							action: 'delete',
							model: response
						} );

					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
				} ).always( function () {
					$( document ).unbind( 'keydown', self.keyAction );
					TVE_Dash.hideLoader();
					self.close();
				} );
			},
			closeModal: function () {
				$( document ).unbind( 'keydown', this.keyAction );
				ThriveApp.globals.selected_items.reset();
				ThriveApp.globals.selected_items.trigger( 'tva_hide_bulk_actions' );
				ThriveApp.globals.selected_items.trigger( 'tva_clear_selected' );
				this.close();
			}
		} );

		/**
		 * View to render move items modal
		 */
		ThriveApp.views.ConfirmBulkMove = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/bulkmove' ),
			events: {
				'click .tva-modal-next-step': 'next',
				'click .tva-modal-prev-step': 'prev',
				'click .tva-receiver-course-item': function ( e ) {
					this.$( '.tva-available-receivers-courses' ).slideToggle();
				},
				'click .tva-receiver-module-item': function ( e ) {
					this.$( '.tva-available-receivers-modules' ).slideToggle();
				},
				'click .tva-receiver-chapter-item': function () {
					this.$( '.tva-available-receivers-chapters' ).slideToggle();
				},
				'click .tva-single-course-item': 'renderReceiverCourseChange',
				'click .tva-single-module-item': 'renderReceiverModuleChange',
				'click .tva-single-chapter-item': 'renderReceiverChapterChange',
				'click .tvd-modal-submit': 'moveBulkItems',
				'click .tvd-modal-close': 'closeModal'
			},
			receiverCourseModel: null,
			receiverModuleModel: null,
			receiverChapterModel: null,
			type: null,
			afterInitialize: function () {
				var first_item = ThriveApp.globals.selected_items.at( 0 );
				$( document ).bind( 'keydown', this.keyAction );
				this.item_level = ThriveApp.globals.selected_items.getHighestSelectedLevel();

				this.selected = new ThriveApp.collections.SelectedItems( JSON.parse( JSON.stringify( ThriveApp.globals.selected_items ) ) );

				this.type = first_item.get( 'item_type' ).toLowerCase();
			},
			afterRender: function () {
				this.steps = this.$el.find( this.stepClass ).hide();
				this.renderOptions();
				this.closeOnLoseFocus();
			},
			keyAction: function ( e ) {
				var code = e.which;
				if ( code == 13 ) {
					this.moveBulkItems();
				}
			},
			closeOnLoseFocus: function () {
				var self = this;

				this.$el.on( 'click', function ( e ) {
					var $container = self.$( '.tva-material-list' ),
						$element = self.$( '.tva-dropdown' );

					if ( ! $container.is( e.target ) && $container.has( e.target ).length === 0 && ! $element.is( e.target ) ) {
						$container.slideUp();
					}
				} );
			},
			renderOptions: function () {
				this.renderReceiverCourse();
				this.renderReceiverModule();
				this.renderReceiverChapter();
			},
			renderReceiverCourse: function () {
				var view = new ThriveApp.views.ReceiverMoveItem( {
					el: this.$( '.tva-receiver-course' ),
					model: this.receiverCourseModel ? this.receiverCourseModel : new ThriveApp.models.Base(),
					collection: this.collection,
					type: 'course'
				} );

				view.render();
			},
			renderReceiverModule: function () {

				this.$( '.tva-receiver-module' ).empty();

				if ( this.receiverCourseModel && this.receiverCourseModel.hasChildren() === 'modules' && ThriveApp.util.userHasToDecide( this.item_level, this.receiverCourseModel ) > 0 ) {
					var view = new ThriveApp.views.ReceiverMoveItem( {
						el: this.$( '.tva-receiver-module' ),
						model: this.receiverModuleModel ? this.receiverModuleModel : new ThriveApp.models.Base(),
						collection: this.receiverCourseModel.get( 'modules' ),
						type: 'module'
					} );

					view.render();
				}
			},
			renderReceiverChapter: function () {
				this.$( '.tva-receiver-chapter' ).empty();
				var collection;

				if ( this.receiverModuleModel && ThriveApp.util.userHasToDecide( this.item_level, this.receiverModuleModel ) > 0 ) {
					collection = this.receiverModuleModel.get( 'chapters' )
				} else if ( this.receiverCourseModel && this.receiverCourseModel.hasChildren() === 'chapters' && ThriveApp.util.userHasToDecide( this.item_level, this.receiverCourseModel ) > 0 ) {
					collection = this.receiverCourseModel.get( 'chapters' )
				}

				if ( collection ) {
					var view = new ThriveApp.views.ReceiverMoveItem( {
						el: this.$( '.tva-receiver-chapter' ),
						model: this.receiverChapterModel ? this.receiverChapterModel : new ThriveApp.models.Base(),
						collection: collection,
						type: 'chapter'
					} );
					view.render();
				}
			},
			renderReceiverCourseChange: function ( e ) {
				var course_id = e.currentTarget.getAttribute( 'data-course-id' );

				this.receiverCourseModel = ThriveApp.globals.courses.findWhere( {ID: parseInt( course_id )} );
				this.receiverModuleModel = null;
				this.receiverChapterModel = null;
				this.post_parent = 0;
				this.old_post_parent = this.selected.at( 0 ).get( 'post_parent' );

				this.selected.each( function ( model ) {
					model.set( {course_id: course_id} );
				} );

				if ( ThriveApp.util.userHasToDecide( this.item_level, this.receiverCourseModel ) === 0 ) {
					this.$( '.tvd-modal-submit' ).removeClass( 'tvd-disabled' );
				}

				this.renderOptions();
			},
			renderReceiverModuleChange: function ( e ) {
				this.post_parent = e.currentTarget.getAttribute( 'data-module-id' );
				this.receiverModuleModel = this.receiverCourseModel.get( 'modules' ).findWhere( {ID: parseInt( this.post_parent )} );
				this.receiverChapterModel = null;

				this.renderReceiverModule();
				this.renderReceiverChapter();

				if ( ThriveApp.util.userHasToDecide( this.item_level, this.receiverModuleModel ) === 0 ) {
					this.$( '.tvd-modal-submit' ).removeClass( 'tvd-disabled' );
				}
			},
			renderReceiverChapterChange: function ( e ) {
				this.post_parent = parseInt( e.currentTarget.getAttribute( 'data-chapter-id' ) );

				this.receiverChapterModel = this.receiverModuleModel ?
					this.receiverModuleModel.get( 'chapters' ).findWhere( {ID: this.post_parent} ) :
					this.receiverCourseModel.get( 'chapters' ).findWhere( {ID: this.post_parent} );

				this.renderReceiverChapter();

				this.$( '.tvd-modal-submit' ).removeClass( 'tvd-disabled' );
			},
			validateBeforeSave: function () {
				if ( ! this.receiverCourseModel ) {
					TVE_Dash.err( 'Please select a course' );
					return false;
				}

				/**
				 * No need to go further if we move modules
				 */
				if ( this.type === 'module' ) {
					return true;
				}

				var children = this.receiverCourseModel.hasChildren();


				if ( ! children ) {
					return true;
				}

				if ( ( children === 'modules' ) && ! this.receiverModuleModel ) {
					TVE_Dash.err( 'Please select a module' );
					return false;
				}

				/**
				 * No need to go further if we move chapters
				 */
				if ( this.type === 'chapter' ) {
					return true;
				}

				if ( this.type === 'lesson' ) {
					return true;
				}

				if ( ! this.receiverChapterModel ) {
					TVE_Dash.err( 'Please select a chapter' );
					return false;
				}

				return true
			},

			moveBulkItems: function () {
				if ( ! this.validateBeforeSave() ) {
					return this;
				}

				TVE_Dash.showLoader();

				this.selected.each( function ( item ) {
					item.set( {old_post_parent: this.old_post_parent} );
					item.set( {post_parent: this.post_parent} );
				}, this );

				var self = this,
					data = this.selected.pluckValues(
						[ 'ID', 'post_parent', 'course_id', 'post_type', 'post_status', 'old_post_parent' ]
					);

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce,
						'X-TVA-Request-ID': ThriveApp.request_id
					},
					type: 'POST',
					url: ThriveApp.routes.courses + '/bulk_action/',
					data: {
						action: 'move',
						items: data,
						high_level: ThriveApp.util.hierarchy[ this.item_level ],
						course_id: ThriveApp.globals.active_course.get( 'ID' )
					}
				} ).done( function ( response, status, options ) {
					if ( response ) {
						ThriveApp.globals.active_course.trigger( 'tva_after_action', {
							action: 'move',
							model: response
						} );
					}

				} ).error( function ( errorObj ) {
					TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
				} ).always( function () {
					$( document ).unbind( 'keydown', self.keyAction );
					TVE_Dash.hideLoader();
					self.closeModal();
				} );
			},
			closeModal: function () {
				$( document ).unbind( 'keydown', this.keyAction );
				this.close();
			}
		} );

		ThriveApp.views.ConfirmBulkMoveNo = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/bulkmoveno' )
		} );

		/**
		 * Base view for grouping items
		 */
		ThriveApp.views.BaseItemGroup = ThriveApp.views.ModalSteps.extend( {
			events: {
				'click .tva-modal-next-step': 'next',
				'click .tva-modal-prev-step': 'prev',
				'click .tvd-group-modal-submit': 'save',
				'click .tvd-modal-close': 'closeModal'
			},
			main_item_model: null,
			optional_item_model: null,
			item_collection: null,
			next_step: null,
			prev_step: null,
			childs: null,
			afterRender: function () {
				this.childs = this.model.hasChildren();

				/**
				 * End it here if no childs are found for given model
				 */
				if ( ( false === this.childs ) || ! ThriveApp.util.isCollection( this.model.get( this.childs ) ) ) {
					return;
				}

				this.steps = this.$el.find( this.stepClass ).hide();
				this.gotoStep( 0 );
				return this;
			},
			next: function () {
				if ( ( this.currentStep === 1 ) && this.optional_item_model && ! this.optional_item_model.isValid() ) {
					return this.tvd_show_errors( this.optional_item_model );
				}

				this.getNextStep();
				this.gotoStep( this.next_step );
				this.renderCurrentStep();

				return this;
			},
			prev: function () {
				this.gotoStep( this.currentStep - 1 );
				this.renderSmallEditor();
				this.bindOnKeyDown();
				return this;
			},
			bindOnKeyDown: function () {
				if ( this.currentStep === 2 ) {
					_.bindAll( this, 'keyAction' );
					$( document ).bind( 'keydown', this.keyAction );
				} else {
					$( document ).unbind( 'keydown', this.keyAction )
				}
			},
			keyAction: function ( e ) {
				var code = e.which;
				if ( code == 13 ) {
					this.save();
				}
			},
			getNextStep: function () {
				return this;
			},
			getPrevStep: function () {
				return this;
			},
			makeOptionalItemModel: function () {
				return this;
			},
			makeMainItemModel: function () {
				return this
			},
			renderEditor: function () {
				return this;
			},
			renderCurrentStep: function () {
				switch ( this.currentStep ) {
					case 1:
						this.makeOptionalItemModel();
						this.renderSmallEditor();
						break;

					case 2:
						this.tvd_clear_errors();

						if ( ! this.main_item_model ) {
							this.makeMainItemModel();
							this.renderEditor();
							this.bindOnKeyDown();
						}

						break;
				}

				return this;
			},
			renderSmallEditor: function () {
				if ( this.inputEditor ) {
					this.inputEditor.render();
					return this;
				}

				this.inputEditor = new ThriveApp.views.InputFieldEditor( {
					el: this.$( '.tva-default-module-editor' ),
					model: this.optional_item_model,
					placeholder: ThriveApp.util.upperFirst( this.group_type )
				} );

				this.inputEditor.render();

				return this;
			},
			save: function () {
				this.tvd_clear_errors();

				if ( ! this.main_item_model.isValid() ) {
					return this.tvd_show_errors( this.main_item_model );
				}

				TVE_Dash.showLoader();

				var data = [],
					self = this;

				data.push( this.main_item_model.toDeepJSON() );

				if ( this.optional_item_model ) {
					data.push( this.optional_item_model.toDeepJSON() );
				}

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce,
						'X-TVA-Request-ID': ThriveApp.request_id
					},
					type: 'POST',
					url: ThriveApp.routes[ this.group_type + 's' ] + '/group_as_' + this.group_type + '/',
					data: {
						items: data,
						course_id: ThriveApp.globals.active_course.get( 'ID' )
					}
				} ).done( function ( response, status, options ) {
					if ( response ) {
						ThriveApp.globals.active_course.trigger( 'tva_after_action', {
							action: 'group',
							model: response
						} );
						ThriveApp.globals.bulk_action_model.clear().set( ThriveApp.globals.bulk_action_model.defaults );
						ThriveApp.globals.selected_items.reset();
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
				} ).always( function () {
					$( document ).unbind( 'keydown', self.keyAction );
					TVE_Dash.hideLoader();
					self.close();
				} );
			},
			closeModal: function () {
				$( document ).unbind( 'keydown', this.keyAction );
				this.close();
			}
		} );

		/**
		 * View to render groping items in a module
		 */
		ThriveApp.views.ConfirmModuleGroup = ThriveApp.views.BaseItemGroup.extend( {
			template: TVE_Dash.tpl( 'course/module/module-group' ),
			getNextStep: function () {
				var type = this.childs.substring( 0, this.childs.length - 1 ),
					type_items = ThriveApp.globals.selected_items.where( {type: type} );

				switch ( this.currentStep ) {
					case 0:
						this.next_step = ( this.model.get( 'modules' ).length === 0 ) &&
						                 ( type_items.length < this.model.get( this.childs ).length ) ? 1 : 2;
						break;

					case 1:
						this.next_step = 2;
						break;
				}

				return this;
			},
			makeMainItemModel: function () {
				var items = ThriveApp.globals.selected_items.where( {type: 'lesson'} ),
					item_ids = [];
				this.item_collection = this.model.get( 'modules' );
				this.main_item_model = new ThriveApp.models.Module( {
					course_id: this.model.get( 'ID' ),
					order: ThriveApp.globals.active_course.get( 'modules' ).length,
					comment_status: this.model.get( 'comment_status' )
				} );

				/**
				 * Only the direct descendent will have his post parent changed
				 * so we don't need to send through the lessons if we have chapters
				 */
				if ( ThriveApp.globals.selected_items.findWhere( {type: 'chapter'} ) ) {
					items = ThriveApp.globals.selected_items.where( {type: 'chapter'} );
				}

				_.each( items, function ( item ) {
					if ( item.get( 'post_status' ) === 'publish' ) {
						this.main_item_model.set( {post_status: 'publish'} );
					}

					item_ids.push( item.get( 'ID' ) );
				}, this );

				this.main_item_model.set( {item_ids: item_ids} );

				return this;
			},
			makeOptionalItemModel: function () {
				var item_ids = [];
				this.optional_item_model = new ThriveApp.models.Module( {
					course_id: this.model.get( 'ID' ),
					order: ThriveApp.globals.active_course.get( 'modules' ).length + 1,
					comment_status: this.model.get( 'comment_status' )
				} );

				var type = ThriveApp.globals.active_course.hasChildren(),
					elements = ThriveApp.globals.active_course.get( type ).reject( function ( item ) {
						return ! ! ThriveApp.globals.selected_items.findWhere( {ID: item.get( 'ID' )} );
					} );

				_.each( elements, function ( item ) {
					if ( item.get( 'post_status' ) === 'publish' ) {
						this.optional_item_model.set( {post_status: 'publish'} );
					}

					item_ids.push( item.get( 'ID' ) );
				}, this );

				this.optional_item_model.set( {item_ids: item_ids} );
			},
			renderEditor: function () {
				var view = new ThriveApp.views.ModuleEditor( {
					el: this.$( '.tva-main-item-editor' ),
					model: this.main_item_model,
					collection: this.item_collection
				} );

				view.footerView = 'ModalGroupFooter';

				view.render();

				return this;
			}
		} );

		/**
		 * View to render chapter group screen
		 */
		ThriveApp.views.ConfirmChapterGroup = ThriveApp.views.BaseItemGroup.extend( {
			template: TVE_Dash.tpl( 'course/chapter/chapter-group' ),
			getNextStep: function () {
				this.parent_model = this.model;
				if ( this.model.get( 'lessons' ).length === 0 ) {

					var self = this;
					ThriveApp.globals.selected_items.each( function ( item ) {
						var module = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: item.get( 'post_parent' )} );

						if ( module ) {
							self.parent_model = module;
						}
					}, this );
				}

				if ( this.currentStep === 0 ) {
					this.next_step = ThriveApp.globals.selected_items.length < this.parent_model.get( 'lessons' ).length ? 1 : 2;
				}

				if ( this.currentStep === 1 ) {
					this.next_step = 2;
				}

				return this;
			},
			makeMainItemModel: function () {
				var items = ThriveApp.globals.selected_items.where( {type: 'lesson'} ),
					item_ids = [],
					module_id,
					collection = ThriveApp.globals.active_course.get( 'chapters' );

				this.main_item_model = new ThriveApp.models.Chapter( {
					course_id: this.model.get( 'ID' ),
					order: collection.length
				} );


				_.each( items, function ( item ) {
					if ( item.get( 'post_status' ) === 'publish' ) {
						this.main_item_model.set( {post_status: 'publish'} );
					}

					if ( item.get( 'post_parent' ) > 0 && ThriveApp.globals.course_elements.get( 'modules' ).length > 0 ) {
						var parent = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: item.get( 'post_parent' )} );

						if ( parent.get( 'type' ) === 'module' ) {
							module_id = item.get( 'post_parent' );
						} else {
							module_id = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: parent.get( 'post_parent' )} ).get( 'ID' )
						}
					}

					item_ids.push( item.get( 'ID' ) );
				}, this );

				if ( module_id ) {
					this.main_item_model.set( {
						post_parent: module_id,
						order: ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: module_id} ).get( 'chapters' ).length
					} )
				}

				this.main_item_model.set( {item_ids: item_ids} );
			},
			makeOptionalItemModel: function () {
				var module_id,
					item_ids = [];
				this.optional_item_model = new ThriveApp.models.Chapter( {
					course_id: this.model.get( 'ID' ),
					order: ThriveApp.globals.active_course.get( 'modules' ).length + 1
				} );

				var type = this.parent_model.hasChildren(),
					elements = this.parent_model.get( type ).reject( function ( item ) {
						return ! ! ThriveApp.globals.selected_items.findWhere( {ID: item.get( 'ID' )} );
					} );


				_.each( elements, function ( item ) {
					if ( item.get( 'post_status' ) === 'publish' ) {
						this.optional_item_model.set( {post_status: 'publish'} );
					}

					if ( item.get( 'post_parent' ) > 0 && ThriveApp.globals.course_elements.get( 'modules' ).length > 0 ) {
						var parent = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: item.get( 'post_parent' )} );

						if ( parent.get( 'type' ) === 'module' ) {
							module_id = item.get( 'post_parent' );
						} else {
							module_id = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: parent.get( 'post_parent' )} ).get( 'ID' )
						}
					}

					item_ids.push( item.get( 'ID' ) );
				}, this );

				if ( module_id ) {
					this.optional_item_model.set( {
						post_parent: module_id,
						order: ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: module_id} ).get( 'chapters' ).length
					} )
				}

				this.optional_item_model.set( {item_ids: item_ids} );
			},
			renderEditor: function () {
				var view = new ThriveApp.views.ChapterEditor( {
					el: this.$( '.tva-chapter-group-editor' ),
					model: this.main_item_model
				} );

				view.footerView = 'ModalGroupFooter';

				view.render();

				return this;
			}
		} );

		ThriveApp.views.ConfirmTopicDelete = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/confirm-topic-delete' ),
			events: {
				'click .tva-confirm-delete': 'deleteTopic'
			},
			afterInitialize: function ( args ) {
				this.$el.addClass( 'tvd-red' );
				var _this = this;
				_.defer( function () {
					_this.$( '.tva-confirm-delete' ).focus();
				} );
			},
			deleteTopic: function ( e ) {
				var self = this;

				TVE_Dash.showLoader();
				var xhr = this.model.destroy();
				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response.edited ) {
							_.each( response.edited, function ( id, index ) {
								var edit_courses = ThriveApp.globals.courses.where( {ID: parseInt( id )} );
								if ( Object.keys( edit_courses ).length > 0 ) {
									_.each( edit_courses, function ( course, index ) {
										course.set( {topic: 0} );
										course.set( {topic_name: course.getTopicName()} );
										course.set( {topic_color: course.getTopicColor()} );
									} );
								}
							} );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
						self.close();
					} );
				}

			}
		} );

		/**
		 * Label modal to confirm deletion
		 */
		ThriveApp.views.ConfirmLabelDelete = ThriveApp.views.ConfirmTopicDelete.extend( {
			template: TVE_Dash.tpl( 'modals/confirm-label-delete' ),
			events: {
				'click .tva-confirm-delete': 'deleteTopic'
			},
			afterInitialize: function ( args ) {
				this.$el.addClass( 'tvd-red' );
				var _this = this;
				_.defer( function () {
					_this.$( '.tva-confirm-delete' ).focus();
				} );
			},
			deleteTopic: function ( e ) {
				var self = this;

				TVE_Dash.showLoader();
				var xhr = this.model.destroy();
				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response.edited ) {
							_.each( response.edited, function ( id, index ) {
								var edit_courses = ThriveApp.globals.courses.where( {ID: parseInt( id )} );
								if ( Object.keys( edit_courses ).length > 0 ) {
									_.each( edit_courses, function ( course, index ) {
										course.set( {topic: 0} );
										course.set( {topic_name: course.getTopicName()} );
										course.set( {topic_color: course.getTopicColor()} );
									} );
								}
							} );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
						self.close();
					} );
				}

			}
		} );

		/**
		 * Modal for notifying the user he can change between live and demo previews
		 */
		ThriveApp.views.ModalFirstTimeNotification = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/notification' ),
			afterInitialize: function ( args ) {
				this.$el.addClass( 'tva-first-time-notification' );
			},
			afterClose: function () {
				$( 'body' ).css( {overflowY: 'hidden'} );
				$( 'body' ).css( {overflowX: 'hidden'} );
			}
		} );

		ThriveApp.views.ConfirmEditorClose = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/close-editor' ),
			events: {
				'click .tva-confirm-delete': 'closeEditor'
			},
			afterInitialize: function ( args ) {
				this.$el.addClass( 'tvd-red' );
				var _this = this;
				_.defer( function () {
					_this.$( '.tva-confirm-delete' ).focus();
				} );
			},
			closeEditor: function () {
				if ( this.model.get( 'first_time' ) ) {
					ThriveApp.router.navigate( "#greeting", {trigger: true} );
				} else {
					ThriveApp.router.navigate( "#dashboard", {trigger: true} );
				}
				this.close()
			}
		} );

		/**
		 * Base View
		 */
		ThriveApp.views.Base = Backbone.View.extend( {
			/**
			 * Always try to return this !!!
			 *
			 * @returns {ThriveChurn.views.Base}
			 */
			render: function () {
				return this;
			},
			/**
			 *
			 * Instantiate and open a new modal which has the view constructor assigned and send params further along
			 *
			 * @param ViewConstructor View constructor
			 * @param params
			 */
			modal: function ( ViewConstructor, params ) {
				return TVE_Dash.modal( ViewConstructor, params );
			},
			bind_zclip: function () {
				TVE_Dash.bindZClip( this.$el.find( 'a.tvd-copy-to-clipboard' ) );
			}
		} );

		ThriveApp.views.RegisterPage = TVE_Dash.views.Modal.extend( {
			template: TVE_Dash.tpl( 'modals/register-page' ),

			render: function () {
				var self = this;

				self.$el.html( self.template( {model: self.model} ) );
				TVE_Dash.materialize( self.$el );

				new ThriveApp.PostSearch( self.$( '#tva-apprentice-register-page' ), {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',

					select: function ( event, ui ) {
						self.model.get( 'register_page' ).ID = parseInt( ui.item.id );
						self.model.get( 'register_page' ).name = ui.item.label;
					},
					search: function () {
						self.model.set( {register_page: {}} );
					},
					open: function () {
						self.model.set( {register_page: {}} );
					},

					fetch_single: self.model.get( 'register_page' )
				} );

				return this;
			}
		} );

		/**
		 * View to render add content modal
		 */
		ThriveApp.views.AddContentModal = ThriveApp.views.ModalSteps.extend( {
			template: TVE_Dash.tpl( 'modals/add-content' ),
			events: {
				'click .tva-modal-next-step': 'next',
				'click .tva-modal-prev-step': 'prev'
			},
			item: null,
			item_type: null,
			footerView: 'StepModalFooter',
			next: function ( e ) {
				this.beforeNext();

				var step = 1;
				if ( this.currentStep === 0 ) {
					this.item_type = e.currentTarget.getAttribute( 'id' );
					var item_level = parseInt( e.currentTarget.getAttribute( 'data-level' ) ),
						decision = ThriveApp.util.userHasToDecide( item_level );

					this.makeItemModel();

					if ( decision ) {
						var collection;
						if ( decision === 1 ) {
							collection = ThriveApp.globals.active_course.get( ThriveApp.util.getPlural( this.item_type ) );
						} else {
							var children = ThriveApp.globals.active_course.hasChildren();
							collection = ThriveApp.globals.active_course.get( children );
						}


						var decision_maker = new ThriveApp.views.ItemDecisionMaker( {
							model: this.item,
							collection: collection,
							modal_view: this
						} );

						this.$( '.tva-decision-container' ).html( decision_maker.render().$el );

					} else {

						if ( ! ThriveApp.views[ ThriveApp.util.upperFirst( this.item_type ) + 'Editor' ] ) {
							return;
						}

						/**
						 * Instantiate the correct view of inputs
						 */
						var view = new ThriveApp.views[ ThriveApp.util.upperFirst( this.item_type ) + 'Editor' ]( {
							model: this.item,
							collection: this.model.get( this.item_type + 's' ),
							modal_view: this
						} );

						this.$( '.tva-editor-container' ).html( view.render().$el );
						step ++;
					}
				}

				this.gotoStep( this.currentStep + step );
				this.afterNext();
			},
			prev: function () {
				this.beforePrev();
				if ( this.currentStep === 0 ) {
					this.item.set( {post_parent: 0} );
				}

				var step = this.currentStep - 1;

				var decision = ThriveApp.util.userHasToDecide( ThriveApp.util.levels[ this.item.get( 'type' ) ] );

				if ( ! decision ) {
					this.gotoStep( 0 );

					return;
				}

				this.gotoStep( step );

				if ( step === 0 ) {
					this.item_type = null;
					this.item = null;
				}

				this.afterPrev();
				return this;
			},
			makeItemModel: function () {
				if ( this.item ) {
					this.item = null;
				}

				this.item = new ThriveApp.models[ ThriveApp.util.upperFirst( this.item_type ) ]( {
					course_id: this.model.get( 'ID' ),
					comment_status: this.model.get( 'comment_status' )
				} );

				if ( this.item_type === 'lesson' ) {
					this.item.set( {
						lesson_type: ThriveApp.globals.active_course.get( 'lesson_template' ).lesson_type,
						post_media: ThriveApp.globals.active_course.get( 'lesson_template' ).post_media,
						comment_status: ThriveApp.globals.active_course.get( 'lesson_template' ).comment_status
					} );
				}

			}
		} );

		/**
		 * Test modal
		 */
		ThriveApp.views.ModalEditor = TVE_Dash.views.Modal.extend( {
			events: {
				'click .tvd-modal-submit': 'save'
			},
			editor: null,
			footerView: '',
			afterInitialize: function ( options ) {
				this.footerView = options.footerView ? options.footerView : 'ModalFooter';
			},
			afterRender: function () {
				this.renderEditor();

				return this;
			},
			renderEditor: function () {
				if ( ! ThriveApp.views[ this.viewName ] ) {
					return;
				}

				this.editor = new ThriveApp.views[ this.viewName ]( {
					el: this.getEl(),
					model: this.model,
					collection: this.collection,
					footerView: this.footerView,
					modal_view: this
				} );

				this.editor.render();

				return this;
			}
		} );

		ThriveApp.views.ModalDelete = ThriveApp.views.ModalEditor.extend( {
			events: {
				'click .tva-delete-yes': 'deleteItem',
				'click .tva-delete-no': 'closeItem'
			},
			afterInitialize: function ( options ) {
				this.footerView = options.footerView ? options.footerView : 'ModalFooterDelete';
				_.bindAll( this, 'keyAction' );
				$( document ).bind( 'keydown', this.keyAction );
			},
			keyAction: function ( e ) {
				e.stopPropagation();

				var code = e.which;
				if ( code == 13 ) {
					e.preventDefault();
					this.deleteItem();
				}

				/**
				 * Unbind when esc is pressed
				 */
				if ( code === 27 ) {
					$( document ).unbind( 'keydown', this.keyAction );
					this.close();
				}
			},
			deleteItem: function () {
				TVE_Dash.showLoader();
				var xhr = this.model.destroy(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						TVE_Dash.success( ThriveApp.t.ItemDeleted );
						self.collection.updateValues( {order: ''} );

						if ( response.length > 0 ) {
							_.each( response, function ( id ) {
								var element = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: id} );
								if ( element ) {
									element.set( {post_status: 'draft'} );
								}
							} );
						}

						ThriveApp.globals.active_course.trigger( 'tva_render_content' );

					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						$( document ).unbind( 'keydown', this.keyAction );
						TVE_Dash.hideLoader();
						self.close();
					} );
				}
			},
			closeItem: function () {
				$( document ).unbind( 'keydown', this.keyAction );
				this.close();
			}
		} );

		/**
		 * View to render Module editor modal
		 */
		ThriveApp.views.ModuleModalEditor = ThriveApp.views.ModalEditor.extend( {
			template: TVE_Dash.tpl( 'course/module/modal-module-editor' ),
			getEl: function () {
				return this.$( '.tva-add-module' );
			}
		} );

		ThriveApp.views.ModuleModalDelete = ThriveApp.views.ModalDelete.extend( {
			template: TVE_Dash.tpl( 'course/module/modal-module-delete' ),
			getEl: function () {
				return this.$( '.tva-delete-module' );
			}
		} );

		ThriveApp.views.ChapterModalDelete = ThriveApp.views.ModalDelete.extend( {
			template: TVE_Dash.tpl( 'course/chapter/modal-chapter-delete' ),
			getEl: function () {
				return this.$( '.tva-delete-chapter' );
			}
		} );


		/**
		 * View to render chapter modal editor
		 */
		ThriveApp.views.ChapterModalEditor = ThriveApp.views.AddContentModal.extend( {
			afterInitialize: function ( options ) {
				this.module = options.module;
				this.item_type = 'chapter';
			},
			afterRender: function () {
				this.steps = this.$el.find( this.stepClass ).hide();
				var item_level = 1,
					decision = ThriveApp.util.userHasToDecide( item_level, this.module );

				this.makeItemModel();

				if ( decision ) {
					var collection = this.module.get( 'chapters' );

					var decision_maker = new ThriveApp.views.ItemDecisionMaker( {
						model: this.item,
						collection: collection,
						decision_model: this.module,
						modal_view: this,
						restrict_back: true
					} );

					this.$( '.tva-decision-container' ).html( decision_maker.render().$el );

					this.gotoStep( 1 );
				} else {
					/**
					 * Instantiate the correct view of inputs
					 */
					var view = new ThriveApp.views.ChapterEditor( {
						model: this.item,
						collection: this.module.get( 'chapters' ),
						modal_view: this,
						footerView: 'ModalFooter'
					} );

					this.$( '.tva-editor-container' ).html( view.render().$el );

					this.gotoStep( 2 );
				}
			},
			makeItemModel: function () {
				if ( this.item ) {
					this.item = null;
				}

				if ( this.module instanceof ThriveApp.models.Module ) {

					this.item = new ThriveApp.models.Chapter( {
						post_parent: this.module.get( 'ID' ),
						course_id: this.module.get( 'course_id' ),
						comment_status: this.model.get( 'comment_status' )
					} );
				} else {
					this.item = this.model;
				}
			}
		} );

		/**
		 * View to render lesson modal editor
		 */
		ThriveApp.views.LessonModalEditor = ThriveApp.views.ModalEditor.extend( {
			template: TVE_Dash.tpl( 'course/lesson/modal-lesson-editor' ),
			getEl: function () {
				return this.$( '.tva-lesson-editor' );
			}
		} );

		/**
		 * View to render Add customer modal
		 */
		ThriveApp.views.AddCustomerModal = ThriveApp.views.ModalSteps.extend( {
			template: TVE_Dash.tpl( 'modals/sendowl/add-customer' ),
			events: {
				'click .tva-modal-next-step': 'next',
				'click .tva-modal-prev-step': 'prev',
				'click .tva-import-type': 'setImportType',
				'click .tva-process-csv-file': 'processFile',
				'click .tva-import-customers': 'importCustomers',
				'click .tva-add-file': function () {
					this.$( '#tva-upload-file' ).click();
				},
				'change #tva-upload-file': function ( e ) {
					this.files = e.currentTarget.files;
					this.renderBulkImport();
					this.registerEventListeners();
				},
				'change .tva-membership-change': 'setMembership',
				'change .tva-bundle-change': 'setBundle',
				'input #tva-customer-name': 'seName',
				'input #tva-list-import': 'setCustomersList',
				'input #tva-customer-email': 'setEmail',
				'input .tva-to-bulk-import-customers': 'renderBulkImport',
				'click .tvd-modal-submit': 'save'
			},
			imported_users: 0,
			files: [],
			customers: '',
			membership_ids: [],
			bundle_ids: [],
			customers_list: '',
			errors: {
				invalid_lines: [],
				invalid_emails: []
			},
			afterRender: function () {
				this.resetData();
				this.steps = this.$el.find( this.stepClass ).hide();

				if ( this.import_type ) {
					this.gotoStep( 2 );
					this.renderBulkImport();

					if ( this.import_type === 'csvFile' ) {
						this.registerEventListeners();
					}
					return this;
				} else {
					this.gotoStep( 0 );
					this.renderSingleCustomerImport();
					this.$products_el = this.$( '.tva-sendowl-products' );
					this.$bundles_el = this.$( '.tva-sendowl-bundles' );
					this.renderOptions();
				}

				return this;
			},
			next: function () {
				this.beforeNext();

				if ( ! this.isValid() ) {
					return;
				}

				this.gotoStep( this.currentStep + 1 );

				if ( this.currentStep === 2 ) {
					this.renderBulkImport();

					if ( this.import_type === 'csvFile' ) {
						this.registerEventListeners();
					}
				}

				if ( this.currentStep === 3 ) {
					if ( this.import_type === 'listNames' ) {
						this.renderImportPreview();
					} else if ( this.files.length > 0 ) {
						this.processFile();
					}
				}

				this.afterNext();
			},
			prev: function () {
				this.beforePrev();
				this.gotoStep( this.currentStep - 1 );

				this.$( '.tva-invalid-file-type, .tva-invalid-file, .tva-invalid-list-message, .tva-invalid-customers' ).hide();

				if ( this.currentStep === 0 ) {
					this.files = [];
					this.import_type = '';
					this.model = new ThriveApp.models.SendowlCustomer( {} );
					this.renderSingleCustomerImport();
					this.$products_el = this.$( '.tva-sendowl-products' );
					this.$bundles_el = this.$( '.tva-sendowl-bundles' );
					this.$( '.tva-import-type' ).removeClass( 'tva-selected-import-option' );
					this.$( '.tva-toggle-disable' ).addClass( 'tvd-disabled' );
					this.renderOptions();
				}

				this.customers = '';
				this.afterPrev();
			},
			/**
			 * Register the listeners for drop file area
			 */
			registerEventListeners: function () {
				var dropArea = document.getElementById( 'tva-drop-layer' ),
					events = [ 'dragover', 'dragleave', 'drop' ],
					self = this;

				dropArea.addEventListener( 'drop', function ( ev ) {
					self.$( '.tva-drop-file-elem' ).removeClass( 'tva-can-drop' );
					self.files = ev.dataTransfer.files;
					self.renderBulkImport();
					self.registerEventListeners();
				}, false );

				events.forEach( function ( eventName ) {
					dropArea.addEventListener( eventName, preventDefaults, false );
					document.body.addEventListener( eventName, preventDefaults, false )
				} );

				dropArea.addEventListener( 'dragover', function ( ev ) {
					self.$( '.tva-drop-file-elem' ).addClass( 'tva-can-drop' );
				}, false );
				dropArea.addEventListener( 'dragleave', function ( ev ) {
					self.$( '.tva-drop-file-elem' ).removeClass( 'tva-can-drop' );
				}, false );

				function preventDefaults( e ) {
					e.preventDefault();
					e.stopPropagation()
				}
			},
			renderOptions: function () {
				this.settings.get( 'sendowl_products' ).each( this.renderProduct, this );
				this.settings.get( 'sendowl_bundles' ).each( this.renderBundles, this );
			},
			renderProduct: function ( model ) {
				var view = new ThriveApp.views.SendowlLevel( {
					tag: 'sendowl',
					model: model
				} );

				this.$products_el.append( view.render().$el );
			},
			renderBundles: function ( model ) {
				var view = new ThriveApp.views.SendowlBundle( {
					tag: 'sendowl',
					model: model
				} );

				this.$bundles_el.append( view.render().$el );
			},
			renderBulkImport: function () {
				if ( ! ThriveApp.views[ 'ImportFrom' + ThriveApp.util.upperFirst( this.import_type ) ] ) {
					return;
				}

				this.model = new ThriveApp.models.SendowlCustomer( {} );
				this.$( '.tva-add-single-customer' ).empty();

				var self = this,
					view = new ThriveApp.views[ 'ImportFrom' + ThriveApp.util.upperFirst( this.import_type ) ]( {
						el: this.$( '.tva-import-container' ),
						files: this.files
					} );

				view.render();

				this.$( '#tva-drop-layer' ).mouseover( function () {
					self.$( '.tva-drop-file-elem' ).zIndex( 1 );
				} );
				this.$( '.tva-drop-file-elem' ).mouseleave( function () {
					self.$( this ).zIndex( - 1 );
				} );
			},
			renderImportPreview: function () {
				this.model = new ThriveApp.models.SendowlCustomer( {} );
				this.$( '.tva-csv-customers-list' ).empty();
				this.customers.each( this.renderOne, this );

				var is_valid = this.customers.findWhere( {is_valid: 0} );

				if ( is_valid ) {
					this.$( '.tva-invalid-customers' ).show();
				}

				this.$products_el = this.$( '.tva-csv-sendowl-products' );
				this.$bundles_el = this.$( '.tva-csv-sendowl-bundles' );
				this.$products_el.empty();
				this.$bundles_el.empty();
				this.renderOptions();
				this.$( '.tva-available-options' ).addClass( 'tva-bulk-options' )
			},
			renderOne: function ( model ) {
				var view = new ThriveApp.views.SendowlCsvCustomer( {
					model: model,
					el: this.$( '.tva-csv-customers-list' )
				} );

				view.render();
			},
			renderSingleCustomerImport: function () {
				var view = new ThriveApp.views.SingleCustomerImport( {
					model: this.model,
					el: this.$( '.tva-add-single-customer ' )
				} );

				view.render();
			},
			setMembership: function ( e ) {
				var ids = this.model.get( 'membership_ids' ),
					value = $( e.currentTarget ).val(),
					$bulk = this.$( '.tva-bulk-options' );

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = ThriveApp.util.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}

				$bulk ? this.membership_ids = ids : this.model.set( {membership_ids: ids} );
			},
			setBundle: function ( e ) {
				var ids = this.model.get( 'bundle_ids' ),
					value = $( e.currentTarget ).val(),
					$bulk = this.$( '.tva-bulk-options' );

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = ThriveApp.util.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}

				$bulk ? this.bundle_ids = ids : this.model.get( {bundle_ids: ids} );
			},
			seName: function ( e ) {
				this.model.set( {display_name: e.target.value} );
				this.model.set( {user_login: e.target.value} );
			},
			setEmail: function ( e ) {
				this.model.set( {user_email: e.target.value} );
			},
			setImportType: function ( e ) {
				this.import_type = e.currentTarget.getAttribute( 'data-option' );

				var self = this;

				this.$( '.tva-selected-import-option' ).each( function () {
					self.$( this ).removeClass( 'tva-selected-import-option' )
				} );

				this.$( e.currentTarget ).addClass( 'tva-selected-import-option' );
				this.$( '.tva-toggle-disable' ).removeClass( 'tvd-disabled' );
			},
			setCustomersList: function ( e ) {
				this.customers_list = e.currentTarget.value;
			},
			resetData: function () {
				this.files = [];
				this.membership_ids = [];
				this.bundle_ids = [];
				this.errors.invalid_lines = [];
				this.errors.invalid_emails = [];
			},
			importCustomers: function () {
				$( '.tvd-hiddendiv' ).empty();

				if ( ( this.membership_ids.length === 0 &&
				       this.bundle_ids.length === 0 ) ) {
					TVE_Dash.err( ThriveApp.t.MissingAccessRights );
					return;
				}

				var req_limit = ThriveApp.globals.settings.get( 'customers_per_request' );

				this.current_request_index = 0;
				this.total_requests_numer = Math.ceil( this.customers.length / req_limit );

				this.json_customers = this.customers.pluckValues(
					[ 'user_email', 'user_login', 'display_name', 'is_valid' ]
				);

				this.json_customers = _.filter( this.json_customers, function ( item ) {
					if ( item.is_valid ) {
						return parseInt( item.is_valid ) === 1;
					}
				} );

				this.doImportAjaxCall();
			},
			processFile: function () {
				TVE_Dash.showLoader();
				var form_data = new FormData(),
					self = this;

				$.each( this.files, function ( key, value ) {
					form_data.append( 'tva_csv_file', value )
				} );

				$.ajax( {
					url: ThriveApp.routes.users + '/upload_file',
					type: 'POST',
					data: form_data,
					cache: false,
					dataType: 'json',
					processData: false,
					contentType: false
				} ).done( function ( response, status, options ) {
					if ( response ) {

						if ( response.invalid_file ) {
							self.$( '.tva-customers-import-options' ).hide();
							self.$( '.tva-invalid-file' ).show();
							return;
						}

						self.$( '.tva-customers-import-options' ).show();
						self.$( '.tva-csv-customers-list' ).empty();
						self.model = new ThriveApp.models.SendowlCustomer( {} );
						self.customers = new ThriveApp.collections.SendowlCustomers( response );
						self.customers.each( self.renderOne, self );

						var is_valid = self.customers.findWhere( {is_valid: 0} );

						if ( is_valid ) {
							self.$( '.tva-invalid-customers' ).show();
						}

						self.$products_el = self.$( '.tva-csv-sendowl-products' );
						self.$bundles_el = self.$( '.tva-csv-sendowl-bundles' );
						self.$products_el.empty();
						self.$bundles_el.empty();
						self.renderOptions();
						self.$( '.tva-available-options' ).addClass( 'tva-bulk-options' );
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( errorObj.responseText );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			parseCustomersList: function () {
				this.customers_arr = this.customers_list.split( '\n' );
				this.customers = new ThriveApp.collections.SendowlCustomers();

				_.each( this.customers_arr, function ( line, key ) {
					var valid = this.isValidLine( line, key ),
						chunks = line.split( ', ' );

					this.customers.add( new ThriveApp.models.SendowlCustomer( {
						display_name: chunks[ 0 ] ? chunks[ 0 ] : '',
						user_login: chunks[ 0 ] ? chunks[ 0 ] : '',
						user_email: chunks[ 1 ] ? chunks[ 1 ].replace( ',', '' ) : '',
						is_valid: valid ? 1 : 0
					} ) );
				}, this );
			},
			isValid: function () {
				switch ( this.currentStep ) {
					case 1:
						if ( ! this.import_type ) {
							TVE_Dash.err( ThriveApp.t.ChooseImportType );
							return;
						}
						break;

					case 2:
						if ( this.import_type === 'csvFile' && this.files.length === 0 ) {
							TVE_Dash.err( ThriveApp.t.ChooseCsv );
							return;
						}

						if ( this.import_type === 'csvFile' ) {
							var ext = this.files[ 0 ].name.split( '.' ).pop();

							if ( ext !== 'csv' ) {
								this.$( '.tva-invalid-file-type' ).show();
								return;
							}
						}

						if ( this.import_type === 'listNames' ) {
							this.errors.invalid_emails = [];
							this.errors.invalid_lines = [];

							this.$( '.tva-invalid-list-message' ).hide();
							this.$( '.tva-invalid-lines' ).empty();

							this.customers_list = ThriveApp.util.removeEmptyLine( this.customers_list );

							if ( ! this.customers_list.length ) {
								TVE_Dash.err( ThriveApp.t.MissingCustomers );
								return;
							}

							this.parseCustomersList();
						}

						break;

					default:
						break;
				}

				return true;
			},
			isValidLine: function ( line, key ) {
				var chunks = line.split( ', ' );

				// we only accept two fields here
				if ( chunks.length !== 2 ) {
					this.errors.invalid_lines.push( {line: key, data: line} );
					return false
				}

				if ( chunks[ 1 ].substr( chunks[ 1 ].length - 1 ) === ',' ) {
					chunks[ 1 ] = chunks[ 1 ].replace( ',', '' )
				}

				if ( ! ThriveApp.util.isEmail( chunks[ 1 ].trim() ) ) {
					this.errors.invalid_emails.push( {line: key, email: chunks[ 1 ]} );
					return false;
				}

				return true;
			},
			doImportAjaxCall: function () {
				if ( this.json_customers.length === 0 ) {
					TVE_Dash.err( ThriveApp.t.noValidCustomers );
					return;
				}

				TVE_Dash.showLoader();

				var self = this,
					req_limit = ThriveApp.globals.settings.get( 'customers_per_request' ),
					left_limit = this.current_request_index * req_limit,
					right_limit = req_limit * ( this.current_request_index + 1 ),
					data = {
						membership_ids: this.membership_ids,
						bundle_ids: this.bundle_ids,
						customers: this.json_customers.slice( left_limit, right_limit ),
						is_last_request: this.current_request_index + 1 === this.total_requests_numer ? 1 : 0
					};

				$.ajax( {
					url: ThriveApp.routes.users + '/import_customers',
					type: 'POST',
					data: data
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.current_request_index ++;
						self.imported_users = self.imported_users + parseInt( response.imported_users.length );

						if ( self.current_request_index < self.total_requests_numer ) {
							self.doImportAjaxCall();
						} else {
							ThriveApp.globals.settings.set( {customers_count: response.count} );
							ThriveApp.globals.sendowl_customers.trigger( 'tva_render_customers', {
								data: response.customers,
								count: response.count
							} );
							TVE_Dash.success( ThriveApp.t.Success + self.imported_users + ThriveApp.t.NewUsersAdded );
							self.close();
						}
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( JSON.parse( errorObj.responseText ).message, 5000 );
				} ).always( function ( response, textStatus, options ) {
					if ( response === null ) {
						TVE_Dash.err( ThriveApp.t.UnknownError, '10000' );
						TVE_Dash.hideLoader();
						return;
					}

					if ( self.current_request_index >= self.total_requests_numer ) {
						TVE_Dash.hideLoader();
					}
				} );
			},
			save: function () {
				this.tvd_clear_errors();

				if ( ! this.model.isValid() ) {
					this.tvd_show_errors( this.model );
					this.$( '[data-error]' ).addClass( 'tva-custom-err-label' );
					return;
				}

				if ( ( this.model.get( 'membership_ids' ).length === 0 &&
				       this.model.get( 'bundle_ids' ).length === 0 ) ) {
					TVE_Dash.err( ThriveApp.t.MissingAccessRights );
					return;
				}

				TVE_Dash.showLoader();
				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response && response.customers ) {
							ThriveApp.globals.settings.set( {customers_count: response.count} );
							ThriveApp.globals.sendowl_customers.trigger( 'tva_render_customers', {
								data: response.customers,
								count: response.count
							} );

							var names = '',
								total = self.model.get( 'membership_ids' ).length + self.model.get( 'bundle_ids' ).length,
								added = 0;

							_.each( self.model.get( 'membership_ids' ), function ( id ) {
								var model = ThriveApp.globals.settings.get( 'sendowl_products' ).findWhere( {id: id} );

								if ( model ) {
									added ++;
									names += model.get( 'name' );
									names += added < total ? ', ' : '';
								}
							} );

							_.each( self.model.get( 'bundle_ids' ), function ( id ) {
								var model = ThriveApp.globals.settings.get( 'sendowl_bundles' ).findWhere( {id: id} );

								if ( model ) {
									added ++;
									names += model.get( 'name' );
									names += added < total ? ', ' : '';
								}
							} );
						}

						var msg = ThriveApp.t.Success + '  ' + self.model.get( 'display_name' ) + ' ' + ThriveApp.t.HasAccess + ' ' + names;

						TVE_Dash.success( msg, 3000 );

						self.close();
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( JSON.parse( errorObj.responseText ).message, 5000 );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * View to render Edit customer modal
		 */
		ThriveApp.views.EditCustomerModal = ThriveApp.views.ModalEditor.extend( {
			template: TVE_Dash.tpl( 'modals/sendowl/edit-customer' ),
			events: {
				'change .tva-membership-change': 'setMembership',
				'change .tva-bundle-change': 'setBundle',
				'click .tvd-modal-submit': 'save'
			},
			afterRender: function () {
				this.settings.get( 'sendowl_products' ).each( this.renderProduct, this );
				this.settings.get( 'sendowl_bundles' ).each( this.renderBundles, this );
			},
			renderProduct: function ( model ) {
				var view = new ThriveApp.views.SendowlLevel( {
					model: model,
					checked: this.model.get( 'membership_ids' ),
					tag: 'sendowl'
				} );

				this.$( '.tva-sendowl-products' ).append( view.render().$el );
			},
			renderBundles: function ( model ) {
				var view = new ThriveApp.views.SendowlBundle( {
					model: model,
					checked: this.model.get( 'bundle_ids' ),
					tag: 'sendowl'
				} );

				this.$( '.tva-sendowl-bundles' ).append( view.render().$el );
			},
			renderBulkImport: function () {
				if ( ! ThriveApp.views[ 'ImportFrom' + ThriveApp.util.upperFirst( this.import_type ) ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'ImportFrom' + ThriveApp.util.upperFirst( this.import_type ) ]( {
					el: this.$( '.tva-import-container' )
				} );

				view.render();
			},
			setMembership: function ( e ) {
				var ids = this.model.get( 'membership_ids' ),
					value = $( e.currentTarget ).val();

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = ThriveApp.util.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}

				this.model.set( {membership_ids: ids} );
			},
			setBundle: function ( e ) {
				var ids = this.model.get( 'bundle_ids' ),
					value = $( e.currentTarget ).val();

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = ThriveApp.util.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}

				this.model.set( {bundle_ids: ids} );
			},
			save: function () {
				TVE_Dash.showLoader();
				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response ) {
							self.close();
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

	} ); //end document ready
} )( jQuery );;/**
 * Created by Pop Aurelian on 21-Feb-17.
 */
var ThriveApp = ThriveApp || {};
ThriveApp.views = ThriveApp.views || {};

( function ( $ ) {
	$( function () {

		/**
		 * remove tvd-invalid class for all inputs in the view's root element
		 *
		 * @returns {Backbone.View}
		 */
		Backbone.View.prototype.tvd_clear_errors = function () {
			this.$( '.tvd-invalid' ).removeClass( 'tvd-invalid' );
			this.$( 'select' ).trigger( 'tvdclear' );
			return this;
		};

		/**
		 *
		 * @param {Backbone.Model|object} [model] backbone model or error object with 'field' and 'message' properties
		 *
		 * @returns {Backbone.View|undefined}
		 */
		Backbone.View.prototype.tvd_show_errors = function ( model ) {
			model = model || this.model;

			if ( ! model ) {
				return;
			}

			var err = model instanceof Backbone.Model ? model.validationError : model,
				self = this,
				$all = $();

			function show_error( error_item ) {
				if ( typeof error_item === 'string' ) {
					return TVE_Dash.err( error_item );
				}
				$all = $all.add( self.$( '[data-field=' + error_item.field + ']' ).addClass( 'tvd-invalid' ).each( function () {
					var $this = $( this );
					if ( $this.is( 'select' ) ) {
						$this.trigger( 'tvderror', error_item.message );
					} else {
						$this.next( 'label' ).attr( 'data-error', error_item.message )
					}
				} ) );
			}

			if ( $.isArray( err ) ) {
				_.each( err, function ( item ) {
					show_error( item );
				} );
			} else {
				show_error( err );
			}
			$all.not( '.tvd-no-focus' ).first().focus();
			/* if the first error message is not visible, scroll the contents to include it in the viewport. At the moment, this is only implemented for modals */
			this.scroll_first_error( $all.first() );

			return this;
		};

		/**
		 * scroll the contents so that the first errored input is visible
		 * currently this is only implemented for modals
		 *
		 * @param {Object} $input first input element that has the error
		 *
		 * @returns {Backbone.View}
		 */
		Backbone.View.prototype.scroll_first_error = function ( $input ) {
			if ( ! ( this instanceof TVE_Dash.views.Modal ) || ! $input.length ) {
				return this;
			}
			var input_top = $input.offset().top,
				content_top = this.$_content.offset().top,
				scroll_top = this.$_content.scrollTop(),
				content_height = this.$_content.outerHeight();
			if ( input_top >= content_top && input_top < content_height + content_top - 50 ) {
				return this;
			}

			this.$_content.animate( {
				'scrollTop': scroll_top + input_top - content_top - 40 // 40px difference
			}, 200, 'swing' );
		};

		/**
		 * Base View
		 */
		ThriveApp.views.Base = Backbone.View.extend( {
			/**
			 * Always try to return this !!!
			 *
			 * @returns {ThriveChurn.views.Base}
			 */
			render: function () {
				return this;
			},
			/**
			 *
			 * Instantiate and open a new modal which has the view constructor assigned and send params further along
			 *
			 * @param ViewConstructor View constructor
			 * @param params
			 */
			modal: function ( ViewConstructor, params ) {
				return TVE_Dash.modal( ViewConstructor, params );
			},
			bind_zclip: function () {
				console.log()
				/**
				 * Keep the old ZClip working
				 */
				TVE_Dash.bindZClip( this.$el.find( 'a.tvd-copy-to-clipboard' ) );

				var $element = this.$el.find( '.tva-sendowl-search' );

				function bind_it() {
					$element.each( function () {
						var $elem = $( this ),
							$input = $elem.prev().on( 'click', function ( e ) {
								this.select();
								e.preventDefault();
								e.stopPropagation();
							} ),
							_default_btn_color_class = $elem.attr( 'data-tvd-btn-color-class' ) || 'tva-copied';

						try {
							$elem.zclip( {
								path: TVE_Dash_Const.dash_url + '/js/util/jquery.zclip.1.1.1/ZeroClipboard.swf',
								copy: function () {
									return jQuery( this ).prev().val();
								},
								afterCopy: function () {
									var $link = jQuery( this );
									$input.select();
									$link.removeClass( _default_btn_color_class ).addClass( 'tva-copied' );
									setTimeout( function () {
										$link.removeClass( 'tva-copied' );
									}, 2000 );
								}
							} );
						} catch ( e ) {
							console.error && console.error( 'Error embedding zclip - most likely another plugin is messing this up' ) && console.error( e );
						}
					} );
				}

				setTimeout( bind_it, 200 );
			}
		} );

		/**
		 * Header View
		 */
		ThriveApp.views.Header = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'header' ),
			events: {
				'click #tva-general-settings': 'generalSettings',
				'click #tva-disable-apprentice': 'disableApprentice',
				'click #tva-template-settings': 'templateSettings',
				'click #tva-label-management': 'labelManagement'
			},
			initialize: function () {
				this.listenTo( this.model, 'change:apprentice', this.render );
				this.model.on( 'tva_render_header', function () {
					this.render();
				}, this )
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				TVE_Dash.materialize( this.$el );
				return this;
			},
			generalSettings: function () {
				this.modal( ThriveApp.views.ModalSettings, {
					model: ThriveApp.globals.settings,
					'max-width': '60%',
					width: '800px',
					in_duration: 200,
					out_duration: 0
				} );
			},
			templateSettings: function () {
				ThriveApp.router.navigate( "#template_settings", {trigger: true} );
			},
			labelManagement: function () {
				ThriveApp.router.navigate( "#label_management", {trigger: true} );
			},
			disableApprentice: function () {
				TVE_Dash.showLoader();
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/disable_apprentice/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.model.set( {apprentice: false} );
						TVE_Dash.success( ThriveApp.t.SuccessDisabledApprentice );
						self.render();
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( errorObj.responseText );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		/**
		 * breadcrumbs view - renders breadcrumb links
		 */
		ThriveApp.views.Breadcrumbs = ThriveApp.views.Base.extend( {
			el: $( '#tva-breadcrumbs-wrapper' )[ 0 ],
			template: TVE_Dash.tpl( 'breadcrumbs' ),
			/**
			 * setup collection listeners
			 */
			initialize: function () {
				this.$title = $( 'head > title' );
				this.original_title = this.$title.html();
				this.listenTo( this.collection, 'change', this.render );
				this.listenTo( this.collection, 'add', this.render );
			},
			/**
			 * render the html
			 */
			render: function () {
				this.$el.empty().html( this.template( {links: this.collection} ) );
				return this;
			}
		} );
		/**
		 * Start With The greeting modal
		 */
		ThriveApp.views.Start = ThriveApp.views.Base.extend( {
			render: function () {
				this.modal( ThriveApp.views.ModalGreetings, {
					model: this.model,
					'max-width': '60%',
					width: '800px',
					// height: '500',
					no_close: true,
					dismissible: false
				} );

				return this;
			}
		} );
		/**
		 * Wizard view
		 */
		ThriveApp.views.Wizard = ThriveApp.views.Base.extend( {
			className: 'tva-wizard-container',
			template: TVE_Dash.tpl( 'wizard' ),
			events: {
				'click .save-template': 'save',
				'click .tva-close': 'close',
				'click .tva-advanced-settings': 'showAdvancedSettings',
				'click .tva-apprentice-template': 'showCourseList',
				'click .tva-go-back': 'showTemplateSettings',
				'click .tva-live': 'showLive',
				'click .tva-mock-up': 'showMockUp',
				'click .tva-card-title': 'openCardContent'
			},
			preview: '',
			settings: '',
			initialize: function ( options ) {
				TVE_Dash.showLoader();
				var self = this;
				ThriveApp.globals.wizzard = this;

				ThriveApp.globals.wizzard.on( 'tva_iframe_loaded', function () {
					ThriveApp.util.Editor.init();
					self.renderSettings();
					TVE_Dash.hideLoader();
				} );

				$( '.dismiss-notice, .notice-dismiss' ).on( 'click', function () {
					setTimeout( function () {
						self.calcHeight();
					}, 200 );
				} );

				this.listenTo( this.model, 'change:template', this.render );
				this.listenTo( this.model, 'change:current_page', this.render );
				this.edit = options.edit ? options.edit : false;
				this.original_model = options.original_model;
				this.model.set( {advanced: false} );
			},
			render: function () {
				/**
				 * if we don't have any courses and the preview is set to see live move the switch to mock-up
				 */
				var published = ThriveApp.globals.courses.where( {status: 'publish'} );

				if ( this.model.get( 'preview_option' ) && published.length === 0 ) {
					this.model.set( {preview_option: false} );
				}

				if ( published.length === 0 && this.model.get( 'preview_notification' ) ) {
					this.modal( ThriveApp.views.ModalFirstTimeNotification, {
						model: this.model,
						'max-width': '60%',
						width: '800px'
					} );
				}

				this.$el.html( this.template( {model: this.model, published: published} ) );

				this.renderPreview();

				return this
			},
			calcHeight: function () {
				var height = $( '.notice' ).outerHeight() + $( '.update-nag' ).outerHeight() + 'px';
				this.$el.css( {height: height} );
			},
			renderPreview: function () {
				this.preview = new ThriveApp.views.Template( {
					model: this.model,
					el: this.$( '.tva-template-container' )
				} );

				this.preview.render();
			},
			renderSettings: function () {
				var state = this.model.get( 'current_page' );


				this.settings = new ThriveApp.views[ ThriveApp.util.upperFirst( state ) + 'Settings' ]( {
					model: this.model,
					collection: this.collection,
					available_settings: ThriveApp.globals.available_settings,
					edit: this.edit,
					el: this.$( '.tva-options-container' )
				} );

				this.settings.render();

				//Hide notices on template settings view

				if ( $( '.tva-header' ).prevAll().length > 1 ) {

					$( '.tva-header' ).prevAll().css( "display", "none" );

				}

				TVE_Dash.materialize( this.$el );

			},
			showLive: function () {
				this.switchPreview( true );
			},
			showMockUp: function () {
				this.switchPreview( false );
			},
			switchPreview: function ( value ) {
				var self = this;
				TVE_Dash.showLoader();

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/switch_preview/',
					data: {
						preview_option: value
					}
				} ).done( function ( response, status, options ) {
					if ( response.url ) {

						var iframe_url = self.model.get( 'current_page' ) === 'index' ? self.model.get( 'apprentice_url' ) : response.url;

						self.model.set( {preview_option: value, iframe_url: iframe_url, preview_url: response.url} );
						ThriveApp.globals.settings.set( {
							preview_option: value,
							iframe_url: iframe_url,
							preview_url: response.url
						} );

						self.render();
					}
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			showTemplateSettings: function () {

				this.model.set( {iframe_url: this.model.get( 'preview_url' ), current_page: 'template'} );
			},
			showAdvancedSettings: function () {
				TVE_Dash.showLoader();

				var self = this;

				self.model.set( {
					membership_plugin: {},
					sendowl_products: {}
				} );

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/set_advanced_user_settings/',
					data: {
						template: self.model.toDeepJSON()
					}
				} ).done( function ( response, status, options ) {
					self.model.set( {
						iframe_url: response.url,
						current_page: 'advanced',
						preview_notification: false,
						advanced: true
					} );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );

			},
			showCourseList: function () {
				if ( this.model.get( 'apprentice_url' ) ) {
					this.model.set( {iframe_url: this.model.get( 'apprentice_url' )} );
				}

				this.model.set( {current_page: 'index'} );
			},
			openCardContent: function ( e ) {
				var currentCard = e.currentTarget;
				if ( jQuery( currentCard ).siblings( ".tva-card-content" ).hasClass( "open" ) ) {
					jQuery( currentCard ).siblings( ".tva-card-content" ).removeClass( "open" );
					jQuery( currentCard ).children( ".tva-arrow" ).css( "transform", "rotate(45deg)" );
				} else {
					jQuery( currentCard ).siblings( ".tva-card-content" ).addClass( "open" );
					jQuery( currentCard ).children( ".tva-arrow" ).css( "transform", "rotate(225deg)" );
				}

			},
			save: function () {
				TVE_Dash.showLoader();
				this.model.set( {first_time: false} );

				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response ) {
							self.original_model.set( {template: response.template} );
							self.original_model.set( 'first_time', false );
						}

						TVE_Dash.success( ThriveApp.t.TemplateSaved );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();

					} );
				}


			},
			close: function () {
				TVE_Dash.showLoader();
				if ( this.model.get( 'old_template' ) ) {
					this.model.set( {template: this.model.get( 'old_template' )} );
					this.model.unset( 'old_template' );

					this.save();
				} else {
					/**
					 * Set the preview option in case it was changed
					 */
					this.original_model.set( {preview_option: this.model.get( 'preview_option' )} );
					this.original_model.set( {iframe_url: this.model.get( 'iframe_url' )} );

					if ( this.model.get( 'first_time' ) ) {
						this.showCloseNotification();

						TVE_Dash.hideLoader();
					} else {
						if ( _.isEqual( this.model.get( 'template' ), this.original_model.get( 'template' ) ) ) {
							ThriveApp.router.navigate( "#dashboard", {trigger: true} );
						} else {
							this.showCloseNotification();
							TVE_Dash.hideLoader();
						}
					}
				}

			},
			showCloseNotification: function () {
				this.modal( ThriveApp.views.ConfirmEditorClose, {
					'max-width': '60%',
					width: '800px',
					model: this.model,
					in_duration: 200,
					out_duration: 0
				} );
			}
		} );

		/**
		 * Render the template settings
		 */
		ThriveApp.views.TemplateSettings = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/template-settings' ),
			events: {
				'change #tva-fonts-source': 'setSource',
				'change #tva-fonts-select': 'setFont',
				'change #tva-regular-select': 'setRegular',
				'change #tva-bold-select': 'setBold',
				'change #tva-charset-select': 'setCharset',
				'change #tva-template-select': 'changeTemplate',
				'change #tva-main-color-select': 'setSpectrum',
				'click .tva-change-logo': 'changeLogo',
				'input .tva-text-change': 'inputLogo',
				'click .tva-logo-type': 'changeLogoType',
				'input #tva-logo-text': 'textLogo'
			},
			initialize: function ( options ) {
				this.edit = options.edit ? options.edit : false;
				this.available_settings = options.available_settings;
			},
			render: function () {
				var self = this;
				this.$el.empty().html( this.template( {
					model: this.model,
					edit: this.edit,
					available_settings: this.available_settings
				} ) );
				this.collection.each( this.renderTemplate, this );
				this.renderFonts();
				this.renderTextLogo();

				this.$( '#tva-fonts-select' ).select2( {
					multiple: false
				} );

				this.$( '#tva-main-color-select' ).spectrum( {
					containerClassName: 'tva-color-picker',
					allowEmpty: false,
					showInitial: true,
					showButtons: true,
					chooseText: "Apply",
					cancelText: "Cancel",
					preferredFormat: "hex",
					showInput: true,
					change: function ( color ) {
						var _color = color ? color.toHexString() : '',
							name = this.name;

						self.model.get( 'template' )[ name ] = _color;
					},
					move: function ( color ) {
						var _color = color ? color.toHexString() : '',
							name = this.name;

						ThriveApp.util.Editor.apply( '.tva_' + name, 'color', _color );
						ThriveApp.util.Editor.apply( '.tva_' + name, 'fill', _color, '', '#D0D0D0' );
						ThriveApp.util.Editor.apply( '.tva_' + name + '_bg, header.tva-header ul.menu > li.h-cta', 'background-color', _color );
						ThriveApp.util.Editor.apply( '.tva-sidebar-container ul a', 'color', _color );
						ThriveApp.util.Editor.apply( '.tva-cm-redesigned-breadcrumbs ul li a', 'color', _color, true, '#666666' );
					}
				} );

				return this;
			},
			inputLogo: function ( e ) {
				var value = $( e.currentTarget ).val();

				ThriveApp.util.Editor.image( '.lg', value );
				this.model.get( 'template' ).logo_url = value;
			},
			changeLogoType: function ( e ) {
				e.stopPropagation();
				var checked = $( e.currentTarget ).is( ':checked' );

				ThriveApp.util.Editor.emptyElement( '.lg' );

				this.model.get( 'template' ).logo_type = checked;
				this.model.get( 'template' ).logo_url = '';

				this.$( '.tva-change-logo-input' ).val( '' );
				this.renderTextLogo();
			},
			renderTextLogo: function () {

				var view = new ThriveApp.views.TextLogoInput( {
					el: this.$( '.tva-text-logo-wrapper' )[ 0 ],
					model: this.model
				} );

				view.render();
			},
			textLogo: function ( e ) {
				var target = e.currentTarget;

				ThriveApp.util.Editor.text( '.lg', target.value );
				this.model.get( 'template' ).logo_url = target.value;
			},
			setSpectrum: function ( e ) {
				var name = $( e.currentTarget ).attr( 'name' ),
					color = $( e.currentTarget ).val();

				$( e.currentTarget ).spectrum( "set", color );

				this.model.get( 'template' )[ name ] = color;

				ThriveApp.util.Editor.apply( '.tva_' + name, 'color', color );
				ThriveApp.util.Editor.apply( '.tva_' + name + '_bg', 'background-color', color );
				ThriveApp.util.Editor.apply( '.tva-header > div ul li, header.tva-header ul.menu > li.h-cta', 'background-color', color, true, '#fff' );
				ThriveApp.util.Editor.apply( 'header.tva-header ul.menu > li.h-cta', 'background-color', color, true, this.model.get( 'template' ).main_color );
			},
			renderTemplate: function ( template ) {
				var view = new ThriveApp.views.TemplateOption( {
					el: this.$( '#tva-template-select' ),
					model: template,
					selected: this.model.get( 'template' ).ID
				} );

				view.render();
			},
			renderFonts: function () {
				var source = this.model.get( 'template' ).font_source,
					collection = ThriveApp.globals.fonts;
				if ( source === 'google' ) {
					collection = ThriveApp.globals.googlefonts;
				}

				this.fonts = new ThriveApp.views[ ThriveApp.util.upperFirst( source ) + 'FontsSettings' ]( {
					model: this.model,
					collection: collection,
					el: this.$( '#tva-fonts' )
				} );

				this.fonts.render();
			},
			setSource: function ( e ) {
				this.model.get( 'template' ).font_source = e.currentTarget.value;
				this.model.get( 'template' ).font_family = '';
				this.renderFonts();
				this.setFont( this.$( '#tva-fonts-select' ) );
			},
			setRegular: function ( e ) {
				this.model.get( 'template' ).font_regular = e.currentTarget.value;
				this.applyFontToHead();
			},
			setBold: function ( e ) {
				this.model.get( 'template' ).font_bold = e.currentTarget.value;
				this.applyFontToHead();
			},
			setCharset: function ( e ) {
				this.model.get( 'template' ).font_charset = e.currentTarget.value;
				this.applyFontToHead();
			},
			setFont: function ( e ) {
				var font = e.currentTarget ? e.currentTarget.value : e.val();
				var source = this.model.get( 'template' ).font_source;
				this.model.get( 'template' ).font_family = font;

				if ( source === 'google' ) {
					var model = ThriveApp.globals.googlefonts.findWhere( {family: font} ),
						regular = '',
						bold = '';

					_.each( model.get( 'variants' ), function ( variant ) {
						if ( variant === 'regular' || ( parseInt( variant ) < 400 && ! isNaN( parseInt( variant ) ) && regular < parseInt( variant ) ) ) {
							regular = variant;
						}

						if ( variant === 'bold' || ( parseInt( variant ) > 400 && ! isNaN( parseInt( variant ) ) && bold < parseInt( variant ) ) ) {
							bold = variant;
						}
					} );
					this.model.get( 'template' ).font_regular = regular;
					this.model.get( 'template' ).font_charset = model.get( 'subsets' )[ 0 ];
					this.model.get( 'template' ).font_bold = bold;

					this.applyFontToHead();
				}

				ThriveApp.util.Editor.apply( 'body a', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body p', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body h1', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body h2', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body h3', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body h4', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body h5', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body span', 'font-family', font );
				ThriveApp.util.Editor.apply( 'body strong', 'font-family', font );
				this.model.trigger( 'fonts_changed' );

			},
			applyFontToHead: function () {
				params = {
					family: '',
					variants: [],
					subset: ''
				};

				params.family = this.model.get( 'template' ).font_family;
				params.subset = this.model.get( 'template' ).font_charset;
				params.variants.push( this.model.get( 'template' ).font_regular );
				params.variants.push( this.model.get( 'template' ).font_bold );

				var font_url = ThriveApp.util.FontManager.generate_link( params );

				ThriveApp.util.Editor.font_to_head( font_url );

				this.model.get( 'template' ).font_url = font_url;
			},
			changeTemplate: function ( e ) {
				var template = e.currentTarget.value,
					model = this.collection.findWhere( {ID: parseInt( template )} );

				this.model.changeTemplate( model );

			},
			changeLogo: function () {
				var frame,
					self = this;

				if ( frame ) {
					frame.open();
					return;
				}

				frame = wp.media( {
					title: 'Select or Upload Your Logo',
					button: {
						text: 'Use this logo'
					},
					multiple: false,  // Set to true to allow multiple files to be selected
					library: {type: 'image'}
				} );

				frame.on( 'select', function () {
					$( '.tva-logo-type' ).prop( 'checked', false );
					$( '#tva-logo-text' ).val( '' );
					self.model.get( 'template' ).logo_type = false;

					$( '.tva-change-logo-input' ).val( '' );

					// Get media attachment details from the frame state
					var attachment = frame.state().get( 'selection' ).first().toJSON();
					ThriveApp.util.Editor.image( '.lg', attachment.url, attachment );
					self.model.get( 'template' ).logo_url = attachment.url;

					self.render();
					TVE_Dash.materialize( self.$el );
				} );

				// Finally, open the modal on click
				frame.open();
			}
		} );

		ThriveApp.views.SafeFontsSettings = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/safe-fonts' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderFonts();
				TVE_Dash.materialize( this.$el );
			},
			renderFonts: function () {
				this.collection.each( this.renderFont, this );
			},
			renderFont: function ( font ) {
				var view = new ThriveApp.views.FontOption( {
					el: this.$( '#tva-fonts-select' ),
					model: font,
					selected: this.model.get( 'template' ).font_family
				} );

				view.render();
			}
		} );

		ThriveApp.views.GoogleFontsSettings = ThriveApp.views.SafeFontsSettings.extend( {
			template: TVE_Dash.tpl( 'wizard/google-fonts' ),
			initialize: function () {
				var self = this;
				this.model.on( 'fonts_changed', function () {
					self.selected_model = self.collection.findWhere( {family: self.model.get( 'template' ).font_family} );

					self.renderVariants();
					self.renderCharsets();
				} );
			},
			renderFonts: function () {
				this.selected_model = this.collection.at( 0 );
				if ( this.model.get( 'template' ).font_family ) {
					this.selected_model = this.collection.findWhere( {family: this.model.get( 'template' ).font_family} );
				}

				this.collection.each( this.renderFont, this );
				this.renderVariants();
				this.renderCharsets();
			},
			renderFont: function ( font ) {
				var selected = this.model.get( 'template' ).font_family,
					view = new ThriveApp.views.FontOption( {
						el: this.$( '#tva-fonts-select' ),
						model: font,
						selected: selected
					} );

				view.render();

			},
			renderVariants: function () {
				if ( ! this.selected_model ) {
					this.selected_model = this.collection.at( 0 )
				}
				this.$( '#tva-regular-select' ).empty();
				this.$( '#tva-bold-select' ).empty();
				_.each( this.selected_model.get( 'variants' ), this.renderVariant, this );
				TVE_Dash.materialize( this.$el );

			},
			renderVariant: function ( variant ) {
				if ( variant.indexOf( 'italic' ) !== - 1 ) {
					return;
				}
				var $element = this.$( '#tva-regular-select' ),
					selected = this.model.get( 'template' ).font_regular;

				if ( variant === 'bold' || ( parseInt( variant ) > 400 && ! isNaN( parseInt( variant ) ) ) ) {
					this.$( '.tva-bold-holder' ).show();
					$element = this.$( '#tva-bold-select' );
					selected = this.model.get( 'template' ).font_bold;
				} else {
					this.$( '.tva-bold-holder' ).hide();
				}

				var view = new ThriveApp.views.VariantsOption( {
					el: $element,
					model: variant,
					selected: selected
				} );

				view.render();

			},
			renderCharsets: function () {
				if ( ! this.selected_model ) {
					this.selected_model = this.collection.at( 0 )
				}
				this.$( '#tva-charset-select' ).empty();
				_.each( this.selected_model.get( 'subsets' ), this.renderCharset, this );
				TVE_Dash.materialize( this.$el );

			},
			renderCharset: function ( charset ) {
				var view = new ThriveApp.views.VariantsOption( {
					el: this.$( '#tva-charset-select' ),
					model: charset,
					selected: this.model.get( 'template' ).font_charset
				} );

				view.render();
			}

		} );

		/**
		 * Render the fonts
		 */
		ThriveApp.views.FontOption = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/font-row' ),
			initialize: function ( options ) {
				this.selected = options.selected;
			},
			render: function () {
				this.$el.append( this.template( {model: this.model, selected: this.selected} ) );
				return this;
			}
		} );

		ThriveApp.views.VariantsOption = ThriveApp.views.FontOption.extend( {
			template: TVE_Dash.tpl( 'wizard/variants-row' )
		} );

		/**
		 * Render the input for the text logo Option
		 */
		ThriveApp.views.TextLogoInput = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/logo-text' ),
			render: function () {
				this.$el.empty();
				if ( this.model.get( 'template' ).logo_type ) {
					this.$el.html( this.template( {model: this.model} ) );
				}

				return this;
			}
		} );

		/**
		 * Render the template options
		 */
		ThriveApp.views.TemplateOption = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/template-row' ),
			initialize: function ( options ) {
				this.selected = options.selected;
			},
			render: function () {
				this.$el.append( this.template( {model: this.model, selected: this.selected} ) );
				return this;
			}
		} );

		/**
		 * Render the template preview
		 */
		ThriveApp.views.Template = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/preview' ),
			render: function () {
				var url = this.model.get( 'iframe_url' ).replace( /^https?\:/i, "" );
				this.model.set( 'no_protocol_url', url );
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.AdvancedSettings = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'wizard/template-advanced-settings' ),
			events: {
				'input .tva-no-ui-slider': 'slide',
				'change .tva-no-ui-slider': 'setValue',
				'change .tva_color_input_change': 'setSpectrum',
				'input .tva-slider-input': 'slide',
				'input .tva-text-change': 'text',
				'input .tva-attr-change': 'attrChange'
			},
			initialize: function ( options ) {
				this.available_settings = options.available_settings;
			},
			render: function () {
				this.$el.empty().html( this.template( {
					template: this.model.get( 'template' ),
					available_settings: this.available_settings
				} ) );
				var self = this;
				this.$( '#tva-headline-color, #tva-paragraph-color, #tva-course-title-color' ).spectrum( {
					containerClassName: 'tva-color-picker',
					allowEmpty: false,
					showInitial: true,
					showButtons: true,
					chooseText: "Apply",
					cancelText: "Cancel",
					preferredFormat: "hex",
					change: function ( color ) {
						var _color = color ? color.toHexString() : '',
							name = this.name,
							element = name.replace( '_color', '' );

						self.model.get( 'template' )[ name ] = _color;
					},
					move: function ( color ) {
						var _color = color ? color.toHexString() : '',
							name = this.name,
							element = name.replace( '_color', '' );

						ThriveApp.util.Editor.apply( '.tva_' + element, 'color', _color );
					}
				} );

				ThriveApp.util.Editor.init();

				return this;
			},
			setSpectrum: function ( e ) {
				var name = $( e.currentTarget ).attr( 'name' ),
					color = $( e.currentTarget ).val(),
					element = name.replace( '_color', '' );
				$( e.currentTarget ).spectrum( "set", color );

				this.model.get( 'template' )[ name ] = color;

				ThriveApp.util.Editor.apply( '.tva_' + element, 'color', color );
			},
			slide: function ( e ) {
				var type = e.currentTarget.type,
					target = e.currentTarget,
					name = e.currentTarget.name,
					row = $( target ).closest( '.tva-card' );

				if ( type == 'range' ) {
					row.find( '#tva_' + name + '_input' ).val( target.value );
				} else {
					row.find( '#tva_' + name ).val( target.value );

					this.setValue( e );
				}

				ThriveApp.util.Editor.apply( '.tva_' + name, 'font-size', target.value, 'px' );

			},
			text: function ( e ) {
				var target = e.currentTarget,
					name = e.currentTarget.name;
				ThriveApp.util.Editor.text( '.tva_' + name, target.value );
				this.setValue( e )
			},
			attrChange: function ( e ) {
				var target = e.currentTarget,
					name = e.currentTarget.name;
				ThriveApp.util.Editor.attrChange( '.tva_' + name, target.value, 'placeholder' );
				this.setAttrValue( e )
			},
			setValue: function ( e ) {
				var name = e.currentTarget.name,
					value = $( e.currentTarget ).hasClass( 'tva-text-change' ) ? e.currentTarget.value : parseFloat(
						e.currentTarget.value.replace( /,/g, "." ) );

				this.model.get( 'template' )[ name ] = value;
			},
			setAttrValue: function ( e ) {
				var name = e.currentTarget.name,
					value = $( e.currentTarget ).hasClass( 'tva-attr-change' ) ? e.currentTarget.value : parseFloat(
						e.currentTarget.value.replace( /,/g, "." ) );

				this.model.get( 'template' )[ name ] = value;
			}
		} );

		/**
		 * Render the index page settings
		 */
		ThriveApp.views.IndexSettings = ThriveApp.views.AdvancedSettings.extend( {
			template: TVE_Dash.tpl( 'wizard/template-index-settings' ),
			render: function () {
				this.$el.empty().html( this.template( {
					template: this.model.get( 'template' ),
					available_settings: this.available_settings
				} ) );
			}
		} );

		/**
		 * Import Dashboard
		 */
		ThriveApp.views.Import = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'import' ),
			events: {
				'click .tva-import-courses': 'import',
				'click .tva-select-all-courses': 'toggleChecked'
			},
			checked: '',
			render: function () {

				this.$el.html( this.template() );
				this.collection.each( this.renderCourse, this );
				this.checkAllCourses();

				$( " body " ).css( "overflow", "hidden" );

				if ( $( '.tva-header' ).prevAll().length > 1 ) {
					$( '.tva-header' ).prevAll().css( "display", "none" );
				}

				return this
			},
			renderCourse: function ( course ) {
				var view = new ThriveApp.views.OldCourse( {
					model: course
				} );
				this.$( '.tva-courses-container' ).append( view.render().$el );
			},
			checkAllCourses: function () {
				var check = true,
					not_checked = this.collection.findWhere( {checked: false} );
				if ( ! not_checked ) {
					this.collection.each( function ( course ) {
						var lessons_not_checked = course.get( 'lessons' ).findWhere( {checked: false} );
						if ( lessons_not_checked ) {
							check = false;
						}
					}, this );
				} else {
					check = false;
				}

				this.$( '.tva-select-all-courses' ).prop( 'checked', check )
			},
			toggleChecked: function ( e ) {
				this.checked = $( e.currentTarget ).is( ':checked' );
				this.$( '.tva-course-checkbox' ).prop( 'checked', this.checked );
				this.collection.each( this.toggleCourses, this );

			},
			toggleCourses: function ( course ) {
				var lessons = course.get( 'lessons' );
				course.set( {checked: this.checked} );

				this.$( '.tva-lesson-checkbox' ).prop( 'checked', this.checked );

				lessons.each( function ( model ) {
					model.set( {checked: this.checked} );
				}, this );
			},
			import: function () {
				TVE_Dash.showLoader();
				var courses = this.collection.where( {checked: true} ),
					/** Before saving and adding the courses to the collection we should set the order */
					order = ThriveApp.globals.courses.length,
					length = courses.length - 1;

				if ( courses.length == 0 ) {
					TVE_Dash.hideLoader();
					TVE_Dash.err( ThriveApp.t.SelectLessonCourse );
				}

				_.each( courses, function ( course, index ) {
					var exists = ThriveApp.globals.courses.findWhere( {name: course.get( 'name' )} );

					if ( ! exists ) {
						course.set( {order: order} );
						order ++;
					} else {
						course.set( {order: exists.get( 'order' )} );
					}
					var xhr = course.save();

					if ( xhr ) {
						xhr.done( function ( response, status, options ) {
							var exists = ThriveApp.globals.courses.findWhere( {name: course.get( 'name' )} );
							if ( ! exists ) {
								course.set( 'ID', response.term_id );
								course.set( 'url', response.url );
								if ( response.lessons ) {
									course.set( {lessons: new ThriveApp.collections.Lessons( response.lessons )} );
								} else {
									course.set( {lessons: new ThriveApp.collections.Lessons()} );
								}

								course.set( {lessons_count: course.get( 'lessons' ).length} );
								ThriveApp.globals.courses.add( course, {at: ThriveApp.globals.courses.length} );
							} else {
								if ( response.lessons ) {
									exists.get( 'lessons' ).add( response.lessons );
									exists.set( {lessons_count: exists.get( 'lessons' ).length} );
								}
							}
							if ( index == length ) {
								ThriveApp.router.navigate( "#dashboard", {trigger: true} );
							}
						} );
						xhr.error( function ( errorObj ) {
							TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
						} );
						xhr.always( function () {
							if ( index == length ) {
								TVE_Dash.hideLoader();
							}
						} );
					}
				}, this );
			}
		} );

		ThriveApp.views.OldCourse = ThriveApp.views.Base.extend( {
			className: 'tva-course-container',
			tagName: 'div',
			template: TVE_Dash.tpl( 'import/course' ),
			events: {
				'click .tva-course-checkbox': 'toggleChecked'
			},
			render: function () {
				this.model.set( {exists: false} );
				//check if the course already exists
				var exists = ThriveApp.globals.courses.findWhere( {name: this.model.get( 'name' )} ),
					lessons = this.model.get( 'lessons' ).findWhere( {imported: ''} );

				if ( exists && ! lessons ) {
					this.model.set( {exists: true} );
					this.model.set( {checked: false} );
				}

				this.$el.html( this.template( {model: this.model} ) );

				this.model.get( 'lessons' ).each( this.renderLessons, this );

				return this;
			},
			renderLessons: function ( lesson ) {
				var view = new ThriveApp.views.OldLesson( {
					model: lesson,
					course: this.model
				} );
				this.$( '.tva-lessons-container' ).append( view.render().$el );
			},
			toggleChecked: function ( e ) {
				var checked = $( e.currentTarget ).is( ':checked' ),
					lessons = this.model.get( 'lessons' );
				this.model.set( {checked: checked} );

				this.$( '.tva-lesson-checkbox' ).prop( 'checked', checked );

				lessons.each( function ( model ) {
					model.set( {checked: checked} );
				}, this );
			}
		} );

		ThriveApp.views.OldLesson = ThriveApp.views.Base.extend( {
			tagName: 'li',
			template: TVE_Dash.tpl( 'import/lesson' ),
			events: {
				'click .tva-lesson-checkbox': 'toggleChecked'
			},
			initialize: function ( options ) {
				this.course = options.course;
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, course: this.course} ) );

				return this
			},
			toggleChecked: function ( e ) {
				var checked = $( e.currentTarget ).is( ':checked' );

				this.model.set( {checked: checked} );

				if ( checked ) {
					this.course.set( {checked: true} );
					$( '#tva-course-' + this.course.get( 'term_id' ) ).prop( 'checked', true );
				}
			}
		} );

		/**
		 * Dashboard list view
		 */
		ThriveApp.views.Dashboard = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'dashboard' ),
			events: {
				'click .tva-add-course': 'addCourse',
				'click .tva-toggle-filters': 'toggleFilters'
			},
			courseList: '',
			render: function () {
				this.$el.html( this.template( {} ) );

				if ( this.model.get( 'is_thrivetheme' ) && this.model.get( 'apprentice' ) && ! this.model.get( 'import' ) && ! this.model.get( 'first_time' ) && this.model.get( 'has_importcourses' ) ) {
					this.modal( ThriveApp.views.ModalImportLessons, {
						model: this.model,
						'max-width': '60%',
						width: '800px'
					} );
				}
				this.renderFilters();

				$( 'body' ).on( 'click', function ( e ) {
					var container = $( '.tva-filters-wrapper' );
					var btn = $( '.tva-toggle-filters' );

					if ( ! container.is( e.target ) && container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						container.slideUp();
					}
				} );

				this.courseList = new ThriveApp.views.CourseList( {
					el: this.$el.find( '#tva-courses-list' ),
					model: this.model,
					collection: this.collection
				} );

				this.courseList.render();
				this.toggleNoCoursesText();

				TVE_Dash.hideLoader();
				return this;
			},
			addCourse: function () {
				ThriveApp.router.navigate( "#add_course", {trigger: true} );
			},
			toggleNoCoursesText: function () {

				if ( this.collection.length ) {
					this.$( '#tva-no-courses-text' ).hide();
				} else {
					this.$( '#tva-no-courses-text' ).show();
				}

				if ( this.collection.length == 1 ) {
					this.$el.find( '#tva-one-courses-text' ).show();
				} else {
					this.$el.find( '#tva-one-courses-text' ).hide();
				}
			},
			renderFilters: function () {
				var view = new ThriveApp.views.Filters( {
					el: this.$( '.tva-filters-container' ),
					collection: ThriveApp.globals.topics
				} );

				view.render();
			},
			toggleFilters: function () {
				this.$( '.tva-filters-wrapper' ).slideToggle();
			}
		} );


		ThriveApp.views.SendowlSettings = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/settings' ),
			render: function () {
				this.$el.html( this.template() );
				return this;
			}
		} );

		ThriveApp.views.SendowlQuickStart = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/quick-start' ),
			render: function () {
				var sendowl = this.model.get( 'membership_plugin' ).findWhere( {tag: 'sendowl'} ),
					access = false;

				ThriveApp.globals.courses.each( function ( course ) {
					if ( course.get( 'membership_ids' ).sendowl.length > 0 ) {
						access = true;
					}
				} );
				this.$el.html( this.template( {model: this.model, sendowl: sendowl, access: access} ) );

				return this;
			}
		} );

		ThriveApp.views.SendowlCheckout = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/checkout' ),
			events: {
				'click .tva-create-checkout-page': 'showCreatePage',
				'click .tva-create-thankyou-page': 'showCreateThankyouPage',
				'input #tva-new-sendowl-checkout-page': 'createPageInput',
				'input #tva-new-sendowl-thankyou-page': 'createThankyouPageInput',
				'click .tva-create-checkout-page-submit': function () {
					this.model.set( {checkout_changed: true} );
					this.saveCheckoutSettings();
				},
				'click .tva-create-thankyou-page-submit': 'saveCheckoutSettings',
				'click .tva-save-message': 'saveCheckoutSettings',
				'click .tva-thankyou-type': 'changeType'
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				this.initializeCheckout();
				this.initializeThankyou();
				ThriveApp.util.rebindWistiaFancyBoxes();

				return this;
			},
			initializeCheckout: function () {
				var $post_search = this.$( '#tva-sendowl-checkout-page' ),
					model = this.model,
					self = this;

				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',
					select: function ( event, ui ) {
						model.get( 'checkout_page' ).ID = parseInt( ui.item.id );
						model.get( 'checkout_page' ).name = ui.item.label;
						model.set( {checkout_changed: true} );
						self.saveCheckoutSettings();
					},
					search: function () {
						if ( model.get( 'checkout_page' ).ID ) {
							model.get( 'checkout_page' ).old_ID = model.get( 'checkout_page' ).ID;
						}
						model.get( 'checkout_page' ).ID = '';
						model.get( 'checkout_page' ).name = '';
						self.$( '#tva-new-sendowl-checkout-page' ).removeAttr( 'disabled' );
					},
					open: function () {
						if ( model.get( 'checkout_page' ).ID ) {
							model.get( 'checkout_page' ).old_ID = model.get( 'checkout_page' ).ID;
						}
						model.get( 'checkout_page' ).ID = '';
						model.get( 'checkout_page' ).name = '';
						self.$( '#tva-new-sendowl-checkout-page' ).attr( 'disabled', 'disabled' );
					},
					close: function ( event, ui ) {
						if ( ! model.get( 'checkout_page' ).ID ) {
							self.$( '#tva-new-sendowl-checkout-page' ).removeAttr( 'disabled' );
						}
					},
					fetch_single: model.get( 'checkout_page' )
				} );
			},
			initializeThankyou: function () {
//				var type = this.model.get( 'thankyou_page' ).type;
//
//				if ( ! ThriveApp.views['SendowlThankYou' + ThriveApp.util.upperFirst( type )] ) {
//					return;
//				}
//
//
//				var view = new ThriveApp.views['SendowlThankYou' + ThriveApp.util.upperFirst( type )]( {
//					el: this.$( '.tva-thankyou-wrapper' ),
//					model: this.model
//				} );
//
//				view.render();

				var $post_search = this.$( '#tva-sendowl-thankyou-page' ),
					model = this.model,
					self = this;

				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',
					select: function ( event, ui ) {
						model.get( 'thankyou_page' ).ID = parseInt( ui.item.id );
						model.get( 'thankyou_page' ).name = ui.item.label;
						self.saveCheckoutSettings();
					},
					search: function () {
						model.set( {thankyou_page: {}} );
						self.$( '#tva-new-sendowl-thankyou-page' ).removeAttr( 'disabled' );
					},
					open: function () {
						model.set( {thankyou_page: {}} );
						self.$( '#tva-new-sendowl-thankyou-page' ).attr( 'disabled', 'disabled' );
					},
					close: function ( event, ui ) {
						if ( ! model.get( 'thankyou_page' ).ID ) {
							self.$( '#tva-new-sendowl-thankyou-page' ).removeAttr( 'disabled' );
						}
					},
					fetch_single: model.get( 'thankyou_page' )
				} );

			},
			changeType: function ( e ) {
				this.model.get( 'thankyou_page' ).type = $( e.currentTarget ).attr( 'data-type' );

				this.saveCheckoutSettings();

			},
			showCreatePage: function () {
				if ( this.model.get( 'checkout_page' ).ID ) {
					this.model.get( 'checkout_page' ).old_ID = this.model.get( 'checkout_page' ).ID;
				}

				this.model.get( 'checkout_page' ).ID = '';
				this.model.get( 'checkout_page' ).name = '';
				this.$( '#tva-sendowl-checkout-page' ).val( '' );
				this.$( '#tva-create-checkout-page' ).show();
				this.$( '.tva-create-checkout-page' ).addClass( 'tvd-disabled' );
				this.$( '#tva-new-sendowl-checkout-page' ).removeAttr( 'disabled' );
			},
			showCreateThankyouPage: function () {
				this.model.set( {thankyou_page: {name: '', ID: ''}} );
				this.$( '#tva-sendowl-thankyou-page' ).val( '' );
				this.$( '#tva-create-thankyou-page' ).show();
				this.$( '.tva-create-thankyou-page' ).addClass( 'tvd-disabled' );
				this.$( '#tva-new-sendowl-thankyou-page' ).removeAttr( 'disabled' );
			},
			createPageInput: function ( e ) {
				this.model.get( 'checkout_page' ).name = e.target.value;
			},
			createThankyouPageInput: function ( e ) {
				this.model.get( 'thankyou_page' ).name = e.target.value;
			},
			saveCheckoutSettings: function ( e ) {
				if ( e && ( ( $( e.target ).hasClass( 'tva-create-checkout-page-submit' ) && ! this.model.get( 'checkout_page' ).name ) || ( $( e.target ).hasClass( 'tva-create-thankyou-page-submit' ) && ! this.model.get( 'thankyou_page' ).name ) ) ) {
					TVE_Dash.err( ThriveApp.t.PageNameError );
					return;
				}

				var data = {
					checkout_page: this.model.get( 'checkout_page' ),
					thankyou_page: this.model.get( 'thankyou_page' )
				};
				data.thankyou_page.message = this.model.get( 'thankyou_page_message' );

				var self = this;
				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/save_checkout_settings/',
					data: data
				} ).done( function ( response, status, options ) {
					self.model.get( 'checkout_page' ).ID = response.checkout.ID;
					self.model.get( 'checkout_page' ).name = response.checkout.name;
					self.model.set( {checkout_page_url: response.checkout.url} );
					self.model.set( {checkout_tcb_edit_url: response.checkout.tcb_edit_url} );
					self.model.get( 'thankyou_page' ).ID = response.thankyou.ID;
					self.model.get( 'thankyou_page' ).name = response.thankyou.name;
					self.model.get( 'thankyou_page' ).type = response.thankyou.type;
					self.model.set( {
						'thankyou_page_message': response.thankyou.message,
						thankyou_page_url: response.thankyou.url,
						thankyou_tcb_edit_url: response.thankyou.tcb_edit_url
					} );

					TVE_Dash.success( ThriveApp.t.SuccessSettings );
					self.render();
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		/**
		 * Static Options view
		 */
		ThriveApp.views.SendowlThankYouStatic = ThriveApp.views.SendowlCheckout.extend( {
			template: TVE_Dash.tpl( 'sendowl/static' ),
			events: {},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );


				var $post_search = this.$( '#tva-sendowl-thankyou-page' ),
					model = this.model,
					self = this;

				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.settings + '/search_pages/',
					type: 'POST',
					select: function ( event, ui ) {
						model.get( 'thankyou_page' ).ID = parseInt( ui.item.id );
						model.get( 'thankyou_page' ).name = ui.item.label;
						self.saveCheckoutSettings();
					},
					search: function () {
						model.set( {thankyou_page: {}} );
						self.$( '#tva-new-sendowl-thankyou-page' ).removeAttr( 'disabled' );
					},
					open: function () {
						model.set( {thankyou_page: {}} );
						self.$( '#tva-new-sendowl-thankyou-page' ).attr( 'disabled', 'disabled' );
					},
					close: function ( event, ui ) {
						if ( ! model.get( 'thankyou_page' ).ID ) {
							self.$( '#tva-new-sendowl-thankyou-page' ).removeAttr( 'disabled' );
						}
					},
					fetch_single: model.get( 'thankyou_page' )
				} );

				return this;
			}
		} );

		/**
		 * Unique option view
		 */
		ThriveApp.views.SendowlThankYouUnique = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/unique' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * Redirect option view
		 */
		ThriveApp.views.SendowlThankYouRedirect = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/redirect' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderMCE();

				return this;
			},
			renderMCE: function () {
				var self = this;
				setTimeout( function () {
					var editors = [ 'tva-thankyou-message' ];
					ThriveApp.util.clearMCEEditor( editors );
					ThriveApp.util.editorInit( 'tva-thankyou-message', self.model, 'thankyou_page_message' );
					TVE_Dash.materialize( self.$el );
				}, 0 );

			},
		} );

		ThriveApp.views.SendowlLinks = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/links' ),
			events: {
				'click .tva-product-select': 'changeProduct',
				'click .tva-bundle-select': 'changeBundle',
				'click .tva-refresh-products': 'refreshProducts',
				'click .tva-refresh-bundles': 'refreshBundles',
				'click .ttw-tpm-dropdown-container': 'toggleElement'
			},
			initialize: function () {
				var self = this;
				this.$el.on( 'click', function ( e ) {
					var $container = self.$( 'ul' ),
						$element = self.$( '.ttw-tpm-dropdown-selected' );

					if ( ! $container.is( e.target ) && $container.has( e.target ).length === 0 && ! $element.is( e.target ) ) {
						$container.slideUp();
					}
				} );
			},
			render: function () {

				this.$el.html( this.template( {model: this.model} ) );
				this.bind_zclip();
				ThriveApp.util.rebindWistiaFancyBoxes();

				return this;
			},
			toggleElement: function ( e ) {
				$( e.currentTarget ).find( 'ul' ).slideToggle();
			},
			changeProduct: function ( e ) {
				var product = this.model.get( 'sendowl_products' ).findWhere( {id: parseInt( e.target.value )} ),
					$element = this.$( '.tva-sendowl-products' );

				$( e.currentTarget ).parent().prev().html( product.get( 'name' ) );

				if ( product ) {
					var url = this.model.get_checkout_page_url( product ),
						shortcode = "[tva_sendowl_buy product=" + product.get( 'id' ) + " pp='Sendowl' title='Buy Now']",
						html = url ? '<a href="' + url + '">Buy now</a>' : '';

					$element.find( '.tva-product-url' ).val( url );
					$element.find( '.tva-product-shortcode' ).val( shortcode );
					$element.find( '.tva-product-html' ).val( html );
				}
			},
			refreshProducts: function () {

				var self = this;

				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'GET',
					url: ThriveApp.routes.settings + '/refresh_sendowl_products/'
				} ).done( function ( response, status, options ) {
					if ( response.error ) {
						TVE_Dash.err( response.error, 5000 );

						return;
					}
					self.model.set( {sendowl_products: new ThriveApp.collections.Base( response )} );
					var model = ThriveApp.globals.settings.get( 'membership_plugin' ).findWhere( {tag: 'sendowl'} );

					if ( model ) {
						model.set( {membership_levels: new ThriveApp.collections.MembermouseCollection( response )} )
					}

					_.each( ThriveApp.data.settings.membership_plugin, function ( value, index ) {
						if ( value.tag === 'sendowl' ) {
							var data = ThriveApp.data.settings.membership_plugin[ index ];
							data.membership_levels = response
						}
					} );

					self.render();
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			applyTo: function ( $element, product ) {

			},

			refreshBundles: function () {
				var self = this;

				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'GET',
					url: ThriveApp.routes.settings + '/refresh_sendowl_bundles/'
				} ).done( function ( response, status, options ) {

					if ( response.error ) {
						TVE_Dash.err( response.error, 5000 );

						return;
					}

					self.model.set( {sendowl_bundles: new ThriveApp.collections.Base( response )} );

					var model = ThriveApp.globals.settings.get( 'membership_plugin' ).findWhere( {tag: 'sendowl'} );

					if ( model ) {
						model.set( {bundles: new ThriveApp.collections.MembermouseCollection( response )} )
					}

					_.each( ThriveApp.data.settings.membership_plugin, function ( value, index ) {
						if ( value.tag === 'sendowl' ) {
							var data = ThriveApp.data.settings.membership_plugin[ index ];
							data.bundles = response;
						}
					} );

					self.render();
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			changeBundle: function ( e ) {
				var bundle = this.model.get( 'sendowl_bundles' ).findWhere( {id: parseInt( e.target.value )} ),
					$element = this.$( '.tva-sendowl-bundles' );

				$( e.currentTarget ).parent().prev().html( bundle.get( 'name' ) );

				if ( bundle ) {
					var url = this.model.get_checkout_page_url( bundle, 'bundles' ),
						shortcode = "[tva_sendowl_buy bundle=" + bundle.get( 'id' ) + " pp='Sendowl' title='Buy Now']",
						html = url ? '<a href="' + url + '">Buy now</a>' : '';

					$element.find( '.tva-product-url' ).val( url );
					$element.find( '.tva-product-shortcode' ).val( shortcode );
					$element.find( '.tva-product-html' ).val( html );
				}
			}
		} );

		ThriveApp.views.SendowlIntegration = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/integration' ),
			events: {
				'input #tva-account-key': 'setKey',
				'input #tva-account-secret': 'setSecret',
				'click .tva-save-keys': 'saveKeys'
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.bind_zclip();
				ThriveApp.util.rebindWistiaFancyBoxes();

				return this;
			},
			setKey: function ( e ) {
				this.model.get( 'account_keys' ).key = e.target.value;

				if ( this.model.get( 'account_keys' ).key && this.model.get( 'account_keys' ).secret ) {
					this.$( '.tva-save-keys' ).removeClass( 'tvd-disabled' );
				} else {
					this.$( '.tva-save-keys' ).addClass( 'tvd-disabled' );
				}
			},
			setSecret: function ( e ) {
				this.model.get( 'account_keys' ).secret = e.target.value;

				if ( this.model.get( 'account_keys' ).key && this.model.get( 'account_keys' ).secret ) {
					this.$( '.tva-save-keys' ).removeClass( 'tvd-disabled' );
				} else {
					this.$( '.tva-save-keys' ).addClass( 'tvd-disabled' );
				}
			},
			saveKeys: function () {
				TVE_Dash.showLoader();
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					data: this.model.get( 'account_keys' ),
					url: ThriveApp.routes.settings + '/set_account_keys/'
				} ).done( function ( response, status, options ) {
					TVE_Dash.success( ThriveApp.t.accountKeysSaved );
					self.model.set( response );
					self.render();
				} ).error( function ( errorObj ) {
					self.model.set( JSON.parse( errorObj.responseText ).message );
					self.render();
					TVE_Dash.err( ThriveApp.t.accountKeysNotSaved );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		/**
		 * Course list view
		 */
		ThriveApp.views.CourseList = ThriveApp.views.Base.extend( {
			initialize: function () {
				var self = this;
				this.listenTo( this.collection, 'add', function () {
					self.collection.sort();
					self.render();
				} );
				this.listenTo( ThriveApp.globals.settings, 'change:comment_status', this.setNewCommentStatus );

				this.collection.on( 'tva_filter_courses', function () {
					TVE_Dash.showLoader();
					self.itemViews = [];
					self.$( '.tva-course-item' ).remove();

					var topics = ThriveApp.globals.topics.where( {checked: 1} );

					if ( topics.length == 0 ) {
						self.collection.each( function ( model ) {
							model.set( {display: 0} );
						}, this );
					} else if ( topics.length == ThriveApp.globals.topics.length ) {
						self.collection.each( function ( model ) {
							model.set( {display: 1} );
						}, this );
					} else {
						self.collection.each( function ( model ) {
							model.set( {display: 0} );
						}, this );

						_.each( topics, function ( topic ) {
							var id = topic.get( 'ID' );
							self.collection.each( function ( model ) {
								if ( model.get( 'topic' ) == id ) {
									model.set( {display: 1} );
								}

							}, this );
						}, this );
					}

					self.render();

					TVE_Dash.hideLoader();
				} );
				this.collection.on( 'tva_update_courses', function () {
					self.updateOrder();
				} )
			},
			setNewCommentStatus: function () {
				this.collection.each( this.setCourseCommentStatus, this );
			},
			setCourseCommentStatus: function ( course ) {
				course.set( {comment_status: ThriveApp.globals.settings.get( 'comment_status' )} );
			},
			/**
			 * @param {Array} used for sortable
			 */
			itemViews: [],
			render: function () {
				/**
				 * remove the views if they're already created so we don't have duplicate events
				 */
				var self = this,
					total = this.collection.length;

				this.collection.each( this.renderOne, this );

				function show_position( event, ui ) {
					var $placeholder = $( ui.placeholder ),
						position = total - $placeholder.prevAll().not( ui.item ).length;

					$placeholder.html( "<div class=tva-inside-placeholder'><span>" + position + (
						ThriveApp.t.n_suffix[ position ] ? ThriveApp.t.n_suffix[ position ] : ThriveApp.t.n_th
					) + ' ' + ThriveApp.t.position + "</span></div>" );
				}

				this.$el.sortable( {
					placeholder: 'tvd-card-new tva-course-ui-sortable-placeholder tvd-col tvd-s6 tvd-ms6 tvd-m4 tvd-l3',
					items: '.tva-course-item',
					forcePlaceholderSize: true,
					handle: '.tva-drag-card',
					update: _.bind( self.updateOrder, this ),
					tolerance: 'pointer',
					change: show_position,
					start: function ( event, ui ) {
						show_position( event, ui );
						$( 'body' ).addClass( 'tva-sorting' );
					},
					stop: function () {
						setTimeout( function () {
							$( 'body' ).removeClass( 'tva-sorting' );
						}, 200 );
					}
				} );

				$( '#collapse-button' ).on( 'click', function () {
					self.recalcWidth();
				} );

				if ( ! this.ribbon ) {
					this.renderRibbon();
				}

				return this;
			},
			renderOne: function ( course ) {
				if ( course.get( 'display' ) ) {
					var $lastItem = this.$el.find( '.tva-course-item' ).last(),
						view = new ThriveApp.views.Course( {
							model: course,
							collection: this.collection
						} );

					if ( $lastItem.length ) {
						$lastItem.after( view.render().$el );
					} else {
						this.$el.prepend( view.render().$el );
					}

					this.itemViews.push( view );
				}


				return this;
			},
			renderRibbon: function () {

				if ( this.model.get( 'is_thrivetheme' ) && this.model.get( 'apprentice' ) && this.collection.where( {status: 'publish'} ).length > 0 && ! this.model.get( 'apprentice_ribbon' ) ) {
					this.ribbon = new ThriveApp.views.Ribbon( {
						model: this.model
					} );

					this.$el.append( this.ribbon.render().$el );
					this.itemViews.push( this.ribbon );

					this.recalcWidth();
				}

			},
			recalcWidth: function () {
				var bar_width = $( '#adminmenuback' ).outerWidth();
				var body_width = $( 'body' ).outerWidth();

				// this.$( '.tva-ribbon-container' ).css( {'width': body_width - bar_width + 'px', 'left': bar_width + 'px'} );
			},
			updateOrder: function () {
				var to_update = {},
					$items = this.$( '.tva-course-item' ).reverse(),
					self = this;

				_.each( $items, function ( item, index ) {
					var id = self.$( item ).attr( 'data-id' ),
						model = self.collection.findWhere( {ID: parseInt( id )} );

					if ( model ) {
						model.set( {order: index} )
					}

					to_update[ id ] = index;
				} );

				if ( this.collection.length !== Object.keys( to_update ).length ) {
					TVE_Dash.hideLoader();
					return;
				}

				this.collection.sort();
				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.courses + '/update_order/',
					data: {
						new_order: to_update
					}
				} ).done( function ( response, status, options ) {
					ThriveApp.globals.available_settings.fetch().success( function ( response, status, options ) {
						ThriveApp.globals.settings.set( {
							iframe_url: ThriveApp.globals.settings.get( 'preview_url' ),
							preview_url: response.preview_url
						} );
					} );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		/**
		 * Ribbon for raking the decission to disable the apprentice
		 */
		ThriveApp.views.Ribbon = ThriveApp.views.Base.extend( {
			className: 'tva-ribbon-container',
			template: TVE_Dash.tpl( 'course/ribbon' ),
			events: {
				'click .tva-close-ribbon': 'closeRibbon',
				'click .tva-disable-apprentice': 'disableApprentice'
			},
			render: function () {
				this.$el.html( this.template( {} ) );

				return this;
			},
			disableApprentice: function () {
				TVE_Dash.showLoader();
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/disable_apprentice/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.model.set( {apprentice: false} );
						TVE_Dash.success( ThriveApp.t.SuccessDisabledApprentice );
						self.$el.remove();
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( errorObj.responseText );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			closeRibbon: function () {
				var self = this;
				this.$el.remove();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					url: ThriveApp.routes.settings + '/disable_apprentice_ribbon/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.model.set( {apprentice_ribbon: true} );
						self.$el.remove();
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( errorObj.responseText );
				} ).always( function () {
				} );
			}

		} );

		/**
		 * Filters wrapper view
		 */
		ThriveApp.views.Filters = ThriveApp.views.Base.extend( {

			render: function () {
				this.collection.each( this.renderOne, this );
				this.renderSelect();
			},
			renderOne: function ( filter ) {
				var view = new ThriveApp.views.Filter( {
					tagName: 'li',
					model: filter,
					collection: this.collection
				} );

				this.$el.append( view.render().$el )
			},
			renderSelect: function () {
				var model = new ThriveApp.models.Topic();
				model.set( {
					ID: 'none',
					title: 'Select All',
					checked: 1
				} );

				var select = new ThriveApp.views.Filter( {
					tagName: 'li',
					model: model,
					collection: this.collection
				} );

				this.$el.append( select.render().$el )
			}
		} );

		/**
		 * Single filter view
		 */
		ThriveApp.views.Filter = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'filters/item' ),
			events: {
				'click .tva-filter-topic': 'filterTopics'
			},
			checked: true,
			render: function () {
				this.$el.prepend( this.template( {model: this.model} ) );

				return this;
			},
			filterTopics: function ( e ) {
				var id = this.model.get( 'ID' );
				this.checked = $( e.currentTarget ).is( ':checked' ) ? 1 : 0;

				if ( id == 'none' && $( e.currentTarget ).is( ':checked' ) ) {
					$( '.tva-filter-topic' ).prop( 'checked', true );
					this.collection.each( this.toggleChecked, this );
				} else if ( id == 'none' && ! $( e.currentTarget ).is( ':checked' ) ) {
					$( '.tva-filter-topic' ).prop( 'checked', false );
					this.collection.each( this.toggleChecked, this );
				} else {
					this.toggleChecked( this.model );
					var count = this.collection.where( {checked: 1} ).length,
						check = count == this.collection.length;
					$( '#tva-filter-topic-none' ).prop( 'checked', check );
				}

				ThriveApp.globals.courses.trigger( 'tva_filter_courses' );
			},
			toggleChecked: function ( model ) {
				model.set( {checked: this.checked} );
			}
		} );

		/**
		 * View for Cover Image Normal State (with image)
		 */
		ThriveApp.views.CoverNormalState = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/cover-normal' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				return this;
			}
		} );

		/**
		 * View for Cover Image Empty State (without image)
		 */
		ThriveApp.views.CoverEmptyState = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/cover-empty' ),
			render: function () {
				this.$el.html( this.template( {} ) );
				return this;
			}
		} );

		/**
		 * View for Lesson Cover Image Normal State (with image)
		 */
		ThriveApp.views.LessonCoverNormalState = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/lesson-cover-normal' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				return this;
			}
		} );

		/**
		 * View for Lesson Cover Image Empty State (without image)
		 */
		ThriveApp.views.LessonCoverEmptyState = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/lesson-cover-empty' ),
			render: function () {
				this.$el.html( this.template( {} ) );

				return this;
			}
		} );

		/**
		 * Topic View
		 */
		ThriveApp.views.Topic = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/course-topic' ),
			initialize: function ( options ) {
				this.selected = options.selected;
			},
			render: function () {
				this.$el.append( this.template( {model: this.model, selected: this.selected} ) );
				return this;
			}
		} );

		/**
		 * Render the controls for the levels
		 */
		ThriveApp.views.LevelControls = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/level-controls' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * Render the controls for the labels
		 */
		ThriveApp.views.LabelsControls = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/label-controls' ),
			render: function () {
				if ( ! this.model.get( 'label_name' ) ) {
					this.model.set( {label_name: ThriveApp.t.DefaultMembersOnly} );
				}
				if ( ! this.model.get( 'label_color' ) ) {
					this.model.set( {label_color: '#58a545'} );
				}

				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * Level view
		 */
		ThriveApp.views.Level = ThriveApp.views.Base.extend( {
			className: 'tva-course-level-item tvd-pointer',
			template: TVE_Dash.tpl( 'level-item' ),
			events: {
				'click .tva-course-level-delete': 'deleteLevel'
			},
			initialize: function ( options ) {
				this.selected = options.selected;
				this.$el.attr( 'data-id', this.model.get( 'ID' ) );
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, selected: this.selected} ) );
				return this;
			},
			deleteLevel: function ( e ) {
				e.preventDefault();
				e.stopPropagation();
				TVE_Dash.showLoader();

				var $element = this.$( '.tva-course-level-delete' ),
					id = $element.attr( 'data-id' ),
					model = ThriveApp.globals.levels.findWhere( {ID: parseInt( id )} );

				var xhr = model.destroy();
				if ( xhr ) {
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Label view
		 */
		ThriveApp.views.Label = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'label-item' ),
			events: {
				'click .tva-course-label-delete': 'deleteLabel'
			},
			initialize: function ( options ) {
				this.selected = options.selected;
			},
			render: function () {
				var self = this,
					color = this.model.get( 'color' );
				this.$el.html( this.template( {model: this.model, selected: this.selected} ) );

				this.$el.find( '.tva-label-color-picker' ).spectrum( {
					containerClassName: 'tva-color-picker',
					allowEmpty: false,
					showInitial: true,
					showButtons: true,
					chooseText: "Apply",
					cancelText: "Cancel",
					showInput: true,
					disabled: true,
					preferredFormat: "hex",
					hide: function ( color ) {
						var _color = color ? color.toHexString() : '';

						self.model.set( {label_color: _color} );
						self.saveModel();
					}
				} );

				return this;
			},
			deleteLabel: function ( e ) {
				e.preventDefault();
				e.stopPropagation();
				TVE_Dash.showLoader();

				var $element = this.$( '.tva-course-label-delete' ),
					id = $element.attr( 'data-id' ),
					model = ThriveApp.globals.labels.findWhere( {ID: parseInt( id )} );

				var xhr = model.destroy();
				if ( xhr ) {
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Role view
		 */
		ThriveApp.views.Role = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/course-role' ),
			initialize: function ( options ) {
				this.checked = options.checked;
			},
			render: function () {
				if ( this.model.get( 'ID' ) !== 'administrator' ) {
					this.$el.append( this.template( {model: this.model, checked: this.checked} ) );
				}
				return this;
			}
		} );
		/**
		 * View for Adding / Removing / Editing topics
		 */
		ThriveApp.views.TopicsManagement = ThriveApp.views.Base.extend( {
			className: 'tva-topics-container',
			template: TVE_Dash.tpl( 'topics' ),
			_type: 'topic',
			events: {
				'click .tva-add-topic': 'addItem',
				'click .tva-delete-topic': 'removeItem'
			},
			initialize: function () {
				this.listenTo( this.collection, 'add', this.render );
				this.listenTo( this.collection, 'remove', this.render );
			},
			render: function () {
				this.$el.html( this.template( {} ) );
				this.renderHeader();
				this.renderItems();
				return this;
			},
			renderHeader: function () {
				var $el = this.$el.find( '#tva-topics-collection' );
				$el.empty();
				this.header = new ThriveApp.views[ ThriveApp.util.upperFirst( this._type ) + 'sHeader' ]( {
					el: $el
				} );
				this.header.render();
			},
			renderItems: function () {
				this.collection.each( this.renderOne, this );
			},
			renderOne: function ( item ) {
				this.topic = new ThriveApp.views[ ThriveApp.util.upperFirst( this._type ) + 'sItem' ]( {
					model: item
				} );

				this.$el.find( '#tva-' + this._type + 's-collection' ).append( this.topic.render().$el );
			},
			addItem: function () {
				TVE_Dash.showLoader();
				var new_model = new ThriveApp.models[ ThriveApp.util.upperFirst( this._type ) ](),
					collection = this.collection,
					xhr = new_model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {

						new_model.set( 'ID', response.ID );
						collection.add( new_model );
						self.$el.find( '.tva-edit-' + self._type + '-name' ).first().trigger( 'click' );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			},
			removeItem: function ( e ) {
				var $element = $( e.currentTarget ),
					id = $element.attr( 'data-id' ),
					model = this.collection.findWhere( {ID: parseInt( id )} );

				this.handleRemoveTopic( model );
			},
			handleRemoveTopic: function ( model ) {
				if ( model ) {
					this.modal( ThriveApp.views[ 'Confirm' + ThriveApp.util.upperFirst( this._type ) + 'Delete' ], {
						model: model,
						'max-width': '60%',
						width: '800px',
						in_duration: 200,
						out_duration: 0
					} );
				}
			}

		} );

		/**
		 * Label management view [ add/edit/delete labels ]
		 */
		ThriveApp.views.LabelManagement = ThriveApp.views.TopicsManagement.extend( {
			className: 'tva-topics-container tva-labels-container',
			template: TVE_Dash.tpl( 'labels' ),
			_type: 'label',
			events: {
				'click .tva-add-label': 'addItem',
				'click .tva-delete-label': 'removeItem'
			}
		} );

		/**
		 * Render the header for the topics
		 */
		ThriveApp.views.TopicsHeader = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'topics-header' ),
			render: function () {
				this.$el.empty().html( this.template( {} ) );
				return this;
			}
		} );

		/**
		 * Render the header for the labels
		 */
		ThriveApp.views.LabelsHeader = ThriveApp.views.TopicsHeader.extend( {
			template: TVE_Dash.tpl( 'labels-header' )
		} );

		/**
		 * View for one topic
		 */
		ThriveApp.views.TopicsItem = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'topic' ),
			tagName: 'div',
			className: 'tva-topic-item',
			_type: 'topic',
			$title: null,
			events: {
				'click .tva-edit-topic-name': 'editItem',
				'click .tva-reset-icon': 'resetIcon',
				'click .tva-change-icon': 'changeIcon'
			},
			render: function () {
				this.renderItem();

				return this;
			},
			renderItem: function () {
				var self = this;
				this.$el.html( this.template( {item: this.model} ) );
				this.$title = this.$el.find( '.tva-' + this._type + '-title' );

				this.$el.find( '.tva-' + this._type + '-color' ).spectrum( {
					containerClassName: 'tva-color-picker',
					allowEmpty: false,
					showInitial: true,
					showButtons: true,
					chooseText: "Apply",
					cancelText: "Cancel",
					showInput: true,
					preferredFormat: "hex",
					hide: function ( color ) {
						self.onSpectrumHide( color )
					}
				} );

				return this;
			},
			editItem: function ( e ) {
				var self = this,
					edit_btn = this.$el.find( '.tva-edit-' + this._type + '-name' ),
					edit_model = new Backbone.Model( {
						title: this.model.get( 'title' ),
						required: true
					} );
				edit_btn.hide();
				edit_model.on( 'change:value', function () {
					self.saveTitle.apply( self, arguments );
					self.$title.show();

					if ( self.$el.find( '.tva-inline-edit' ) ) {
						self.$el.find( '.tva-inline-edit' ).hide();
					}
					edit_btn.show();
				} );
				edit_model.on( 'tvu_no_change', function () {
					self.$title.html( self.model.get( 'title' ) ).show();
					self.$el.find( '.tva-inline-edit' ).remove();
					edit_btn.show();
				} );

				var textEdit = new ThriveApp.views.InputTitle( {
					model: edit_model,
					el: this.$el.find( '.tva-title-container' ),
					tagName: 'div'
				} );

				this.$title.hide();
				textEdit.render();
				textEdit.focus();
			},
			/**
			 * Saves the new title and hides the input value
			 */
			saveTitle: function ( edit_model, new_value ) {
				TVE_Dash.showLoader();
				var self = this;

				this.model.set( {
					title: new_value
				} );
				this.$title.html( new_value );
				var xhr = this.model.save();
				if ( xhr ) {
					xhr.always( function () {
						self.$title.html( new_value );
						TVE_Dash.hideLoader();
					} );
				}
			},
			saveModel: function () {
				TVE_Dash.showLoader();

				var xhr = this.model.save();
				if ( xhr ) {
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			},
			resetIcon: function () {
				this.model.set( {icon: ''} );
				this.saveModel();
				this.render();
			},
			changeIcon: function () {
				var frame,
					self = this;


				// If the media frame already exists, reopen it.
				if ( frame ) {
					frame.open();
					return;
				}

				// Create a new media frame
				frame = wp.media( {
					title: 'Select or Upload an Icon',
					button: {
						text: 'Use this Icon'
					},
					library: {type: 'image'},
					multiple: false  // Set to true to allow multiple files to be selected
				} );


				// When an image is selected in the media frame...
				frame.on( 'select', function () {
					var attachment = frame.state().get( 'selection' ).first().toJSON();
					self.model.set( {icon: attachment.url} );
					self.saveModel();
					self.render();
				} );

				// Finally, open the modal on click
				frame.open();
			},
			onSpectrumHide: function ( color ) {
				var _color = color ? color.toHexString() : '';
				this.model.set( {color: _color} );
				this.saveModel();

				return this;
			}
		} );

		ThriveApp.views.LabelsItem = ThriveApp.views.TopicsItem.extend( {
			template: TVE_Dash.tpl( 'label' ),
			className: 'tva-label-item',
			_type: 'label',
			$title: null,
			events: {
				'click .tva-edit-label-name': 'editItem'
			},
			onSpectrumHide: function ( color ) {
				var _color = color ? color.toHexString() : '',
					courses = ThriveApp.globals.courses.where( {label: parseInt( this.model.get( 'ID' ) )} );


				this.model.set( {
					color: _color,
					label_color: _color
				} );

				_.each( courses, function ( course ) {
					course.set( {
						label_name: course.getLabelName(),
						label_color: course.getLabelColor()
					} );
				}, this );

				this.saveModel();
				return this;
			}
		} );

		/**
		 * Input View
		 */
		ThriveApp.views.InputTitle = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'textedit' ),
			method: 'append',
			events: {
				'keyup input': 'keyup',
				'click input': function ( e ) {
					e.stopPropagation();
				},
				'change input': function ( e ) {
					if ( ! $.trim( this.input.val() ) ) {
						this.input.addClass( 'tvd-invalid' );
						return false;
					}
					this.model.set( 'value', this.input.val() );
					return false;
				},
				'blur input': function () {
					this.model.trigger( 'tvu_no_change' );
				}
			},
			initialize: function ( options ) {
				this.method = options.method ? options.method : this.method;
			},
			keyup: function ( event ) {
				if ( event.which === 27 ) {
					this.model.trigger( 'tvu_no_change' );
				}
			},
			render: function () {
				this.$el[ this.method ]( this.template( {item: this.model} ) );
				this.input = this.$el.find( 'input' );

				return this;
			},
			focus: function () {
				this.input.focus().select();
			}
		} );

		ThriveApp.views.RenderBulkActions = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/bulk-actions' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * Course Details view
		 */
		ThriveApp.views.CourseDetails = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/manage/course-details' ),
			render: function () {
				var children = this.model.hasChildren(),
					can_publish = false;

				if ( children ) {
					var pb_ch = this.model.get( children ).findWhere( {post_status: 'publish'} );
					can_publish = ! ! pb_ch;
				}

				this.$el.html( this.template( {model: this.model, can_publish: can_publish} ) );

				return this;
			}
		} );

		ThriveApp.views.CourseOption = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/course-option' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.LessonVideoYoutubeOptions = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/youtube-extra-options' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.CourseVideoYoutubeOptions = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/youtube-extra-options' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * Base view for media select options
		 */
		ThriveApp.views.MediaSelectView = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'media-select' ),
			events: {
				'click .tva-media-option': 'changeMediaType',
				'click .tva-selected-media': 'toggleMediaOptions',
				'input #tva-media-url': 'setMediaUrl',
				'input #tva-course-custom': 'setMediaUrl',
				'click .tva-extra-checkbox': 'changeCheckbox'
			},
			initialize: function ( options ) {
				/**
				 * the source may be either a lesson model, or the course model itself
				 */
				this.source = options.source;
				this.listenTo( this.model, 'change', this.setInSource );
			},
			render: function () {
				this.$el.html( this.template( {} ) );
				this.renderSingleMediaOption();

				this.renderMediaInput();
				this.renderExtraOptions();
				TVE_Dash.materialize( this.$el );

				$( 'body' ).on( 'click', function ( e ) {
					var container = $( '.tva-media-options' ),
						btn = $( '.tva-selected-media' );

					if ( ! container.is( e.target ) && container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						container.slideUp();
					}
				} );

				return this;
			},
			renderSingleMediaOption: function () {
				var view = new ThriveApp.views.SingleMediaOption( {
					el: this.$( '.tva-media-select-holder' ),
					model: this.model
				} );

				view.render();

				return this;
			},
			renderMediaInput: function () {
				var view = new ThriveApp.views.MediaInputField( {
					className: 'tva-media-url-container',
					model: this.model
				} );

				this.$el.append( view.render().$el );

				return this;
			},
			toggleMediaOptions: function () {
				this.$( '.tva-media-options' ).slideToggle();

				return this;
			},
			setMediaUrl: function ( e ) {
				var value = e.currentTarget.value;
				this.model.set( {media_url: value} );
			},
			changeMediaType: function ( e ) {
				var value = e.currentTarget.getAttribute( 'data-option' );

				this.model.set( {
					media_type: value,
					media_url: '',
					media_extra_options: {}
				} );

				this.render();

				return this;
			},
			renderExtraOptions: function () {
				var view = new ThriveApp.views.BaseExtraOptions( {
					className: 'tva-media-extra-options',
					model: this.model
				} );

				this.$el.append( view.render().$el );
				return this;
			},
			changeCheckbox: function ( e ) {
				var extra = this.model.get( 'media_extra_options' ),
					name = this.$( e.currentTarget ).attr( 'name' );

				if ( this.$( e.currentTarget ).prop( 'checked' ) ) {
					extra[ name ] = 1;
				} else {
					delete extra[ name ];
				}

				this.model.set( {media_extra_options: extra} );
				this.setInSource();
			},
			setInSource: function () {
				var json = this.model.toJSON();
				delete json.media_options;

				this.source.set( {post_media: json} );

			}
		} );

		ThriveApp.views.SingleMediaOption = ThriveApp.views.Base.extend( {
			render: function () {
				this.renderSelectedMedia();
				this.renderMediaOptions();

				return this;
			},
			renderSelectedMedia: function () {
				var view = new ThriveApp.views.SelectedMedia( {
					model: this.model
				} );

				this.$el.prepend( view.render().$el );

				return this;
			},
			renderMediaOptions: function () {
				var view = new ThriveApp.views.MediaOptionsList( {
					model: this.model
				} );

				this.$el.append( view.render().$el );

				return this;
			}
		} );

		/**
		 * View to render a single media option
		 */
		ThriveApp.views.MediaOption = ThriveApp.views.Base.extend( {
			option: null,
			initialize: function ( options ) {
				this.option = options.option;
			},
			render: function () {
				this.$el.attr( 'data-option', this.option );
				this.$el.html( ThriveApp.util.upperFirst( this.option ) );

				return this;
			}
		} );

		ThriveApp.views.SelectedMedia = ThriveApp.views.Base.extend( {
			tagName: 'div',
			className: 'tva-selected-media',
			render: function () {
				this.$el.append( this.model.get( 'media_options' )[ this.model.get( 'media_type' ) ].name );

				return this;
			}
		} );

		ThriveApp.views.MediaOptionsList = ThriveApp.views.Base.extend( {
			className: 'tva-media-options',
			tagName: 'ul',
			render: function () {
				var self = this;
				_.each( this.model.get( 'media_options' ), function ( value, option ) {
					self.renderOne( option );
				} );

				return this;
			},
			renderOne: function ( option ) {
				var view = new ThriveApp.views.MediaOption( {
					className: 'tva-media-option',
					tagName: 'li',
					model: this.model,
					option: option
				} );

				this.$el.append( view.render().$el );

				return this
			}
		} );

		/**
		 * Render Input field for media options
		 */
		ThriveApp.views.MediaInputField = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/media-input-field' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this
			}
		} );
		/**
		 * Base view to render extra options for item media
		 */
		ThriveApp.views.BaseExtraOptions = ThriveApp.views.Base.extend( {
			extra_view: null,
			render: function () {
				this.renderMediaExtraOptions();
				return this;
			},
			renderMediaExtraOptions: function () {
				var media_type = this.model.get( 'media_type' );
				if ( ! ThriveApp.views[ 'CourseVideo' + ThriveApp.util.upperFirst( media_type ) + 'Options' ] ) {
					return;
				}

				if ( ! this.extra_view ) {
					var view = ThriveApp.views[ 'CourseVideo' + ThriveApp.util.upperFirst( media_type ) + 'Options' ];
					this.extra_view = new view( {model: this.model} );
				}

				this.$el.append( this.extra_view.render().$el );
				return this;
			}
		} );

		/**
		 * View to render Youtube extra options
		 */
		ThriveApp.views.CourseVideoYoutubeOptions = ThriveApp.views.BaseExtraOptions.extend( {
			template: TVE_Dash.tpl( 'youtube-extra-options' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.ModuleContentOptions = ThriveApp.views.Base.extend( {
			className: 'tva-item-content-options',
			render: function () {
				var type = this.model.get( 'post_type' ) ? this.model.get( 'post_type' ).split( '_' )[ 1 ] : 'course';

				if ( ( this.model.get( 'lessons' ).length === 0 ) && ( this.model.get( 'chapters' ).length === 0 ) ) {
					this.addLesson( type );

				} else {
					if ( this.model.get( 'lessons' ).length > 0 ) {
						this.addLesson( type );
					}
				}

				return this;
			},
			addLesson: function ( type ) {
				var view = new ThriveApp.views.AddLesson( {
					className: 'tva-add-' + type + '-lesson'
				} );

				this.$el.append( view.render().$el );

				return this;
			},
			addChapter: function () {
				var view = new ThriveApp.views.AddChapter();
				this.$el.append( view.render().$el );

				return this
			}
		} );

		ThriveApp.views.ChapterContentOptions = ThriveApp.views.ModuleContentOptions.extend( {
			render: function () {
				this.addLesson( 'chapter' );

				return this;
			}
		} );

		ThriveApp.views.CourseContentOptions = ThriveApp.views.ModuleContentOptions.extend( {
			render: function () {
				this.addLesson( 'c' );

				return this;
			}
		} );

		ThriveApp.views.CourseOption = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/course-option' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this;
			}
		} );

		/**
		 * View for adding a lesson in modal steps
		 */
		ThriveApp.views.LessonEditor = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/lesson/lesson-editor' ),
			events: {
				'input #tva-item-title': 'setTitle',
				'input #tva-item-description': 'setDescription',
				'change .tva-lesson-type': 'renderLessonType',
				'click .tva-selected-lesson-type': 'showLessonType',
				'click .tva-lesson-type': 'setLessonType',
				'click .tva-item-cover-image': 'addCoverImage',
				'click .tva-option-type': 'setCommentStatus',
				'click .tva-selected-option': 'toggleOptions',
				'click .tvd-modal-submit': 'save'
			},
			lessonType: null,
			footerView: null,
			initialize: function ( options ) {
				this.initial_comment_status = this.model.get( 'comment_status' );
				this.modal_view = options.modal_view;
				this.course = options.course;
				this.listenTo( this.model, 'change:cover_image', this.renderCoverImage );
				this.footerView = options.footerView ? options.footerView : 'StepModalFooter'
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderLessonTypesList();
				this.renderLessonType();
				this.renderComments();
				ThriveApp.util.renderMCE( 'tva-item-description', this.model, 'post_excerpt' );
				this.renderFooter();

				var self = this;

				this.$el.on( 'click', function ( e ) {
					var container = self.$( '.tva-lessons-types-list' ),
						btn = self.$( '.tva-selected-lesson-type > span' );

					if ( ! container.is( e.target ) && container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						container.slideUp();
					}
				} );

				return this;
			},
			renderLessonTypesList: function () {
				var view = new ThriveApp.views.LessonTypeOption( {
					el: this.$( '.tva-lesson-type-container' ),
					model: this.model
				} );

				view.render();

				return this;
			},
			renderComments: function () {
				var view = new ThriveApp.views.LessonComments( {
					el: this.$( '.tva-comments-container' ),
					model: this.model
				} );

				view.render();

				return this;
			},
			setTitle: function ( e ) {
				var value = e.currentTarget.value;
				this.model.set( {post_title: value} );
			},
			setDescription: function ( e ) {
				var value = e.currentTarget.value;
				this.model.set( {post_excerpt: value} );
			},
			showLessonType: function () {
				this.$( '.tva-lessons-types-list' ).slideToggle();
			},
			toggleOptions: function () {
				this.$( '.tva-available-options' ).slideToggle();
			},
			setLessonType: function ( e ) {
				var type = e.currentTarget.getAttribute( 'data-type' ),
					media = '';
				this.model.set( {
					lesson_type: type
				} );

				if ( type === 'audio' ) {
					media = 'soundcloud';
				} else if ( type === 'video' ) {
					media = 'youtube';
				} else {
					media = '';
				}

				this.model.get( 'post_media' ).media_type = media;
				this.model.get( 'post_media' ).media_url = '';

				this.renderLessonTypesList();
				this.renderLessonType();

				return this;
			},
			setCommentStatus: function ( e ) {
				var comment_status = e.currentTarget.getAttribute( 'data-status' );

				this.model.set( {comment_status: comment_status} );
				this.renderComments();
			},
			renderLessonType: function ( e ) {
				if ( this.lessonType ) {
					this.lessonType.remove();
				}

				this.lessonType = new ThriveApp.views.BaseModalLesson( {
					model: this.model,
					collection: this.collection
				} );

				this.$( '.tva-editor-extra-option' ).append( this.lessonType.render().$el );

				return this
			},
			addCoverImage: function () {
				ThriveApp.util.addCoverImage( this.model );
				return this;

			},
			renderFooter: function () {
				if ( ! ThriveApp.views[ this.footerView ] ) {
					return;
				}

				var view = new ThriveApp.views[ this.footerView ]( {} );

				this.$el.append( view.render().$el );

				return this;
			},
			save: function () {
				this.tvd_clear_errors();

				if ( ! this.model.isValid() ) {
					return this.tvd_show_errors( this.model );
				}

				if ( this.initial_comment_status !== this.model.get( 'comment_status' ) ) {
					this.model.set( {comment_status_changed: true} );
				}

				TVE_Dash.showLoader();
				$( '#tvd-modal-base' ).scrollTop( 0 );
				$( '.tvd-modal-preloader' ).css( 'top', 50 + '%' );

				if ( ! this.model.get( 'order' ) && this.model.get( 'order' ) !== 0 ) {
					this.model.set( {order: this.collection.length} );
				}

				/**
				 * Publish the element if any of it's children are published
				 */
				var children = this.model.hasChildren();

				if ( children ) {
					var published = this.model.get( children ).findWhere( {post_status: 'publish'} );

					if ( published ) {
						this.model.set( {post_status: 'publish'} );
					}
				}

				this.beforeSave();

				var extra_options = this.model.get( 'post_media' ),
					obj = {};

				if ( typeof extra_options === 'object' && typeof extra_options[ 'media_extra_options' ] === 'object' ) {
					Object.keys( extra_options[ 'media_extra_options' ] ).forEach( function ( key ) {
						obj[ key ] = extra_options[ 'media_extra_options' ][ key ];
					} );

					this.model.get( 'post_media' )[ 'media_extra_options' ] = obj;
				}

				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( self.collection.where( {'ID': self.model.get( 'ID' )} ).length === 0 ) {
							self.collection.add( JSON.parse( JSON.stringify( self.model ) ), {at: self.collection.length} );

							/**
							 * Re-sync the course elements with the course model
							 */
							ThriveApp.globals.course_elements = ThriveApp.util.getCourseElements( ThriveApp.globals.active_course );

						} else {
							var model = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: self.model.get( 'ID' )} );

							model.set( JSON.parse( JSON.stringify( self.model ) ) );
							model.set_data();
						}

						self.beforeCheckData();

						ThriveApp.globals.focused_item = self.model;
						ThriveApp.globals.active_course.checkData();
						ThriveApp.globals.active_course_original.set( JSON.parse( JSON.stringify( ThriveApp.globals.active_course ) ) );
						ThriveApp.globals.active_course_original.set_data();

						if ( self.model.get( 'post_type' ) === 'tva_lesson' ) {
							var template = {
								lesson_type: self.model.get( 'lesson_type' ),
								comment_status: self.model.get( 'comment_status' ),
								post_media: JSON.parse( JSON.stringify( self.model.get( 'post_media' ) ) ),
							};

							template.post_media.media_url = '';
							ThriveApp.globals.active_course.set( {lesson_template: template} );
						}

						ThriveApp.globals.active_course.trigger( 'tva_render_content' );

					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
					} );
					xhr.always( function () {
						$( document ).unbind( 'keydown', self.keyAction );
						$( '.tvd-hiddendiv' ).empty();
						TVE_Dash.hideLoader();
						if ( self.modal_view ) {
							self.modal_view.close();
						}
					} );
				}
			},
			beforeSave: function () {
				ThriveApp.globals.active_course.get( 'modules' ).updateValues( {expanded: false}, true );

				if ( this.model.get( 'post_parent' ) > 0 ) {
					var parent = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: this.model.get( 'post_parent' )} );
					parent.set( {expanded: true} );

					if ( parent.get( 'post_parent' ) > 0 ) {
						var module = ThriveApp.globals.active_course.get( 'modules' ).findWhere( {ID: parent.get( 'post_parent' )} );
						if ( module ) {
							module.set( {expanded: true} );
						}
					}
				}
			},
			beforeCheckData: function () {
				/**
				 * Hook so we can put in more data if needed
				 */
			}
		} );

		ThriveApp.views.ChapterEditor = ThriveApp.views.LessonEditor.extend( {
			template: TVE_Dash.tpl( 'course/chapter/chapter-editor' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderFooter();

				return this;
			},
			beforeSave: function () {
				ThriveApp.globals.active_course.get( 'modules' ).updateValues( {expanded: false}, true );
				var module = ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: this.model.get( 'post_parent' )} );

				if ( module ) {
					module.set( {expanded: true} );
				}
				this.model.set( {expanded: true} );
			},
			beforeCheckData: function () {
				if ( this.model.get( 'post_parent' ) > 0 ) {
					var module = ThriveApp.globals.active_course.get( 'modules' ).findWhere( {ID: parseInt( this.model.get( 'post_parent' ) )} );
					if ( module ) {
						module.checkData();
						module.set( {expanded: true} );
					}
				}
			}
		} );

		/**
		 * View to render modal editor for a module
		 */
		ThriveApp.views.ModuleEditor = ThriveApp.views.LessonEditor.extend( {
			template: TVE_Dash.tpl( 'course/module/module-editor' ),
			events: {
				'click .tva-item-cover-image': 'addCoverImage',
				'click .tva-remove-module-cover-image': 'removeCoverImage',
				'input #tva-item-title': 'setTitle',
				'input #tva-item-description': 'setDescription',
				'click .tva-option-type': 'setCommentStatus',
				'click .tva-selected-option': 'toggleOptions',
				'click .tvd-modal-submit': 'save'
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				ThriveApp.util.renderMCE( 'tva-item-description', this.model, 'post_excerpt' );
				this.renderComments();
				this.renderCoverImage();
				this.renderFooter();

				return this;
			},
			beforeSave: function () {
				this.collection.updateValues( {expanded: false}, true );
				this.model.set( {expanded: true} );
			},
			renderCoverImage: function () {
				var state = this.model.get( 'cover_image' ) ? 'normal' : 'empty';

				if ( ! ThriveApp.views[ 'Cover' + ThriveApp.util.upperFirst( state ) + 'State' ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'ModuleCover' + ThriveApp.util.upperFirst( state ) + 'State' ]( {
					el: this.$( '.tva-item-cover-image' ),
					model: this.model
				} );

				view.render();

				return this;
			},
			removeCoverImage: function ( e ) {
				e.stopPropagation();

				this.model.set( {cover_image: ''} );
				this.renderCoverImage();
			},
			renderComments: function () {
				var view = new ThriveApp.views.ModuleComments( {
					el: this.$( '.tva-comments-container' ),
					model: this.model
				} );

				view.render();

				return this;
			},
			setCommentStatus: function ( e ) {
				var comment_status = e.currentTarget.getAttribute( 'data-status' ),
					children = this.model.hasChildren();

				this.model.set( {comment_status: comment_status} );
				this.renderComments();

				if ( children ) {
					this.model.get( children ).updateValues( {comment_status: comment_status}, true );
				}
			}
		} );

		ThriveApp.views.ModuleDelete = ThriveApp.views.LessonEditor.extend( {
			template: TVE_Dash.tpl( 'course/module/module-delete' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderFooter();

				return this;
			}
		} );

		ThriveApp.views.ChapterDelete = ThriveApp.views.ModuleDelete.extend( {
			template: TVE_Dash.tpl( 'course/chapter/chapter-delete' )
		} );

		ThriveApp.views.InputFieldEditor = ThriveApp.views.LessonEditor.extend( {
			template: TVE_Dash.tpl( 'course/input-field-editor' ),
			placeholder: null,
			initialize: function ( options ) {
				this.placeholder = options.placeholder;
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, placeholder: this.placeholder} ) );

				return this;
			}
		} );

		ThriveApp.views.MoveNotification = ThriveApp.views.InputFieldEditor.extend( {
			template: TVE_Dash.tpl( 'modals/move-notification' ),
			args: null,
			initialize: function ( options ) {
				this.args = options.args;
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, args: this.args} ) );

				return this;
			}
		} );

		ThriveApp.views.LessonTypeOption = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/lesson/lesson-option' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				var self = this;
				_.each( this.model.get( 'types' ), function ( item ) {
					self.renderOne( item );
				} );
				return this;
			},
			renderOne: function ( item ) {
				var view = new ThriveApp.views.LessonOption( {
					el: this.$( '.tva-lessons-types-list' ),
					model: item
				} );

				this.$el.append( view.render().$el );

				return this;
			}
		} );

		/**
		 * Base view for lessons. Detect lesson's type and render the editor accordingly
		 */
		ThriveApp.views.BaseModalLesson = ThriveApp.views.Base.extend( {
			mediaModel: null,
			mediaView: null,
			render: function () {
				this.renderLessonType();

				return this;
			},
			renderLessonType: function () {
				var viewName = ThriveApp.util.upperFirst( this.model.get( 'lesson_type' ) ) + 'ModalLesson';

				if ( ! ThriveApp.views[ viewName ] ) {
					return;
				}

				this.mediaView = new ThriveApp.views[ viewName ]( {
					el: this.el,
					model: this.model
				} );

				this.mediaView.render();

				return this;
			}
		} );

		/**
		 * View for text lesson type
		 */
		ThriveApp.views.TextModalLesson = ThriveApp.views.BaseModalLesson.extend( {
			initialize: function () {
				this.listenTo( this.model, 'change:cover_image', this.renderCoverImage )
			},
			render: function () {
				this.renderCoverImage();

				return this;
			},
			renderCoverImage: function () {
				var view = new ThriveApp.views.LessonCoverImage( {
					el: this.el,
					model: this.model,
					labelText: 'Would you like to add a custom lesson cover image?',
				} );

				view.render();

				return this;
			}
		} );

		/**
		 * View for video lesson type
		 */
		ThriveApp.views.VideoModalLesson = ThriveApp.views.BaseModalLesson.extend( {
			render: function () {
				this.renderMediaOptions();
				return this;
			},
			renderMediaOptions: function () {

				if ( ! this.model.get( 'post_media' ).media_type ) {
					/**
					 * Set the default
					 *
					 * @type {string}
					 */
					this.model.get( 'post_media' ).media_type = 'youtube';
				}
				this.mediaModel = new ThriveApp.models.MediaSelect( this.model.get( 'post_media' ) );

				var view = new ThriveApp.views.MediaSelectView( {
					el: this.el,
					model: this.mediaModel,
					source: this.model
				} );

				view.render();
				return this;
			}
		} );

		/**
		 * View for audio lesson type
		 */
		ThriveApp.views.AudioModalLesson = ThriveApp.views.BaseModalLesson.extend( {
			render: function () {
				this.renderMediaOptions();

				return this;
			},
			renderMediaOptions: function () {
				/**
				 * Initialize media model
				 */
				this.mediaModel = new ThriveApp.models.MediaSelect( {
					media_options: {
						soundcloud: {
							name: 'SoundCloud',
							validate_domain: true
						},
						custom: {
							name: 'Custom',
							validate_domain: false
						}
					}
				} );

				/**
				 * If there are any data pass them further to the MediaSelect model, otherwise use defaults
				 */
				var post_media = this.model.get( 'post_media' );
				post_media ?
					this.mediaModel.set( {
						media_type: post_media.media_type,
						media_url: post_media.media_url
					} ) :
					this.mediaModel.set( {media_type: 'soundcloud'} );

				this.mediaView = new ThriveApp.views.MediaSelectView( {
					el: this.el,
					model: this.mediaModel,
					source: this.model
				} );

				this.mediaView.render();

				return this;
			}
		} );

		ThriveApp.views.LessonCoverImage = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/cover-image' ),
			events: {
				'click .tva-lesson-cover-image': 'addCoverImage',
				'click .tva-remove-lesson-cover-image': 'removeCoverImage'
			},
			labelText: 'Cover Image',
			initialize: function ( options ) {
				this.labelText = options.labelText ? options.labelText : this.labelText;
			},
			render: function () {
				this.$el.html( this.template( {labelText: this.labelText} ) );
				this.renderCoverImage();
				return this;
			},
			removeCoverImage: function ( e ) {
				e.stopPropagation();

				this.model.set( {cover_image: ''} );
				this.renderCoverImage();
			},
			renderCoverImage: function () {
				var state = this.model.get( 'cover_image' ) ? 'normal' : 'empty';

				if ( ! ThriveApp.views[ 'LessonCover' + ThriveApp.util.upperFirst( state ) + 'State' ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'LessonCover' + ThriveApp.util.upperFirst( state ) + 'State' ]( {
					el: this.$( '.tva-lesson-cover-image' ),
					model: this.model
				} );

				view.render();
			},
			addCoverImage: function () {
				ThriveApp.util.addCoverImage( this.model );
			}
		} );

		ThriveApp.views.ReceiverMoveItem = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/receiver-course' ),
			type: null,
			initialize: function ( option ) {
				this.type = option.type;
				this.$el.addClass( 'tva-receiver-' + this.type );
			},
			render: function () {
				var name = this.type === 'course' ? this.model.get( 'name' ) : this.model.get( 'post_title' ),
					items = [];

				this.$el.html( this.template( {model: this.model, type: this.type, name: name} ) );

				var item_level = ThriveApp.globals.selected_items.getHighestSelectedLevel();

				this.collection.each( function ( item ) {
					var decision = ThriveApp.util.userHasToDecide( item_level, item );

					if ( decision === 1 || ( ThriveApp.globals.active_course.hasChildren() === ThriveApp.util.getPlural( ThriveApp.util.hierarchy[ item_level ] ) && item.get( 'ID' ) === ThriveApp.globals.active_course.get( 'ID' ) ) ) {
						/**
						 * Don't render elements which would have the user
						 * create an element in order to move what he has selected
						 */
						return;
					}
					items.push( item );


				}, this );

				if ( items.length > 0 ) {
					_.each( items, function ( item ) {
						this.renderOne( item );
					}, this );
				} else {
					$( '.tva-receiver-module' ).empty();
					$( '.tvd-modal-submit' ).addClass( 'tvd-disabled' );
					TVE_Dash.err( 'There are no elements that can receive this item', 4000 );
				}

				return this;
			},
			renderOne: function ( model ) {

				var view = new ThriveApp.views.SingleReceiverItem( {
					el: this.$( '.tva-available-receivers-' + this.type + 's' ),
					model: model,
					type: this.type
				} );

				view.render();

				return this;
			}
		} );

		/**
		 * This view instantiates the path to the decision the user needs to make
		 * Please see comments on ThriveApp.util.userHasToDecide();
		 */
		ThriveApp.views.ItemDecisionMaker = ThriveApp.views.Base.extend( {
			restrict_back: false,
			initialize: function ( options ) {
				this.decision_model = options.decision_model ? options.decision_model : ThriveApp.globals.active_course;
				this.modal_view = options.modal_view;
				this.restrict_back = options.restrict_back;

				this.$el.on( 'click', function ( e ) {
					var container = self.$( '.tva-reciever-container' ),
						btn = self.$( '.tva-parent > span' );

					if ( ! container.is( e.target ) && container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						container.slideUp();
					}
				} );
			},
			render: function () {
				var item_level = ThriveApp.util.levels[ this.model.get( 'type' ) ],
					decision = ThriveApp.util.userHasToDecide( item_level, this.decision_model );

				if ( decision === 1 ) {
					this.notifyUser();
				} else {
					this.selectParents();
				}
				return this;
			},
			notifyUser: function () {
				/**
				 * the user needs to include items within this element
				 */
				var view = new ThriveApp.views.NotifyUser( {
					model: this.model,
					collection: this.collection,
					modal_view: this.modal_view,
					decision_model: this.decision_model,
					restrict_back: this.restrict_back,
					el: this.$el
				} );

				view.render();
			},
			selectParents: function () {
				/**
				 * the user needs to include this element within an item
				 */
				var view = new ThriveApp.views.SelectParents( {
					model: this.model,
					collection: this.collection,
					modal_view: this.modal_view,
					el: this.$el
				} );

				view.render();
			}
		} );

		/**
		 * TODO: LONG TERM, We should refactor these events for general purposes
		 */
		ThriveApp.views.SelectParents = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/select-parent' ),
			events: {
				'click .tva-modal-validate-data': 'validateData'
			},
			receiver_model: null,
			initialize: function ( options ) {
				this.modal_view = options.modal_view;
				this.low_decision_view = null;
				this.receiver_model = new ThriveApp.models[ ThriveApp.util.upperFirst( this.collection.at( 0 ).get( 'type' ) ) ];
				var self = this;

				this.model.on( 'item_decission_low', function ( view ) {
					self.low_decision_view = view;
				} );

				this.model.on( 'tva_select_chapters', function ( collection ) {
					self.receiver_model = new ThriveApp.models[ ThriveApp.util.upperFirst( collection.at( 0 ).get( 'type' ) ) ];
					self.renderOptions( collection, self.$( '.tva-item-optional-container' ) );
				} );

				this.model.on( 'tva_item_was_selected', function ( model ) {
					if ( model.get( 'type' ) === 'module' ) {
						self.$( '.tva-item-optional-container' ).empty();
					}
				} );
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, parent: this.collection.at( 0 ).get( 'type' )} ) );
				this.renderOptions( this.collection );

				return this;
			},
			renderOptions: function ( collection, $element ) {
				this.model.set( {post_parent: 0} );
				var view = new ThriveApp.views.ReceiverItems( {
					model: this.model,
					collection: collection,
					receiver_model: this.receiver_model,
					modal_view: this.modal_view
				} );
				$element = ! $element ? this.$( '.tva-item-parent-container' ) : $element;
				$element.append( view.render().$el );
			},
			validateData: function () {
				if ( ! this.model.get( 'post_parent' ) ) {
					TVE_Dash.err( ThriveApp.t.AddContentNoItem );
					return;
				}

				if ( this.low_decision_view ) {
					$( '.tva-decision-container' ).html( this.low_decision_view.render().$el );
				} else {
					this.modal_view.next();
				}
			}
		} );

		ThriveApp.views.ReceiverItems = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/receiver-item' ),
			events: {
				'click .tva-parent': function () {
					this.$( '.tva-reciever-container' ).slideToggle();
				}
			},
			item: null,
			initialize: function ( options ) {
				var self = this;
				this.modal_view = options.modal_view;
				this.receiver_model = options.receiver_model;
				this.receiver_model.on( 'tva_item_was_selected', function ( model ) {
					self.$( '.tva-reciever-container' ).slideToggle();
					self.receiver_model.set( {post_title: model.get( 'post_title' )} );
					self.render();
				} );
			},
			render: function () {
				this.$el.html( this.template( {model: this.receiver_model} ) );

				this.collection.each( this.renderItems, this );

				return this;
			},
			renderItems: function ( item ) {
				var view = new ThriveApp.views.ReceiverItem( {
					model: item,
					new_item: this.model,
					modal_view: this.modal_view,
					receiver_model: this.receiver_model,
					collection: this.collection
				} );

				this.$( '.tva-material-list' ).append( view.render().$el );
			}
		} );

		ThriveApp.views.ReceiverItem = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/single-receiver-item' ),
			tagName: 'li',
			className: 'tva-module-item tva-parent',
			initialize: function ( options ) {
				var self = this;
				this.$el.on( 'click', function () {
					self.setParent();
				} );
				this.new_item = options.new_item;
				this.modal_view = options.modal_view;
				this.receiver_model = options.receiver_model;
				this.item_level = ThriveApp.util.levels[ this.new_item.get( 'type' ) ];

			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			},
			setParent: function () {
				/**
				 * Send trigger to parent views so we can re-render everything correctly
				 */
				this.new_item.trigger( 'tva_item_was_selected', this.model );
				this.receiver_model.trigger( 'tva_item_was_selected', this.model );


				if ( this.view ) {
					this.view.remove();
				}

				var decision = ThriveApp.util.userHasToDecide( this.item_level, this.model );

				if ( decision ) {
					/**
					 * The user needs to also select a chapter in order to set the parent
					 * or he needs to confirm that some lessons will be moved in this item
					 */

					if ( decision === 1 ) {
						this.new_item.set( {post_parent: this.model.get( 'ID' )} );
						this.view = new ThriveApp.views.ItemDecisionMaker( {
							model: this.new_item,
							collection: this.model.get( 'chapters' ),
							modal_view: this.modal_view,
							decision_model: this.model
						} );

						this.new_item.trigger( 'item_decission_low', this.view );
						$( '.tva-modal-validate-data' ).removeClass( 'tva-disabled-button' );
					} else {
						this.new_item.trigger( 'tva_select_chapters', this.model.get( 'chapters' ) );
						$( '.tva-modal-validate-data' ).addClass( 'tva-disabled-button' );
					}
				} else {
					this.new_item.set( {post_parent: this.model.get( 'ID' )} );

					this.view = new ThriveApp.views[ ThriveApp.util.upperFirst( this.new_item.get( 'type' ) ) + 'Editor' ]( {
						model: this.new_item,
						collection: this.model.get( this.new_item.get( 'type' ) + 's' ),
						modal_view: this.modal_view
					} );

					$( '.tva-editor-container' ).html( this.view.render().$el );
					$( '.tva-modal-validate-data' ).removeClass( 'tva-disabled-button' );
				}
			}
		} );

		ThriveApp.views.NotifyUser = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/confirm-children-move' ),
			events: {
				'click .tva-modal-next-step': 'renderEditor'
			},
			initialize: function ( options ) {
				this.modal_view = options.modal_view;
				this.decision_model = options.decision_model;
				this.restrict_back = options.restrict_back;
			},
			render: function () {

				this.children = this.decision_model.hasChildren();
				this.elements = this.decision_model.get( this.children );

				this.$el.html( this.template( {
					model: this.model,
					current_item: this.children,
					restrict_back: this.restrict_back
				} ) );

				return this;
			},
			renderEditor: function () {

				var collection = this.model.get( this.children );

				collection.reset();
				collection.add( JSON.parse( JSON.stringify( this.elements ) ) );

				var view = new ThriveApp.views[ ThriveApp.util.upperFirst( this.model.get( 'type' ) ) + 'Editor' ]( {
					model: this.model,
					collection: this.collection,
					modal_view: this.modal_view
				} );

				this.modal_view.$( '.tva-editor-container' ).html( view.render().$el );
			}
		} );

		//////////////////////////////////////////////////////////////////////////
		///                         BASIC VIEWS                                ///
		//////////////////////////////////////////////////////////////////////////

		ThriveApp.views.StepModalFooter = ThriveApp.views.CourseOption.extend( {
			template: TVE_Dash.tpl( 'modals/step-modal-footer' )
		} );

		ThriveApp.views.ModalFooter = ThriveApp.views.CourseOption.extend( {
			template: TVE_Dash.tpl( 'modals/modal-footer' )
		} );

		ThriveApp.views.ModalGroupFooter = ThriveApp.views.CourseOption.extend( {
			template: TVE_Dash.tpl( 'modals/modal-group-footer' )
		} );


		ThriveApp.views.ModalFooterDelete = ThriveApp.views.CourseOption.extend( {
			template: TVE_Dash.tpl( 'modals/modal-footer-delete' )
		} );

		ThriveApp.views.ModuleCoverNormalState = ThriveApp.views.CoverNormalState.extend( {
			template: TVE_Dash.tpl( 'course/module/cover-normal' )
		} );

		ThriveApp.views.ModuleCoverEmptyState = ThriveApp.views.CoverEmptyState.extend( {
			template: TVE_Dash.tpl( 'course/module/cover-empty' )
		} );

		ThriveApp.views.LessonOption = ThriveApp.views.CourseOption.extend( {
			template: TVE_Dash.tpl( 'course/lesson/singel-option' )
		} );

		ThriveApp.views.LessonComments = ThriveApp.views.CoverNormalState.extend( {
			template: TVE_Dash.tpl( 'course/lesson/comments' ),
		} );

		ThriveApp.views.ModuleComments = ThriveApp.views.CoverNormalState.extend( {
			template: TVE_Dash.tpl( 'course/module/comments' ),
		} );

		ThriveApp.views.SingleReceiverItem = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/course-option' ),
			initialize: function ( options ) {
				this.type = options.type;
			},
			render: function () {

				this.$el.append( this.template( {model: this.model, type: this.type} ) );

				return this;
			}
		} );

		/**
		 * View to render available chapters to move lessons in
		 */
		ThriveApp.views.ReceiverChapter = ThriveApp.views.ReceiverMoveItem.extend( {
			template: TVE_Dash.tpl( 'course/chapter/receiver-chapter' ),
			className: 'tva-receiver-chapter'
		} );

		/**
		 * LOGS VIEWS
		 */

		ThriveApp.views.SendowlStacks = ThriveApp.views.Base.extend( {
			className: 'tve-debug-container',
			template: TVE_Dash.tpl( 'sendowl/logs' ),
			events: {
				'input .tve-debug-search': 'searchStacks'
			},
			view: this,
			select_all: '',
			initialize: function () {
				ThriveApp.globals.logs_settings.set( {limit: 20} );
				this.listenTo( ThriveApp.globals.stack_types, 'change', this.filterStacks );
				this.listenTo( this.collection, 'reset', this.renderTables );

				$( window ).off( 'scroll' ).on( 'scroll', {view: this}, this.handleScrolling );
			},
			render: function () {
				var self = this;
				ThriveApp.globals.stack_types.on( 'collection_changed', function () {
					self.$( '.tve-debug-type-change' ).prop( 'checked', self.select_all.get( 'checked' ) );
					self.filterStacks();
				} );

				this.$el.html( this.template( {} ) );
				this.renderTypes();
				this.renderTables();
				this.hideRows();

				return this;
			},
			handleScrolling: function ( e ) {
				if ( ( Backbone.history.getFragment().indexOf( 'sendowl_stacks' ) !== - 1 && $( window ).scrollTop() + $( window ).height() ) + 10 > $( document ).height() ) {
					var self = e.data.view;
					$( '#tva-admin-wrapper' ).css( 'margin-bottom', '100px' );
					if ( self.flag ) {
						self.flag = false;
						self.getLogsAjax( self );

					}

				}
			},
			getLogsAjax: function ( self ) {
				TVE_Dash.showLoader();

				var collection = {};
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					data: {
						settings: ThriveApp.globals.logs_settings.toDeepJSON()
					},
					url: ThriveApp.routes.logs + '/get_logs/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						ThriveApp.globals.logs_settings.set( {limit: ThriveApp.globals.logs_settings.get( 'limit' ) + 20} );

						collection = new ThriveApp.collections.Logs( response );
						collection.each( self.renderTable, self );
						self.hideRows();
						$( '#tva-admin-wrapper' ).css( 'margin-bottom', '0' );
					}
				} ).error( function ( errorObj ) {
					var response = JSON.parse( errorObj.responseText );
					TVE_Dash.err( response.message );
				} ).always( function () {
					self.flag = true;
					TVE_Dash.hideLoader();
				} );
			},
			remove: function () {
				$( window ).off( 'scroll', this.handleScrolling );
				return Backbone.View.prototype.remove.apply( this );
			},
			hideRows: function () {
				this.$( '.tvd-collection-header' ).not( '.ttw-table-header' ).hide();
				this.$( '.tvd-collection-item' ).not( '.ttw-table-header' ).hide();
			},
			searchStacks: function ( e ) {
				var value = e.target.value;
				ThriveApp.globals.logs_settings.set( {s: value, limit: 0} );
				this.getStacks();
			},
			filterStacks: function () {
				TVE_Dash.showLoader();
				var checked = ThriveApp.globals.stack_types.where( {checked: true} );
				this.collection.reset();
				/**
				 * reset types
				 * @type {Array}
				 */

				if ( checked.length > 0 ) {
					ThriveApp.globals.logs_settings.set( {limit: 0, types: []} );
					var types = [];
					_.each( checked, function ( model ) {
						types.push( model.get( 'type' ) );
					} );

					ThriveApp.globals.logs_settings.set( {types: types} );
					this.getStacks();

				} else {
					TVE_Dash.hideLoader();
				}
			},
			getStacks: function () {
				var self = this;
				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					data: {
						settings: ThriveApp.globals.logs_settings.toDeepJSON()
					},
					url: ThriveApp.routes.stacks + '/get_stacks/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.collection.reset();
						self.$( '#tve-debug-tables-wrapper' ).empty();
						self.collection.add( response );
						self.collection.each( self.renderTable, self );
						ThriveApp.globals.logs_settings.set( {limit: 20} );
						self.hideRows();

					}
				} ).error( function ( errorObj ) {
					self.$( '#tve-debug-tables-wrapper' ).empty();
					var response = JSON.parse( errorObj.responseText );
					TVE_Dash.err( response.message );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			renderTypes: function () {
				if ( ThriveApp.globals.stack_types.length > 0 ) {
					ThriveApp.globals.stack_types.each( this.renderType, this );
				}

				var unchecked = ThriveApp.globals.stack_types.findWhere( {checked: false} );
				this.select_all = new ThriveApp.models.Type( {type: 'Select All', checked: true} );
				if ( unchecked ) {
					this.select_all.set( {checked: false} );
				}

				var view = new ThriveApp.views.Type( {
					collection: ThriveApp.globals.stack_types,
					model: this.select_all
				} );

				this.$( '#tve-debug-type-filters' ).append( view.render().$el );

			},
			renderType: function ( type ) {
				var view = new ThriveApp.views.Type( {
					collection: ThriveApp.globals.stack_types,
					model: type
				} );

				this.$( '#tve-debug-type-filters' ).append( view.render().$el );

			},
			renderTables: function () {
				this.$( '#tve-debug-tables-wrapper' ).empty();
				this.collection.each( this.renderTable, this )
			},
			renderTable: function ( table ) {
				var view = new ThriveApp.views.Table( {
					model: table
				} );

				this.$( '#tve-debug-tables-wrapper' ).append( view.render().$el );
			}
		} );

		ThriveApp.views.SendowlLogs = ThriveApp.views.SendowlStacks.extend( {
			select_all: '',
			events: {
				'input .tve-debug-search': 'searchLogs'
			},
			flag: true,
			all: false,
			initialize: function () {
				ThriveApp.globals.logs_settings.set( {limit: 20} );
				this.listenTo( ThriveApp.globals.log_types, 'change', this.filterLogs );
				this.listenTo( this.collection, 'reset', this.renderTables );
				this.all = this.collection.length < 20;

				$( window ).off( 'scroll' ).on( 'scroll', {view: this}, this.handleScrolling );
			},
			render: function () {
				var self = this;
				ThriveApp.globals.log_types.on( 'collection_changed', function () {
					self.$( '.tve-debug-type-change' ).prop( 'checked', self.select_all.get( 'checked' ) );
					self.filterLogs();
				} );

				this.$el.html( this.template( {} ) );
				this.renderTypes();
				this.renderTables();
				this.hideRows();

				return this;
			},
			handleScrolling: function ( e ) {
				if ( e.data.view.all ) {
					return;
				}

				if ( ( Backbone.history.getFragment().indexOf( 'sendowl_logs' ) !== - 1 && $( window ).scrollTop() + $( window ).height() ) + 10 > $( document ).height() ) {
					var self = e.data.view;
					$( '#tva-admin-wrapper' ).css( 'margin-bottom', '100px' );

					if ( self.flag ) {
						self.flag = false;
						self.getLogsAjax( self );
					}
				}
			},
			getLogsAjax: function ( self ) {
				TVE_Dash.showLoader();

				var collection = {};
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					data: {
						settings: ThriveApp.globals.logs_settings.toDeepJSON()
					},
					url: ThriveApp.routes.logs + '/get_logs/'
				} ).done( function ( response, status, options ) {
					if ( response ) {

						if ( response.all ) {
							self.all = true;
							TVE_Dash.err( ThriveApp.t.NoLogsFound );
							return;
						}

						ThriveApp.globals.logs_settings.set( {limit: ThriveApp.globals.logs_settings.get( 'limit' ) + 20} );

						collection = new ThriveApp.collections.Logs( response );
						collection.each( self.renderTable, self );

						self.hideRows();
						$( '#tva-admin-wrapper' ).css( 'margin-bottom', '0' );
					}
				} ).always( function () {
					self.flag = true;
					TVE_Dash.hideLoader();
				} );
			},
			searchLogs: function ( e ) {
				var value = e.target.value;
				ThriveApp.globals.logs_settings.set( {s: value, limit: 0} );
				this.getLogs();
			},
			renderTypes: function () {

				var unchecked = ThriveApp.globals.log_types.findWhere( {checked: false} );
				this.select_all = new ThriveApp.models.Type( {type: 'Select All', checked: true} );
				if ( unchecked ) {
					this.select_all.set( {checked: false} );
				}

				var view = new ThriveApp.views.Type( {
					collection: ThriveApp.globals.log_types,
					model: this.select_all
				} );

				this.$( '#tve-debug-type-filters' ).append( view.render().$el );

				if ( ThriveApp.globals.log_types.length > 0 ) {
					ThriveApp.globals.log_types.each( this.renderType, this );
				}
			},
			renderType: function ( type ) {
				var view = new ThriveApp.views.Type( {
					collection: ThriveApp.globals.log_types,
					model: type
				} );

				this.$( '#tve-debug-type-filters' ).append( view.render().$el );

			},
			renderTable: function ( table ) {
				var view = new ThriveApp.views.LogTable( {
					model: table
				} );

				this.$( '#tve-debug-tables-wrapper' ).append( view.render().$el );
			},
			filterLogs: function () {
				TVE_Dash.showLoader();
				var checked = ThriveApp.globals.log_types.where( {checked: true} );
				this.collection.reset();
				/**
				 * reset types
				 * @type {Array}
				 */


				if ( checked.length > 0 ) {
					ThriveApp.globals.logs_settings.set( {limit: 0, types: []} );
					var types = [];
					_.each( checked, function ( model ) {
						types.push( model.get( 'type' ) );
					} );

					ThriveApp.globals.logs_settings.set( {types: types} );
					this.getLogs();

				} else {
					TVE_Dash.hideLoader();
				}
			},
			getLogs: function () {
				var self = this;
				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'POST',
					data: {
						settings: ThriveApp.globals.logs_settings.toDeepJSON()
					},
					url: ThriveApp.routes.logs + '/get_logs/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.collection.reset();
						self.$( '#tve-debug-tables-wrapper' ).empty();
						self.collection.add( response );
						self.collection.each( self.renderTable, self );
						ThriveApp.globals.logs_settings.set( {limit: 20} );
						self.hideRows();

					}
				} ).error( function ( errorObj ) {
					self.$( '#tve-debug-tables-wrapper' ).empty();
					var response = JSON.parse( errorObj.responseText );
					TVE_Dash.err( response.message );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			}
		} );

		ThriveApp.views.Type = ThriveApp.views.Base.extend( {
			className: 'tvd-col tvd-s3',
			template: TVE_Dash.tpl( 'sendowl/type' ),
			events: {
				'click .tve-debug-type-change': 'setFilters'
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				return this;
			},
			setFilters: function ( e ) {
				var checked = $( e.target ).prop( 'checked' );

				this.model.set( {checked: checked} );

				if ( typeof this.model.get( 'id' ) === 'undefined' ) {
					TVE_Dash.showLoader();
					this.collection.each( function ( model ) {
						model.set( {checked: checked}, {silent: true} );
					}, this );

					this.collection.trigger( 'collection_changed' );
					this.render();
				}
			}
		} );

		ThriveApp.views.Table = ThriveApp.views.Base.extend( {
			className: 'tvd-col tvd-s12',
			template: TVE_Dash.tpl( 'sendowl/table-wrapper' ),
			headers: [],
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderData();
				return this;
			},
			renderData: function () {
				this.model.get( 'data' ).each( function ( model ) {
					var view = new ThriveApp.views.Data( {
						model: model
					} );

					this.$( '.tve-debug-table' ).append( view.render().$el );
				}, this );
			}
		} );

		ThriveApp.views.LogTable = ThriveApp.views.Table.extend( {
			renderData: function () {
				var view = new ThriveApp.views.Data( {
					model: this.model
				} );

				this.$( '.tve-debug-table' ).append( view.render().$el );
			}
		} );

		ThriveApp.views.Data = ThriveApp.views.Base.extend( {
			tagName: 'ul',
			className: 'tvd-collection tvd-with-header',
			events: {
				'click .ttw-table-header': 'toggleLog'
			},
			toggleLog: function ( e ) {
				e.stopPropagation();
				this.$( '.tvd-collection-item, .tvd-collection-header' ).not( '.ttw-table-header' ).slideToggle();
				$( e.currentTarget ).find( '.tve-debug-slide > span' ).toggleClass( "tve-churn-icon-keyboard_arrow_up" );
				$( e.currentTarget ).find( '.tve-debug-slide > span' ).toggleClass( "tve-churn-icon-keyboard_arrow_down" );
			},
			render: function () {
				var self = this,
					data = this.model.get( 'data' );
				if ( this.model.get( 'type' ) && this.model.get( 'identifier' ) ) {
					this.renderInfo();
				}

				if ( typeof ( data ) === 'object' ) {
					_.each( data, function ( value, attribute ) {

						var left_column = new ThriveApp.views.LeftColumn( {
							value: attribute
						} );
						self.$el.append( left_column.render().$el );
						if ( typeof ( value ) === 'object' && value !== null ) {

							var model = new ThriveApp.models.Base( {data: value} );
							var right_table_column = new ThriveApp.views.RightTableColumn( {
								model: model
							} );
							self.$el.append( right_table_column.render().$el );
						} else {
							var right_column = new ThriveApp.views.RightColumn( {
								value: value
							} );

							self.$el.append( right_column.render().$el );
						}
					} );
				}

				return this;
			},
			renderInfo: function () {
				var view = new ThriveApp.views.InfoRow( {
					model: this.model
				} );

				this.$el.append( view.render().$el );
			}
		} );

		ThriveApp.views.InfoRow = ThriveApp.views.Base.extend( {
			tagName: 'li',
			className: 'tvd-collection-header ttw-table-header tvd-col tvd-s12',
			template: TVE_Dash.tpl( 'sendowl/table-info-row' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				return this;
			}
		} );

		ThriveApp.views.LeftColumn = ThriveApp.views.Base.extend( {
			tagName: 'li',
			className: 'tvd-collection-header tvd-col tvd-s2',
			template: TVE_Dash.tpl( 'sendowl/table-item-column' ),
			initialize: function ( options ) {
				this.value = options.value;

				this.$( '.tvd-collection-item' ).hide();
			},
			render: function () {
				this.$el.html( this.template( {value: this.value} ) );
				return this;
			}
		} );

		ThriveApp.views.RightColumn = ThriveApp.views.LeftColumn.extend( {
			className: 'tvd-collection-item tvd-col tvd-s10 tve-debug-column'
		} );

		ThriveApp.views.RightTableColumn = ThriveApp.views.Base.extend( {
			tagName: 'li',
			className: 'tvd-collection-item tvd-col tvd-s10 tve-debug-column tve-debug-sub-table',
			template: TVE_Dash.tpl( 'sendowl/table-data-column' ),
			render: function () {
				this.$el.html( this.template( {} ) );
				this.renderData();
				return this;
			},
			renderData: function () {
				var data = new ThriveApp.views.Data( {
					model: this.model
				} );

				this.$el.append( data.render().$el );
			}
		} );

		/**
		 * View to render Sendowl Customers
		 */
		ThriveApp.views.SendowlCustomers = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'sendowl/customers' ),
			events: {
				'click #tva-add-customer': function () {
					this.import_type = null;
					this.addNewCustomer();
				},
				'click .tva-import-customer': function ( e ) {
					this.$( '.tva-import-customer' ).toggleClass( 'dropdown-visible' );
				},
				'click .tva-selected-product-item': function ( e ) {
					this.$( '.tva-available-products' ).slideToggle();
				},
				'click .tva-selected-bundle-item': function () {
					this.$( '.tva-available-bundles' ).slideToggle();
				},
				'click .tva-product-item': 'setCurrentProduct',
				'click .tva-bundle-item': 'setCurrentBundle',
				'click .tva-pg-prev': 'prev',
				'click .tva-pg-next': 'next',
				'click .tva-pg-item': 'goToPage',
				'input #tva-search-customers': 'search',
				'click .tva-csv-import': 'csvImport',
				'click .tva-list-import': 'listImport',
				'click .tva-edit-customer': 'editCustomer'
			},
			initialize: function () {
				this.initData();
				var self = this,
					settings = ThriveApp.globals.settings;

				this.collection.on( 'tva_render_customers', function ( args ) {
					self.initData();
					self.pagination.total_pages = Math.ceil( args.count / settings.get( 'customers_per_page' ) );
					self.collection = new ThriveApp.collections.SendowlCustomers( args.data );
					self.render();
				} );

				this.pagination.total_pages = Math.ceil( settings.get( 'customers_count' ) / settings.get( 'customers_per_page' ) );

				this.products = new ThriveApp.collections.SendowlCollection( settings.get( 'sendowl_products' ).toJSON() );
				this.bundles = new ThriveApp.collections.SendowlCollection( settings.get( 'sendowl_bundles' ).toJSON() );
			},
			initData: function () {
				this.timeout = null;
				this.import_type = null;

				this.filters = {
					limit: ThriveApp.globals.settings.get( 'customers_per_page' ),
					offset: 0,
					current_product: new ThriveApp.models.Base(),
					current_bundle: new ThriveApp.models.Base(),
					s: '',
					count: false
				};

				this.pagination = {
					pages: [],
					current_page: 1,
					total_pages: 1,
					near: 2
				};
			},
			render: function () {
				this.getPages();

				this.$el.html( this.template( {
					model: this.model,
					filters: this.filters,
					pagination: this.pagination,
					customers_count: ThriveApp.globals.settings.get( 'customers_count' )
				} ) );

				this.renderFilters();

				this.collection.each( function ( model ) {
					this.$( '.tva-customers-table' ).append( new ThriveApp.views.SendowlCustomer( {model: model} ).render().$el );
				}, this );

				return this;
			},
			renderFilters: function () {
				if ( ( this.filters.current_product.get( 'id' ) > 0 ) && ( ! this.products.findWhere( {id: 0} ) ) ) {
					this.products.add( new ThriveApp.models.SendowlModel( {
						id: 0,
						name: 'All products'
					} ), {at: 0} );
				} else {
					var model = this.products.findWhere( {ID: 0} );

					if ( model ) {
						this.products.remove( model );
					}
				}

				if ( ( this.filters.current_bundle.get( 'id' ) > 0 ) && ( ! this.bundles.findWhere( {id: 0} ) ) ) {
					this.bundles.add( new ThriveApp.models.SendowlModel( {
						id: 0,
						name: 'All bundles'
					} ), {at: 0} );
				} else {
					var model = this.bundles.findWhere( {ID: 0} );

					if ( model ) {
						this.bundles.remove( model );
					}
				}

				this.products.each( function ( model ) {
					new ThriveApp.views.SendowlSingleProduct( {
						model: model,
						el: this.$( '.tva-available-products' ),
						filters: this.filters
					} ).render();
				}, this );

				this.bundles.each( function ( model ) {
					new ThriveApp.views.SendowlBundleProduct( {
						model: model,
						el: this.$( '.tva-available-bundles' ),
						filters: this.filters
					} ).render();
				}, this );
			},
			addNewCustomer: function () {
				this.modal( ThriveApp.views.AddCustomerModal, {
					model: new ThriveApp.models.SendowlCustomer( {} ),
					settings: ThriveApp.globals.settings,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal tva-extra-space',
					import_type: this.import_type,
					prevent_back: ! ! this.import_type,
					dismissible: false
				} );
			},
			editCustomer: function ( e ) {
				var id = parseInt( e.currentTarget.getAttribute( 'data-customer-id' ) );

				this.modal( ThriveApp.views.EditCustomerModal, {
					model: this.collection.findWhere( {ID: id} ),
					settings: ThriveApp.globals.settings,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal tva-extra-space'
				} );
			},
			setCurrentProduct: function ( e ) {
				var id = parseInt( e.currentTarget.getAttribute( 'data-product-id' ) ),
					model = this.products.findWhere( {id: id} );

				this.filters.current_bundle = new ThriveApp.models.Base();
				this.filters.current_product = model ? model : this.filters.current_product;
				this.filters.count = true;
				this.filters.offset = 0;
				this.pagination.current_page = 1;
				this.doAjaxRequest();
			},
			setCurrentBundle: function ( e ) {
				var id = parseInt( e.currentTarget.getAttribute( 'data-bundle-id' ) ),
					model = this.bundles.findWhere( {id: id} );

				this.filters.current_product = new ThriveApp.models.Base();
				this.filters.current_bundle = model ? model : this.filters.current_bundle;
				this.filters.count = true;
				this.filters.offset = 0;
				this.pagination.current_page = 1;
				this.doAjaxRequest();
			},
			prev: function () {
				this.pagination.current_page --;
				this.filters.offset = this.filters.offset >= this.filters.limit ? this.filters.offset - this.filters.limit : 0;
				this.doAjaxRequest();
			},
			next: function () {
				this.pagination.current_page ++;
				this.filters.offset = this.filters.offset + this.filters.limit;
				this.doAjaxRequest();
			},
			goToPage: function ( e ) {
				var page = parseInt( e.currentTarget.getAttribute( 'data-page' ) );

				this.pagination.current_page = parseInt( e.currentTarget.getAttribute( 'data-page' ) );
				this.filters.offset = this.pagination.current_page - 1 === 0 ? 0 : ( this.pagination.current_page - 1 ) * this.filters.limit;

				this.doAjaxRequest();
			},
			search: function ( e ) {
				if ( this.timeout ) {
					clearTimeout( this.timeout )
				}

				var self = this;

				this.timeout = setTimeout( function () {
					self.$( '#tva-search-customers' ).blur();
					self.filters.count = true;
					self.filters.offset = 0;
					self.filters.s = _.escape( e.target.value );
					self.pagination.current_page = 1;

					self.doAjaxRequest().always( function () {
						var input = self.$( '.ui-autocomplete-input' ).first(),
							val = input.val();

						input.focus().val( '' ).val( val );
					} );

				}, 400 );
			},
			doAjaxRequest: function () {
				TVE_Dash.showLoader();

				var self = this;

				return $.ajax( {
					url: ThriveApp.routes.users + '/get_customers',
					type: 'POST',
					data: self.getAjaxData()
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.collection = new ThriveApp.collections.SendowlCustomers( response.customers );
						self.pagination.total_pages = Math.ceil( response.count / ThriveApp.globals.settings.get( 'customers_per_page' ) );
						self.render();
					}
				} ).error( function ( errorObj ) {
					TVE_Dash.err( errorObj.responseText );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			getAjaxData: function () {
				var data = {};

				data.limit = this.filters.limit;
				data.offset = this.filters.offset;
				data.product_id = 0;
				data.s = this.filters.s;
				data.count = this.filters.count;
				data.count_value = this.pagination.total_pages;

				if ( this.filters.current_bundle.get( 'id' ) > 0 ) {
					data.product_id = this.filters.current_bundle.get( 'id' );
				} else if ( this.filters.current_product.get( 'id' ) > 0 ) {
					data.product_id = this.filters.current_product.get( 'id' );
				}

				return data;
			},
			getPages: function () {
				this.pagination.pages = [];

				var self = this,
					left_x = this.pagination.current_page - this.pagination.near,
					right_x = this.pagination.current_page + this.pagination.near;

				for ( var i = 1; i <= this.pagination.total_pages; i ++ ) {

					var left = ( i >= left_x ) && ( i < self.pagination.current_page ),
						right = ( i > self.pagination.current_page ) && i <= right_x;

					if ( ! _.contains( self.pagination.pages, i ) ) {
						// always push first page
						if ( i === 1 ) {
							self.pagination.pages.push( i );
							continue;
						}

						// push the current page
						if ( i === self.pagination.current_page ) {
							self.pagination.pages.push( i );
							continue;
						}

						//always push last page
						if ( i === self.pagination.total_pages ) {
							self.pagination.pages.push( i );
							continue;
						}

						// push first x pages
						if ( left ) {
							self.pagination.pages.push( i );
							continue;
						}

						// push last x pages
						if ( right ) {
							self.pagination.pages.push( i );
						}
					}
				}
			},
			listImport: function () {
				this.import_type = 'listNames';
				this.addNewCustomer();
			},
			csvImport: function () {
				this.import_type = 'csvFile';
				this.addNewCustomer();
			}
		} );

		ThriveApp.views.SendowlCustomer = ThriveApp.views.Base.extend( {
			tagName: 'div',
			className: 'tva-sendowl-customer',
			template: TVE_Dash.tpl( 'sendowl/customer' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				return this;
			}
		} );

		ThriveApp.views.SendowlModalMembershipOptions = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/sendowl-options' )
		} );

		ThriveApp.views.ImportFromCsvFile = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/sendowl/csv-import' ),
			files: '',
			initialize: function ( options ) {
				this.files = options.files;
			},
			render: function () {
				this.$el.html( this.template( {files: this.files} ) );
				return this;
			}
		} );

		ThriveApp.views.ImportFromListNames = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'modals/sendowl/list-import' ),
			render: function () {
				this.$el.html( this.template( {} ) );
				return this;
			}
		} );

		ThriveApp.views.SendowlSingleProduct = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/single-product-option' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.SendowlBundleProduct = ThriveApp.views.SendowlSingleProduct.extend( {
			template: TVE_Dash.tpl( 'sendowl/single-bundle-option' )
		} );

		ThriveApp.views.SendowlCsvFile = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/csv-option' ),
			csv_name: '',
			initialize: function ( options ) {
				this.csv_name = options.csv_name
			},
			render: function () {
				this.$el.html( this.template( {csv_name: this.csv_name} ) );

				return this;
			}
		} );

		ThriveApp.views.SendowlCsvCustomer = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/csv-customer' ),
			render: function () {
				this.$el.append( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.ManualImportPreview = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/list-option' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );

		ThriveApp.views.SingleCustomerImport = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'sendowl/single-customer-import' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this;
			}
		} );
	} );
} )( jQuery );;var ThriveApp = ThriveApp || {};
ThriveApp.views = ThriveApp.views || {};

( function ( $ ) {

	$( function () {

		/**
		 * Course view
		 */
		ThriveApp.views.Course = ThriveApp.views.Base.extend( {
			events: {
				'click .tva-course-status': 'changeCourseStatus'
			},
			initialize: function () {
				this.listenTo( this.model, 'change:state', this.renderState );
			},
			render: function () {
				this.renderState();
				return this;
			},
			renderState: function () {
				var state = this.model.get( 'state' );
				if ( ! ThriveApp.views[ 'Course' + ThriveApp.util.upperFirst( state ) + 'State' ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'Course' + ThriveApp.util.upperFirst( state ) + 'State' ]( {
					model: this.model,
					collection: this.collection
				} );

				view.render();
				this.$el.replaceWith( view.$el );
				this.setElement( view.$el );

				return this;
			},
			changeCourseStatus: function ( e ) {
				e.stopPropagation();
				var current = this.model.get( 'status' ),
					can_publish = true;

				if ( current === 'publish' ) {
					this.model.set( {status: 'draft'} );
					this.save();
					return;
				}

				if ( ! this.model.hasChildren() ) {
					can_publish = false;
				}

				if ( this.model.get( 'lessons' ).length > 0 ) {
					var published_lesson = this.model.get( 'lessons' ).findWhere( {post_status: 'publish'} );

					if ( ! published_lesson ) {
						can_publish = false;
					}
				}

				if ( this.model.get( 'chapters' ).length > 0 ) {
					var published_ch = this.model.get( 'chapters' ).findWhere( {post_status: 'publish'} );

					if ( ! published_ch ) {
						can_publish = false;
					}
				}

				if ( can_publish === false ) {
					TVE_Dash.err( ThriveApp.t.NumberLessons );
					return;
				}

				if ( this.model.get( 'modules' ).length > 0 ) {
					var published_module = this.model.get( 'modules' ).findWhere( {post_status: 'publish'} );

					if ( ! published_module ) {
						TVE_Dash.err( ThriveApp.t.NoPublishedModule );
						return;
					}
				}

				this.model.set( {status: current == 'draft' ? 'publish' : 'draft'} );

				this.save();

				return this
			},
			save: function () {
				TVE_Dash.cardLoader( this.$el );
				var self = this,
					xhr = self.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response ) {
							self.collection.trigger( 'tva_filter_courses' );

							ThriveApp.globals.available_settings.fetch().success( function ( response, status, options ) {
								ThriveApp.globals.settings.set( {
									iframe_url: ThriveApp.globals.settings.get( 'preview_url' ),
									preview_url: response.preview_url,
									preview_option: ! ! ThriveApp.globals.courses.findWhere( {status: 'publish'} )
								} );
							} );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
				}
			}
		} );

		/**
		 * Course card normal state
		 */
		ThriveApp.views.CourseNormalState = ThriveApp.views.Base.extend( {
			className: 'tva-course-item',
			template: TVE_Dash.tpl( 'course/item' ),
			initialize: function () {
				this.listenTo( this.model.get( 'lessons' ), 'add', this.render );
			},
			events: {
				'click .tva-go-to-course': 'goToCourse',
				'click .tva-delete-course': 'changeState',
				'click .tva-preview-course-url': 'previewCourse',
				'click .tva-drag-card': function ( e ) {
					e.stopPropagation();
				}
			},
			render: function () {
				var published = this.collection.where( {status: 'publish'} ),
					topic = ThriveApp.globals.topics.findWhere( {ID: parseInt( this.model.get( 'topic' ) )} ),
					display = this.collection.where( {display: 0} ),
					show_order = display.length == 0;

				this.model.set( {tooltiped: false} );

				if ( published.length == 0 && this.collection.at( 0 ) == this.model ) {
					var pb_lesson = this.model.get( 'lessons' ).findWhere( {post_status: 'publish'} ),
						pb_chapter = this.model.get( 'chapters' ).findWhere( {post_status: 'publish'} ),
						pb_module = this.model.get( 'modules' ).findWhere( {post_status: 'publish'} );

					if ( pb_lesson || pb_chapter || pb_module ) {
						this.model.set( {tooltiped: true} );
					}
				}

				this.$el.html( this.template( {model: this.model, topic: topic, show_order: show_order} ) );
				this.$el.attr( 'data-id', this.model.get( 'ID' ) );
				return this;
			},
			goToCourse: function ( e ) {
				var id = $( e.currentTarget ).attr( 'data-id' );
				if ( ! $( e.target ).hasClass( 'tva-delete-course' ) ) {
					ThriveApp.router.navigate( '#course/' + id, {trigger: true} );
				}
			},
			changeState: function ( e ) {
				e.stopPropagation();
				var model = this.collection.findWhere( {state: ThriveApp.t.delete_state} );
				if ( model ) {
					model.set( {state: ThriveApp.t.normal_state} );
				}
				this.model.set( {state: ThriveApp.t.delete_state} );
			},
			previewCourse: function ( e ) {
				e.stopPropagation();
			}
		} );

		/**
		 * Delete a course state
		 */
		ThriveApp.views.CourseDeleteState = ThriveApp.views.Base.extend( {
			className: 'tvd-col tvd-s6 tvd-ms6 tvd-m4 tvd-l3 tva-course-item',
			template: TVE_Dash.tpl( 'course/delete-state' ),
			events: {
				'click .tva-delete-no': function () {
					this.model.set( {state: ThriveApp.t.normal_state} );
				},
				'click .tva-delete-yes': 'deleteCourse'
			},
			initialize: function () {
				this.listenTo( this.collection, 'remove', this.remove );
				_.bindAll( this, 'keyAction' );
				$( document ).bind( 'keydown', this.keyAction );
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				return this
			},
			keyAction: function ( e ) {
				var code = e.which;
				if ( code == 13 ) {
					this.deleteCourse();
				} else if ( code == 27 ) {
					this.model.set( {state: ThriveApp.t.normal_state} );
				}
			},
			deleteCourse: function () {
				TVE_Dash.cardLoader( this.$el );
				var self = this,
					xhr = self.model.destroy( {wait: true} );
				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						self.collection.trigger( 'tva_update_courses' );
						$( document ).unbind( 'keydown', this.keyAction );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.hideLoader();
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );
		/**
		 * Add Course view
		 */
		ThriveApp.views.addCourse = ThriveApp.views.Base.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'course/edit/course' ),
			render: function () {

				this.$el.html( this.template( {model: this.model} ) );

				this.course = new ThriveApp.views.CourseFields( {
					el: this.$( '.tva-course-container' ),
					model: this.model
				} );

				this.course.render();
				TVE_Dash.hideLoader();
				return this;
			}
		} );

		/**
		 * Course fields view (all inputs for setting up a course)
		 */
		ThriveApp.views.CourseFields = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/course-fields' ),
			events: {
				'click .tva-expand-levels': 'toggleLevels',
				'click .tva-expand-labels': 'toggleLabels',
				'click .tva-add-course-level': 'addLevel',
				'click .tva-add-course-label': 'addLabel',
				'click .tva-cover-image': 'addCoverImage',
				'click .tva-course-level-item': 'selectLevel',
				'click .tva-course-label-item': 'selectLabel',
				'click #tva-course-logged-in': 'toggleLoggedIn',
				'click .tva-show-advanced': 'toggleSettings',
				'input #tva-course-title': 'setTitle',
				'input #tva-course-slug': 'setSlug',
				'change .tva-course-topic': 'setTopic',
				'change .tva-course-role': 'setRole',
				'input #tva-course-excluded': 'setExcluded',
				'input #tva-course-description': 'setDescription',
				'input #tva-course-message': 'setMessage',
				'click .tva-save-course': 'save',
				'click .tva-remove-course-cover-image': 'removeCoverImage',
				'change #tva-course-featured-video': 'setVideoStatus',
				'change #tva-course-comments': 'setCommentStatus'
			},
			video_view: null,
			initial_comment_status: null,
			comment_status_view: null,
			initialize: function () {
				this.initial_comment_status = this.model.get( 'comment_status' );
				this.listenTo( ThriveApp.globals.levels, 'add', this.renderLevels );
				this.listenTo( ThriveApp.globals.levels, 'remove', this.renderLevels );
				this.listenTo( ThriveApp.globals.settings, 'change:comment_status', this.onCommentStatusChange );

				this.listenTo( ThriveApp.globals.labels, 'add', this.renderLabels() );
				this.listenTo( ThriveApp.globals.labels, 'remove', this.renderLabels );

				$( 'body' ).on( 'click', function ( e ) {
					var container = $( '.tva-course-level-values' );
					var labels_container = $( '.tva-course-label-values' );
					var btn = $( '.tva-expand-levels' );

					if ( ! container.is( e.target ) && container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						container.slideUp();
					}

					if ( ! labels_container.is( e.target ) && labels_container.has( e.target ).length === 0 && ! btn.is( e.target ) ) {
						labels_container.slideUp();
					}
				} );
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				this.renderCoverImage();
				this.renderTopics();
				this.renderLevels();
				this.renderLabels();
				this.renderRoles();
				this.renderCommentStatus();
				this.renderMembershipOptions();
				this.renderMCE();

				if ( this.model.get( 'video_status' ) == true ) {
					this.renderVideo();
				}

				var model = this.model,
					self = this,
					color = this.$( '.tva-label-add-color' ).val() ? this.$( '.tva-label-add-color' ).val() : '#58a545',
					$post_search = this.$( '#tva-course-author' );

				new ThriveApp.PostSearch( $post_search, {
					url: ThriveApp.routes.courses + '/search_users/',
					type: 'POST',
					select: function ( event, ui ) {
						self.tvd_clear_errors();
						model.get( 'author' ).ID = parseInt( ui.item.id );
						model.get( 'author' ).user_login = ui.item.label;
						model.get( 'author' ).url = ui.item.url;
					},
					search: function () {
						self.tvd_clear_errors();
						model.set( {author: {}} );
					},
					open: function () {
						model.set( {author: {}} );
					},
					fetch_single: model.get( 'author' )
				} );

				this.$el.find( '.tva-label-add-color' ).spectrum( {
					color: '#58a545',
					containerClassName: 'tva-color-picker',
					allowEmpty: false,
					showInitial: true,
					showButtons: true,
					chooseText: "Apply",
					cancelText: "Cancel",
					showInput: true,
					preferredFormat: "hex",
					hide: function ( color ) {
						var _color = color ? color.toHexString() : '';
						model.set( {label_color: _color} );
						model.save();
					}
				} );

				$( 'body' ).trigger( 'course_fields', this.model );

				this.$el.addClass( 'tva-course-editor' );
				this.bind_zclip();
				TVE_Dash.hideLoader();
				return this;
			},
			onCommentStatusChange: function () {
				this.model.set( {comment_status: ThriveApp.globals.settings.get( 'comment_status' )} );

				var children = this.model.hasChildren();

				if ( children ) {
					this.model.get( children ).updateValues( {comment_status: this.model.get( 'comment_status' )}, true )
				}

				this.comment_status_view.render();
			},
			renderCommentStatus: function () {
				this.comment_status_view = new ThriveApp.views.CommentStatusView( {
					el: this.$( '#tva-comment-status-container' ),
					model: this.model
				} );

				this.comment_status_view.render();
			},
			renderMCE: function () {
				var self = this;
				setTimeout( function () {
					var editors = [ 'tva-course-description', 'tva-course-message' ];
					ThriveApp.util.clearMCEEditor( editors );
					self.editorInit( 'tva-course-description', self.model.get( 'description' ) );
					self.editorInit( 'tva-course-message', self.model.get( 'message' ) );
					TVE_Dash.materialize( self.$el );
				}, 0 );

			},
			removeCoverImage: function ( e ) {
				e.stopPropagation();

				this.model.set( {cover_image: ''} );
				this.renderCoverImage();

			},
			renderCoverImage: function () {
				var state = this.model.get( 'cover_image' ) ? 'normal' : 'empty';

				if ( ! ThriveApp.views[ 'Cover' + ThriveApp.util.upperFirst( state ) + 'State' ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'Cover' + ThriveApp.util.upperFirst( state ) + 'State' ]( {
					el: this.$( '.tva-cover-image' ),
					model: this.model
				} );

				view.render();
			},
			renderTopics: function () {
				ThriveApp.globals.topics.each( this.renderTopic, this );
			},
			renderTopic: function ( topic ) {
				this.topic = new ThriveApp.views.Topic( {
					model: topic,
					el: this.$( '#tva-course-topic' ),
					selected: this.model.get( 'topic' )

				} );

				this.topic.render();
			},
			renderLevels: function () {
				this.level_controls = new ThriveApp.views.LevelControls( {
					model: this.model,
					el: this.$( '.tva-course-level-controls' )
				} );
				this.level_controls.render();

				this.$( '.tva-course-levels-list' ).empty();
				ThriveApp.globals.levels.each( this.renderLevel, this );
			},
			renderLabels: function () {
				this.label_controls = new ThriveApp.views.LabelsControls( {
					model: this.model,
					el: this.$( '.tva-course-label-controls' )
				} );

				this.label_controls.render();

				this.$( '.tva-course-labels-list' ).empty();
				ThriveApp.globals.labels.each( this.renderLabel, this );
			},
			renderLevel: function ( level ) {
				this.level = new ThriveApp.views.Level( {
					tagName: 'li',
					model: level,
					selected: this.model.get( 'level' )

				} );

				this.$( '.tva-course-levels-list' ).append( this.level.render().$el );
			},
			renderLabel: function ( label ) {
				this.label = new ThriveApp.views.Label( {
					tagName: 'li',
					model: label,
					attributes: {
						"data-id": label.get( 'ID' )
					},
					className: 'tva-course-label-item tvd-pointer',
					selected: this.model.get( 'label' )
				} );

				this.$( '.tva-course-labels-list' ).append( this.label.render().$el );
			},
			renderRoles: function () {
				if ( ThriveApp.globals.settings.get( 'membership_plugin' ).length === 0 ) {
					ThriveApp.globals.roles.each( this.renderRole, this );
				}
			},
			renderRole: function ( role ) {
				var roleView = new ThriveApp.views.Role( {
					el: this.$( '#tva-course-roles' ),
					model: role,
					checked: this.model.get( 'roles' )
				} );

				roleView.render();
			},
			renderMembershipOptions: function () {
				var self = this,
					membership_plugins = new ThriveApp.collections.MembershipsCollection( ThriveApp.data.settings.membership_plugin );
				if ( membership_plugins.length > 0 ) {
					/**
					 * Render membership options for each membership plugin the user has active
					 */
					membership_plugins.each( function ( membership_plugin ) {
						var tag = membership_plugin.get( 'tag' ),
							view = null;

						/**
						 * Instantiate the base for the tag if no view was created for the membership plugin
						 */
						if ( ! ThriveApp.views[ ThriveApp.util.upperFirst( tag ) + 'MembershipOptions' ] ) {
							view = new ThriveApp.views.MembershipOptionsBase( {
								model: self.model,
								settings: membership_plugin,
								tag: tag
							} );
						} else {
							view = new ThriveApp.views[ ThriveApp.util.upperFirst( tag ) + 'MembershipOptions' ]( {
								model: self.model,
								settings: membership_plugin,
								tag: tag
							} );
						}

						self.$( '.tva-membership-options' ).append( view.render().$el );
					}, membership_plugins );

				}
			},
			renderVideo: function () {
				var media_model = new ThriveApp.models.MediaSelect( this.model.get( 'term_media' ) );

				if ( this.video_view ) {
					this.video_view.remove();
				}

				this.video_view = new ThriveApp.views.MediaSelectView( {
					model: media_model,
					source: this.model
				} );

				this.$( '#tva-course-video' ).show();
				this.$( '#tva-course-video' ).append( this.video_view.render().$el );

				return this;
			},
			setVideoStatus: function ( e ) {
				var value = e.currentTarget.checked;
				this.model.set( {video_status: value} );

				if ( value == true ) {
					this.renderVideo();
				} else {
					this.video_view.remove();
					this.$( '#tva-course-video' ).hide()
				}

				return this;
			},
			setTitle: function () {
				var value = this.$( '#tva-course-title' ).val();
				this.model.set( {name: value} );
				if ( value ) {
					$( '.tva-main-course-title' ).text( value );
					var slug = ThriveApp.util.slugify( value, "-" ).toLowerCase();
					this.model.set( {slug: slug} );
					this.$( '#tva-course-slug' ).val( slug );
					this.$( '.tva-slug-url' ).val( ThriveApp.globals.settings.get( 'apprentice_url' ) + slug );
				}
			},
			setSlug: function ( e ) {
				this.model.set( {slug: e.currentTarget.value} );
			},
			setTopic: function () {
				this.model.set( {topic: parseInt( this.$( '.tva-course-topic' ).val() )} );
				this.model.set( {topic_name: this.model.getTopicName()} );
				this.model.set( {topic_color: this.model.getTopicColor()} );
			},
			setRole: function ( e ) {
				var roles = this.model.get( 'roles' ),
					key = $( e.currentTarget ).val();

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					roles[ key ] = 1;
				} else {
					delete roles[ key ];
				}

				this.model.set( {roles: roles} );
			},
			setExcluded: function ( e ) {
				this.model.set( {excluded: e.currentTarget.value} );
			},
			setCommentStatus: function ( e ) {
				var comment_status = $( e.currentTarget ).val() === 'open' ? 'open' : 'closed',
					lesson_template = this.model.get( 'lesson_template' );

				lesson_template.comment_status = comment_status;

				this.model.set( {comment_status: comment_status, lesson_template: lesson_template} );
			},
			toggleLevels: function ( e ) {
				e.stopPropagation();
				this.$( '.tva-course-level-values' ).slideToggle();
			},
			toggleLabels: function ( e ) {
				e.stopPropagation();
				this.$( '.tva-course-label-values' ).slideToggle();
			},
			toggleLoggedIn: function ( e ) {
				this.$( '.tva-logged-in-extra' ).slideToggle();

				$( e.target ).is( ':checked' ) ? this.model.set( {logged_in: 1} ) : this.model.set( {logged_in: 0} );
			},
			toggleSettings: function () {
				var element = this.$( '.tva-advanced-settings' ),
					style = element.css( 'display' );
				element.slideToggle();

				var text = style == 'none' ? ThriveApp.t.HideAdvanced : ThriveApp.t.ShowAdvanced;
				this.$( '.tva-change-settings' ).text( text );
				this.$( '.ta-arrow-upd' ).toggleClass( 'ta-arr-inv' );
			},
			addLevel: function () {

				var value = this.$( '#tva-add-level' ).val(),
					element = this.$( '#tva-add-level' ),
					model = new ThriveApp.models.Level( {name: value} );

				if ( ! model.isValid() ) {
					return this.tvd_show_errors( model );
				}

				TVE_Dash.showLoader();
				var xhr = model.save();
				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						model.set( 'ID', response.ID );
						ThriveApp.globals.levels.add( model );
						element.val( '' );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			},
			addLabel: function () {

				var value = this.$( '#tva-add-label' ).val(),
					self = this,
					element = this.$( '#tva-add-label' );

				if ( ! value ) {
					TVE_Dash.err( ThriveApp.t.InvalidLabelName );
					return;
				}

				TVE_Dash.showLoader();
				setTimeout( function () {
					var color = $( '.tva-label-add-color' ).val() ? $( '.tva-label-add-color' ).val() : '#58a545',
						model = new ThriveApp.models.Label( {
							title: value,
							label: value,
							color: color,
							label_color: color,
							label_name: value
						} ),
						xhr = model.save();

					if ( ! model.isValid() ) {
						return this.tvd_show_errors( model );
					}

					if ( xhr ) {
						xhr.done( function ( response, status, options ) {
							model.set( 'ID', response.ID );
							self.model.set( {
								label: model.get( 'ID' ),
								color: color,
								label_color: color,
								label_name: value
							} );
							ThriveApp.globals.labels.add( model );
							element.val( '' );
							self.renderLabels();
						} );
						xhr.error( function ( errorObj ) {
							TVE_Dash.err( errorObj.responseText );
						} );
						xhr.always( function () {
							TVE_Dash.hideLoader();
						} );
					}
				}, 900 );

			},
			selectLevel: function ( e ) {
				var id = parseInt( $( e.target ).attr( 'data-id' ) ),
					level = ThriveApp.globals.levels.findWhere( {ID: id} );
				this.model.set( {level: parseInt( id )} );
				this.model.set( {level_name: level.get( 'name' )} );

				this.renderLevels();
			},
			selectLabel: function ( e ) {
				var id = parseInt( $( e.target ).attr( 'data-id' ) ),
					label = ThriveApp.globals.labels.findWhere( {ID: id} );

				this.model.set( {label: parseInt( id )} );
				this.model.set( {label_name: label.get( 'title' )} );
				this.model.set( {label_color: label.get( 'color' )} );

				this.renderLabels();
			},
			addCoverImage: function () {
				var frame,
					self = this;

				// If the media frame already exists, reopen it.
				if ( frame ) {
					frame.open();
					return;
				}

				// Create a new media frame
				frame = wp.media( {
					title: 'Select or Upload a Cover Image',
					button: {
						text: 'Use this Image'
					},
					library: {type: 'image'},
					multiple: false  // Set to true to allow multiple files to be selected
				} );

				// When an image is selected in the media frame...
				frame.on( 'select', function () {
					var attachment = frame.state().get( 'selection' ).first().toJSON();
					self.model.set( {cover_image: attachment.url} );
					self.renderCoverImage();
				} );

				// Finally, open the modal on click
				frame.open();
			},
			editorInit: function ( id, content ) {
				if ( typeof tinymce === 'undefined' || ! tinymce ) {
					return;
				}
				var self = this,
					mce_reinit = ThriveApp.util.build_mce_init( {
						mce: window.tinyMCEPreInit.mceInit[ 'tve_tinymce_tpl' ],
						qt: window.tinyMCEPreInit.qtInit[ 'tve_tinymce_tpl' ]
					}, id );

				tinyMCEPreInit.mceInit = $.extend( tinyMCEPreInit.mceInit, mce_reinit.mce_init );
				tinyMCEPreInit.mceInit[ id ].setup = function ( editor ) {
					editor.on( 'init', function () {
						editor.setContent( content );
					} );
					editor.on( 'change', function ( e ) {
						var description = tinymce.get( 'tva-course-description' ),
							message = tinymce.get( 'tva-course-message' );

						self.model.set( {description: description.getContent()} );
						self.model.set( {message: message.getContent()} );
					} );
				};
				tinymce.init( tinyMCEPreInit.mceInit[ id ] );

			},
			setDescription: function ( e ) {
				this.model.set( {description: $( e.currentTarget ).val()} );
			},
			setMessage: function ( e ) {
				this.model.set( {message: $( e.currentTarget ).val()} );
			},
			getPreviewUrl: function () {
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'GET',
					url: ThriveApp.routes.settings + '/get_preview_url/'
				} ).done( function ( response, status, options ) {
					if ( response ) {
						ThriveApp.globals.settings.set( {preview_url: response} );
					}
				} )
			},
			save: function () {
				this.tvd_clear_errors();
				this.initial_comment_status !== this.model.get( 'comment_status' ) ? this.model.set( {comment_status_changed: true} ) : '';
				if ( ! this.model.isValid() ) {
					return this.tvd_show_errors( this.model );
				}

				if ( ( this.model.get( 'video_status' ) == true ) && this.video_view && ! this.video_view.model.isValid() ) {
					return this.tvd_show_errors( this.video_view.model );
				}

				if ( this.video_view ) {
					this.model.set( {
						term_media: {
							media_type: this.video_view.model.get( 'media_type' ),
							media_url: this.video_view.model.get( 'media_url' ),
							media_extra_options: this.video_view.model.get( 'media_extra_options' )
						}
					} );
				}

				if ( ThriveApp.globals.settings.get( 'membership_plugin' ).length === 0 && this.model.get( 'logged_in' ) && Object.keys( this.model.get( 'roles' ) ).length === 0 ) {
					TVE_Dash.err( ThriveApp.t.NoRolesSelected );
					return;
				}

				if ( ! this.model.get( 'description' ) ) {
					TVE_Dash.err( ThriveApp.t.InvalidDescription );
					return;
				}

				var exclude_value = this.model.get( 'excluded' ),
					reg = /^\d+$/,
					exclude_num = reg.test( exclude_value );

				if ( exclude_num === false ) {
					TVE_Dash.err( ThriveApp.t.InvalidNumberType );
					return;
				}

				/**
				 * Let's check if the membership plunghis are selected
				 */
				var valid = this.model.validateMemberships();

				if ( ! valid ) {
					TVE_Dash.err( ThriveApp.t.noProductSelected );
					return;
				}

				var collection = this.model.hasChildren();

				if ( collection ) {
					this.model.set( {post_ids: this.model.get( collection ).listValue( 'ID' )} );
				}

				TVE_Dash.showLoader();
				var route = Backbone.history.getFragment().indexOf( 'add_course' ) !== - 1 ? 'add' : 'edit',
					self = this,
					xhr = self.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						self.model.set( 'ID', response.term_id );
						self.model.set( 'url', response.url );

						var children = self.model.hasChildren();

						if ( children ) {
							self.model.get( children ).updateValues( {comment_status: self.model.get( 'comment_status' )}, true )
						}

						ThriveApp.globals.courses.add( self.model, {at: ThriveApp.globals.courses.length} );

						if ( ( ThriveApp.globals.settings.get( 'preview_url' ) === false ) ) {
							self.getPreviewUrl();
						}

						if ( route == 'add' ) {
							ThriveApp.router.navigate( '#course/' + response.term_id, {trigger: true} );
						} else {
							$( '.tva-course-section' ).trigger( 'tva_show_details' );
						}

					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
					} );
					xhr.always( function () {
						$( '.tva-course-section' ).removeClass( 'tva-course-editor' )
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		ThriveApp.views.CommentStatusView = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/course-comment' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				TVE_Dash.materialize( this.$el );
			}
		} );

		/**
		 * Render video url view
		 */
		ThriveApp.views.CourseVideoUrl = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/video-url-options' ),
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				TVE_Dash.materialize( this.$el );

				return this;
			}
		} );

		/**
		 * View for rendering video options for the course
		 */
		ThriveApp.views.ChangeVideoTypeView = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/video-options' ),
			events: {
				'change .tva-course-video-type': 'changeVideoType',
				'change #tva-course-featured-video': 'renderExtraOptions'
			},
			render: function () {
				if ( this.model.get( 'video_status' ) == true ) {
					this.$el.html( this.template( {model: this.model} ) );
					this.renderExtraOptions( this.model.get( 'video_type' ) );
					this.renderInputVideoUrl();
				}

				return this;
			},
			changeVideoType: function ( e ) {
				var value = e.currentTarget.value;
				this.model.set( {video_type: value, video_url: '', video_extra: {}} );
				this.renderExtraOptions( value );
				this.renderInputVideoUrl();
			},
			renderExtraOptions: function ( video_type ) {
				this.$( '.tva-course-video-extra-options' ).empty();
				if ( video_type && this.model.get( 'video_status' ) == true ) {
					if ( ! ThriveApp.views[ 'CourseVideo' + ThriveApp.util.upperFirst( video_type ) + 'Options' ] ) {
						return;
					}

					var view = new ThriveApp.views[ 'CourseVideo' + ThriveApp.util.upperFirst( video_type ) + 'Options' ]( {
						el: this.$( '.tva-course-video-extra-options' ),
						model: this.model
					} );

					view.render();
				}
			},
			renderInputVideoUrl: function () {
				var view = new ThriveApp.views.CourseVideoUrl( {
					el: this.$( '.tva-video-url' ),
					model: this.model
				} );
				view.render();
			}
		} );

		/**
		 * A base for all membership plugins
		 */
		ThriveApp.views.MembershipOptionsBase = ThriveApp.views.Base.extend( {
			className: 'tva-membership-plugin-wrapper',
			template: TVE_Dash.tpl( 'course/edit/base-options' ),
			events: {
				'change .tva-membership-change': 'setMembership',
				'change .tva-bundle-change': 'setBundle'
			},
			initialize: function ( options ) {
				this.settings = options.settings;
				this.tag = options.tag;
				this.$el.addClass( 'tva-' + this.tag + 'wrapper' );
			},
			render: function () {
				if ( ! ( this.settings.get( 'membership_levels' ) instanceof Backbone.Collection ) ) {
					this.settings.set( {membership_levels: new ThriveApp.collections.MembermouseCollection( this.settings.get( 'membership_levels' ) )} );
				}
				if ( ! ( this.settings.get( 'bundles' ) instanceof Backbone.Collection ) ) {
					this.settings.set( {bundles: new ThriveApp.collections.MembermouseCollection( this.settings.get( 'bundles' ) )} );
				}

				this.$el.html( this.template( {model: this.settings} ) );

				this.renderLevels();
				this.renderBundles();

				return this;
			},
			setMembership: function ( e ) {
				var ids = this.model.get( 'membership_ids' )[ this.tag ],
					value = $( e.currentTarget ).val();

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = this.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}
				this.model.get( 'membership_ids' )[ this.tag ] = ids;
			},
			setBundle: function ( e ) {
				var ids = this.model.get( 'bundle_ids' )[ this.tag ],
					value = $( e.currentTarget ).val();

				if ( $( e.currentTarget ).prop( 'checked' ) ) {
					ids.push( parseInt( value ) );
				} else {
					var key = this.getKeyByValue( ids, value );
					if ( key ) {
						ids.splice( key, 1 );
					}
				}

				this.model.get( 'bundle_ids' )[ this.tag ] = ids;
			},
			getKeyByValue: function ( array, value ) {
				for ( var key in array ) {
					if ( array[ key ] === parseInt( value ) ) {
						return key;
					}
				}
			},
			renderLevels: function () {
				if ( this.settings.get( 'membership_levels' ).length > 0 ) {
					this.settings.get( 'membership_levels' ).each( this.renderLevel, this );
				} else {
					this.$( '#tva-' + this.tag + '-levels' ).append( '<p>' + ThriveApp.t.NoMembershipLevels + '</p>' );
				}
			},
			renderLevel: function ( level ) {
				var view = null;
				/**
				 * if we don't have a specific view for the levels for this memberhip plugin then render a base
				 */
				if ( ! ThriveApp.views[ ThriveApp.util.upperFirst( this.tag ) + 'Level' ] ) {
					view = new ThriveApp.views.BaseLevel( {
						model: level,
						checked: this.model.get( 'membership_ids' )[ this.tag ],
						tag: this.tag
					} );

				} else {
					view = new ThriveApp.views[ ThriveApp.util.upperFirst( this.tag ) + 'Level' ]( {
						model: level,
						checked: this.model.get( 'membership_ids' )[ this.tag ],
						tag: this.tag
					} );
				}

				this.$( '#tva-' + this.tag + '-levels' ).append( view.render().$el )

			},
			renderBundles: function () {
				if ( this.settings.get( 'bundles' ).length > 0 ) {
					this.settings.get( 'bundles' ).each( this.renderBundle, this );
				} else {
					this.$( '#tva-' + this.tag + '-bundles' ).append( '<p>' + ThriveApp.t.NoMembershipBundles + '</p>' );
				}
			},
			renderBundle: function ( bundle ) {
				var view = null;
				/**
				 * if we don't have a specific view for the bundles for this memberhip plugin then render a base
				 */
				if ( ! ThriveApp.views[ ThriveApp.util.upperFirst( this.tag ) + 'Bundle' ] ) {
					view = new ThriveApp.views.BaseBundle( {
						model: bundle,
						checked: this.model.get( 'bundle_ids' )[ this.tag ],
						tag: this.tag
					} );

				} else {
					view = new ThriveApp.views[ ThriveApp.util.upperFirst( this.tag ) + 'Bundle' ]( {
						model: bundle,
						checked: this.model.get( 'bundle_ids' )[ this.tag ],
						tag: this.tag
					} );
				}

				this.$( '#tva-' + this.tag + '-bundles' ).append( view.render().$el )
			}
		} );

		/**
		 * Render one Base level checkbox
		 */
		ThriveApp.views.BaseLevel = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/base-level' ),
			className: 'tva-base-level',
			initialize: function ( options ) {
				this.checked = options.checked;
				this.tag = options.tag;
				this.$el.addClass( 'tva-' + this.tag + '-level' )
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, checked: this.checked, tag: this.tag} ) );

				return this;
			}
		} );

		ThriveApp.views.SendowlLevel = ThriveApp.views.BaseLevel.extend( {
			template: TVE_Dash.tpl( 'course/edit/sendowl-level' )
		} );

		/**
		 * Render one Base bundle checkbox
		 */
		ThriveApp.views.BaseBundle = ThriveApp.views.Base.extend( {
			template: TVE_Dash.tpl( 'course/edit/base-bundle' ),
			className: 'tva-base-bundle',
			initialize: function ( options ) {
				this.checked = options.checked;
				this.tag = options.tag;
				this.$el.addClass( 'tva-' + this.tag + '-bundle' )
			},
			render: function () {
				this.$el.html( this.template( {model: this.model, checked: this.checked, tag: this.tag} ) );

				return this;
			}
		} );

		ThriveApp.views.SendowlBundle = ThriveApp.views.BaseBundle.extend( {
			template: TVE_Dash.tpl( 'course/edit/sendowl-bundle' ),
		} );

		/**
		 * Membermouse Options view extended from the Membership options base
		 * (we keep this just to keep track of our plugins, the base would be just as good)
		 */
		ThriveApp.views.MembermouseMembershipOptions = ThriveApp.views.MembershipOptionsBase.extend( {} );

		/**
		 * Memberpress Options view extended from the Membership options base
		 */
		ThriveApp.views.MemberpressMembershipOptions = ThriveApp.views.MembershipOptionsBase.extend( {
			template: TVE_Dash.tpl( 'course/edit/memberpress-options' )
		} );

		/**
		 * Wishlist Options view extended from the Membership options base
		 */
		ThriveApp.views.WishlistMembershipOptions = ThriveApp.views.MembershipOptionsBase.extend( {
			template: TVE_Dash.tpl( 'course/edit/wishlist-options' )
		} );

		/**
		 * Sendowl options
		 */
		ThriveApp.views.SendowlMembershipOptions = ThriveApp.views.MembershipOptionsBase.extend( {
			template: TVE_Dash.tpl( 'course/edit/sendowl-options' ),
			events: function () {
				return _.extend( {}, ThriveApp.views.MembershipOptionsBase.prototype.events, {
					'click .tva-refresh-memberships': 'refreshMembership'
				} );
			},
			render: function () {
				if ( ! ( this.settings.get( 'membership_levels' ) instanceof Backbone.Collection ) ) {
					this.settings.set( {membership_levels: new ThriveApp.collections.MembermouseCollection( this.settings.get( 'membership_levels' ) )} );
				}
				if ( ! ( this.settings.get( 'bundles' ) instanceof Backbone.Collection ) ) {
					this.settings.set( {bundles: new ThriveApp.collections.MembermouseCollection( this.settings.get( 'bundles' ) )} );
				}

				var disabled = ! ThriveApp.globals.settings.get( 'checkout_page_url' ) || ! ThriveApp.globals.settings.get( 'account_keys' ).secret;

				this.$el.html( this.template( {model: this.settings, disabled: disabled} ) );

				this.renderLevels();
				this.renderBundles();

				return this;
			},
			renderLevels: function () {
				if ( this.settings.get( 'membership_levels' ).length > 0 ) {
					this.settings.get( 'membership_levels' ).each( this.renderLevel, this );
				} else {
					this.$( '#tva-' + this.tag + '-levels' ).append( '<p>' + ThriveApp.t.NoProducts + '</p>' );
				}
			},

			refreshMembership: function () {

				$( '.tva-membership-options' ).addClass( 'tva-sendowl-spinner' );

				var self = this;

				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce
					},
					type: 'GET',
					url: ThriveApp.routes.settings + '/refresh_sendowl_memberships/'
				} ).done( function ( response, status, options ) {

					if ( response.error ) {
						TVE_Dash.err( response.error, 5000 );

						return;
					}

					_.each( ThriveApp.data.settings.membership_plugin, function ( value, index ) {
						if ( value.tag === 'sendowl' ) {
							ThriveApp.data.settings.membership_plugin[ index ] = response
						}
					} );

					self.settings.set( {membership_levels: new ThriveApp.collections.MembermouseCollection( response.membership_levels )} );
					self.settings.set( {bundles: new ThriveApp.collections.MembermouseCollection( response.bundles )} );

					ThriveApp.globals.settings.set( {
						sendowl_bundles: new ThriveApp.collections.Base( response.bundles ),
						sendowl_products: new ThriveApp.collections.Base( response.membership_levels )
					} );

					self.render();
				} ).always( function () {
					$( '.tva-membership-options' ).removeClass( 'tva-sendowl-spinner' );
				} );
			}
		} );

		/**
		 * TODO: If this is an UTILITY view, it should be moved in _utils.js
		 * View to provide helper methods for
		 */
		ThriveApp.views.CourseUtils = ThriveApp.views.Base.extend( {
			render: function () {
				return this;
			},
			isCollection: function ( collection ) {
				return ( typeof collection === 'object' ) && ( collection instanceof Backbone.Collection );
			},
			isCollectionEmpty: function ( collection ) {
				return this.isCollection( collection ) && ( collection.length === 0 );
			},
			getPlural: function ( str ) {
				return str + 's';
			},
			hasContent: function ( model ) {
				var self = this;

				model = model ? model : this.model;

				return ! ! _.find( this.allowedItems, function ( item ) {
					return ! self.isCollectionEmpty( model.get( self.getPlural( item ) ) );
				} );
			},
			hasPublishedContent: function () {
				var self = this;

				return ! ! _.find( this.allowedItems, function ( allowed_item ) {
					return self.hasPublishedItem( self.model.get( self.getPlural( allowed_item ) ) );
				} );
			},
			hasUnpublishedContent: function () {
				var self = this;

				return ! ! _.find( this.allowedItems, function ( allowed_item ) {
					return self.hasUnpublishedItem( self.model.get( self.getPlural( allowed_item ) ) );
				} );
			},
			hasPublishedItem: function ( collection ) {
				return this.isCollection( collection ) && ! ! collection.findWhere( {post_status: 'publish'} );
			},
			hasUnpublishedItem: function ( collection ) {
				return this.isCollection( collection ) && ! ! collection.findWhere( {post_status: 'draft'} );
			},
			isCourse: function ( model ) {
				return model.get( 'taxonomy' ) === 'tva_courses';
			},
			isModule: function ( model ) {
				return model && model.get( 'post_type' ) === 'tva_module';
			},
			isChapter: function ( model ) {
				return model && model.get( 'post_type' ) === 'tva_chapter';
			},
			isLesson: function ( model ) {
				return model && model.get( 'post_type' ) === 'tva_lesson';
			}
		} );

		/**
		 * Edit course and lessons view
		 */
		ThriveApp.views.ManageCourse = ThriveApp.views.CourseUtils.extend( {
			className: 'tvd-container',
			template: TVE_Dash.tpl( 'course/manage/manage' ),
			events: {
				'click .tva-edit-course': 'renderCourse',
				'click .tva-edit-course-structure': function () {
					$( '#tva-manage-course-wrapper' ).addClass( 'tva-editor-visible' ).animate( {right: 0}, 500 );
				},
				'click .tva-cancel-edit': 'cancelEdit',
				'click .tva-publish-course': 'changeCourseStatus',
				'click .tva-delete-lessons': 'bulkDeleteItems',
				'click #tva-select-all-items': 'selectAll',
				'click .tva-publish-lessons': 'bulkPublishItems',
				'click .tva-unpublish-lessons': 'bulkUnPublishItems',
				'click .tva-move-items': 'bulkMoveItems',
				'click .tva-add-content': 'addContent',
				'click .tva-group-into-chapter': 'bulkChapterGroup',
				'click .tva-group-into-module': 'bulkModuleGroup'
			},
			course_edit: '',
			content: null,
			allowedItems: [ 'module', 'chapter', 'lesson' ],
			select_all: false,
			initialize: function () {
				var self = this;

				this.model.on( 'tva_after_action', function ( args ) {
					self.afterAction( args );
					self.select_all = false;
					self.render();
				} );
				this.listenTo( this.model.get( 'lessons' ), 'remove', function () {
					if ( self.model.get( 'lessons' ).length === 0 ) {
						self.render();
					}
				} );

				this.model.on( 'reset_select_all', function () {
					self.select_all = false;
				} );
				this.model.on( 'tva_render_content', function () {
					self.model.checkStatus();
					self.render();
				} );

				this.model.on( 'render_bulk_actions', function () {
					self.renderBulkActions();
				} );

				this.listenTo( ThriveApp.globals.settings, 'change:comment_status', this.onCommentStatusChange );
			},
			render: function () {
				var $body = $( 'body' );

				/**
				 * Because we re-render content often on some browsers will appear a slideup-slidetotarget effect which is very ugly.
				 * That happens because the scroll is reset when the template is inserted into the DOM
				 * To avoid that we temporally add a min-height to body so the scroll won't be reset
				 */
				if ( this.pageHeight ) {
					$body.css( 'min-height', this.pageHeight );
				}

				this.$el.html( this.template( {model: this.model, select_all: this.select_all} ) );
				this.renderDetails();
				this.renderBulkActions();

				var self = this;

				this.$( '.tva-course-section' ).on( 'tva_show_details', function () {
					self.renderDetails();
				} );

				this.renderContent();
				window.addEventListener( 'scroll', ThriveApp.util.makeBulkActionsStiky );

				setTimeout( function () {
					var body = document.body,
						html = document.documentElement;

					self.pageHeight = Math.max( body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight );
				}, 100 );

				$body.css( 'min-height', 'auto' );

				return this;
			},
			renderContent: function () {
				if ( this.content ) {
					this.content.remove();
					this.content.$el.empty();
				}

				this.content = new ThriveApp.views.CourseContent( {
					model: this.model
				} );

				this.$( '.tva-course-container' ).html( this.content.render().$el );

				return this;
			},
			renderBulkActions: function () {
				this.getAllowedBulkActions();

				var view = new ThriveApp.views.RenderBulkActions( {
					model: ThriveApp.globals.bulk_action_model
				} );

				this.$( '.tva-bulk-actions' ).html( view.render().$el );
			},
			renderDetails: function () {
				var view = new ThriveApp.views.CourseDetails( {
					el: this.$( '.tva-course-section' ),
					model: this.model
				} );

				view.render();
			},
			renderCourse: function () {
				var self = this;
				this.$( '.tva-course-section' ).slideUp( 'slow', function () {

					if ( self.course_edit ) {
						self.course_edit.undelegateEvents();
						self.course_edit.$el.empty();
					}

					self.course_edit = new ThriveApp.views.CourseFields( {
						el: self.$( '.tva-course-section' ),
						model: self.model
					} );

					self.course_edit.render().$el.slideDown( 'slow' );
				} );
			},
			afterAction: function ( args ) {
				var fn = 'after' + ThriveApp.util.upperFirst( args.action );
				this[ fn ]( args );

				return this;
			},
			afterDelete: function ( args ) {
				var course = args.model;

				this.model.set( course );
				this.model.set_data();

				ThriveApp.globals.course_elements = ThriveApp.util.getCourseElements( this.model );

				ThriveApp.globals.bulk_action_model.clear().set( ThriveApp.globals.bulk_action_model.defaults );
				ThriveApp.globals.selected_items.reset();
			},
			afterPublish: function () {
				ThriveApp.globals.selected_items.updateValues( {checked: false} );
				ThriveApp.globals.selected_items.reset();
			},
			afterUnpublish: function () {
				ThriveApp.globals.selected_items.updateValues( {checked: false} );
				ThriveApp.globals.active_course.checkStatus();
				ThriveApp.globals.selected_items.reset();
			},
			afterMove: function ( args ) {
				var self = this;

				_.each( args.model, function ( model ) {
					var course = ThriveApp.globals.courses.findWhere( {ID: parseInt( model.ID )} );
					course.set( model );
					course.set_data();

					if ( course.get( 'ID' ) === self.model.get( 'ID' ) ) {
						self.model = course;
					}
				} );

				ThriveApp.globals.selected_items.updateValues( {checked: false} );
				ThriveApp.globals.bulk_action_model.clear().set( ThriveApp.globals.bulk_action_model.defaults );
				ThriveApp.globals.selected_items.reset();
			},
			afterGroup: function ( args ) {
				ThriveApp.globals.bulk_action_model.clear().set( ThriveApp.globals.bulk_action_model.defaults );
				ThriveApp.globals.active_course.set( JSON.parse( JSON.stringify( args.model ) ) );
				ThriveApp.globals.active_course.set_data();
				ThriveApp.globals.active_course_original.set( JSON.parse( JSON.stringify( args.model ) ) );
				ThriveApp.globals.active_course_original.set_data();
				ThriveApp.globals.selected_items.reset();

				ThriveApp.globals.course_elements = ThriveApp.util.getCourseElements( ThriveApp.globals.active_course );
			},
			onCommentStatusChange: function () {
				var children = this.model.hasChildren();

				if ( children ) {
					this.model.get( children ).updateValues( {comment_status: this.model.get( 'comment_status' )}, true )
				}

				this.model.set( {comment_status: ThriveApp.globals.settings.get( 'comment_status' )} );
			},
			cancelEdit: function ( e ) {
				e.preventDefault();
				if ( this.video_view ) {
					this.video_view.remove();
				}
				$( '.tva-course-section' ).removeClass( 'tva-course-editor' );
				this.renderDetails();
			},
			selectAll: function ( e ) {
				var checked = e.currentTarget.checked,
					children = this.model.hasChildren();

				this.model.get( children ).each( function ( model ) {
					ThriveApp.globals.selected_items.checkItems( model, checked );
				} );

				this.select_all = checked;
				this.render();
			},
			changeCourseStatus: function () {
				var current = this.model.get( 'status' ),
					childs = this.model.hasChildren();

				if ( current === 'publish' ) {
					this.model.set( {status: 'draft'} );
					this.save();
					return this;
				}

				if ( ! childs ) {
					TVE_Dash.err( ThriveApp.t.NumberLessons );
					return;
				}

				var collection = this.model.get( childs ),
					published_item = collection.findWhere( {post_status: 'publish'} );

				if ( ! published_item ) {
					if ( childs === 'modules' ) {
						TVE_Dash.err( ThriveApp.t.NoPublishedModule );
						return;
					}

					TVE_Dash.err( ThriveApp.t.NumberLessons );
					return;
				}

				this.model.set( {status: current == 'draft' ? 'publish' : 'draft'} );
				this.save();

				return this
			},
			bulkDeleteItems: function () {
				this.modal( ThriveApp.views.ConfirmBulkDelete, {
					model: ThriveApp.globals.bulk_action_model,
					'max-width': '600px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-delete-modal-style tvd-modal'
				} );
			},
			bulkPublishItems: function () {
				TVE_Dash.showLoader();
				var valid_ids = [];
				ThriveApp.globals.selected_items.each( function ( model ) {
					if ( model.get( 'type' ) === 'lesson' && model.get( 'post_status' ) === 'draft' ) {
						valid_ids.push( model.get( 'ID' ) );
					}
				} );

				if ( valid_ids.length > 0 ) {
					this.save_bulk_action( 'publish', valid_ids );
				} else {
					TVE_Dash.hideLoader();
				}
			},
			bulkUnPublishItems: function () {
				TVE_Dash.showLoader();
				var valid_ids = [];
				ThriveApp.globals.selected_items.each( function ( model ) {
					if ( model.get( 'type' ) === 'lesson' && model.get( 'post_status' ) === 'publish' ) {
						valid_ids.push( model.get( 'ID' ) );
					}
				} );


				if ( valid_ids.length > 0 ) {
					this.save_bulk_action( 'unpublish', valid_ids );
				} else {
					TVE_Dash.hideLoader();
				}
			},
			save_bulk_action: function ( action, data ) {
				var self = this;
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce,
						'X-TVA-Request-ID': ThriveApp.request_id
					},
					type: 'POST',
					url: ThriveApp.routes.courses + '/bulk_action/',
					data: {
						action: action,
						items: data,
						course_id: this.model.get( 'ID' )
					}
				} ).done( function ( response, status, options ) {
					var post_status = action === 'unpublish' ? 'draft' : 'publish';

					_.each( response, function ( id ) {
						var element = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: id} );

						if ( element ) {
							element.set( {post_status: post_status} );
						}
					} );

					self.model.trigger( 'tva_after_action', {action: action} );

				} ).error( function ( errorObj ) {
					TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
				} ).always( function () {
					TVE_Dash.hideLoader();
				} );
			},
			bulkMoveItems: function () {
				this.modal( ThriveApp.views.ConfirmBulkMove, {
					model: this.model,
					collection: ThriveApp.globals.courses,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal'
				} );
			},
			bulkChapterGroup: function () {
				this.modal( ThriveApp.views.ConfirmChapterGroup, {
					model: this.model,
					collection: this.collection,
					group_type: 'chapter',
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal'
				} );
			},
			bulkModuleGroup: function () {

				this.modal( ThriveApp.views.ConfirmModuleGroup, {
					model: this.model,
					collection: this.collection,
					group_type: 'module',
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal'
				} );
			},
			addContent: function () {
				this.modal( ThriveApp.views.AddContentModal, {
					model: this.model,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal tva-extra-space'
				} );

				return this;
			},
			getAllowedBulkActions: function () {
				var module = ThriveApp.globals.selected_items.findWhere( {post_type: 'tva_module'} ),
					chapter = ThriveApp.globals.selected_items.findWhere( {post_type: 'tva_chapter'} ),
					lesson = ThriveApp.globals.selected_items.findWhere( {post_type: 'tva_lesson'} ),
					all_selected_lessons = ThriveApp.globals.selected_items.where( {type: 'lesson'} ),
					chapter_lessons = [],
					model = ThriveApp.globals.bulk_action_model,
					full_chapters = true,
					full_modules = ThriveApp.globals.selected_items.where( {type: 'module'} ).length > 0;

				/**
				 * Check if we have full chapters, this way we could group them into a module
				 */
				_.each( ThriveApp.globals.selected_items.where( {type: 'chapter'} ), function ( chapter ) {
					var items = ThriveApp.globals.course_elements.get( 'lessons' ).where( {post_parent: chapter.get( 'ID' )} );

					/**
					 * chapter_lessons = [ ...chapter_lessons, ...items]; would have worked here well but the uglification doesn't recognize the syntax
					 */
					_.each( items, function ( item ) {
						chapter_lessons.push( item );
					} );
				} );

				if ( all_selected_lessons.length !== chapter_lessons.length ) {
					full_chapters = false;
				}


				if ( ! model ) {
					model = new ThriveApp.models.BulkActions();
				}
				model.set( {publish: ! ! ThriveApp.globals.selected_items.findWhere( {type: 'lesson'} )} );

				model.set( {
					module_group: (
						( ( ! module ) &&                           // no modules
						  ( ( ! ! lesson && ! chapter ) ||     // lessons only
						    ( ! ! chapter && full_chapters ) ) )    // chapters only
					)
				} );
				model.set( {
					chapter_group: (
						! module && ! chapter   // no modules and no chapters
					)
				} );
				model.set( {
					items_move: (
						( ( full_modules ) ||                           // no modules
						  ( ( ! ! lesson && ! chapter ) ||     // lessons only
						  ( ! ! chapter && full_chapters ) ) )    // chapters only
					)
				} );

				return this;
			},
			save: function () {
				this.tvd_clear_errors();
				//Todo: Some validation to the fields is required
				TVE_Dash.showLoader();
				var self = this,
					xhr = self.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						if ( response ) {
							self.renderDetails();

							ThriveApp.globals.available_settings.fetch().success( function ( response, status, options ) {
								ThriveApp.globals.settings.set( {
									iframe_url: ThriveApp.globals.settings.get( 'preview_url' ),
									preview_url: response.preview_url
								} );
							} );
						}
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Course content view
		 */
		ThriveApp.views.CourseContent = ThriveApp.views.ManageCourse.extend( {
			className: '',
			template: TVE_Dash.tpl( 'course/manage/course-content' ),
			$_list: null,
			newOrder: [],
			lesson: null,
			events: {
				'click .tva-add-c-lesson': 'addLesson',
				'click .tva-add-chapter-single': 'addChapter',
				'click .tva-add-module-single': 'addModule'
			},
			initialize: function () {
				var self = this;
				this.model.on( 'tva_apply_sortable', function () {
					self.applySortable();
				} )
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );
				this.renderContent();
				this.applySortable();

				return this;
			},
			renderContent: function () {
				var childs = this.model.hasChildren();

				if ( ! childs || ! ThriveApp.util.isCollection( this.model.get( childs ) ) ) {
					return
				}

				this.renderList();

				return this;
			},
			renderList: function () {
				if ( this.$_list ) {
					this.$_list.remove();
				}

				var childs = this.model.hasChildren();

				this.$_list = new ThriveApp.views.ItemsList( {
					model: this.model,
					collection: this.model.get( childs )
				} );

				this.$( '.tva-elements-container' ).prepend( this.$_list.render().$el );

				return this;
			},
			renderOne: function ( item ) {
				var type = item.get( 'item_type' ).toLowerCase();

				if ( ! ThriveApp.views[ item.get( 'item_type' ) ] ) {
					return;
				}

				var $lastItem = this.$el.find( '.tva-' + type + '-item' ).last(),
					view = new ThriveApp.views[ ThriveApp.util.upperFirst( type ) ]( {
						model: item,
						collection: this.model.get( this.getPlural( type ) )
					} );

				if ( $lastItem.length ) {
					$lastItem.after( view.render().$el );
				} else {
					this.$el.prepend( view.render().$el );
				}

				return this;
			},
			updateOrder: function () {
				var self = this;
				this.updated = true;

				TVE_Dash.showLoader();
				$.ajax( {
					headers: {
						'X-WP-Nonce': ThriveApp.nonce,
						'X-TVA-Request-ID': ThriveApp.request_id
					},
					type: 'POST',
					url: ThriveApp.routes.courses + '/update_posts_order/',
					data: {
						items: self.newOrder,
						course_id: self.model.get( 'ID' )
					}
				} ).done( function ( response, status, options ) {
					if ( response ) {
						self.model.set( response );
						self.model.set_data();

						if ( self.receiver_item_model ) {
							ThriveApp.globals.course_elements = ThriveApp.util.getCourseElements( ThriveApp.globals.active_course );
							ThriveApp.globals.course_elements.get( 'all' ).findWhere(
								{ID: self.receiver_item_model.get( 'ID' )} ).set( {expanded: true}
							);

							// expand the module if needed
							if ( self.receiver_item_model.get( 'post_parent' ) > 0 ) {
								self.model.get( 'modules' ).findWhere(
									{ID: self.receiver_item_model.get( 'post_parent' )} ).set( {expanded: true}
								)
							}

							self.receiver_item_model = null;
						}

						self.render();
					}
				} ).always( function () {
					self.newOrder = [];
					$( '.tvd-material-tooltip' ).hide();
					TVE_Dash.hideLoader();
				} );

				return this;
			},
			applySortable: function () {
				/**
				 * When called this should always be a ThriveApp.views.CourseContent instance
				 * and have ThriveApp.globals.active_course as model
				 */

				this._sortable();
				this._droppable();
			},
			_sortable: function () {
				var self = this;

				function show_position( event, ui ) {
					var $placeholder = $( ui.placeholder );

					self.position = $placeholder.prevAll().not( ui.item ).length + 1;

					$placeholder.html( "<div class='tvd-col tvd-s12 tva-inside-placeholder' style='height: 100px;'><span>" + self.position + (
						ThriveApp.t.n_suffix[ self.position ] ? ThriveApp.t.n_suffix[ self.position ] : ThriveApp.t.n_th
					) + ' ' + ThriveApp.t.position + "</span></div>" );
				}

				/**
				 * Sortable for modules
				 */
				this.$( '.tva-modules-section' ).sortable( {
					placeholder: 'tva-sortable-placeholder',
					items: '.tva-module-item',
					forceHelperSize: true,
					dropOnEmpty: true,
					refreshPositions: true,
					handle: '.tva-drag-module',
					update: _.bind( self.updateModulesOrder, this ),
					change: show_position,
					start: function ( event, ui ) {
						var expanded = self.model.get( 'modules' ).where( {expanded: true} );

						if ( expanded.length > 0 ) {
							_.each( expanded, function ( model ) {
								model.set( {expanded: false} );
								model.trigger( 'tva_render_content' );
							} );
						}
					},
					stop: function ( event, ui ) {
					}
				} );

				/**
				 * Sortable for chapters
				 */
				this.$( '.tva-chapters-section' ).sortable( {
					connectWith: this.$( '.tva-chapters-section' ),
					placeholder: 'tva-sortable-placeholder',
					items: '.tva-chapter-item',
					forceHelperSize: true,
					dropOnEmpty: true,
					refreshPositions: true,
					handle: '.tva-drag-chapter',
					update: _.bind( self.updateChaptersOrder, this ),
					change: show_position,
					start: function ( event, ui ) {
						self.updated = false;
						ui.helper.outerHeight();
						show_position( event, ui );
						$( 'body' ).addClass( 'tva-sorting' );

						self.$( '.tva-module-has-lessons' ).parent().addClass( 'tva-module-disabled' );

						var expanded = ThriveApp.globals.course_elements.get( 'chapters' ).where( {expanded: true} );

						if ( expanded.length > 0 ) {
							_.each( expanded, function ( model ) {
								model.set( {expanded: false} );
								model.trigger( 'tva_render_content' );
							} );
						}
					},
					stop: function ( event, ui ) {
						self.$( '.tva-module-has-lessons' ).parent().removeClass( 'tva-module-disabled' );
					}
				} );

				/**
				 * Sortable for lessons
				 */
				this.$( '.tva-lessons-section' ).sortable( {
					connectWith: this.$( '.tva-lessons-section' ),
					placeholder: 'ui-sortable-placeholder',
					items: '.tva-lesson-item',
					forceHelperSize: true,
					dropOnEmpty: true,
					refreshPositions: true,
					forcePlaceholderSize: true,
					handle: '.tva-drag-card',
					update: _.bind( self.updateLessonsOrder, this ),
					change: show_position,
					start: function ( event, ui ) {
						self.updated = false;

						self.$( '.tva-chapter-item' ).droppable( {
							accept: '.tva-lesson-item',
							hoverClass: 'tva-drop-hover',
							drop: function ( event, ui ) {
								var id = parseInt( self.$( event.target ).attr( 'data-id' ) );

								self.receiver_item_model = new ThriveApp.models.Chapter( JSON.parse( JSON.stringify(
									ThriveApp.globals.course_elements.get( 'chapters' ).findWhere( {ID: id} )
								) ) );
							},
							over: function ( event, ui ) {
								setTimeout( function () {
									self.eventTarget = event.target;

									if ( self.eventTarget === event.target ) {
										self.$( event.target ).find( '.tva-drag-chapter' ).remove();
										self.$( event.target ).find( '.tva-lessons-section' )
										    .css( 'min-height', self.$( ui.draggable ).height() );

										self.$( event.target )
										    .find( '.tva-elements-container' )
										    .slideDown( 'slow', function () {
											    self.$( '.tva-lessons-section' ).sortable( 'refresh' );
										    } );
									}
								}, self.model.hasChildren() === 'modules' ? 0 : 1000 );
							},
							out: function ( event, ui ) {
								self.receiver_item_model = null;
							}
						} );

						ui.helper.outerHeight();
						show_position( event, ui );
						$( 'body' ).addClass( 'tva-sorting' );
					},
					stop: function ( event, ui ) {
						if ( ! self.receiver_item_model ) {
							self.$( '.tva-lessons-section' ).sortable( 'cancel' );
							ThriveApp.globals.moved_item = null;
						}

						setTimeout( function () {
							$( 'body' ).removeClass( 'tva-sorting' );
						}, 200 );
					}
				} );
			},
			_droppable: function () {
				var self = this;

				this.$( '.tva-module-item' ).droppable( {
					accept: '.tva-chapter-item, .tva-lesson-item',
					hoverClass: 'tva-drop-hover',
					drop: function ( event, ui ) {
						var id = parseInt( self.$( event.target ).attr( 'data-id' ) );

						self.receiver_item_model = new ThriveApp.models.Module( JSON.parse( JSON.stringify(
							ThriveApp.globals.course_elements.get( 'modules' ).findWhere( {ID: id} )
						) ) );
					},
					over: function ( event, ui ) {
						self.eventTarget = event.target;

						setTimeout( function () {
							if ( self.eventTarget === event.target ) {
								var $elements = self.$( event.target ).find( '.tva-elements-container:first' ),
									slide = self.$( $elements ).hasClass( 'tva-module-has-lessons' ) ? self.isLesson( ThriveApp.globals.moved_item ) : true;

								self.$( $elements ).find( '.tva-childs-section' ).css( 'min-height', '10px' );

								if ( slide ) {
									self.$( event.target ).find( '.tva-drag-module' ).remove();
									self.$( $elements )
									    .css( 'min-height', self.$( ui.draggable ).height() )
									    .slideDown( 'slow', function () {
										    self.$( '.tva-chapters-section' ).sortable( 'refresh' );
										    self.$( '.tva-lessons-section' ).sortable( 'refresh' );
									    } );
								}
							}
						}, 1000 );
					}
				} );
			},
			addLesson: function () {
				this.lesson_model = new ThriveApp.models.Lesson( {
					order: this.model.get( 'lessons' ).length,
					course_id: this.model.get( 'term_id' ) ? this.model.get( 'term_id' ) : this.model.get( 'course_id' ),
					lesson_type: ThriveApp.globals.active_course.get( 'lesson_template' ).lesson_type,
					post_media: ThriveApp.globals.active_course.get( 'lesson_template' ).post_media,
					comment_status: ThriveApp.globals.active_course.get( 'lesson_template' ).comment_status
				} );

				this.afterAddLessonModel();

				this.modal( ThriveApp.views.LessonModalEditor, {
					model: this.lesson_model,
					collection: this.model.get( 'lessons' ),
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal',
					viewName: 'LessonEditor'
				} );
			},
			addChapter: function ( e ) {
				this.chapter_model = new ThriveApp.models.Chapter( {
					order: this.model.get( 'chapters' ).length,
					course_id: this.model.get( 'ID' ),
					comment_status: this.model.get( 'comment_status' )
				} );

				this.afterAddChapterModel();

				this.modal( ThriveApp.views.ChapterModalEditor, {
					model: this.chapter_model,
					collection: this.model.get( 'chapters' ),
					module: this.model,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal',
					viewName: 'ChapterEditor'
				} );
			},
			addModule: function () {
				this.module_model = new ThriveApp.models.Module( {
					order: this.model.get( 'modules' ).length,
					course_id: this.model.get( 'ID' ),
					comment_status: this.model.get( 'comment_status' )
				} );

				this.afterAddModuleModel();

				this.modal( ThriveApp.views.ModuleModalEditor, {
					model: this.module_model,
					collection: this.model.get( 'modules' ),
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal',
					viewName: 'ModuleEditor'
				} );
			},
			afterAddLessonModel: function () {
			},
			afterAddChapterModel: function () {
			},
			afterAddModuleModel: function () {
			},
			updateCollection: function ( model, collection, add ) {
				var values = [ 'ID', 'order', 'post_type', 'post_parent' ];

				model.get( collection ).remove( ThriveApp.globals.moved_item, {silent: true} );

				if ( add === true ) {
					model.get( collection ).add( ThriveApp.globals.moved_item, {at: this.position - 1} );
				}

				model.get( collection ).updateValues( {order: ''} );
				model.get( collection ).sort();

				this.newOrder = model.get( collection ).pluckValues( values ).concat( this.newOrder );
			},
			updateItemsOrder: function ( items ) {
				this.updateCollection( this.model, items, true );
				ThriveApp.globals.moved_item = null;
				this.updateOrder();
			},
			updateModulesOrder: function () {
				this.updateItemsOrder( 'modules' )
			},
			updateChaptersOrder: function () {
				if ( this.model.hasChildren() === 'chapters' ) {
					this.updateItemsOrder( 'chapters' );

					return;
				}

				/**
				 * Because we may move a chapter from one module to another sortable will trigger change event twice,
				 * for each item involved in move action. So here we ensure a single execution!
				 */
				if ( this.updated === true ) {
					this.updated = false;
					return;
				}

				if ( ! this.receiver_item_model ) {
					this.render();
					return;
				}

				/**
				 * Here we know for sure that we either move chapter from one module to another or within the same module
				 */
				if ( this.isModule( this.receiver_item_model ) ) {

					/**
					 * Cancel the sortable if a chapter is dropped over a module with lessons
					 */
					if ( this.isChapter( ThriveApp.globals.moved_item ) && this.receiver_item_model.hasChildren() === 'lessons' ) {
						this.$( '.tva-chapters-section' ).sortable( 'cancel' );
						return
					}

					if ( this.receiver_item_model.hasChildren() === 'chapters' && this.isLesson( ThriveApp.globals.moved_item ) ) {
						this.$( '.tva-lessons-section' ).sortable( 'cancel' );
						this.render();
						return;
					}

					var sender_module = this.model.get( 'modules' ).findWhere( {ID: ThriveApp.globals.moved_item.get( 'post_parent' )} );

					ThriveApp.globals.moved_item.set( {post_parent: this.receiver_item_model.get( 'ID' )} );

					if ( sender_module.get( 'ID' ) !== this.receiver_item_model.get( 'ID' ) ) {
						this.updateCollection( sender_module, 'chapters', false );
					}

					this.updateCollection( this.receiver_item_model, 'chapters', true );
				}

				ThriveApp.globals.moved_item = null;
				this.updateOrder();

				return this;
			},
			updateLessonsOrder: function () {
				if ( this.model.hasChildren() === 'lessons' ) {
					this.updateItemsOrder( 'lessons' );

					return;
				}

				if ( this.updated === true ) {
					this.updated = false;
					return;
				}

				if ( ! this.receiver_item_model ) {
					this.render();
					return;
				}

				/**
				 * Make sure that the receiver item is a chapter
				 */
				if ( this.receiver_item_model.hasChildren() === 'chapters' && ! this.isChapter( ThriveApp.globals.moved_item ) ) {
					this.$( '.tva-lessons-section' ).sortable( 'cancel' );
					this.render();
					return;
				}


				var sender_item = ThriveApp.globals.course_elements.get( 'all' ).findWhere( {ID: ThriveApp.globals.moved_item.get( 'post_parent' )} );

				ThriveApp.globals.moved_item.set( {post_parent: this.receiver_item_model.get( 'ID' )} );
				this.updateCollection( this.receiver_item_model, 'lessons', true );

				if ( sender_item.get( 'ID' ) !== this.receiver_item_model.get( 'ID' ) ) {
					this.updateCollection( sender_item, 'lessons', false );
				}

				ThriveApp.globals.moved_item = null;
				this.updateOrder();
			}
		} );

		/**
		 * View to render items list
		 */
		ThriveApp.views.ItemsList = ThriveApp.views.CourseContent.extend( {
			className: 'tva-items-list',
			expanded: null,
			events: {
				'click .tva-parent-item': 'animateItem',
				'click .tva-checkbox': 'setItems'
			},
			initialize: function () {
			},
			render: function () {
				var type = this.model.hasChildren();
				this.collection.each( this.renderOne, this );
				this.$el.addClass( 'tva-' + type + '-section' );

				if ( this.model.hasChildren() && ! this.model.get( 'taxonomy' ) ) {
					this.$el.addClass( 'tva-childs-section' );
				}

				return this
			},
			setItems: function ( e ) {
				e.stopPropagation();

				ThriveApp.globals.active_course.trigger( 'reset_select_all' );

				this.showActions( e );
			},
			animateItem: function ( e ) {
				e.stopPropagation();

				this.expanded = this.collection.findWhere( {expanded: true} );
				this.model.set( {expanded: ! this.model.get( 'expanded' )} );
				ThriveApp.globals.focused_item = this.model;
			},
			showActions: function ( e ) {
				/**
				 * Because we're doing actions on the wrapper
				 * the checked prop will always return the opposite of what we expect
				 * @type {*|void}
				 */
				var checked = ! $( e.currentTarget ).find( 'input' ).prop( 'checked' );
				$( e.currentTarget ).find( 'input' ).prop( 'checked', checked );

				if ( ! checked ) {
					$( '#tva-select-all-items' ).prop( 'checked', checked )
				}

				ThriveApp.globals.selected_items.checkItems( this.model, checked );

				ThriveApp.globals.active_course.trigger( 'render_bulk_actions' );

				ThriveApp.util.makeBulkActionsStiky();
			},
			setChecked: function () {
				this.$( '.tva-select-items:first' ).prop( 'checked', this.model.get( 'checked' ) );
			}
		} );

		/**
		 * View to render a single module item
		 */
		ThriveApp.views.Module = ThriveApp.views.ItemsList.extend( {
			template: TVE_Dash.tpl( 'course/module/module' ),
			className: 'tva-module-item tva-single-item',
			$_list: null,
			events: function () {
				return _.extend( {}, ThriveApp.views.ItemsList.prototype.events, {
					'click .tva-add-module-lesson': 'addLesson',
					'click .tva-add-module-chapter': 'addChapter',
					'click .tva-add-module-chapter-single': 'addChapter',
					'click .tva-edit-module': 'editModule',
					'click .tva-delete-module': 'deleteModule',
					'click .tva-edit-module-name': 'editTitle',
					'mousedown .tva-drag-module': function () {
						ThriveApp.globals.moved_item = this.model;
					},
					'click .tva-drag-module': function ( e ) {
						e.stopPropagation();
					}
				} );
			},
			initialize: function () {
				this.listenTo( this.model, 'change:checked', this.setChecked );
				this.$el.attr( 'data-id', this.model.get( 'ID' ) );
				this.$el.attr( 'id', 'tva-module-' + this.model.get( 'ID' ) );

				var self = this;

				this.model.on( 'tva_render_content', function () {
					self.render();
				} );
			},
			render: function () {
				this.$el.empty();
				var status = 'tva-status-red',
					unpublished_lessons = false;

				if ( this.model.get( 'post_status' ) === 'publish' ) {
					var children = this.model.hasChildren();

					if ( children ) {
						var items_status = this.model.get( children ).listValue( 'post_status' ),
							drafts = _.find( items_status, function ( item_status ) {
								return item_status === 'draft'
							} );

						status = drafts ? 'tva-status-yellow' : 'tva-status-green';
					}
				}


				if ( this.model.hasChildren() === 'chapters' ) {
					this.model.get( 'chapters' ).each( function ( chapter ) {
						if ( unpublished_lessons === false ) {
							unpublished_lessons = ! ! chapter.get( 'lessons' ).findWhere( {post_status: 'draft'} );
						}
					} )
				}

				this.$el.html( this.template( {
					model: this.model,
					status: status,
					unpublished_lessons: unpublished_lessons
				} ) );
				this.$moduleTitle = this.$( '.tva-module-title' );
				this.renderContent();

				if ( ! this.model.get( 'expanded' ) ) {
					this.$el.addClass( 'tva-module-collapsed' );
				}

				return this;
			},
			animateItem: function () {
				ThriveApp.views.ItemsList.prototype.animateItem.apply( this, arguments );
				if ( this.expanded && ( this.expanded.get( 'ID' ) !== this.model.get( 'ID' ) ) ) {
					this.expanded.set( {expanded: false} );
				}
				ThriveApp.globals.active_course.trigger( 'tva_render_content' );
			},
			afterAddChapterModel: function () {
				this.chapter_model.set( {
					course_id: this.model.get( 'course_id' ),
					post_parent: this.model.get( 'ID' )
				} );
			},
			afterAddLessonModel: function () {
				this.lesson_model.set( {post_parent: this.model.get( 'ID' )} );
			},
			editModule: function ( e ) {
				e.stopPropagation();
				this.modal( ThriveApp.views.ModuleModalEditor, {
					model: new ThriveApp.models.Module( this.model.toDeepJson() ),
					collection: this.collection,
					'max-width': '850px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-default-modal-style tvd-modal',
					viewName: 'ModuleEditor'
				} );
			},
			deleteModule: function ( e ) {
				e.stopPropagation();
				ThriveApp.globals.selected_items.reset();
				this.modal( ThriveApp.views.ModuleModalDelete, {
					model: this.model,
					collection: this.collection,
					viewName: 'ModuleDelete',
					'max-width': '600px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					className: 'tva-delete-modal-style tvd-modal'
				} );
			},
			editTitle: function ( e ) {
				e.stopPropagation();
				var self = this,
					edit_btn = this.$el.find( '.tva-edit-module-name' ),
					edit_model = new Backbone.Model( {
						title: this.model.get( 'post_title' ),
						required: true
					} );
				edit_btn.hide();
				edit_model.on( 'change:value', function () {

					self.model.set( {post_title: edit_model.get( 'value' )} );
					self.save.apply( self, arguments );
					self.$moduleTitle.show();
					self.$el.find( '.tva-inline-edit' ).hide();
					edit_btn.css( 'display', 'inline-block' );
					self.$moduleTitle.text( edit_model.get( 'value' ) );


				} );
				edit_model.on( 'tvu_no_change', function () {
					self.$moduleTitle.html( self.model.get( 'post_title' ) ).show();
					self.$el.find( '.tva-inline-edit' ).remove();
					edit_btn.css( 'display', 'inline-block' );
				} );

				var textEdit = new ThriveApp.views.InputTitle( {
					model: edit_model,
					el: this.$el.find( '.tva-module-title-container' ),
					tagName: 'div',
					method: 'append'
				} );

				this.$moduleTitle.hide();

				textEdit.render();
				textEdit.focus();
			},
			save: function () {
				TVE_Dash.showLoader();
				var xhr = this.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Chapter view
		 */
		ThriveApp.views.Chapter = ThriveApp.views.ItemsList.extend( {
			template: TVE_Dash.tpl( 'course/chapter/chapter' ),
			className: 'tva-chapter-item tva-single-item',
			allowedItems: [ 'lesson' ],
			$_list: null,
			events: function () {
				return _.extend( {}, ThriveApp.views.ItemsList.prototype.events, {
					'click .tva-expand-chapter': 'expandChapter',
					'click .tva-collapse-chapter': 'collapseChapter',
					'click .tva-add-chapter-lesson': 'addLesson',
					'click .tva-delete-chapter': 'deleteChapter',
					'click .tva-edit-chapter-name': 'editTitle',
					'click .tva-drag-chapter': function ( e ) {
						e.stopPropagation();
					},
					'mousedown .tva-drag-chapter': function () {
						ThriveApp.globals.moved_item = this.model;
					}
				} );
			},
			initialize: function () {
				var self = this;

				this.listenTo( this.model, 'change:checked', this.setChecked );

				this.model.on( 'tva_render_content', function () {
					self.render();
				} );

				this.$el.attr( 'data-id', this.model.get( 'ID' ) );

				return this;
			},
			render: function () {
				this.$el.empty();
				var status = 'tva-status-red';

				if ( this.model.get( 'post_status' ) === 'publish' ) {
					status = this.model.get( 'lessons' ).findWhere( {post_status: 'draft'} ) ? 'tva-status-yellow' : 'tva-status-green';
				}

				this.$el.html( this.template( {model: this.model, status: status} ) );
				this.$chapterTitle = this.$( '.tva-chapter-title' );

				this.renderContent();

				if ( ! this.model.get( 'expanded' ) ) {
					this.$el.addClass( 'tva-chapter-collapsed' );
				}

				return this;
			},
			afterAddLessonModel: function () {
				this.lesson_model.set( {post_parent: this.model.get( 'ID' )} );
			},
			deleteChapter: function () {
				ThriveApp.globals.selected_items.reset();
				this.modal( ThriveApp.views.ChapterModalDelete, {
					model: this.model,
					collection: this.collection,
					'max-width': '600px',
					width: 'auto',
					in_duration: 200,
					out_duration: 300,
					viewName: 'ChapterDelete',
					className: 'tva-delete-modal-style tvd-modal'
				} );
			},
			editTitle: function ( e ) {
				e.stopPropagation();
				var self = this,
					edit_btn = this.$el.find( '.tva-edit-chapter-name' ),
					edit_model = new Backbone.Model( {
						title: this.model.get( 'post_title' ),
						required: true
					} );
				edit_btn.hide();
				edit_model.on( 'change:value', function () {

					self.model.set( {post_title: edit_model.get( 'value' )} );
					self.save.apply( self, arguments );
					self.$chapterTitle.show();
					self.$el.find( '.tva-inline-edit' ).remove();
					edit_btn.css( 'display', 'inline-block' );
					self.$chapterTitle.text( edit_model.get( 'value' ) );
				} );
				edit_model.on( 'tvu_no_change', function () {
					self.$chapterTitle.html( self.model.get( 'post_title' ) ).show();
					self.$el.find( '.tva-inline-edit' ).remove();
					edit_btn.css( 'display', 'inline-block' );
				} );

				var textEdit = new ThriveApp.views.InputTitle( {
					model: edit_model,
					el: this.$el.find( '.tva-chapter-title-container' ),
					tagName: 'div',
					method: 'append'
				} );

				this.$chapterTitle.hide();
				textEdit.render();
				textEdit.focus();
			},
			animateItem: function () {
				ThriveApp.views.ItemsList.prototype.animateItem.apply( this, arguments );
				ThriveApp.globals.active_course.trigger( 'tva_render_content' );
			},
			save: function () {
				TVE_Dash.showLoader();
				var xhr = this.model.save();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Lesson view
		 */
		ThriveApp.views.Lesson = ThriveApp.views.ItemsList.extend( {
			initialize: function () {
				this.listenTo( this.model, 'change:state', this.renderState );
				this.listenTo( this.model, 'change:checked', this.setChecked );
			},
			render: function () {
				this.renderState();

				return this;
			},
			renderState: function () {
				var state = this.model.get( 'state' );
				if ( ! ThriveApp.views[ 'Lesson' + ThriveApp.util.upperFirst( state ) + 'State' ] ) {
					return;
				}

				var view = new ThriveApp.views[ 'Lesson' + ThriveApp.util.upperFirst( state ) + 'State' ]( {
					model: this.model,
					collection: this.collection
				} );

				view.render();
				this.$el.replaceWith( view.$el );
				this.setElement( view.$el );

				return this;
			}
		} );

		/**
		 * Lesson normal state
		 */
		ThriveApp.views.LessonNormalState = ThriveApp.views.Lesson.extend( {
			className: 'tva-lesson-item tva-single-item',
			template: TVE_Dash.tpl( 'course/edit/lesson-normal' ),
			events: {
				'click .tva-edit-lesson': 'editLesson',
				'click .tva-delete-lesson': 'changeState',
				'click .tva-publish-lesson': 'publishLesson',
				'click .tva-edit-lesson-name': 'editTitle',
				'click .tva-select-lesson': 'toggleSelectedClass',
				'mousedown .tva-drag-lesson': function () {
					ThriveApp.globals.moved_item = this.model;
				}
			},
			changed_models: [],
			initialize: function () {

				this.$el.attr( 'data-id', this.model.get( 'ID' ) );

				this.listenTo( this.model, 'remove', function () {
					this.collection.remove( this.model );
					this.$el.remove();
				} );
			},
			render: function () {
				var published = this.collection.where( {post_status: 'publish'} );

				this.model.set( {tooltiped: false} );

				if ( published.length === 0 && this.collection.at( 0 ) == this.model ) {
					this.model.set( {tooltiped: true} );
				}

				this.$el.html( this.template( {model: this.model} ) );

				this.$lessonTitle = this.$( '.tva-lesson-title' );

				return this;
			},
			editLesson: function ( e ) {
				if ( $( '.tva-course-section' ).hasClass( 'tva-course-editor' ) ) {
					$( '.tva-cancel-edit' ).click();
				}

				var id = $( e.currentTarget ).attr( 'data-id' ),
					model = this.collection.findWhere( {ID: parseInt( id )} ),
					clone_model = new ThriveApp.models.Lesson( model.toDeepJSON() );

				this.modal( ThriveApp.views.LessonModalEditor, {
					model: clone_model,
					collection: this.collection,
					'max-width': '60%',
					width: '800px',
					in_duration: 200,
					out_duration: 0,
					className: 'tva-default-modal-style tvd-modal',
					viewName: 'LessonEditor'
				} );
			},
			changeState: function () {
				var model = this.collection.findWhere( {state: ThriveApp.t.delete_state} );

				if ( model ) {
					model.set( {state: ThriveApp.t.normal_state} );
				}

				this.model.set( {state: ThriveApp.t.delete_state} )
			},
			publishLesson: function () {
				var current = this.model.get( 'post_status' ),
					post_status = current === 'draft' ? 'publish' : 'draft';

				this.changed_models = this.model.changeParentsStatus();

				this.model.set( {post_status: post_status} );
				this.save();

				return this;
			},
			editTitle: function ( e ) {
				var self = this,
					edit_btn = this.$el.find( '.tva-edit-lesson-name' ),
					edit_model = new Backbone.Model( {
						title: this.model.get( 'post_title' ),
						required: true
					} );
				edit_btn.hide();
				edit_model.on( 'change:value', function () {

					self.model.set( {post_title: edit_model.get( 'value' )} );
					self.save.apply( self, arguments );
					self.$lessonTitle.show();
					self.$el.find( '.tva-inline-edit' ).hide();
					edit_btn.show();
				} );
				edit_model.on( 'tvu_no_change', function () {
					self.$lessonTitle.html( self.model.get( 'post_title' ) ).show();
					self.$el.find( '.tva-inline-edit' ).remove();
					edit_btn.show();
				} );

				var textEdit = new ThriveApp.views.InputTitle( {
					model: edit_model,
					el: this.$el.find( '.tva-title-container' ),
					tagName: 'div',
					method: 'prepend'
				} );

				this.$lessonTitle.hide();
				textEdit.render();
				textEdit.focus();
			},
			save: function () {
				TVE_Dash.showLoader();
				var xhr = this.model.save(),
					self = this;

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {

						/**
						 * It's very sad that this need to be here but seems that after every save options object
						 * which holds all xhr data is inserted in the model
						 * having as key response var: https://www.screencast.com/t/KGRDtctLreL
						 * This leeds to issues and unexpected behaviour if the model need to be used later on
						 * into a normal ajax call
						 */

						if ( self.model.get( response ) ) {
							self.model.unset( response )
						}

						if ( self.changed_models.length > 0 ) {
							_.each( self.changed_models, function ( model ) {
								model.save();
							} );
						}

						ThriveApp.globals.active_course.trigger( 'tva_render_content' );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.err( JSON.parse( errorObj.responseText ).message );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );

		/**
		 * Delete a lesson state
		 */
		ThriveApp.views.LessonDeleteState = ThriveApp.views.Base.extend( {
			className: 'tvd-col tvd-s12 tva-lesson-item',
			template: TVE_Dash.tpl( 'course/edit/lesson-delete' ),
			events: {
				'click .tva-delete-no': function () {
					this.model.set( {state: ThriveApp.t.normal_state} );
				},
				'click .tva-delete-yes': 'deleteLesson'
			},
			changed_models: [],
			initialize: function () {
				_.bindAll( this, 'keyAction' );
				$( document ).bind( 'keydown', this.keyAction );
			},
			render: function () {
				this.$el.html( this.template( {model: this.model} ) );

				if ( this.model.get( 'post_status' ) === 'publish' ) {
					this.$el.addClass( 'tva-lesson-pb' )
				}

				return this;
			},
			keyAction: function ( e ) {
				var code = e.which;
				if ( code == 13 ) {
					this.deleteLesson();
				} else if ( code == 27 ) {
					this.model.set( {state: ThriveApp.t.normal_state} );
				}
			},
			deleteLesson: function () {
				TVE_Dash.cardLoader( this.$el );
				/**
				 * We should only change the status if the lesson is the last one and is published
				 * or if we have more than one lesson in the parents
				 */
				if ( this.model.get( 'post_status' ) === 'publish' ) {
					this.changed_models = this.model.changeParentsStatus();
				}

				var self = this,
					xhr = self.model.destroy();

				if ( xhr ) {
					xhr.done( function ( response, status, options ) {
						TVE_Dash.success( ThriveApp.t.ItemDeleted );
						$( document ).unbind( 'keydown', this.keyAction );
						self.collection.remove( self.model );

						if ( self.changed_models.length > 0 ) {
							_.each( self.changed_models, function ( model ) {
								model.save();
							} );
						}

						ThriveApp.globals.active_course.trigger( 'tva_render_content' );
					} );
					xhr.error( function ( errorObj ) {
						TVE_Dash.hideLoader();
						TVE_Dash.err( errorObj.responseText );
					} );
					xhr.always( function () {
						TVE_Dash.hideLoader();
					} );
				}
			}
		} );
	} )

} )( jQuery );;/**
 * Created by Pop Aurelian on 21-Feb-17.
 */

var ThriveApp = ThriveApp || {};
ThriveApp.globals = ThriveApp.globals || {};

( function ( $ ) {
	var Router = Backbone.Router.extend( {
		$el: $( '#tva-admin-wrapper' ),
		routes: {
			'greeting': 'greeting',
			'wizard': 'wizard',
			'template_settings': 'template_settings',
			'label_management': 'label_management',
			'import': 'import',
			'dashboard': 'dashboard',
			'topics': 'editTopics',
			'purge-cache': 'purgeCache',
			'add_course': 'addCourse',
			'sendowl_settings': 'sendowl_settings',
			'sendowl_quick_start': 'sendowl_quick_start',
			'sendowl_checkout': 'sendowl_checkout',
			'sendowl_links': 'sendowl_links',
			'sendowl_integration': 'sendowl_integration',
			'sendowl_logs': 'sendowl_logs',
			'sendowl_stacks': 'sendowl_stacks',
			'sendowl_customers': 'sendowl_customers',
			'course/:id': 'course'
		},
		greeting_view: null,
		wizard_view: null,
		dash_view: null,
		course_view: null,
		topics_view: null,
		sendowl_view: null,
		sendowl_quick_start_view: null,
		sendowl_checkout_view: null,
		sendowl_links_view: null,
		sendowl_integration_view: null,
		sendowl_logs_view: null,
		sendowl_stacks_view: null,
		sendowl_customers_view: null,
		breadcrumbs: {
			col: null,
			view: null
		},
		/**
		 * set the current page - adds the structure to breadcrumbs and sets the new document title
		 *
		 * @param {string} section page hierarchy
		 * @param {string} label current page label
		 *
		 * @param {Array} [structure] optional the structure of the links that lead to the current page
		 */
		set_page: function ( section, label, structure ) {
			this.breadcrumbs.col.reset();
			structure = structure || {};

			/* Thrive Admin Dashboard is always the first element */
			this.breadcrumbs.col.add_page( ThriveApp.dash_url, ThriveApp.t.Thrive_Dashboard, true );

			_.each( structure, _.bind( function ( item ) {
				this.breadcrumbs.col.add_page( item.route, item.label );
			}, this ) );

			/**
			 * last link - no need for route
			 */
			this.breadcrumbs.col.add_page( '', label );

			/* update the page title */
			var $title = $( 'head > title' );
			if ( ! this.original_title ) {
				this.original_title = $title.html();
			}

			$title.html( label + ' &lsaquo; ' + this.original_title )
		},

		/**
		 * Greeting view
		 */
		greeting: function () {
			$( '.tvd-material-tooltip' ).hide();

			this.renderHeader();

			if ( this.greeting_view ) {
				this.greeting_view.remove();
			}

			this.greeting_view = new ThriveApp.views.Start( {
				model: ThriveApp.globals.settings
			} );

			this.$el.html( this.greeting_view.render().$el );

			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

		},

		/**
		 * Wizard view
		 */
		wizard: function () {
			this.set_page( 'dashboard', ThriveApp.t.TemplateSettings );
			$( '.tvd-material-tooltip' ).hide();
			this.hideAdminBar();

			TVE_Dash.showLoader();

			this.renderHeader();

			if ( this.wizard_view ) {
				this.wizard_view.remove();
			}

			this.wizard_view = new ThriveApp.views.Wizard( {
				model: ThriveApp.globals.settings,
				collection: new ThriveApp.collections.Templates( ThriveApp.data.templates )
			} );

			this.$el.html( this.wizard_view.render().$el );

			window.scrollTo( 0, 0 );
		},
		addFullHeight: function () {
			$( '#wpwrap' ).addClass( 'tva-fullheight' );
		},
		removeFullHeight: function () {
			$( '#wpwrap' ).removeClass( 'tva-fullheight' );
		},

		/**
		 * Sendowl settings page
		 */
		sendowl_settings: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'SendOwl Settings', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				}
			] );

			if ( this.sendowl_view ) {
				this.sendowl_view.remove();
			}

			this.renderHeader();

			this.sendowl_view = new ThriveApp.views.SendowlSettings( {} );

			this.$el.html( this.sendowl_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		/**
		 * Sendowl quick start guide
		 */
		sendowl_quick_start: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'Quick Start Guide', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_quick_start_view ) {
				this.sendowl_quick_start_view.remove();
			}

			this.renderHeader();

			this.sendowl_quick_start_view = new ThriveApp.views.SendowlQuickStart( {
				model: ThriveApp.globals.settings
			} );

			this.$el.html( this.sendowl_quick_start_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_checkout: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'Checkout Settings', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_checkout_view ) {
				this.sendowl_checkout_view.remove();
			}

			this.renderHeader();

			this.sendowl_checkout_view = new ThriveApp.views.SendowlCheckout( {
				model: ThriveApp.globals.settings
			} );

			this.$el.html( this.sendowl_checkout_view.render().$el );
			ThriveApp.util.rebindWistiaFancyBoxes();
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_links: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'Purchase Links', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_links_view ) {
				this.sendowl_links_view.remove();
			}

			this.renderHeader();

			this.sendowl_links_view = new ThriveApp.views.SendowlLinks( {
				model: ThriveApp.globals.settings
			} );

			this.$el.html( this.sendowl_links_view.render().$el );
			ThriveApp.util.rebindWistiaFancyBoxes();
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_integration: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'Integration Settings', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_integration_view ) {
				this.sendowl_integration_view.remove();
			}

			this.renderHeader();

			this.sendowl_integration_view = new ThriveApp.views.SendowlIntegration( {
				model: ThriveApp.globals.settings
			} );

			this.$el.html( this.sendowl_integration_view.render().$el );
			ThriveApp.util.rebindWistiaFancyBoxes();
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_logs: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'SendOwl Logs', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_logs_view ) {
				this.sendowl_logs_view.remove();
			}

			this.renderHeader();
			this.sendowl_logs_view = new ThriveApp.views.SendowlLogs( {
				collection: ThriveApp.globals.logs
			} );

			this.$el.html( this.sendowl_logs_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_stacks: function () {
			TVE_Dash.showLoader();

			this.set_page( '', 'SendOwl Stacks', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_stacks_view ) {
				this.sendowl_stacks_view.remove();
			}

			this.renderHeader();

			this.sendowl_stacks_view = new ThriveApp.views.SendowlStacks( {
				collection: ThriveApp.globals.stacks
			} );

			this.$el.html( this.sendowl_stacks_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();

			TVE_Dash.hideLoader();
		},

		sendowl_customers: function() {
			TVE_Dash.showLoader();

			this.set_page( '', 'SendOwl Customers List', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				},
				{
					route: 'sendowl_settings',
					label: ThriveApp.t.SendowlSettings
				}
			] );

			if ( this.sendowl_customers_view ) {
				this.sendowl_customers_view.remove();
			}

			this.renderHeader();

			this.sendowl_customers_view = new ThriveApp.views.SendowlCustomers( {
				model: new ThriveApp.models.SendowlCustomer(),
				collection: ThriveApp.globals.sendowl_customers
			} );

			this.$el.html( this.sendowl_customers_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();
		},

		/**
		 * Template view same as Wizard view but we're sending a flag and making the route look better
		 *
		 */
		template_settings: function () {

			var prev_url = ThriveApp.globals.settings.get( 'preview_url' );
			if ( prev_url === false ) {
				ThriveApp.router.navigate( "#dashboard", {trigger: true} );

				return;
			}

			$( '.tva-import-wrapper' ).css( {
				"zoom": "1",
				"-moz-transform": "scale(1)",
				"-moz-transform-origin": "left top"
			} );

			var self = this;
			$( '.tvd-material-tooltip' ).hide();
			this.addFullHeight();
			TVE_Dash.showLoader();

			$( '#wpwrap' ).addClass( 'tva-fullheight' );

			this.renderHeader();
			this.breadcrumbs.col.reset();
			this.breadcrumbs.view.render();
			this.hideAdminBar();
			ThriveApp.globals.available_settings.clear().set( ThriveApp.globals.available_settings.defaults );
			ThriveApp.globals.available_settings.fetch().success( function ( response ) {
				if ( self.wizard_view ) {
					self.wizard_view.remove();
				}

				ThriveApp.globals.settings.set( {iframe_url: response.preview_url, current_page: 'template'} );

				var settings_model = ThriveApp.globals.settings,
					settings_clone = new ThriveApp.models.Settings( settings_model.toDeepJSON() );

				self.wizard_view = new ThriveApp.views.Wizard( {
					model: settings_clone,
					original_model: settings_model,
					collection: new ThriveApp.collections.Templates( ThriveApp.data.templates ),
					edit: true
				} );

				self.$el.html( self.wizard_view.render().$el );

				window.scrollTo( 0, 0 );
			} );


		},

		import: function () {
			this.set_page( '', ThriveApp.t.Import, [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				}
			] );
			this.showAdminBar();

			$( "#tva-admin-wrapper" ).addClass( "tva-import-wrapper" );

			if ( $( window ).width() < 1400 ) {
				this.hideScrollBar();

				$( '.tva-import-wrapper' ).css( {
					"zoom": "0.6",
					"-moz-transform": "scale(0.6)",
					"-moz-transform-origin": "left -75px"
				} );
			}

			this.renderHeader();
			var self = this,
				collection = new ThriveApp.collections.ImportCourses();

			collection.fetch().success( function () {
				if ( self.import_view ) {
					self.import_view.remove();
				}

				self.import_view = new ThriveApp.views.Import( {
					model: ThriveApp.globals.settings,
					collection: collection
				} );

				self.$el.html( self.import_view.render().$el );
			} );
		},
		/**
		 * Dashboard Route
		 */
		dashboard: function () {

			$( '.tva-import-wrapper' ).css( {
				"zoom": "1",
				"-moz-transform": "scale(1)",
				"-moz-transform-origin": "left top"
			} );

			window.removeEventListener( 'scroll', ThriveApp.util.makeBulkActionsStiky );
			ThriveApp.globals.selected_items.reset();

			this.set_page( 'dashboard', ThriveApp.t.Dashboard );
			this.removeFullHeight();
			$( '.tvd-material-tooltip' ).hide();
			this.showAdminBar();

			TVE_Dash.showLoader();

			if ( this.dash_view ) {
				this.dash_view.remove();
			}

			this.renderHeader();

			this.dash_view = new ThriveApp.views.Dashboard( {
				model: ThriveApp.globals.settings,
				collection: ThriveApp.globals.courses
			} );

			this.$el.html( this.dash_view.render().$el );
			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();
		},
		/**
		 * Add a course route
		 */
		addCourse: function () {
			TVE_Dash.showLoader();
			$( '.tvd-material-tooltip' ).hide();
			this.set_page( '', 'Add Course', [
				{
					route: 'purge-cache',
					label: ThriveApp.t.Dashboard
				}
			] );

			if ( this.course_view ) {
				this.course_view.remove();
			}

			this.renderHeader();

			this.course_view = new ThriveApp.views.addCourse( {
				model: new ThriveApp.models.Course( {
					order: ThriveApp.globals.courses.length,
					comment_status: ThriveApp.globals.settings.get( 'comment_status' )
				} ),
				collection: ThriveApp.globals.courses
			} );

			this.$el.html( this.course_view.render().$el );
			window.scrollTo( 0, 0 );
		},
		/**
		 * Editing a course route
		 *
		 * @param id
		 */
		course: function ( id ) {
			TVE_Dash.showLoader();

			if ( this.view ) {
				this.view.remove();
			}

			$( '.tvd-material-tooltip' ).hide();

			if ( ! id ) {
				ThriveApp.router.navigate( '#dashboard', {trigger: true} );
				return;
			}

			ThriveApp.globals.active_course = ThriveApp.globals.courses.findWhere( {ID: parseInt( id )} );

			if ( ! ThriveApp.globals.active_course ) {
				ThriveApp.router.navigate( '#dashboard', {trigger: true} );
				return;
			}

			ThriveApp.globals.course_elements = ThriveApp.util.getCourseElements( ThriveApp.globals.active_course );

			this.set_page( '', ThriveApp.globals.active_course.get( 'name' ), [
				{
					route: 'purge-cache',
					label: ThriveApp.t.Dashboard
				}
			] );

			this.renderHeader();

			ThriveApp.globals.active_course_original = new ThriveApp.models.Course(
				JSON.parse( JSON.stringify( ThriveApp.globals.active_course ) )
			);

			this.view = new ThriveApp.views.ManageCourse( {
				model: ThriveApp.globals.active_course,
				collection: ThriveApp.globals.courses
			} );

			this.$el.html( this.view.render().$el );

			window.scrollTo( 0, 0 );
			TVE_Dash.hideLoader();
		},
		/**
		 * Edit topics route
		 */
		editTopics: function () {


			$( '.tva-import-wrapper' ).css( {
				"zoom": "1",
				"-moz-transform": "scale(1)",
				"-moz-transform-origin": "left top"
			} );

			window.removeEventListener( 'scroll', ThriveApp.util.makeBulkActionsStiky );

			TVE_Dash.showLoader();
			$( '.tvd-material-tooltip' ).hide();
			this.set_page( '', 'Topics', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				}
			] );
			this.renderHeader();

			this.topics_view = new ThriveApp.views.TopicsManagement( {
				collection: ThriveApp.globals.topics
			} );

			this.$el.html( this.topics_view.render().$el );


			TVE_Dash.hideLoader();

			this.showAdminBar();

		},

		/**
		 * Route for the labels management
		 */
		label_management: function () {

			TVE_Dash.showLoader();

			this.renderHeader();
			this.set_page( '', 'Labels Management', [
				{
					route: 'dashboard',
					label: ThriveApp.t.Dashboard
				}
			] );

			this.labels_view = new ThriveApp.views.LabelManagement( {
				collection: ThriveApp.globals.labels
			} );

			this.$el.html( this.labels_view.render().$el );

			TVE_Dash.hideLoader();

			this.showAdminBar();

		},

		/**
		 * Creates a view for the header
		 */
		renderHeader: function () {
			if ( ! this.header ) {
				this.header = new ThriveApp.views.Header( {
					el: '.tva-header',
					model: ThriveApp.globals.settings
				} );
			} else {
				this.header.setElement( $( '.tva-header' ) );
			}

			this.header.render();

		},
		init_breadcrumbs: function () {
			this.breadcrumbs.col = new ThriveApp.collections.Breadcrumbs();
			this.breadcrumbs.view = new ThriveApp.views.Breadcrumbs( {
				collection: this.breadcrumbs.col
			} )
		},
		hideAdminBar: function () {
			$( '#adminmenuwrap, #adminmenuback, #wpfooter ' ).hide();
			$( '#wpcontent' ).css( {margin: 0} );
			$( 'body' ).css( {overflowY: 'hidden'} );
			$( 'body' ).css( {overflowX: 'hidden'} );
		},
		hideScrollBar: function () {
			$( 'body' ).css( {overflowY: 'hidden'} );
			$( 'body' ).css( {overflowX: 'hidden'} );
		},
		showAdminBar: function () {
			$( '#adminmenuwrap, #adminmenuback, #wpfooter' ).css( 'display', '' );
			$( '#wpcontent' ).css( 'margin-left', '' );
			$( 'body' ).css( {overflowY: 'scroll'} );
			$( 'body' ).css( {overflowX: 'auto'} );
		},
		purgeCache: function () {
			TVE_Dash.showLoader();
			$.ajax( {
				headers: {
					'X-WP-Nonce': ThriveApp.nonce
				},
				type: 'GET',
				url: ThriveApp.routes.courses
			} ).done( _.bind( function ( response ) {
				ThriveApp.globals.courses.reset( response );
				this.navigate( "#dashboard", {trigger: true} );
			}, this ) ).always( function () {
				TVE_Dash.hideLoader();
			} );
		}
	} );
	/**
	 * DOM Ready
	 */
	$( function () {

		ThriveApp.globals.topics = new ThriveApp.collections.TopicsCollection( ThriveApp.data.topics );
		ThriveApp.globals.labels = new ThriveApp.collections.LabelsCollection( ThriveApp.data.labels );
		ThriveApp.globals.levels = new ThriveApp.collections.Levels( ThriveApp.data.levels );
		ThriveApp.globals.roles = new ThriveApp.collections.Roles( ThriveApp.data.defaults.roles );
		ThriveApp.globals.courses = new ThriveApp.collections.Courses( ThriveApp.data.courses );
		ThriveApp.globals.settings = new ThriveApp.models.Settings( ThriveApp.data.settings );
		ThriveApp.globals.settings.set( {membership_plugin: new ThriveApp.collections.MembershipsCollection( ThriveApp.globals.settings.get( 'membership_plugin' ) )} );
		ThriveApp.globals.available_settings = new ThriveApp.models.AvailableSettings();
		ThriveApp.globals.fonts = new ThriveApp.collections.Fonts( ThriveApp.data.fonts );
		ThriveApp.globals.googlefonts = new ThriveApp.collections.GoogleFonts( ThriveApp.util.FontManager.get_google_fonts() );
		ThriveApp.globals.tva_iframe_loaded = '';
		ThriveApp.globals.selected_items = new ThriveApp.collections.SelectedItems();
		ThriveApp.globals.bulk_action_model = new ThriveApp.models.BulkActions();
		ThriveApp.globals.moved_item = null;
		ThriveApp.globals.receiver_item_model = null;
		ThriveApp.globals.sendowl_customers = new ThriveApp.collections.SendowlCustomers( ThriveApp.data.customers );

		/**
		 * Logs Data
		 */
		ThriveApp.globals.logs = new ThriveApp.collections.Logs( ThriveApp.data.logs );
		ThriveApp.globals.stacks = new ThriveApp.collections.Stacks( ThriveApp.data.stacks );
		ThriveApp.globals.log_types = new ThriveApp.collections.Types( ThriveApp.data.log_types );
		ThriveApp.globals.stack_types = new ThriveApp.collections.Types( ThriveApp.data.stack_types );
		ThriveApp.globals.logs_settings = new ThriveApp.models.LogsSettings( ThriveApp.data.logs_settings );

		ThriveApp.router = new Router;
		ThriveApp.router.init_breadcrumbs();
		Backbone.history.start( {hashchange: true} );

		if ( ! ThriveApp.wizard ) {
			ThriveApp.router.navigate( '#greeting', {trigger: true} );
		}

		if ( ! Backbone.history.fragment ) {
			ThriveApp.router.navigate( '#dashboard', {trigger: true} );
		}
	} );

} )( jQuery );